@@ -6,23 +6,36 @@
  * Descr.: boolean and Meta-constraint predicate management - C part       *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: fd_bool_c.c,v 1.15 2009/01/23 11:24:13 diaz Exp $ */
+/* $Id: fd_bool_c.c,v 1.18 2011/03/28 16:20:05 diaz Exp $ */
 
 #define OBJ_INIT Fd_Bool_Initializer
 
@@ -834,7 +847,7 @@ Add_Fd_Variables(WamWord e_word)
   if (tag_mask == TAG_STC_MASK)
     {
       adr = UnTag_STC(word);
-      
+
       i = Arity(adr);
       do
 	Add_Fd_Variables(Arg(adr, --i));
@@ -862,7 +875,7 @@ Add_Fd_Variables(WamWord e_word)
 static Bool
 Load_Bool_Into_Word(WamWord *exp, int result, WamWord *load_word)
 {
-  unsigned long op = *exp;
+  PlULong op = *exp;
 
   if (op >= EQ_F && op <= LTE_F)
     {
@@ -1155,7 +1168,7 @@ Set_Or(WamWord *exp, int result, WamWord *load_word)
 
 				/* L \/ R = B */
   *load_word = Tag_REF(Pl_Fd_New_Bool_Variable());
-  BOOL_CSTR_3(pl_x_or_y_eq_b, load_l, load_r, *load_word);	
+  BOOL_CSTR_3(pl_x_or_y_eq_b, load_l, load_r, *load_word);
   return TRUE;
 }
 
@@ -1201,7 +1214,7 @@ Set_Eq(WamWord *exp, int result, WamWord *load_word)
   WamWord le_word, re_word;
   int mask;
   WamWord l_word, r_word;
-  long c;
+  PlLong c;
 
   le_word = exp[1];
   re_word = exp[2];
@@ -1273,7 +1286,7 @@ Set_Neq(WamWord *exp, int result, WamWord *load_word)
   WamWord le_word, re_word;
   int mask;
   WamWord l_word, r_word;
-  long c;
+  PlLong c;
 
   le_word = exp[1];
   re_word = exp[2];
@@ -1346,7 +1359,7 @@ Set_Lt(WamWord *exp, int result, WamWord *load_word)
   WamWord le_word, re_word;
   int mask;
   WamWord l_word, r_word;
-  long c;
+  PlLong c;
 
   le_word = exp[1];
   re_word = exp[2];
@@ -1418,7 +1431,7 @@ Set_Lte(WamWord *exp, int result, WamWord *load_word)
   WamWord le_word, re_word;
   int mask;
   WamWord l_word, r_word;
-  long c;
+  PlLong c;
 
   le_word = exp[1];
   re_word = exp[2];
