@@ -6,23 +6,36 @@
  * Descr.: mathematical predicate management - C part                      *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: fd_math_c.c,v 1.13 2009/01/23 11:24:13 diaz Exp $ */
+/* $Id: fd_math_c.c,v 1.16 2011/03/28 16:20:05 diaz Exp $ */
 
 #include "engine_pl.h"
 #include "bips_pl.h"
@@ -72,7 +85,7 @@ Pl_Fd_Eq_2(WamWord le_word, WamWord re_word)
 {
   int mask;
   WamWord l_word, r_word;
-  long c;
+  PlLong c;
 
 
 #ifdef DEBUG
@@ -132,7 +145,7 @@ Pl_Fd_Neq_2(WamWord le_word, WamWord re_word)
 {
   int mask;
   WamWord l_word, r_word;
-  long c;
+  PlLong c;
 
 
 #ifdef DEBUG
@@ -200,7 +213,7 @@ Pl_Fd_Lt_2(WamWord le_word, WamWord re_word)
 {
   int mask;
   WamWord l_word, r_word;
-  long c;
+  PlLong c;
 
 
 #ifdef DEBUG
@@ -266,7 +279,7 @@ Pl_Fd_Lte_2(WamWord le_word, WamWord re_word)
 {
   int mask;
   WamWord l_word, r_word;
-  long c;
+  PlLong c;
 
 
 #ifdef DEBUG
