@@ -6,23 +6,36 @@
  * Descr.: FD variable values management - C part                          *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: fd_values_c.c,v 1.15 2009/01/23 11:24:13 diaz Exp $ */
+/* $Id: fd_values_c.c,v 1.19 2011/03/28 16:20:05 diaz Exp $ */
 
 #include <stdlib.h>
 
@@ -97,17 +110,17 @@ static Bool Cmp_Max_Regret(WamWord *last_fdv_adr, WamWord *new_fdv_adr);
 
 
 
-#define INDOMAIN_MIN_ALT           X24696E646F6D61696E5F6D696E5F616C74
+#define INDOMAIN_MIN_ALT           X1_24696E646F6D61696E5F6D696E5F616C74
 
-#define INDOMAIN_MAX_ALT           X24696E646F6D61696E5F6D61785F616C74
+#define INDOMAIN_MAX_ALT           X1_24696E646F6D61696E5F6D61785F616C74
 
-#define INDOMAIN_MIDDLE_ALT        X24696E646F6D61696E5F6D6964646C655F616C74
+#define INDOMAIN_MIDDLE_ALT        X1_24696E646F6D61696E5F6D6964646C655F616C74
 
-#define INDOMAIN_LIMITS_ALT        X24696E646F6D61696E5F6C696D6974735F616C74
+#define INDOMAIN_LIMITS_ALT        X1_24696E646F6D61696E5F6C696D6974735F616C74
 
-#define INDOMAIN_RANDOM_ALT        X24696E646F6D61696E5F72616E646F6D5F616C74
+#define INDOMAIN_RANDOM_ALT        X1_24696E646F6D61696E5F72616E646F6D5F616C74
 
-#define EXTRA_CSTR_ALT             X2465787472615F637374725F616C74
+#define EXTRA_CSTR_ALT             X1_2465787472615F637374725F616C74
 
 
 
@@ -184,7 +197,7 @@ Pl_Fd_Domain_2(WamWord list_word, WamWord r_word)
 
 
   DEREF(list_word, word, tag_mask);
-  if (tag_mask == TAG_REF_MASK || tag_mask == TAG_INT_MASK || 
+  if (tag_mask == TAG_REF_MASK || tag_mask == TAG_INT_MASK ||
       tag_mask == TAG_FDV_MASK)
     return pl_fd_domain_r(word, r_word);
 
@@ -424,7 +437,7 @@ Pl_Indomain_Max_Alt_0(void)
 
   if (cur != end)
     {
-#if 0 /* the following data is unchanged */ 
+#if 0 /* the following data is unchanged */
       AB(B, 0) = (WamWord) fdv_adr;
       AB(B, 1) = (WamWord) range;
       AB(B, 2) = end;
@@ -819,7 +832,7 @@ Pl_Fd_Sel_Array_Pick_Var_4(WamWord sel_array_word, WamWord method_word,
   WamWord **array;
   WamWord **p, **end;
   CmpFct cmp_meth;
-  long n;
+  PlLong n;
   int i;
   WamWord *fdv_adr;
   WamWord **res_elem = NULL;
@@ -832,7 +845,7 @@ Pl_Fd_Sel_Array_Pick_Var_4(WamWord sel_array_word, WamWord method_word,
 
   array = (WamWord **) (Cstr_Stack + Pl_Rd_Integer_Check(sel_array_word));
 
-  n = (long) array[0];
+  n = (PlLong) array[0];
   if (n == 0)
     return FALSE;
 
