@@ -6,23 +6,36 @@
  * Descr.: mathematical support                                            *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: math_supp.c,v 1.14 2009/01/23 11:24:13 diaz Exp $ */
+/* $Id: math_supp.c,v 1.17 2011/03/28 16:20:05 diaz Exp $ */
 
 #include <stdlib.h>
 #include <stdarg.h>
@@ -101,7 +114,7 @@
 
 typedef struct			/* Monomial term information      */
 {				/* ------------------------------ */
-  long a;			/* coefficient                    */
+  PlLong a;			/* coefficient                    */
   WamWord x_word;		/* variable a tagged <REF,adr>    */
 }
 Monom;
@@ -111,7 +124,7 @@ Monom;
 
 typedef struct			/* Polynomial term information    */
 {				/* ------------------------------ */
-  long c;			/* the constant                   */
+  PlLong c;			/* the constant                   */
   int nb_monom;			/* nb of monomial terms           */
   Monom m[MAX_MONOMS];		/* table of monomial terms        */
 }
@@ -166,7 +179,7 @@ static Bool Load_Term_Into_Word(WamWord e_word, WamWord *load_word);
 static WamWord Push_Delayed_Cstr(int cstr, WamWord a1, WamWord a2,
 				 WamWord a3);
 
-static void Add_Monom(Poly *p, int sign, long a, WamWord x_word);
+static void Add_Monom(Poly *p, int sign, PlLong a, WamWord x_word);
 
 #ifdef DEVELOP_TIMES_2
 static Bool Add_Multiply_Monom(Poly *p, int sign, Monom *m1, Monom *m2);
@@ -243,7 +256,7 @@ Math_Supp_Initializer(void)
  *-------------------------------------------------------------------------*/
 Bool
 Pl_Load_Left_Right(Bool optim_eq, WamWord le_word, WamWord re_word,
-		int *mask, long *c, WamWord *l_word, WamWord *r_word)
+		int *mask, PlLong *c, WamWord *l_word, WamWord *r_word)
 {
 #ifdef DEBUG
   DBGPRINTF("\n*** Math constraint : ");
@@ -307,7 +320,7 @@ Pl_Term_Math_Loading(WamWord l_word, WamWord r_word)
  *-------------------------------------------------------------------------*/
 static Bool
 Load_Left_Right_Rec(Bool optim_eq, WamWord le_word, WamWord re_word,
-		    int *mask, long *c, WamWord *l_word, WamWord *r_word)
+		    int *mask, PlLong *c, WamWord *l_word, WamWord *r_word)
 {
   Poly p;
   Monom *l_m, *r_m;
@@ -385,13 +398,13 @@ Load_Left_Right_Rec(Bool optim_eq, WamWord le_word, WamWord re_word,
   DBGPRINTF("normalization: ");
   for (i = 0; i < l_nb_monom; i++)
     {
-      DBGPRINTF("%ld*", l_m[i].a);
+      DBGPRINTF("%" PL_FMT_d "*", l_m[i].a);
       Pl_Write_1(l_m[i].x_word);
       DBGPRINTF(" + ");
     }
 
   if (p.c > 0)
-    DBGPRINTF("%ld + ", p.c);
+    DBGPRINTF("%" PL_FMT_d " + ", p.c);
   else if (l_nb_monom == 0)
     DBGPRINTF("0 + ");
 
@@ -399,13 +412,13 @@ Load_Left_Right_Rec(Bool optim_eq, WamWord le_word, WamWord re_word,
 
   for (i = 0; i < r_nb_monom; i++)
     {
-      DBGPRINTF("%ld*", r_m[i].a);
+      DBGPRINTF("%" PL_FMT_d "*", r_m[i].a);
       Pl_Write_1(r_m[i].x_word);
       DBGPRINTF(" + ");
     }
 
   if (p.c < 0)
-    DBGPRINTF("%ld + ", -p.c);
+    DBGPRINTF("%" PL_FMT_d " + ", -p.c);
   else if (r_nb_monom == 0 && re_word != NOT_A_WAM_WORD)
     DBGPRINTF("0 + ");
 
@@ -469,10 +482,10 @@ Load_Term_Into_Word(WamWord e_word, WamWord *load_word)
 {
   int mask;
   WamWord l_word, r_word, word;
-  long c;
+  PlLong c;
 
 
-  if (!Load_Left_Right_Rec(FALSE, e_word, NOT_A_WAM_WORD, &mask, &c, 
+  if (!Load_Left_Right_Rec(FALSE, e_word, NOT_A_WAM_WORD, &mask, &c,
 			   &l_word, &r_word))
     return FALSE;
 
@@ -546,7 +559,7 @@ Load_Term_Into_Word(WamWord e_word, WamWord *load_word)
 static int
 Compar_Monom(Monom *m1, Monom *m2)
 {
-  long cmp;
+  PlLong cmp;
 
   if (m1->a > 0)
     cmp = (m2->a > 0) ? m2->a - m1->a : -1;
@@ -593,7 +606,7 @@ Push_Delayed_Cstr(int cstr, WamWord a1, WamWord a2, WamWord a3)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 static void
-Add_Monom(Poly *p, int sign, long a, WamWord x_word)
+Add_Monom(Poly *p, int sign, PlLong a, WamWord x_word)
 {
   int i;
 
@@ -629,7 +642,7 @@ Add_Monom(Poly *p, int sign, long a, WamWord x_word)
 static Bool
 Add_Multiply_Monom(Poly *p, int sign, Monom *m1, Monom *m2)
 {
-  long a;
+  PlLong a;
   WamWord x_word;
 
   a = m1->a * m2->a;
@@ -686,7 +699,7 @@ Normalize(WamWord e_word, int sign, Poly *p)
   WamWord word1, word2, word3;
   WamWord f_n, le_word, re_word;
   int i;
-  long n1, n2, n3;
+  PlLong n1, n2, n3;
 
  terminal_rec:
 
@@ -731,9 +744,9 @@ Normalize(WamWord e_word, int sign, Poly *p)
   if (tag_mask != TAG_STC_MASK)
     goto type_error;
 
-  
+
   adr = UnTag_STC(word);
-  
+
   f_n = Functor_And_Arity(adr);
   for (i = 0; i < NB_OF_OP; i++)
     if (arith_tbl[i] == f_n)
