@@ -6,23 +6,36 @@
  * Descr.: all solution collector management - C part                      *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: all_solut_c.c,v 1.15 2009/01/23 11:24:13 diaz Exp $ */
+/* $Id: all_solut_c.c,v 1.21 2011/03/28 16:20:05 diaz Exp $ */
 
 #include <sys/types.h>
 
@@ -71,7 +84,7 @@ static WamWord new_gen_word;
 
 
 
-static long *bound_var_ptr;
+static PlLong *bound_var_ptr;
 static WamWord *free_var_base;
 
 
@@ -79,9 +92,9 @@ static WamWord *free_var_base;
 static OneSol dummy = { NULL, 0, 0 };
 static OneSol *sol = &dummy;
 
-static long *key_var_ptr;
-static long *save_key_var_ptr;
-static long *next_key_var_ptr;
+static PlLong *key_var_ptr;
+static PlLong *save_key_var_ptr;
+static PlLong *next_key_var_ptr;
 
 
 
@@ -90,17 +103,17 @@ static long *next_key_var_ptr;
  * Function Prototypes             *
  *---------------------------------*/
 
-static void Bound_Var(WamWord *adr);
+static Bool Bound_Var(WamWord *adr);
 
 static WamWord Existential_Variables(WamWord start_word);
 
-static void Free_Var(WamWord *adr);
+static Bool Free_Var(WamWord *adr);
 
 
 
 static void Handle_Key_Variables(WamWord start_word);
 
-static void Link_Key_Var(WamWord *adr);
+static Bool Link_Key_Var(WamWord *adr);
 
 
 
@@ -109,7 +122,7 @@ static WamWord Group(WamWord all_sol_word, WamWord gl_key_word,
 
 
 
-#define GROUP_SOLUTIONS_ALT       X2467726F75705F736F6C7574696F6E735F616C74
+#define GROUP_SOLUTIONS_ALT       X1_2467726F75705F736F6C7574696F6E735F616C74
 
 Prolog_Prototype(GROUP_SOLUTIONS_ALT, 0);
 
@@ -149,7 +162,7 @@ All_Solut_Initializer(void)
  *-------------------------------------------------------------------------*/
 Bool
 Pl_Free_Variables_4(WamWord templ_word, WamWord gen_word, WamWord gen1_word,
-		 WamWord key_word)
+		    WamWord key_word)
 {
   WamWord gl_key_word;
   WamWord *save_H, *arg;
@@ -205,19 +218,20 @@ Pl_Recover_Generator_1(WamWord gen1_word)
  * BOUND_VAR                                                               *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-static void
+static Bool
 Bound_Var(WamWord *adr)
 {
-  long *p;
+  PlLong *p;
 
   for (p = pl_glob_dico_var; p < bound_var_ptr; p++)
-    if (*p == (long) adr)
-      return;
+    if (*p == (PlLong) adr)
+      return TRUE;
 
   if (bound_var_ptr - pl_glob_dico_var >= MAX_VAR_IN_TERM)
     Pl_Err_Representation(pl_representation_too_many_variables);
 
-  *bound_var_ptr++ = (long) adr;
+  *bound_var_ptr++ = (PlLong) adr;
+  return TRUE;
 }
 
 
@@ -234,7 +248,7 @@ Existential_Variables(WamWord start_word)
   WamWord *adr;
 
   DEREF(start_word, word, tag_mask);
-  
+
   if (tag_mask == TAG_STC_MASK)
     {
       adr = UnTag_STC(word);
@@ -255,23 +269,24 @@ Existential_Variables(WamWord start_word)
  * FREE_VAR                                                                *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-static void
+static Bool
 Free_Var(WamWord *adr)
 {
-  long *p;
+  PlLong *p;
   WamWord word;
 
   for (p = pl_glob_dico_var; p < bound_var_ptr; p++)
-    if (*p == (long) adr)
-      return;
+    if (*p == (PlLong) adr)
+      return TRUE;
 
   word = Tag_REF(adr);	/* if an FDV for a Dont_Separate_Tag */
 
   for (p = free_var_base; p < H; p++)
     if (*p == word)
-      return;
+      return TRUE;
 
   *H++ = word;
+  return TRUE;
 }
 
 
@@ -360,7 +375,7 @@ Pl_Store_Solution_1(WamWord term_word)
   s->prev = sol;
   s->sol_no = sol->sol_no + 1;
   s->term_size = size;
-  fix_bug = term_word;	
+  fix_bug = term_word;
   Pl_Copy_Term(&s->term_word, &fix_bug);
   sol = s;
 }
@@ -374,7 +389,7 @@ Pl_Store_Solution_1(WamWord term_word)
  *-------------------------------------------------------------------------*/
 Bool
 Pl_Recover_Solutions_2(WamWord stop_word, WamWord handle_key_word,
-		    WamWord list_word)
+		       WamWord list_word)
 {
   int stop;
   int nb_sol;
@@ -393,6 +408,15 @@ Pl_Recover_Solutions_2(WamWord stop_word, WamWord handle_key_word,
 
 
   H += 2 * nb_sol;
+
+  /* Since we start from the end to the beginning, if nb_sol is very big
+   * when the heap overflow triggers a SIGSEGV the handler will not detect
+   * that the heap is the culprit (and emits a simple Segmentation Violation
+   * message). To avoid this we remain just after the end of the stack.
+   */
+  if (H > Global_Stack + Global_Size)
+    H =  Global_Stack + Global_Size;
+
   p = q = H;
 
   while (nb_sol--)
@@ -443,26 +467,27 @@ Handle_Key_Variables(WamWord start_word)
  * LINK_KEY_VAR                                                            *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-static void
+static Bool
 Link_Key_Var(WamWord *adr)
 {
-  long *p;
+  PlLong *p;
 
   for (p = pl_glob_dico_var; p < key_var_ptr; p++)
-    if (*p == (long) adr)
-      return;
+    if (*p == (PlLong) adr)
+      return TRUE;
 
   if (next_key_var_ptr < save_key_var_ptr)
     {		      /* same as Pl_Unify(Tag_REF(adr), *next_key_var_ptr++) */
       *adr = *(WamWord *) (*next_key_var_ptr);
       next_key_var_ptr++;
-      return;
+      return TRUE;
     }
 
   if (key_var_ptr - pl_glob_dico_var >= MAX_VAR_IN_TERM)
     Pl_Err_Representation(pl_representation_too_many_variables);
 
-  *key_var_ptr++ = (long) adr;
+  *key_var_ptr++ = (PlLong) adr;
+  return TRUE;
 }
 
 
@@ -482,7 +507,7 @@ Link_Key_Var(WamWord *adr)
  *-------------------------------------------------------------------------*/
 Bool
 Pl_Group_Solutions_3(WamWord all_sol_word, WamWord gl_key_word,
-		  WamWord sol_word)
+		     WamWord sol_word)
 {
   WamWord word, tag_mask;
   WamWord key_word;
@@ -490,15 +515,14 @@ Pl_Group_Solutions_3(WamWord all_sol_word, WamWord gl_key_word,
   DEREF(all_sol_word, word, tag_mask);
   if (word == NIL_WORD)
     return FALSE;
-  
+
   word = Group(all_sol_word, gl_key_word, &key_word);
   if (word != NOT_A_WAM_WORD)
     {
       A(0) = word;
       A(1) = gl_key_word;
       A(2) = sol_word;
-      Pl_Create_Choice_Point((CodePtr)
-			  Prolog_Predicate(GROUP_SOLUTIONS_ALT, 0), 3);
+      Pl_Create_Choice_Point((CodePtr) Prolog_Predicate(GROUP_SOLUTIONS_ALT, 0), 3);
     }
 
   Pl_Unify(key_word, gl_key_word);
@@ -519,8 +543,7 @@ Pl_Group_Solutions_Alt_0(void)
   WamWord word;
   WamWord key_word;
 
-  Pl_Update_Choice_Point((CodePtr) Prolog_Predicate(GROUP_SOLUTIONS_ALT, 0),
-		      0);
+  Pl_Update_Choice_Point((CodePtr) Prolog_Predicate(GROUP_SOLUTIONS_ALT, 0), 0);
 
   all_sol_word = AB(B, 0);
   gl_key_word = AB(B, 1);
