@@ -6,23 +6,36 @@
  * Descr.: arithmetic (inline) management - C part                         *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: arith_inl_c.c,v 1.21 2009/01/23 11:24:13 diaz Exp $ */
+/* $Id: arith_inl_c.c,v 1.28 2011/04/08 16:16:16 diaz Exp $ */
 
 #include <stdlib.h>
 #include <string.h>
@@ -33,11 +46,32 @@
 #include "engine_pl.h"
 #include "bips_pl.h"
 
-#ifdef M_ix86_win32
+#ifdef _MSC_VER
 #define rint(x)  (floor((x) + (double) 0.5))
 #endif
 
 
+/* PI */
+
+#ifndef M_PI
+#define M_PI 3.1415926535897932384
+#endif
+
+
+#ifndef M_E
+#define M_E  2.7182818284590452354
+#endif
+
+/* Difference between 1.0 and the minimum double greater than 1.0 */
+#ifndef DBL_EPSILON
+
+#ifdef __DBL_EPSILON__
+#define DBL_EPSILON __DBL_EPSILON__
+#else
+#define DBL_EPSILON 2.2204460492503131e-16  /* C double (64 bits IEEE encoding) */
+#endif
+
+#endif
 
 
 /*---------------------------------*
@@ -69,6 +103,9 @@ ArithInf;
 
 static char *arith_tbl;
 
+static int atom_pi;
+static int atom_e;
+static int atom_epsilon;
 
 
 
@@ -84,9 +121,9 @@ static WamWord Load_Math_Expression(WamWord exp);
 
 
 
-#define ADD_ARITH_OPER(atom_str, arity, f)                      \
+#define ADD_ARITH_OPER(atom_str, arity, f)                         \
   arith_info.f_n = Functor_Arity(Pl_Create_Atom(atom_str), arity); \
-  arith_info.fct = f;                                           \
+  arith_info.fct = f;                                              \
   Pl_Hash_Insert(arith_tbl, (char *) &arith_info, FALSE)
 
 
@@ -113,27 +150,44 @@ Arith_Initializer(void)
   ADD_ARITH_OPER("*", 2, Pl_Fct_Mul);
   ADD_ARITH_OPER("//", 2, Pl_Fct_Div);
   ADD_ARITH_OPER("/", 2, Pl_Fct_Float_Div);
+  ADD_ARITH_OPER("rem", 2, Pl_Fct_Rem);
   ADD_ARITH_OPER("mod", 2, Pl_Fct_Mod);
+  ADD_ARITH_OPER("div", 2, Pl_Fct_Div2);
   ADD_ARITH_OPER("/\\", 2, Pl_Fct_And);
   ADD_ARITH_OPER("\\/", 2, Pl_Fct_Or);
-  ADD_ARITH_OPER("^", 2, Pl_Fct_Xor);
+  ADD_ARITH_OPER("xor", 2, Pl_Fct_Xor);
   ADD_ARITH_OPER("\\", 1, Pl_Fct_Not);
   ADD_ARITH_OPER("<<", 2, Pl_Fct_Shl);
   ADD_ARITH_OPER(">>", 2, Pl_Fct_Shr);
+  ADD_ARITH_OPER("lsb", 1, Pl_Fct_LSB);
+  ADD_ARITH_OPER("msb", 1, Pl_Fct_MSB);
+  ADD_ARITH_OPER("popcount", 1, Pl_Fct_Popcount);
   ADD_ARITH_OPER("abs", 1, Pl_Fct_Abs);
   ADD_ARITH_OPER("sign", 1, Pl_Fct_Sign);
 
+  ADD_ARITH_OPER("gcd", 2, Pl_Fct_GCD);
   ADD_ARITH_OPER("min", 2, Pl_Fct_Min);
   ADD_ARITH_OPER("max", 2, Pl_Fct_Max);
+  ADD_ARITH_OPER("^", 2, Pl_Fct_Integer_Pow);
   ADD_ARITH_OPER("**", 2, Pl_Fct_Pow);
   ADD_ARITH_OPER("sqrt", 1, Pl_Fct_Sqrt);
+  ADD_ARITH_OPER("tan", 1, Pl_Fct_Tan);
   ADD_ARITH_OPER("atan", 1, Pl_Fct_Atan);
+  ADD_ARITH_OPER("atan2", 2, Pl_Fct_Atan2);
   ADD_ARITH_OPER("cos", 1, Pl_Fct_Cos);
   ADD_ARITH_OPER("acos", 1, Pl_Fct_Acos);
   ADD_ARITH_OPER("sin", 1, Pl_Fct_Sin);
   ADD_ARITH_OPER("asin", 1, Pl_Fct_Asin);
+  ADD_ARITH_OPER("tanh", 1, Pl_Fct_Tanh);
+  ADD_ARITH_OPER("atanh", 1, Pl_Fct_Atanh);
+  ADD_ARITH_OPER("cosh", 1, Pl_Fct_Cosh);
+  ADD_ARITH_OPER("acosh", 1, Pl_Fct_Acosh);
+  ADD_ARITH_OPER("sinh", 1, Pl_Fct_Sinh);
+  ADD_ARITH_OPER("asinh", 1, Pl_Fct_Asinh);
   ADD_ARITH_OPER("exp", 1, Pl_Fct_Exp);
   ADD_ARITH_OPER("log", 1, Pl_Fct_Log);
+  ADD_ARITH_OPER("log10", 1, Pl_Fct_Log10);
+  ADD_ARITH_OPER("log", 2, Pl_Fct_Log_Radix);
   ADD_ARITH_OPER("float", 1, Pl_Fct_Float);
   ADD_ARITH_OPER("ceiling", 1, Pl_Fct_Ceiling);
   ADD_ARITH_OPER("floor", 1, Pl_Fct_Floor);
@@ -142,7 +196,9 @@ Arith_Initializer(void)
   ADD_ARITH_OPER("float_fractional_part", 1, Pl_Fct_Float_Fract_Part);
   ADD_ARITH_OPER("float_integer_part", 1, Pl_Fct_Float_Integ_Part);
 
-  ADD_ARITH_OPER("rem", 2, Pl_Fct_Rem);
+  atom_pi = Pl_Create_Atom("pi");
+  atom_e = Pl_Create_Atom("e");
+  atom_epsilon = Pl_Create_Atom("epsilon");
 }
 
 
@@ -229,7 +285,7 @@ Make_Tagged_Float(double d)
 static double
 To_Double(WamWord x)
 {
-  return (Tag_Is_INT(x)) ? (double) (UnTag_INT(x)) : 
+  return (Tag_Is_INT(x)) ? (double) (UnTag_INT(x)) :
     Pl_Obtain_Float(UnTag_FLT(x));
 }
 
@@ -247,6 +303,7 @@ Load_Math_Expression(WamWord exp)
   WamWord *adr;
   WamWord *lst_adr;
   ArithInf *arith;
+  int atom;
 
   DEREF(exp, word, tag_mask);
 
@@ -268,7 +325,7 @@ Load_Math_Expression(WamWord exp)
       if (tag_mask == TAG_REF_MASK)
 	Pl_Err_Instantiation();
 
-      if (tag_mask != TAG_INT_MASK) 
+      if (tag_mask != TAG_INT_MASK)
 	{
 	  Pl_Err_Type(pl_type_integer, word);
 	}
@@ -287,7 +344,7 @@ Load_Math_Expression(WamWord exp)
 	  Pl_Unify_Integer(Arity(adr));
 	  Pl_Err_Type(pl_type_evaluable, word);
 	}
-      
+
       if (Arity(adr) == 1)
 	return (*(arith->fct)) (Load_Math_Expression(Arg(adr, 0)));
 
@@ -300,6 +357,16 @@ Load_Math_Expression(WamWord exp)
 
   if (tag_mask == TAG_ATM_MASK)
     {
+      atom = UnTag_ATM(word);
+      if (atom == atom_pi)
+	return Pl_Fct_PI();
+
+      if (atom == atom_e)
+	return Pl_Fct_E();
+
+      if (atom == atom_epsilon)
+	return Pl_Fct_Epsilon();
+
       word = Pl_Put_Structure(ATOM_CHAR('/'), 2);
       Pl_Unify_Value(exp);
       Pl_Unify_Integer(0);		/* then type_error */
@@ -349,6 +416,8 @@ Pl_Arith_Eval_2(WamWord exp_word, WamWord x_word)
 
 #define DFract(x)    ((x) - DInteg(x))
 
+#define Log_Radix(b, x)  (log(x) / log(b))
+
 
 #define X_and_Y_are_INT(x, y)  Tag_Is_INT(x & y)
 
@@ -367,9 +436,9 @@ Pl_Arith_Eval_2(WamWord exp_word, WamWord x_word)
 
 #define IxItoI(x, y, fast_op)                    \
   if (Tag_Is_FLT(x))		/* error case */ \
-    Pl_Err_Type(pl_type_integer, x);                \
+    Pl_Err_Type(pl_type_integer, x);             \
   if (Tag_Is_FLT(y))		/* error case */ \
-    Pl_Err_Type(pl_type_integer, y);                \
+    Pl_Err_Type(pl_type_integer, y);             \
   return fast_op(x, y)
 
 
@@ -382,7 +451,7 @@ Pl_Arith_Eval_2(WamWord exp_word, WamWord x_word)
 
 #define ItoI(x, fast_op)                         \
   if (Tag_Is_FLT(x))            /* error case */ \
-    Pl_Err_Type(pl_type_integer, x);                \
+    Pl_Err_Type(pl_type_integer, x);             \
   return fast_op(x)
 
 
@@ -392,22 +461,28 @@ Pl_Arith_Eval_2(WamWord exp_word, WamWord x_word)
 
 
 
+       /* FtoI is ONLY used for rounding functions */
 #define FtoI(x, c_op)                            \
   double d;                                      \
   if (Tag_Is_INT(x))            /* error case */ \
-    Pl_Err_Type(pl_type_float, x);                  \
+    {                                            \
+      if (Flag_Value(FLAG_STRICT_ISO))           \
+         Pl_Err_Type(pl_type_float, x);          \
+      else                                       \
+         return x;                               \
+    }						 \
   else                                           \
-    d = Pl_Obtain_Float(UnTag_FLT(x));              \
-  return Tag_INT((long) c_op(d))
+    d = Pl_Obtain_Float(UnTag_FLT(x));           \
+  return Tag_INT((PlLong) c_op(d))
 
 
 
 #define FtoF(x, c_op)                            \
   double d;                                      \
   if (Tag_Is_INT(x))            /* error case */ \
-    Pl_Err_Type(pl_type_float, x);                  \
+    Pl_Err_Type(pl_type_float, x);               \
   else                                           \
-    d = Pl_Obtain_Float(UnTag_FLT(x));              \
+    d = Pl_Obtain_Float(UnTag_FLT(x));           \
   return Make_Tagged_Float(c_op(d))
 
 
@@ -417,53 +492,53 @@ Pl_Arith_Eval_2(WamWord exp_word, WamWord x_word)
 WamWord FC
 Pl_Fct_Fast_Neg(WamWord x)
 {
-  long vx = UnTag_INT(x);
+  PlLong vx = UnTag_INT(x);
   return Tag_INT(-vx);
 }
 
 WamWord FC
 Pl_Fct_Fast_Inc(WamWord x)
 {
-  long vx = UnTag_INT(x);
+  PlLong vx = UnTag_INT(x);
   return Tag_INT(vx + 1);
 }
 
 WamWord FC
 Pl_Fct_Fast_Dec(WamWord x)
 {
-  long vx = UnTag_INT(x);
+  PlLong vx = UnTag_INT(x);
   return Tag_INT(vx - 1);
 }
 
 WamWord FC
 Pl_Fct_Fast_Add(WamWord x, WamWord y)
 {
-  long vx = UnTag_INT(x);
-  long vy = UnTag_INT(y);
+  PlLong vx = UnTag_INT(x);
+  PlLong vy = UnTag_INT(y);
   return Tag_INT(vx + vy);
 }
 
 WamWord FC
 Pl_Fct_Fast_Sub(WamWord x, WamWord y)
 {
-  long vx = UnTag_INT(x);
-  long vy = UnTag_INT(y);
+  PlLong vx = UnTag_INT(x);
+  PlLong vy = UnTag_INT(y);
   return Tag_INT(vx - vy);
 }
 
 WamWord FC
 Pl_Fct_Fast_Mul(WamWord x, WamWord y)
 {
-  long vx = UnTag_INT(x);
-  long vy = UnTag_INT(y);
+  PlLong vx = UnTag_INT(x);
+  PlLong vy = UnTag_INT(y);
   return Tag_INT(vx * vy);
 }
 
 WamWord FC
 Pl_Fct_Fast_Div(WamWord x, WamWord y)
 {
-  long vx = UnTag_INT(x);
-  long vy = UnTag_INT(y);
+  PlLong vx = UnTag_INT(x);
+  PlLong vy = UnTag_INT(y);
 
   if (vy == 0)
     Pl_Err_Evaluation(pl_evluation_zero_divisor);
@@ -474,8 +549,8 @@ Pl_Fct_Fast_Div(WamWord x, WamWord y)
 WamWord FC
 Pl_Fct_Fast_Rem(WamWord x, WamWord y)
 {
-  long vx = UnTag_INT(x);
-  long vy = UnTag_INT(y);
+  PlLong vx = UnTag_INT(x);
+  PlLong vy = UnTag_INT(y);
 
   if (vy == 0)
     Pl_Err_Evaluation(pl_evluation_zero_divisor);
@@ -486,9 +561,9 @@ Pl_Fct_Fast_Rem(WamWord x, WamWord y)
 WamWord FC
 Pl_Fct_Fast_Mod(WamWord x, WamWord y)
 {
-  long vx = UnTag_INT(x);
-  long vy = UnTag_INT(y);
-  long m;
+  PlLong vx = UnTag_INT(x);
+  PlLong vy = UnTag_INT(y);
+  PlLong m;
 
   if (vy == 0)
     Pl_Err_Evaluation(pl_evluation_zero_divisor);
@@ -501,6 +576,26 @@ Pl_Fct_Fast_Mod(WamWord x, WamWord y)
   return Tag_INT(m);
 }
 
+WamWord FC
+Pl_Fct_Fast_Div2(WamWord x, WamWord y)
+{
+  PlLong vx = UnTag_INT(x);
+  PlLong vy = UnTag_INT(y);
+  PlLong m;
+
+  if (vy == 0)
+    Pl_Err_Evaluation(pl_evluation_zero_divisor);
+
+  m = vx % vy;
+
+  if (m != 0 && (m ^ vy) < 0)	/* have m and vy different signs ? */
+    m += vy;
+
+  m = (vx - m) / vy;
+
+  return Tag_INT(m);
+}
+
 WamWord FC
 Pl_Fct_Fast_And(WamWord x, WamWord y)
 {
@@ -522,40 +617,95 @@ Pl_Fct_Fast_Xor(WamWord x, WamWord y)
 WamWord FC
 Pl_Fct_Fast_Not(WamWord x)
 {
-  long vx = UnTag_INT(x);
+  PlLong vx = UnTag_INT(x);
   return Tag_INT(~vx);
 }
 
+
 WamWord FC
 Pl_Fct_Fast_Shl(WamWord x, WamWord y)
 {
-  long vx = UnTag_INT(x);
-  long vy = UnTag_INT(y);
+  PlLong vx = UnTag_INT(x);
+  PlLong vy = UnTag_INT(y);
   return Tag_INT(vx << vy);
 }
 
 WamWord FC
 Pl_Fct_Fast_Shr(WamWord x, WamWord y)
 {
-  long vx = UnTag_INT(x);
-  long vy = UnTag_INT(y);
+  PlLong vx = UnTag_INT(x);
+  PlLong vy = UnTag_INT(y);
   return Tag_INT(vx >> vy);
 }
 
+WamWord FC
+Pl_Fct_Fast_LSB(WamWord x)
+{
+  PlLong vx = UnTag_INT(x);
+  return Tag_INT((vx == 0) ? -1 : Pl_Least_Significant_Bit(vx));
+}
+
+WamWord FC
+Pl_Fct_Fast_MSB(WamWord x)
+{
+  PlLong vx = UnTag_INT(x);
+  return Tag_INT((vx == 0) ? -1 : Pl_Most_Significant_Bit(vx));
+}
+
+WamWord FC
+Pl_Fct_Fast_Popcount(WamWord x)
+{
+  PlLong vx = UnTag_INT(x);
+  return Tag_INT(Pl_Count_Set_Bits(vx));
+}
+
 WamWord FC
 Pl_Fct_Fast_Abs(WamWord x)
 {
-  long vx = UnTag_INT(x);
+  PlLong vx = UnTag_INT(x);
   return (vx < 0) ? Tag_INT(-vx) : x;
 }
 
 WamWord FC
 Pl_Fct_Fast_Sign(WamWord x)
 {
-  long vx = UnTag_INT(x);
+  PlLong vx = UnTag_INT(x);
   return (vx < 0) ? Tag_INT(-1) : (vx == 0) ? Tag_INT(0) : Tag_INT(1);
 }
 
+WamWord FC
+Pl_Fct_Fast_GCD(WamWord x, WamWord y)
+{
+  PlLong vx = UnTag_INT(x);
+  PlLong vy = UnTag_INT(y);
+
+  if (vx < 0)
+    vx = -vx;
+
+  if (vy < 0)
+    vy = -vy;
+
+  while(vy != 0)
+    {
+      PlLong r = vx % vy;
+      vx = vy;
+      vy = r;
+    }
+  return Tag_INT(vx);
+}
+
+
+
+
+WamWord FC
+Pl_Fct_Fast_Integer_Pow(WamWord x, WamWord y)
+{
+  PlLong vx = UnTag_INT(x);
+  PlLong vy = UnTag_INT(y);
+  PlLong p = (PlLong) pow(vx, vy);
+  return Tag_INT(p);
+}
+
 
 
 	  /* standard version */
@@ -620,6 +770,12 @@ Pl_Fct_Mod(WamWord x, WamWord y)
   IxItoI(x, y, Pl_Fct_Fast_Mod);
 }
 
+WamWord FC
+Pl_Fct_Div2(WamWord x, WamWord y)
+{
+  IxItoI(x, y, Pl_Fct_Fast_Div2);
+}
+
 WamWord FC
 Pl_Fct_And(WamWord x, WamWord y)
 {
@@ -656,6 +812,24 @@ Pl_Fct_Shr(WamWord x, WamWord y)
   IxItoI(x, y, Pl_Fct_Fast_Shr);
 }
 
+WamWord FC
+Pl_Fct_LSB(WamWord x)
+{
+  ItoI(x, Pl_Fct_Fast_LSB);
+}
+
+WamWord FC
+Pl_Fct_MSB(WamWord x)
+{
+  ItoI(x, Pl_Fct_Fast_MSB);
+}
+
+WamWord FC
+Pl_Fct_Popcount(WamWord x)
+{
+  ItoI(x, Pl_Fct_Fast_Popcount);
+}
+
 WamWord FC
 Pl_Fct_Abs(WamWord x)
 {
@@ -668,6 +842,15 @@ Pl_Fct_Sign(WamWord x)
   IFtoIF(x, DSign, Pl_Fct_Fast_Sign);
 }
 
+
+WamWord FC
+Pl_Fct_GCD(WamWord x, WamWord y)
+{
+  IxItoI(x, y, Pl_Fct_Fast_GCD);
+}
+
+
+
 WamWord FC
 Pl_Fct_Min(WamWord x, WamWord y)
 {
@@ -698,6 +881,14 @@ Pl_Fct_Max(WamWord x, WamWord y)
   return Tag_Is_INT(x) ? x : y;
 }
 
+WamWord FC
+Pl_Fct_Integer_Pow(WamWord x, WamWord y)
+{
+  IxItoI(x, y, Pl_Fct_Fast_Integer_Pow);
+}
+
+
+
 WamWord FC
 Pl_Fct_Pow(WamWord x, WamWord y)
 {
@@ -710,12 +901,24 @@ Pl_Fct_Sqrt(WamWord x)
   IFtoF(x, sqrt);
 }
 
+WamWord FC
+Pl_Fct_Tan(WamWord x)
+{
+  IFtoF(x, tan);
+}
+
 WamWord FC
 Pl_Fct_Atan(WamWord x)
 {
   IFtoF(x, atan);
 }
 
+WamWord FC
+Pl_Fct_Atan2(WamWord x, WamWord y)
+{
+  IFxIFtoF(x, y, atan2);
+}
+
 WamWord FC
 Pl_Fct_Cos(WamWord x)
 {
@@ -740,6 +943,57 @@ Pl_Fct_Asin(WamWord x)
   IFtoF(x, asin);
 }
 
+WamWord FC
+Pl_Fct_Tanh(WamWord x)
+{
+  IFtoF(x, tanh);
+}
+
+WamWord FC
+Pl_Fct_Atanh(WamWord x)
+{
+#ifdef HAVE_ATANH
+  IFtoF(x, atanh);
+#else
+  Pl_Err_Resource(Pl_Create_Atom("unavailable function"));
+  return 0;			/* anything for the compiler */
+#endif
+}
+
+WamWord FC
+Pl_Fct_Cosh(WamWord x)
+{
+  IFtoF(x, cosh);
+}
+
+WamWord FC
+Pl_Fct_Acosh(WamWord x)
+{
+#ifdef HAVE_ACOSH
+  IFtoF(x, acosh);
+#else
+  Pl_Err_Resource(Pl_Create_Atom("unavailable function"));
+  return 0;			/* anything for the compiler */
+#endif
+}
+
+WamWord FC
+Pl_Fct_Sinh(WamWord x)
+{
+  IFtoF(x, sinh);
+}
+
+WamWord FC
+Pl_Fct_Asinh(WamWord x)
+{
+#ifdef HAVE_ASINH
+  IFtoF(x, asinh);
+#else
+  Pl_Err_Resource(Pl_Create_Atom("unavailable function"));
+  return 0;			/* anything for the compiler */
+#endif
+}
+
 WamWord FC
 Pl_Fct_Exp(WamWord x)
 {
@@ -752,6 +1006,18 @@ Pl_Fct_Log(WamWord x)
   IFtoF(x, log);
 }
 
+WamWord FC
+Pl_Fct_Log10(WamWord x)
+{
+  IFtoF(x, log10);
+}
+
+WamWord FC
+Pl_Fct_Log_Radix(WamWord b, WamWord x)
+{
+  IFxIFtoF(b, x, Log_Radix);
+}
+
 WamWord FC
 Pl_Fct_Float(WamWord x)
 {
@@ -794,6 +1060,27 @@ Pl_Fct_Float_Integ_Part(WamWord x)
   FtoF(x, DInteg);
 }
 
+WamWord FC
+Pl_Fct_PI(void)
+{
+  return Make_Tagged_Float(M_PI);
+}
+
+
+WamWord FC
+Pl_Fct_E(void)
+{
+  return Make_Tagged_Float(M_E);
+}
+
+
+WamWord FC
+Pl_Fct_Epsilon(void)
+{
+  return Make_Tagged_Float(DBL_EPSILON);
+}
+
+
 WamWord FC
 Pl_Fct_Identity(WamWord x)
 {
@@ -801,6 +1088,9 @@ Pl_Fct_Identity(WamWord x)
 }				/* for meta-call */
 
 
+
+
+
 	  /* Mathematic Comparisons */
 
 #define Cmp_IFxIF(x, y, c_op, fast_op)     \
@@ -826,32 +1116,32 @@ Pl_Blt_Fast_Neq(WamWord x, WamWord y)
 Bool FC
 Pl_Blt_Fast_Lt(WamWord x, WamWord y)
 {
-  long vx = UnTag_INT(x);
-  long vy = UnTag_INT(y);
+  PlLong vx = UnTag_INT(x);
+  PlLong vy = UnTag_INT(y);
   return vx < vy;
 }
 
 Bool FC
 Pl_Blt_Fast_Lte(WamWord x, WamWord y)
 {
-  long vx = UnTag_INT(x);
-  long vy = UnTag_INT(y);
+  PlLong vx = UnTag_INT(x);
+  PlLong vy = UnTag_INT(y);
   return vx <= vy;
 }
 
 Bool FC
 Pl_Blt_Fast_Gt(WamWord x, WamWord y)
 {
-  long vx = UnTag_INT(x);
-  long vy = UnTag_INT(y);
+  PlLong vx = UnTag_INT(x);
+  PlLong vy = UnTag_INT(y);
   return vx > vy;
 }
 
 Bool FC
 Pl_Blt_Fast_Gte(WamWord x, WamWord y)
 {
-  long vx = UnTag_INT(x);
-  long vy = UnTag_INT(y);
+  PlLong vx = UnTag_INT(x);
+  PlLong vy = UnTag_INT(y);
   return vx >= vy;
 }
 
