@@ -6,23 +6,36 @@
  * Descr.: dynamic predicate management - C part                           *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: assert_c.c,v 1.13 2009/01/23 11:24:13 diaz Exp $ */
+/* $Id: assert_c.c,v 1.17 2011/03/28 16:20:05 diaz Exp $ */
 
 #include "engine_pl.h"
 #include "bips_pl.h"
@@ -59,18 +72,19 @@ static Bool Retract_Alt(DynCInf *clause, WamWord *w);
 
 
 /*-------------------------------------------------------------------------*
- * PL_ASSERT_4                                                             *
+ * PL_ASSERT_5                                                             *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-Pl_Assert_4(WamWord head_word, WamWord body_word,
-	 WamWord asserta_word, WamWord check_perm_word)
+Pl_Assert_5(WamWord head_word, WamWord body_word,
+	    WamWord asserta_word, WamWord check_perm_word, WamWord pl_file_word)
 {
   Bool asserta = Pl_Rd_Integer(asserta_word);
   Bool check_perm = Pl_Rd_Integer(check_perm_word);
-  
+  int pl_file = Pl_Rd_Atom(pl_file_word);
+
   last_clause = Pl_Add_Dynamic_Clause(head_word, body_word, asserta,
-				   check_perm);
+				      check_perm, pl_file);
 }
 
 
@@ -140,8 +154,8 @@ Pl_Clause_3(WamWord head_word, WamWord body_word, WamWord for_what_word)
   w[1] = body_word;
 
   clause = Pl_Scan_Dynamic_Pred(-1, 0, (DynPInf *) (pred->dyn), word,
-			     (ScanFct) Clause_Alt, DYN_ALT_FCT_FOR_TEST, 2,
-			     w);
+				(ScanFct) Clause_Alt, DYN_ALT_FCT_FOR_TEST, 2,
+				w);
   if (clause == NULL)
     return FALSE;
 
@@ -227,8 +241,8 @@ Pl_Retract_2(WamWord head_word, WamWord body_word)
   w[1] = body_word;
 
   clause = Pl_Scan_Dynamic_Pred(-1, 0, (DynPInf *) (pred->dyn), word,
-			     (ScanFct) Retract_Alt, DYN_ALT_FCT_FOR_TEST, 2,
-			     w);
+				(ScanFct) Retract_Alt, DYN_ALT_FCT_FOR_TEST, 2,
+				w);
   if (clause == NULL)
     return FALSE;
 
@@ -356,7 +370,7 @@ Pl_Retractall_If_Empty_Head_1(WamWord head_word)
     }
 
   if (ret)
-    Pl_Update_Dynamic_Pred(func, arity, 1);
+    Pl_Update_Dynamic_Pred(func, arity, 1, -1);
 
   return ret;
 }
@@ -375,7 +389,7 @@ Pl_Abolish_1(WamWord pred_indic_word)
 
   func = Pl_Get_Pred_Indicator(pred_indic_word, TRUE, &arity);
 
-  Pl_Update_Dynamic_Pred(func, arity, 3);
+  Pl_Update_Dynamic_Pred(func, arity, 3, -1);
 }
 
 
@@ -393,5 +407,5 @@ Pl_Remove_Predicate_2(WamWord name_word, WamWord arity_word)
   func = Pl_Rd_Atom_Check(name_word);
   arity = Pl_Rd_Integer_Check(arity_word);
 
-  Pl_Update_Dynamic_Pred(func, arity, 2);
+  Pl_Update_Dynamic_Pred(func, arity, 2, -1);
 }
