@@ -6,23 +6,36 @@
  * Descr.: atom manipulation management - C part                           *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: atom_c.c,v 1.18 2009/01/23 11:24:13 diaz Exp $ */
+/* $Id: atom_c.c,v 1.27 2011/04/08 16:16:16 diaz Exp $ */
 
 #include <stdlib.h>
 #include <string.h>
@@ -62,11 +75,11 @@ static Bool String_To_Number(char *str, WamWord number_word);
 
 
 
-#define ATOM_CONCAT_ALT            X2461746F6D5F636F6E6361745F616C74
+#define ATOM_CONCAT_ALT            X1_2461746F6D5F636F6E6361745F616C74
 
-#define SUB_ATOM_ALT               X247375625F61746F6D5F616C74
+#define SUB_ATOM_ALT               X1_247375625F61746F6D5F616C74
 
-#define CURRENT_ATOM_ALT           X2463757272656E745F61746F6D5F616C74
+#define CURRENT_ATOM_ALT           X1_2463757272656E745F61746F6D5F616C74
 
 
 
@@ -153,11 +166,12 @@ Pl_Atom_Concat_3(WamWord atom1_word, WamWord atom2_word, WamWord atom3_word)
 
       patom3 = pl_atom_tbl + UnTag_ATM(atom3_word);
       l = patom3->prop.length - patom1->prop.length;
+      if (l < 0 || strncmp(patom1->name, patom3->name, patom1->prop.length) != 0)
+	return FALSE;
       MALLOC_STR(l);
       strcpy(str, patom3->name + patom1->prop.length);
 
-      return strncmp(patom1->name, patom3->name, patom1->prop.length) == 0
-	&& Pl_Get_Atom(Create_Malloc_Atom(str), atom2_word);
+      return Pl_Get_Atom(Create_Malloc_Atom(str), atom2_word);
     }
 
   if (tag2 == TAG_ATM_MASK)	/* here tag1 == REF */
@@ -165,13 +179,14 @@ Pl_Atom_Concat_3(WamWord atom1_word, WamWord atom2_word, WamWord atom3_word)
       patom2 = pl_atom_tbl + UnTag_ATM(atom2_word);
       patom3 = pl_atom_tbl + UnTag_ATM(atom3_word);
       l = patom3->prop.length - patom2->prop.length;
+      if (l < 0 || strncmp(patom2->name, patom3->name + l, patom2->prop.length) != 0)
+	return FALSE;
+
       MALLOC_STR(l);
       strncpy(str, patom3->name, l);
       str[l] = '\0';
 
-      return strncmp(patom2->name, patom3->name + l,
-		     patom2->prop.length) == 0
-	&& Pl_Get_Atom(Create_Malloc_Atom(str), atom1_word);
+      return Pl_Get_Atom(Create_Malloc_Atom(str), atom1_word);
     }
 
   /* A1 and A2 are variables: non deterministic case */
@@ -277,7 +292,7 @@ Pl_Sub_Atom_5(WamWord atom_word, WamWord before_word, WamWord length_word,
   AtomInf *patom;
   AtomInf *psub_atom = NULL;	/* only for the compiler */
   int length;
-  int b, l, a;
+  PlLong b, l, a;
   int b1, l1, a1;
   Bool nondet;
   int mask = 0;
@@ -601,7 +616,7 @@ Pl_Number_Atom_2(WamWord number_word, WamWord atom_word)
   DEREF(number_word, word, tag_mask);
   if (tag_mask == TAG_INT_MASK)
     {
-      sprintf(pl_glob_buff, "%ld", UnTag_INT(word));
+      sprintf(pl_glob_buff, "%" PL_FMT_d, UnTag_INT(word));
       return Pl_Un_String_Check(pl_glob_buff, atom_word);
     }
 
@@ -652,14 +667,14 @@ from_nb:
   DEREF(number_word, word, tag_mask);
   if (tag_mask == TAG_INT_MASK)
     {
-      sprintf(pl_glob_buff, "%ld", UnTag_INT(word));
-      return Pl_Un_Chars(pl_glob_buff, chars_word);
+      sprintf(pl_glob_buff, "%" PL_FMT_d, UnTag_INT(word));
+      return Pl_Un_Chars_Check(pl_glob_buff, chars_word);
     }
 
   if (tag_mask != TAG_REF_MASK)
     {
       str = Pl_Float_To_String(Pl_Rd_Number_Check(word));
-      return Pl_Un_Chars(str, chars_word);
+      return Pl_Un_Chars_Check(str, chars_word);
     }
 
   Pl_Rd_Chars_Check(chars_word);	/* only to raise the correct error */
@@ -679,7 +694,7 @@ Pl_Number_Codes_2(WamWord number_word, WamWord codes_word)
   WamWord word, tag_mask;
   WamWord *lst_adr, list_word;
   char *str = pl_glob_buff;
-  int c;
+  PlLong c;
 
   list_word = codes_word;
   for (;;)
@@ -698,7 +713,7 @@ Pl_Number_Codes_2(WamWord number_word, WamWord codes_word)
       if (tag_mask != TAG_INT_MASK || !Is_Valid_Code(c))
 	goto from_nb;
 
-      *str++ = c;
+      *str++ = (char) c;
       list_word = Cdr(lst_adr);
     }
 
@@ -709,14 +724,14 @@ from_nb:
   DEREF(number_word, word, tag_mask);
   if (tag_mask == TAG_INT_MASK)
     {
-      sprintf(pl_glob_buff, "%ld", UnTag_INT(word));
-      return Pl_Un_Codes(pl_glob_buff, codes_word);
+      sprintf(pl_glob_buff, "%" PL_FMT_d, UnTag_INT(word));
+      return Pl_Un_Codes_Check(pl_glob_buff, codes_word);
     }
 
   if (tag_mask != TAG_REF_MASK)
     {
       str = Pl_Float_To_String(Pl_Rd_Number_Check(word));
-      return Pl_Un_Codes(str, codes_word);
+      return Pl_Un_Codes_Check(str, codes_word);
     }
 
   Pl_Rd_Codes_Check(codes_word);	/* only to raise the correct error */
