@@ -6,23 +6,36 @@
  * Descr.: byte-code support                                               *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: bc_supp.c,v 1.17 2009/02/10 12:48:54 diaz Exp $ */
+/* $Id: bc_supp.c,v 1.23 2011/03/28 16:20:05 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -148,21 +161,14 @@ BCWord;
 typedef union
 {
   double d;
-  unsigned u[2];
-}
-DblUns;
-
-
 #if WORD_SIZE == 64
-
-typedef union
-{
   int *p;
+  PlLong l;
+#endif
   unsigned u[2];
 }
-PtrUns;
+C64To32;
 
-#endif
 
 
 
@@ -180,6 +186,7 @@ static int bc_nb_block;
 
 static int atom_dynamic;
 static int atom_public;
+static int atom_multifile;
 static int atom_built_in;
 static int atom_built_in_fd;
 static int atom_fail;
@@ -223,9 +230,9 @@ static void Prep_Debug_Call(int func, int arity, int caller_func,
 
 
 
-#define BC_EMULATE_CONT            X2462635F656D756C6174655F636F6E74
+#define BC_EMULATE_CONT            X1_2462635F656D756C6174655F636F6E74
 
-#define CALL_INTERNAL_WITH_CUT     X2463616C6C5F696E7465726E616C5F776974685F637574
+#define CALL_INTERNAL_WITH_CUT     X1_2463616C6C5F696E7465726E616C5F776974685F637574
 
 Prolog_Prototype(BC_EMULATE_CONT, 0);
 Prolog_Prototype(CALL_INTERNAL_WITH_CUT, 3);
@@ -250,31 +257,12 @@ Prolog_Prototype(CALL_INTERNAL_WITH_CUT, 3);
 
 #define BC2_Int(w)                 ((w).t2.i24)
 
-#define Fit_In_16bits(n)           ((unsigned) (n) < (1<<16))
+#define Fit_In_16bits(n)           ((PlULong) (n) < (1 << 16))
 
-#define Fit_In_24bits(n)           ((unsigned) (n) < (1<<24))
+#define Fit_In_24bits(n)           ((PlULong) (n) < (1 << 24))
 
 #define Op_In_Tbl(str, op)  BC_Op(*p) = op; BC2_Atom(*p) = Pl_Create_Atom(str); p++
 
-#if WORD_SIZE == 32
-
-#define Compute_Branch_Adr(bc, codep)   \
-{                                       \
-  codep = (WamCont) (bc->word); bc++;   \
-}
-
-#else
-
-#define Compute_Branch_Adr(bc, codep)     \
-{                                         \
-  PtrUns pu;                              \
-  pu.u[0] = (unsigned) (bc->word); bc++;  \
-  pu.u[1] = (unsigned) (bc->word); bc++;  \
-  codep = (WamCont) (pu.p);               \
-}
-
-#endif
-
 
 
 
@@ -339,6 +327,7 @@ Byte_Code_Initializer(void)
 
   atom_dynamic = Pl_Create_Atom("dynamic");
   atom_public = Pl_Create_Atom("public");
+  atom_multifile = Pl_Create_Atom("multifile");
   atom_built_in = Pl_Create_Atom("built_in");
   atom_built_in_fd = Pl_Create_Atom("built_in_fd");
   atom_fail = Pl_Create_Atom("fail");
@@ -395,15 +384,16 @@ Compar_Inst_Code_Op(BCWord *p1, BCWord *p2)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-Pl_BC_Start_Pred_7(WamWord func_word, WamWord arity_word,
-		WamWord pl_file_word, WamWord pl_line_word,
-		WamWord sta_dyn_word, WamWord pub_priv_word,
-		WamWord us_blp_bfd_word)
+Pl_BC_Start_Pred_8(WamWord func_word, WamWord arity_word,
+		   WamWord pl_file_word, WamWord pl_line_word,
+		   WamWord sta_dyn_word, WamWord pub_priv_word,
+		   WamWord mono_multi_word, WamWord us_blp_bfd_word)
 {
   int func, arity;
   int pl_file, pl_line;
   int prop = 0;
   int atom;
+  int multi = 0;
   PredInf *pred;
 
   func = Pl_Rd_Atom_Check(func_word);
@@ -416,6 +406,12 @@ Pl_BC_Start_Pred_7(WamWord func_word, WamWord arity_word,
   else if (Pl_Rd_Atom_Check(pub_priv_word) == atom_public)
     prop = MASK_PRED_PUBLIC;
 
+  if (Pl_Rd_Atom_Check(mono_multi_word) == atom_multifile)
+    {
+      prop |= MASK_PRED_MULTIFILE;
+      multi = 1;
+    }
+
   atom = Pl_Rd_Atom_Check(us_blp_bfd_word);
   if (atom == atom_built_in)
     prop |= MASK_PRED_BUILTIN;
@@ -423,14 +419,19 @@ Pl_BC_Start_Pred_7(WamWord func_word, WamWord arity_word,
     prop |= MASK_PRED_BUILTIN_FD;
 
 
-  pred = Pl_Update_Dynamic_Pred(func, arity, 0);
+  pred = Pl_Update_Dynamic_Pred(func, arity, 0, (multi) ? pl_file : -1);
   if (pred == NULL)
     pred = Pl_Create_Pred(func, arity, pl_file, pl_line, prop, NULL);
   else
     {
-      pred->pl_file = pl_file;
-      pred->pl_line = pl_line;
-      pred->prop = prop;
+      if (multi)
+	pred->prop |= prop;
+      else
+	{
+	  pred->pl_file = pl_file;
+	  pred->pl_line = pl_line;
+	  pred->prop = prop;
+	}
     }
 
 #if 1
@@ -485,6 +486,30 @@ Pl_BC_Stop_Emit_0(void)
 
 
 
+#define ASSEMBLE_INST(bc_sp, op, nb_word, w, w1, w2, w3)	\
+  BC_Op(w) = op;						\
+  *bc_sp++ = w;							\
+  if (nb_word >= 2)						\
+    {								\
+      bc_sp->word = w1;						\
+      bc_sp++;							\
+								\
+      if (nb_word >= 3)						\
+	{							\
+	  bc_sp->word = w2;					\
+	  bc_sp++;						\
+	  if (nb_word >= 4)					\
+	    {							\
+	      bc_sp->word = w3;					\
+	      bc_sp++;						\
+	    }							\
+	}							\
+    }
+
+
+
+
+
 /*-------------------------------------------------------------------------*
  * PL_BC_EMIT_INST_1                                                       *
  *                                                                         *
@@ -498,12 +523,10 @@ Pl_BC_Emit_Inst_1(WamWord inst_word)
   int size_bc;
   BCWord w;			/* code-op word */
   unsigned w1, w2, w3;		/* additional words */
+  PlLong l;
   int nb_word;
-  DblUns du;
+  C64To32 cv;
 
-#if WORD_SIZE == 64
-  PtrUns pu;
-#endif
   PredInf *pred;
 
   arg_adr = Pl_Rd_Callable_Check(inst_word, &func, &arity);
@@ -549,13 +572,21 @@ Pl_BC_Emit_Inst_1(WamWord inst_word)
 
     case GET_INTEGER:
     case PUT_INTEGER:
-      w1 = Pl_Rd_Integer(*arg_adr++);
-      if (Fit_In_16bits(w1))
-	BC1_Int(w) = w1;
+      l = Pl_Rd_Integer(*arg_adr++);
+      if (Fit_In_16bits(l))
+	BC1_Atom(w) = l;
       else
 	{
 	  op++;
+#if WORD_SIZE == 32
+	  w1 = l;
 	  nb_word = 2;
+#else
+	  cv.l = l;
+	  w1 = cv.u[0];
+	  w2 = cv.u[1];
+	  nb_word = 3;
+#endif
 	}
       BC1_X0(w) = Pl_Rd_Integer(*arg_adr);
       break;
@@ -563,10 +594,10 @@ Pl_BC_Emit_Inst_1(WamWord inst_word)
     case GET_FLOAT:
     case PUT_FLOAT:
       nb_word = 3;
-      du.d = Pl_Rd_Float(*arg_adr++);
+      cv.d = Pl_Rd_Float(*arg_adr++);
       BC1_X0(w) = Pl_Rd_Integer(*arg_adr);
-      w1 = du.u[0];
-      w2 = du.u[1];
+      w1 = cv.u[0];
+      w2 = cv.u[1];
       break;
 
     case GET_NIL:
@@ -608,13 +639,21 @@ Pl_BC_Emit_Inst_1(WamWord inst_word)
       break;
 
     case UNIFY_INTEGER:
-      w1 = Pl_Rd_Integer(*arg_adr++);
-      if (Fit_In_24bits(w1))
-	BC2_Int(w) = w1;
+      l = Pl_Rd_Integer(*arg_adr++);
+      if (Fit_In_24bits(l))
+	BC2_Int(w) = l;
       else
 	{
 	  op++;
+#if WORD_SIZE == 32
+	  w1 = l;
 	  nb_word = 2;
+#else
+	  cv.l = l;
+	  w1 = cv.u[0];
+	  w2 = cv.u[1];
+	  nb_word = 3;
+#endif
 	}
       break;
 
@@ -644,9 +683,9 @@ Pl_BC_Emit_Inst_1(WamWord inst_word)
 	  w3 = 0;		/* to avoid MSVC warning */
 #else
 	  nb_word = 4;
-	  pu.p = (int *) (pred->codep);
-	  w2 = pu.u[0];
-	  w3 = pu.u[1];
+	  cv.p = (int *) (pred->codep);
+	  w2 = cv.u[0];
+	  w3 = cv.u[1];
 #endif
 	}
       else
@@ -657,25 +696,8 @@ Pl_BC_Emit_Inst_1(WamWord inst_word)
       break;
     }
 
-  BC_Op(w) = op;
 
-  *bc_sp++ = w;
-  if (nb_word >= 2)
-    {
-      bc_sp->word = w1;
-      bc_sp++;
-
-      if (nb_word >= 3)
-	{
-	  bc_sp->word = w2;
-	  bc_sp++;
-	  if (nb_word >= 4)
-	    {
-	      bc_sp->word = w3;
-	      bc_sp++;
-	    }
-	}
-    }
+  ASSEMBLE_INST(bc_sp, op, nb_word, w, w1, w2, w3);
 
 
 #ifdef DEBUG
@@ -703,6 +725,43 @@ Pl_BC_Emit_Inst_1(WamWord inst_word)
 
 
 
+/*-------------------------------------------------------------------------*
+ * PL_BC_EMIT_INST_EXECUTE_NATIVE                                          *
+ *                                                                         *
+ * This function is called by the compiled code for dynamic or multifile   *
+ * predicate. Each clause has been compiled to native code (aux pred).     *
+ * We here create a call to this clause.                                   *
+ * This function is called between Pl_BC_Start_Emit_0 and Pl_BC_Stop_Emit_0*
+ * the buffer always bc has enough room for our 3 or 4 words.              *
+ *-------------------------------------------------------------------------*/
+void
+Pl_BC_Emit_Inst_Execute_Native(int func, int arity, PlLong *codep)
+{
+  BCWord w;			/* code-op word */
+  unsigned w1, w2, w3;		/* additional words */
+  int nb_word;
+#if WORD_SIZE == 64
+  C64To32 cv;
+#endif
+
+  w1 = func;
+  BC2_Arity(w) = arity;
+#if WORD_SIZE == 32
+  nb_word = 3;
+  w2 = (unsigned) codep;
+  w3 = 0;		/* to avoid MSVC warning */
+#else
+  nb_word = 4;
+  cv.p = (int *) codep;
+  w2 = cv.u[0];
+  w3 = cv.u[1];
+#endif
+
+  ASSEMBLE_INST(bc_sp, EXECUTE_NATIVE, nb_word, w, w1, w2, w3);
+}
+
+
+
 
 /*-------------------------------------------------------------------------*
  * BC_ARG_X_OR_Y                                                           *
@@ -771,7 +830,7 @@ Pl_BC_Call_Terminal_Pred_3(WamWord pred_word, WamWord call_info_word,
 
   debug_call = (call_info_word & (1 << TAG_SIZE_LOW)) != 0;
 
-  if (pl_debug_call_code != NULL && debug_call && 
+  if (pl_debug_call_code != NULL && debug_call &&
       (first_call_word & (1 << TAG_SIZE_LOW)))
     {
       A(0) = pred_word;
@@ -822,7 +881,7 @@ start:
   A(arity + 1) = debug_call;
 
   clause = Pl_Scan_Dynamic_Pred(func, arity, dyn, A(0),
-			     (long (*)()) BC_Emulate_Pred_Alt,
+			     (PlLong (*)()) BC_Emulate_Pred_Alt,
 			     DYN_ALT_FCT_FOR_JUMP, arity + 2, &A(0));
   if (clause == NULL)
     goto fail;
@@ -884,7 +943,7 @@ BC_Emulate_Clause(DynCInf *clause)
   int func, arity;
   int i;
 
-  bc = (BCWord *) clause->pl_byte_code;
+  bc = (BCWord *) clause->byte_code;
 
   if (bc)			/* emulated code */
     return BC_Emulate_Byte_Code(bc);
@@ -919,10 +978,11 @@ BC_Emulate_Byte_Code(BCWord *bc)
   BCWord w;
   int x0, x, y;
   int w1;
+  PlLong l;
   WamCont codep;
   int func, arity;
   PredInf *pred;
-  DblUns du;
+  C64To32 cv;
 
 
 bc_loop:
@@ -977,19 +1037,27 @@ bc_loop:
 
     case GET_INTEGER_BIG:
       x0 = BC1_X0(w);
-      w1 = bc->word;
+#if WORD_SIZE == 32
+      l = bc->word;
       bc++;
-      if (!Pl_Get_Integer(w1, X(x0)))
+#else
+      cv.u[0] = bc->word;
+      bc++;
+      cv.u[1] = bc->word;
+      bc++;
+      l = cv.l;
+#endif
+      if (!Pl_Get_Integer(l, X(x0)))
 	goto fail;
       goto bc_loop;
 
     case GET_FLOAT:
       x0 = BC1_X0(w);
-      du.u[0] = bc->word;
+      cv.u[0] = bc->word;
       bc++;
-      du.u[1] = bc->word;
+      cv.u[1] = bc->word;
       bc++;
-      if (!Pl_Get_Float(du.d, X(x0)))
+      if (!Pl_Get_Float(cv.d, X(x0)))
 	goto fail;
       goto bc_loop;
 
@@ -1069,18 +1137,26 @@ bc_loop:
 
     case PUT_INTEGER_BIG:
       x0 = BC1_X0(w);
-      w1 = bc->word;
+#if WORD_SIZE == 32
+      l = bc->word;
       bc++;
-      X(x0) = Pl_Put_Integer(w1);
+#else
+      cv.u[0] = bc->word;
+      bc++;
+      cv.u[1] = bc->word;
+      bc++;
+      l = cv.l;
+#endif
+      X(x0) = Pl_Put_Integer(l);
       goto bc_loop;
 
     case PUT_FLOAT:
       x0 = BC1_X0(w);
-      du.u[0] = bc->word;
+      cv.u[0] = bc->word;
       bc++;
-      du.u[1] = bc->word;
+      cv.u[1] = bc->word;
       bc++;
-      X(x0) = Pl_Put_Float(du.d);
+      X(x0) = Pl_Put_Float(cv.d);
       goto bc_loop;
 
     case PUT_NIL:
@@ -1169,9 +1245,17 @@ bc_loop:
       goto bc_loop;
 
     case UNIFY_INTEGER_BIG:
-      w1 = bc->word;
+#if WORD_SIZE == 32
+      l = bc->word;
       bc++;
-      if (!Pl_Unify_Integer(w1))
+#else
+      cv.u[0] = bc->word;
+      bc++;
+      cv.u[1] = bc->word;
+      bc++;
+      l = cv.l;
+#endif
+      if (!Pl_Unify_Integer(l))
 	goto fail;
       goto bc_loop;
 
@@ -1239,8 +1323,17 @@ bc_loop:
     case CALL_NATIVE:
       arity = BC2_Arity(w);
       func = bc->word;
-      bc++;      
-      Compute_Branch_Adr(bc, codep);
+      bc++;
+#if WORD_SIZE == 32
+      codep = (WamCont) (bc->word);
+      bc++;
+#else
+      cv.u[0] = bc->word;
+      bc++;
+      cv.u[1] = bc->word;
+      bc++;
+      codep = (WamCont) (cv.p);
+#endif
       BCI = (WamWord) bc | debug_call;
       CP = Adjust_CP(Prolog_Predicate(BC_EMULATE_CONT, 0));
       if (pl_debug_call_code != NULL && debug_call)
@@ -1253,8 +1346,17 @@ bc_loop:
     case EXECUTE_NATIVE:
       arity = BC2_Arity(w);
       func = bc->word;
-      bc++;      
-      Compute_Branch_Adr(bc, codep);
+      bc++;
+#if WORD_SIZE == 32
+      codep = (WamCont) (bc->word);
+      bc++;
+#else
+      cv.u[0] = bc->word;
+      bc++;
+      cv.u[1] = bc->word;
+      bc++;
+      codep = (WamCont) (cv.p);
+#endif
       if (pl_debug_call_code != NULL && debug_call)
 	{
 	  Prep_Debug_Call(func, arity, 0, 0);
