@@ -6,23 +6,36 @@
  * Descr.: byte-Code support - header file                                 *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: bc_supp.h,v 1.13 2009/01/23 11:24:13 diaz Exp $ */
+/* $Id: bc_supp.h,v 1.17 2011/03/28 16:20:05 diaz Exp $ */
 
 /*---------------------------------*
  * Constants                       *
@@ -49,7 +62,7 @@ extern int pl_byte_len;
 
 				/* defined as this to avoid to force the   */
 				/* inclusion of bc_supp.o if not needed    */
-				/* (dynam_supp.c uses pl_byte_code)           */
+				/* (dynam_supp.c uses pl_byte_code)        */
 unsigned *pl_byte_code;
 
 
@@ -59,3 +72,9 @@ unsigned *pl_byte_code;
  *---------------------------------*/
 
 WamCont Pl_BC_Emulate_Pred(int func, DynPInf *dyn);
+
+void Pl_BC_Start_Emit_0(void);
+void Pl_BC_Stop_Emit_0(void);
+void Pl_BC_Emit_Inst_1(WamWord inst_word);
+void Pl_BC_Emit_Inst_Execute_Native(int func, int arity, PlLong *codep);
+
