@@ -6,23 +6,36 @@
  * Descr.: C interface support                                             *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: c_supp.c,v 1.14 2009/01/23 11:24:13 diaz Exp $ */
+/* $Id: c_supp.c,v 1.21 2011/05/27 16:13:52 diaz Exp $ */
 
 #include <string.h>
 
@@ -53,7 +66,7 @@
                                                                             \
   DEREF(start_word, word, tag_mask);                                        \
    if (tag_mask != TAG_REF_MASK)                                            \
-     Pl_Err_Type(pl_type_variable, word)
+     Pl_Err_Uninstantiation(word)
 
 
 #define CHECK_FOR_UN_INTEGER                                                \
@@ -69,7 +82,7 @@
                                                                             \
   DEREF(start_word, word, tag_mask);                                        \
   if (tag_mask != TAG_REF_MASK && tag_mask != TAG_INT_MASK)                 \
-    Pl_Err_Type(pl_type_integer, word);                                        \
+    Pl_Err_Type(pl_type_integer, word);                                     \
   if (tag_mask == TAG_INT_MASK && UnTag_INT(word)<0)                        \
     Pl_Err_Domain(pl_domain_not_less_than_zero, word)
 
@@ -106,7 +119,7 @@
   DEREF(start_word, word, tag_mask);                                        \
   atom = UnTag_ATM(word);                                                   \
   if (tag_mask != TAG_REF_MASK && (tag_mask != TAG_ATM_MASK ||              \
-      (atom != pl_atom_true && atom != pl_atom_false)))                           \
+      (atom != pl_atom_true && atom != pl_atom_false)))                     \
      Pl_Err_Type(pl_type_boolean, word)
 
 
@@ -117,7 +130,7 @@
   DEREF(start_word, word, tag_mask);                                        \
   atom = UnTag_ATM(word);                                                   \
   if (tag_mask != TAG_REF_MASK && (tag_mask != TAG_ATM_MASK ||              \
-      pl_atom_tbl[atom].prop.length != 1))                                     \
+      pl_atom_tbl[atom].prop.length != 1))                                  \
     Pl_Err_Type(pl_type_character, word)
 
 
@@ -128,17 +141,17 @@
   DEREF(start_word, word, tag_mask);                                        \
   atom = UnTag_ATM(word);                                                   \
   if (tag_mask != TAG_REF_MASK && (tag_mask != TAG_ATM_MASK ||              \
-      (atom != pl_atom_end_of_file && pl_atom_tbl[atom].prop.length != 1)))       \
+      (atom != pl_atom_end_of_file && pl_atom_tbl[atom].prop.length != 1))) \
     Pl_Err_Type(pl_type_in_character, word)
 
 
 #define CHECK_FOR_UN_CODE                                                   \
   WamWord word, tag_mask;                                                   \
-  int c;                                                                    \
+  PlLong c;                                                                 \
                                                                             \
   DEREF(start_word, word, tag_mask);                                        \
   if (tag_mask != TAG_REF_MASK && tag_mask != TAG_INT_MASK)                 \
-    Pl_Err_Type(pl_type_integer, word);                                        \
+    Pl_Err_Type(pl_type_integer, word);                                     \
   c = UnTag_INT(word);                                                      \
   if (tag_mask == TAG_INT_MASK && !Is_Valid_Code(c))                        \
     Pl_Err_Representation(pl_representation_character_code)
@@ -146,11 +159,11 @@
 
 #define CHECK_FOR_UN_IN_CODE                                                \
   WamWord word, tag_mask;                                                   \
-  int c;                                                                    \
+  PlLong c;                                                                 \
                                                                             \
   DEREF(start_word, word, tag_mask);                                        \
   if (tag_mask != TAG_REF_MASK && tag_mask != TAG_INT_MASK)                 \
-    Pl_Err_Type(pl_type_integer, word);                                        \
+    Pl_Err_Type(pl_type_integer, word);                                     \
   c = UnTag_INT(word);                                                      \
   if (tag_mask == TAG_INT_MASK && c != -1 && !Is_Valid_Code(c))             \
     Pl_Err_Representation(pl_representation_in_character_code)
@@ -158,7 +171,7 @@
 
 #define CHECK_FOR_UN_BYTE                                                   \
   WamWord word, tag_mask;                                                   \
-  int c;                                                                    \
+  PlLong c;                                                                 \
                                                                             \
   DEREF(start_word, word, tag_mask);                                        \
   c = UnTag_INT(word);                                                      \
@@ -169,7 +182,7 @@
 
 #define CHECK_FOR_UN_IN_BYTE                                                \
   WamWord word, tag_mask;                                                   \
-  int c;                                                                    \
+  PlLong c;                                                                 \
                                                                             \
   DEREF(start_word, word, tag_mask);                                        \
   c = UnTag_INT(word);                                                      \
@@ -202,7 +215,7 @@
  * PL_RD_INTEGER_CHECK                                                     *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-long
+PlLong
 Pl_Rd_Integer_Check(WamWord start_word)
 {
   WamWord word, tag_mask;
@@ -224,7 +237,7 @@ Pl_Rd_Integer_Check(WamWord start_word)
  * PL_RD_INTEGER                                                           *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-long
+PlLong
 Pl_Rd_Integer(WamWord start_word)
 {
   WamWord word, tag_mask;
@@ -241,10 +254,10 @@ Pl_Rd_Integer(WamWord start_word)
  * PL_RD_POSITIVE_CHECK                                                    *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-long
+PlLong
 Pl_Rd_Positive_Check(WamWord start_word)
 {
-  long n = Pl_Rd_Integer_Check(start_word);
+  PlLong n = Pl_Rd_Integer_Check(start_word);
 
   if (n < 0)
     Pl_Err_Domain(pl_domain_not_less_than_zero, start_word);
@@ -259,7 +272,7 @@ Pl_Rd_Positive_Check(WamWord start_word)
  * PL_RD_POSITIVE                                                          *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-long
+PlLong
 Pl_Rd_Positive(WamWord start_word)
 {
   return Pl_Rd_Integer(start_word);
@@ -493,7 +506,7 @@ Pl_Rd_In_Char_Check(WamWord start_word)
     Pl_Err_Instantiation();
 
   atom = UnTag_ATM(word);
-  if (tag_mask != TAG_ATM_MASK || 
+  if (tag_mask != TAG_ATM_MASK ||
       (atom != pl_atom_end_of_file && pl_atom_tbl[atom].prop.length != 1))
     Pl_Err_Type(pl_type_in_character, word);
 
@@ -593,7 +606,7 @@ int
 Pl_Rd_Byte_Check(WamWord start_word)
 {
   WamWord word, tag_mask;
-  int c;
+  PlLong c;
 
   DEREF(start_word, word, tag_mask);
   if (tag_mask == TAG_REF_MASK)
@@ -630,7 +643,7 @@ int
 Pl_Rd_In_Byte_Check(WamWord start_word)
 {
   WamWord word, tag_mask;
-  int c;
+  PlLong c;
 
   DEREF(start_word, word, tag_mask);
   if (tag_mask == TAG_REF_MASK)
@@ -997,6 +1010,49 @@ Pl_Rd_Proper_List_Check(WamWord start_word, WamWord *arg)
 
 
 
+/*-------------------------------------------------------------------------*
+ * PL_RD_PROPER_LIST_CHECK2                                                *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+int 
+Pl_Rd_Proper_List_Check2(WamWord start_word, WamWord *arg, 
+			 WamWord (*elt_fct)(WamWord start_word))
+
+{
+  WamWord word, tag_mask;
+  WamWord save_start_word;
+  WamWord *lst_adr;
+  int n = 0;
+
+  save_start_word = start_word;
+
+  for (;;)
+    {
+      DEREF(start_word, word, tag_mask);
+
+      if (tag_mask == TAG_REF_MASK)
+	Pl_Err_Instantiation();
+
+      if (word == NIL_WORD)
+	break;
+
+      if (tag_mask != TAG_LST_MASK)
+	Pl_Err_Type(pl_type_list, save_start_word);
+
+      lst_adr = UnTag_LST(word);
+
+      *arg++ = (*elt_fct)(Car(lst_adr));
+      n++;
+
+      start_word = Cdr(lst_adr);
+    }
+
+  return n;
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * PL_RD_PROPER_LIST                                                       *
  *                                                                         *
@@ -1347,6 +1403,27 @@ Pl_Check_For_Un_In_Byte(WamWord start_word)
 
 
 
+/*-------------------------------------------------------------------------*
+ * PL_CHECK_FOR_UN_PAIR                                                    *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Pl_Check_For_Un_Pair(WamWord start_word)
+{
+  WamWord word, tag_mask;
+
+static WamWord minus_2 = Functor_Arity(ATOM_CHAR('-'), 2);
+
+
+  DEREF(start_word, word, tag_mask);
+  if (tag_mask != TAG_REF_MASK && (tag_mask != TAG_STC_MASK || 
+      Functor_And_Arity(UnTag_STC(word)) != minus_2))
+    Pl_Err_Type(pl_type_pair, word);
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * PL_CHECK_FOR_UN_CHARS                                                   *
  *                                                                         *
@@ -1354,7 +1431,28 @@ Pl_Check_For_Un_In_Byte(WamWord start_word)
 void
 Pl_Check_For_Un_Chars(WamWord start_word)
 {
-  Pl_Check_For_Un_List(start_word);
+  WamWord word, tag_mask;
+  WamWord save_start_word;
+  WamWord *lst_adr;
+
+  save_start_word = start_word;
+
+  for (;;)
+    {
+      DEREF(start_word, word, tag_mask);
+
+      if (tag_mask == TAG_REF_MASK || word == NIL_WORD)
+	return;
+
+      if (tag_mask != TAG_LST_MASK)
+	Pl_Err_Type(pl_type_list, save_start_word);
+
+      lst_adr = UnTag_LST(word);
+
+      Pl_Check_For_Un_Char(Car(lst_adr));
+
+      start_word = Cdr(lst_adr);
+    }
 }
 
 
@@ -1380,7 +1478,28 @@ Pl_Check_For_Un_String(WamWord start_word)
 void
 Pl_Check_For_Un_Codes(WamWord start_word)
 {
-  Pl_Check_For_Un_List(start_word);
+  WamWord word, tag_mask;
+  WamWord save_start_word;
+  WamWord *lst_adr;
+
+  save_start_word = start_word;
+
+  for (;;)
+    {
+      DEREF(start_word, word, tag_mask);
+
+      if (tag_mask == TAG_REF_MASK || word == NIL_WORD)
+	return;
+
+      if (tag_mask != TAG_LST_MASK)
+	Pl_Err_Type(pl_type_list, save_start_word);
+
+      lst_adr = UnTag_LST(word);
+
+      Pl_Check_For_Un_Code(Car(lst_adr));
+
+      start_word = Cdr(lst_adr);
+    }
 }
 
 
@@ -1400,6 +1519,39 @@ Pl_Check_For_Un_List(WamWord start_word)
 
 
 
+
+/*-------------------------------------------------------------------------*
+ * PL_CHECK_FOR_UN_LIST2                                                   *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Pl_Check_For_Un_List2(WamWord start_word, void (*elt_fct)(WamWord start_word))
+{
+  WamWord start_word0 = start_word;
+  WamWord word, tag_mask;
+  WamWord *adr;
+
+  for (;;)
+    {
+      DEREF(start_word, word, tag_mask);
+
+      if (tag_mask == TAG_REF_MASK || word == NIL_WORD)
+	return;
+
+      if (tag_mask != TAG_LST_MASK)
+	Pl_Err_Type(pl_type_list, start_word0);
+
+      adr = UnTag_LST(word);
+
+      (*elt_fct)(Car(adr));
+
+      start_word = Cdr(adr);
+    }
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * PL_CHECK_FOR_UN_COMPOUND                                                *
  *                                                                         *
@@ -1444,7 +1596,7 @@ Pl_Check_For_Un_Variable(WamWord start_word)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool
-Pl_Un_Integer_Check(long value, WamWord start_word)
+Pl_Un_Integer_Check(PlLong value, WamWord start_word)
 {
   CHECK_FOR_UN_INTEGER;
 
@@ -1459,7 +1611,7 @@ Pl_Un_Integer_Check(long value, WamWord start_word)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool
-Pl_Un_Integer(long value, WamWord start_word)
+Pl_Un_Integer(PlLong value, WamWord start_word)
 {
   return Pl_Get_Integer(value, start_word);
 }
@@ -1472,7 +1624,7 @@ Pl_Un_Integer(long value, WamWord start_word)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool
-Pl_Un_Positive_Check(long value, WamWord start_word)
+Pl_Un_Positive_Check(PlLong value, WamWord start_word)
 {
   CHECK_FOR_UN_POSITIVE;
 
@@ -1487,7 +1639,7 @@ Pl_Un_Positive_Check(long value, WamWord start_word)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool
-Pl_Un_Positive(long value, WamWord start_word)
+Pl_Un_Positive(PlLong value, WamWord start_word)
 {
   return Pl_Get_Integer(value, start_word);
 }
@@ -1530,11 +1682,11 @@ Pl_Un_Float(double value, WamWord start_word)
 Bool
 Pl_Un_Number_Check(double value, WamWord start_word)
 {
-  long n;
+  PlLong n;
 
   CHECK_FOR_UN_NUMBER;
 
-  n = (long) value;
+  n = (PlLong) value;
 
   return (n == value) ? Pl_Get_Integer(n, word) : Pl_Get_Float(value, word);
 }
@@ -1549,9 +1701,9 @@ Pl_Un_Number_Check(double value, WamWord start_word)
 Bool
 Pl_Un_Number(double value, WamWord start_word)
 {
-  long n;
+  PlLong n;
 
-  n = (long) value;
+  n = (PlLong) value;
 
   return (n == value) ? Pl_Get_Integer(n, start_word) :
     Pl_Get_Float(value, start_word);
@@ -1821,7 +1973,11 @@ Pl_Un_String(char *value, WamWord start_word)
 Bool
 Pl_Un_Chars_Check(char *str, WamWord start_word)
 {
+#if 0
   Pl_Check_For_Un_List(start_word);
+#else
+  Pl_Check_For_Un_Chars(start_word);
+#endif
 
   return Pl_Un_Chars(str, start_word);
 }
@@ -1857,7 +2013,11 @@ Pl_Un_Chars(char *str, WamWord start_word)
 Bool
 Pl_Un_Codes_Check(char *str, WamWord start_word)
 {
+#if 0
   Pl_Check_For_Un_List(start_word);
+#else
+  Pl_Check_For_Un_Codes(start_word);
+#endif
 
   return Pl_Un_Codes(str, start_word);
 }
@@ -2094,7 +2254,7 @@ Pl_Un_Term(WamWord term_word, WamWord start_word)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 WamWord
-Pl_Mk_Integer(long value)
+Pl_Mk_Integer(PlLong value)
 {
   return Pl_Put_Integer(value);
 }
@@ -2107,7 +2267,7 @@ Pl_Mk_Integer(long value)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 WamWord
-Pl_Mk_Positive(long value)
+Pl_Mk_Positive(PlLong value)
 {
   return Pl_Put_Integer(value);
 }
@@ -2137,7 +2297,7 @@ Pl_Mk_Number(double value)
 {
   int n;
 
-  n = (long) value;
+  n = (PlLong) value;
 
   if (n == value)
     return Pl_Put_Integer(n);
