@@ -6,23 +6,36 @@
  * Descr.: C interface support - header file                               *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: c_supp.h,v 1.13 2009/01/23 11:24:13 diaz Exp $ */
+/* $Id: c_supp.h,v 1.17 2011/05/27 16:13:52 diaz Exp $ */
 
 /*---------------------------------*
  * Constants                       *
@@ -40,13 +53,13 @@
  * Function Prototypes             *
  *---------------------------------*/
 
-long Pl_Rd_Integer_Check(WamWord start_word);
+PlLong Pl_Rd_Integer_Check(WamWord start_word);
 
-long Pl_Rd_Integer(WamWord start_word);
+PlLong Pl_Rd_Integer(WamWord start_word);
 
-long Pl_Rd_Positive_Check(WamWord start_word);
+PlLong Pl_Rd_Positive_Check(WamWord start_word);
 
-long Pl_Rd_Positive(WamWord start_word);
+PlLong Pl_Rd_Positive(WamWord start_word);
 
 double Pl_Rd_Float_Check(WamWord start_word);
 
@@ -114,6 +127,9 @@ WamWord *Pl_Rd_List(WamWord start_word);
 
 int Pl_Rd_Proper_List_Check(WamWord start_word, WamWord *arg);
 
+int Pl_Rd_Proper_List_Check2(WamWord start_word, WamWord *arg, 
+			     WamWord (*elt_fct)(WamWord start_word));
+
 int Pl_Rd_Proper_List(WamWord start_word, WamWord *arg);
 
 WamWord *Pl_Rd_Compound_Check(WamWord start_word, int *func, int *arity);
@@ -158,6 +174,9 @@ void Pl_Check_For_Un_Codes(WamWord start_word);
 
 void Pl_Check_For_Un_List(WamWord start_word);
 
+void Pl_Check_For_Un_List2(WamWord start_word, 
+			   void (*elt_fct)(WamWord start_word));
+
 void Pl_Check_For_Un_Compound(WamWord start_word);
 
 void Pl_Check_For_Un_Callable(WamWord start_word);
@@ -166,13 +185,13 @@ void Pl_Check_For_Un_Variable(WamWord start_word);
 
 
 
-Bool Pl_Un_Integer_Check(long value, WamWord start_word);
+Bool Pl_Un_Integer_Check(PlLong value, WamWord start_word);
 
-Bool Pl_Un_Integer(long value, WamWord start_word);
+Bool Pl_Un_Integer(PlLong value, WamWord start_word);
 
-Bool Pl_Un_Positive_Check(long value, WamWord start_word);
+Bool Pl_Un_Positive_Check(PlLong value, WamWord start_word);
 
-Bool Pl_Un_Positive(long value, WamWord start_word);
+Bool Pl_Un_Positive(PlLong value, WamWord start_word);
 
 Bool Pl_Un_Float_Check(double value, WamWord start_word);
 
@@ -248,9 +267,9 @@ Bool Pl_Un_Term(WamWord term_word, WamWord start_word);
 
 
 
-WamWord Pl_Mk_Integer(long value);
+WamWord Pl_Mk_Integer(PlLong value);
 
-WamWord Pl_Mk_Positive(long value);
+WamWord Pl_Mk_Positive(PlLong value);
 
 WamWord Pl_Mk_Float(double value);
 
