@@ -6,23 +6,37 @@
  * Descr.: meta call management - C part                                   *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
+#include <string.h>
 
-/* $Id: call_args_c.c,v 1.13 2009/01/23 11:24:13 diaz Exp $ */
+/* $Id: call_args_c.c,v 1.19 2011/06/10 13:58:09 diaz Exp $ */
 
 #define OBJ_INIT Call_Args_Initializer
 
@@ -52,7 +66,7 @@ static int atom_call;
  * Function Prototypes             *
  *---------------------------------*/
 
-#define CALL_INTERNAL              X2463616C6C5F696E7465726E616C
+#define CALL_INTERNAL              X1_2463616C6C5F696E7465726E616C
 
 Prolog_Prototype(CALL_INTERNAL, 2);
 
@@ -77,11 +91,11 @@ Call_Args_Initializer(void)
  * CALL_CLOSURE                                                            *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-static WamCont
-Call_Closure(int atom_bip, int arity_rest)
+WamCont
+Pl_Call_Closure(int atom_bip, int arity_rest)
 {
   int call_info;
-  int func, arity0, arity;
+  int func, arity_clos, arity;
   WamWord *arg_adr;
   PredInf *pred;
   WamWord *w;
@@ -90,26 +104,27 @@ Call_Closure(int atom_bip, int arity_rest)
   Pl_Set_C_Bip_Name(pl_atom_tbl[atom_bip].name, 1 + arity_rest);
   if (atom_bip == atom_call_with_args) {
     func = Pl_Rd_Atom_Check(A(0));
-    arity0 = 0;
+    arity_clos = 0;
   } else {
-    arg_adr = Pl_Rd_Callable_Check(A(0), &func, &arity0);
+    arg_adr = Pl_Rd_Callable_Check(A(0), &func, &arity_clos);
   }
-  
-  arity = arity0 + arity_rest;
+
+  arity = arity_clos + arity_rest;
   if (arity > MAX_ARITY)
     Pl_Err_Representation(pl_representation_max_arity);
 
 
   call_info = Call_Info(func, arity, 1);
 
-  if ((pred = Pl_Lookup_Pred(func, arity)) == NULL)
+  if ((pred = Pl_Lookup_Pred(func, arity)) == NULL || 
+      (pred->prop & MASK_PRED_CONTROL_CONSTRUCT))
     {
       if (arity > 0)
 	{
 	  w = H;
 	  A(0) = Tag_STC(w);
 	  *w++ = Functor_Arity(func, arity);
-	  while(arity0-- > 0)
+	  while(arity_clos-- > 0)
 	    *w++ = *arg_adr++;
 	  for (i = 1; i <= arity_rest; i++)
 	    *w++ = A(i);
@@ -121,150 +136,27 @@ Call_Closure(int atom_bip, int arity_rest)
       return (CodePtr) Prolog_Predicate(CALL_INTERNAL, 2);
     }
 
+
+  /* arity = arity_clos + arity_rest */
+  /* the arity_clos args in the compound term   go in A(0)..A(arity_clos-1) */
+  /* the arity_rest args in A(1)..A(arity_rest) go in A(arity_clos)..A(arity-1) */
+
+  /* first copy the arity_rest args (to avoid the overwrite them copying closure args first) */
+  /* NB: if arity_clos == 0 then dst < src */
+  /*     if arity_clos == 1 then dst == src (optim) */
+  /*     if arity_clos >= 2 then dst > src */
+  /* we use memmove */
+
+  if (arity_clos != 1)		/* optim: no copy needed when closure has 1 arg */
+    memmove((void *) &A(arity_clos), &A(1), sizeof(WamWord) * arity_rest);
+
+  /* then copy the arity_clos args */
   w = &A(0);
-  while(arity0-- > 0)
+  while(arity_clos-- > 0)
     *w++ = *arg_adr++;
-  for (i = 1; i <= arity_rest; i++)
-    *w++ = A(i);
 
   if (pred->prop & MASK_PRED_NATIVE_CODE)	/* native code */
     return (WamCont) (pred->codep);
 
   return Pl_BC_Emulate_Pred(func, (DynPInf *) (pred->dyn));
 }
-
-
-
-
-/*-------------------------------------------------------------------------*
- * CALL_WITH_ARGS_...                                                      *
- *                                                                         *
- *-------------------------------------------------------------------------*/
-WamCont
-Pl_Call_With_Args_1(void)
-{
-  return Call_Closure(atom_call_with_args, 0);
-}
-
-WamCont
-Pl_Call_With_Args_2(void)
-{
-  return Call_Closure(atom_call_with_args, 1);
-}
-
-WamCont
-Pl_Call_With_Args_3(void)
-{
-  return Call_Closure(atom_call_with_args, 2);
-}
-
-WamCont
-Pl_Call_With_Args_4(void)
-{
-  return Call_Closure(atom_call_with_args, 3);
-}
-
-WamCont
-Pl_Call_With_Args_5(void)
-{
-  return Call_Closure(atom_call_with_args, 4);
-}
-
-WamCont
-Pl_Call_With_Args_6(void)
-{
-  return Call_Closure(atom_call_with_args, 5);
-}
-
-WamCont
-Pl_Call_With_Args_7(void)
-{
-  return Call_Closure(atom_call_with_args, 6);
-}
-
-WamCont
-Pl_Call_With_Args_8(void)
-{
-  return Call_Closure(atom_call_with_args, 7);
-}
-
-WamCont
-Pl_Call_With_Args_9(void)
-{
-  return Call_Closure(atom_call_with_args, 8);
-}
-
-WamCont
-Pl_Call_With_Args_10(void)
-{
-  return Call_Closure(atom_call_with_args, 9);
-}
-
-WamCont
-Pl_Call_With_Args_11(void)
-{
-  return Call_Closure(atom_call_with_args, 10);
-}
-
-
-WamCont
-Pl_Call_2(void)
-{
-  return Call_Closure(atom_call, 1);
-}
-
-WamCont
-Pl_Call_3(void)
-{
-  return Call_Closure(atom_call, 2);
-}
-
-WamCont
-Pl_Call_4(void)
-{
-  return Call_Closure(atom_call, 3);
-}
-
-
-WamCont
-Pl_Call_5(void)
-{
-  return Call_Closure(atom_call, 4);
-}
-
-WamCont
-Pl_Call_6(void)
-{
-  return Call_Closure(atom_call, 5);
-}
-
-WamCont
-Pl_Call_7(void)
-{
-  return Call_Closure(atom_call, 6);
-}
-
-WamCont
-Pl_Call_8(void)
-{
-  return Call_Closure(atom_call, 7);
-}
-
-WamCont
-Pl_Call_9(void)
-{
-  return Call_Closure(atom_call, 8);
-}
-
-WamCont
-Pl_Call_10(void)
-{
-  return Call_Closure(atom_call, 9);
-}
-
-
-WamCont
-Pl_Call_11(void)
-{
-  return Call_Closure(atom_call, 10);
-}
