@@ -1,28 +1,41 @@
-/*-------------------------------------------------------------------------* 
- * GNU Prolog                                                              * 
- *                                                                         * 
- * Part  : Prolog buit-in predicates                                       * 
- * File  : debugger.pl                                                     * 
- * Descr.: debugger                                                        * 
- * Author: Daniel Diaz                                                     * 
- *                                                                         * 
- * Copyright (C) 1999-2009 Daniel Diaz                                     * 
- *                                                                         * 
- * GNU Prolog is free software; you can redistribute it and/or modify it   * 
- * under the terms of the GNU General Public License as published by the   * 
- * Free Software Foundation; either version 2, or any later version.       * 
- *                                                                         * 
- * GNU Prolog is distributed in the hope that it will be useful, but       * 
- * WITHOUT ANY WARRANTY; without even the implied warranty of              * 
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        * 
- * General Public License for more details.                                * 
- *                                                                         * 
- * You should have received a copy of the GNU General Public License along * 
- * with this program; if not, write to the Free Software Foundation, Inc.  * 
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               * 
+/*-------------------------------------------------------------------------*
+ * GNU Prolog                                                              *
+ *                                                                         *
+ * Part  : Prolog buit-in predicates                                       *
+ * File  : debugger.pl                                                     *
+ * Descr.: debugger                                                        *
+ * Author: Daniel Diaz                                                     *
+ *                                                                         *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ *                                                                         *
+ * This file is part of GNU Prolog                                         *
+ *                                                                         *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
+ * General Public License for more details.                                *
+ *                                                                         *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: debugger.pl,v 1.16 2009/01/23 11:24:13 diaz Exp $ */
+/* $Id: debugger.pl,v 1.20 2011/06/10 16:24:15 diaz Exp $ */
 
 :-	built_in.
 
@@ -647,7 +660,7 @@ nospyall.
 	repeat,
 	get_code(debugger_input, 10),                         % the last '\n'
 	                              !.
-	
+
 
 
 '$debug_exec_cmd'(C, _, _, _, _, _) :-
@@ -682,7 +695,7 @@ nospyall.
 	'$debug_read_integer'(Invoc),
 	g_assign('$debug_next', skip),
 	g_assign('$debug_skip', s(Invoc, 31)).
-	
+
 '$debug_exec_cmd'(r, _, _, _, call, _) :-                             % retry
 	!,
 	fail.
@@ -783,7 +796,7 @@ nospyall.
 	functor(Goal, N, A),
 	PI = N / A,
 	(   '$current_predicate_any'(PI) ->
-	    (   '$predicate_property_any'(PI, native_code) ->
+	    (   '$predicate_property1'(N, A, native_code) ->
 	        format(debugger_output, 'native code predicate ~a/~d~n', [N, A])
 	    ;   listing(PI),
 	        nl(debugger_output)
@@ -924,14 +937,16 @@ nospyall.
 
 '$debug_disp_alt1'(N, A, _) :-                      % detect system predicate
 	sub_atom(N, 0, 1, _, $),
-	'$predicate_property_any'(N / A, native_code),
+	'$predicate_property1'(N, A, native_code),
 	(   (   sub_atom(N, 1, _, 4, N1),
 	        '$debug_check_bip'(N1, A1)
-	    ;   sub_atom(N, 1, _, 1, N1),
+	    ;
+		sub_atom(N, 1, _, 1, N1),
 	        '$debug_check_bip'(N1, A1)
 	    ) ->
 	    '$debug_disp_alt2'(N1 / A1)
-	;   '$debug_disp_alt2'('system predicate'(N / A))
+	;
+	    '$debug_disp_alt2'('system predicate'(N / A))
 	).
 
 '$debug_disp_alt1'(N, A, _) :-                             % normal predicate
@@ -946,15 +961,17 @@ nospyall.
 
 '$debug_is_debug_predicate'(N) :-
 	sub_atom(N, 0, 7, _, '$debug_').
-	
+
 
 
 
 '$debug_check_bip'(N1, A1) :-
-	predicate_property(N1 / A1, built_in), !.
+	'$predicate_property1'(N1, A1, built_in).
 
+/* useless since now built_in_fd ==> built_in
 '$debug_check_bip'(N1, A1) :-
-	predicate_property(N1 / A1, built_in_fd).
+	'$predicate_property1'(N1, A1, built_in_fd).
+*/
 
 
 
