@@ -6,23 +6,36 @@
  * Descr.: dynamic predicate support                                       *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: dynam_supp.c,v 1.15 2009/01/23 11:24:13 diaz Exp $ */
+/* $Id: dynam_supp.c,v 1.20 2011/03/28 16:20:05 diaz Exp $ */
 
 #include <stdlib.h>
 
@@ -102,16 +115,18 @@ static int size_of_erased = 0;
 
 static DynPInf *Alloc_Init_Dyn_Info(PredInf *pred, int arity);
 
-static int Index_From_First_Arg(WamWord first_arg_word, long *key);
+static int Index_From_First_Arg(WamWord first_arg_word, PlLong *key);
 
-static void Add_To_2Chain(D2ChHdr *hdr, DynCInf *clause, Bool in_seq_chain, 
+static void Add_To_2Chain(D2ChHdr *hdr, DynCInf *clause, Bool in_seq_chain,
 			  Bool asserta);
 
-static void Remove_From_2Chain(D2ChHdr *hdr, DynCInf *clause, 
+static void Remove_From_2Chain(D2ChHdr *hdr, DynCInf *clause,
 			       Bool in_seq_chain);
 
 static void Erase_All(DynPInf *dyn);
 
+static void Erase_All_Clauses_Of_File(DynPInf *dyn, int pl_file);
+
 static void Clean_Erased_Clauses(void);
 
 static void Unlink_Clause(DynCInf *clause);
@@ -138,9 +153,9 @@ static void Check_Chain(D2ChHdr *p, int index_no);
 
 
 
-#define SCAN_DYN_TEST_ALT          X247363616E5F64796E5F746573745F616C74
+#define SCAN_DYN_TEST_ALT          X1_247363616E5F64796E5F746573745F616C74
 
-#define SCAN_DYN_JUMP_ALT          X247363616E5F64796E5F6A756D705F616C74
+#define SCAN_DYN_JUMP_ALT          X1_247363616E5F64796E5F6A756D705F616C74
 
 Prolog_Prototype(SCAN_DYN_TEST_ALT, 0);
 Prolog_Prototype(SCAN_DYN_JUMP_ALT, 0);
@@ -184,7 +199,7 @@ Prolog_Prototype(SCAN_DYN_JUMP_ALT, 0);
  * must ensure that subsequent retracted clause must be selected and all   *
  * subsequent added clause must be ignored.                                *
  * For added clause we use the count_z value when the selection starts (cf.*
- * stop_cl_no). A clause is ignored it its clause number (cl_no) is >= to  *
+ * stop_cl_no). A clause is ignored if its clause number (cl_no) is >= to  *
  * stop_cl_no.                                                             *
  * For retracted clause we use a stamp incremented at each selection. When *
  * a clause is retracted its erase_stamp is set to the current stamp. Then *
@@ -198,6 +213,9 @@ Prolog_Prototype(SCAN_DYN_JUMP_ALT, 0);
  * reached. Then the local stack is scanned to detect all predicates with  *
  * selections and to mark them. All erased clauses of a predicate which is *
  * not marked are physically destroyed (free).                             *
+ *                                                                         *
+ * pl_file is the file name of its definition (or -1). Used for multifile  *
+ * predicates by consult/1 (see Pl_Update_Dynamic_Pred).
  *-------------------------------------------------------------------------*/
 
 /*-------------------------------------------------------------------------*
@@ -206,14 +224,14 @@ Prolog_Prototype(SCAN_DYN_JUMP_ALT, 0);
  *-------------------------------------------------------------------------*/
 DynCInf *
 Pl_Add_Dynamic_Clause(WamWord head_word, WamWord body_word, Bool asserta,
-		   Bool check_perm)
+		      Bool check_perm, int pl_file)
 {
   WamWord word;
   WamWord *first_arg_adr;
   int func, arity;
   PredInf *pred;
   int index_no;
-  long key;
+  PlLong key;
   DynCInf *clause;
   DynPInf *dyn;
   char **p_ind_htbl;
@@ -248,6 +266,9 @@ Pl_Add_Dynamic_Clause(WamWord head_word, WamWord body_word, Bool asserta,
 			pl_permission_type_static_procedure, word);
     }
 
+  if (pl_file == pl_atom_void)
+    pl_file = -1;
+
   dyn = (DynPInf *) (pred->dyn);
   if (dyn == NULL)		/* dynamic info not yet allocated ? */
     dyn = Alloc_Init_Dyn_Info(pred, arity);
@@ -263,7 +284,7 @@ Pl_Add_Dynamic_Clause(WamWord head_word, WamWord body_word, Bool asserta,
   Pl_Write_Simple(head_word);
   DBGPRINTF("\tbody: ");
   Pl_Write_Simple(body_word);
-  DBGPRINTF("\nByte Code at :%p\n", (long) pl_byte_code);
+  DBGPRINTF("\nByte Code at :%p\n", pl_byte_code);
 #endif
 
 
@@ -279,13 +300,14 @@ Pl_Add_Dynamic_Clause(WamWord head_word, WamWord body_word, Bool asserta,
 
   clause->dyn = dyn;
   clause->cl_no = (asserta) ? dyn->count_a-- : dyn->count_z++;
+  clause->pl_file = pl_file;
   clause->erase_stamp = DYN_STAMP_NONE;
   clause->next_erased_cl = NULL;
   clause->term_size = size;
 
   Pl_Copy_Term(&clause->term_word, &lst_h_b);
 
-  clause->pl_byte_code = pl_byte_code;
+  clause->byte_code = pl_byte_code;
   pl_byte_code = NULL;
 
 
@@ -371,7 +393,7 @@ Alloc_Init_Dyn_Info(PredInf *pred, int arity)
   dyn->first_erased_cl = NULL;
   dyn->next_dyn_with_erase = NULL;
 
-  pred->dyn = (long *) dyn;
+  pred->dyn = (PlLong *) dyn;
 
   return dyn;
 }
@@ -384,7 +406,7 @@ Alloc_Init_Dyn_Info(PredInf *pred, int arity)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 static int
-Index_From_First_Arg(WamWord first_arg_word, long *key)
+Index_From_First_Arg(WamWord first_arg_word, PlLong *key)
 {
   WamWord word, tag_mask;
   int index_no;
@@ -407,7 +429,7 @@ Index_From_First_Arg(WamWord first_arg_word, long *key)
 
     case ATM:
       index_no = ATM_INDEX;
-      *key = (long) UnTag_ATM(word);
+      *key = (PlLong) UnTag_ATM(word);
       break;
 
     case FLT:
@@ -420,7 +442,7 @@ Index_From_First_Arg(WamWord first_arg_word, long *key)
 
     default:			/* tag==STC */
       index_no = STC_INDEX;
-      *key = (long) Functor_And_Arity(UnTag_STC(word));
+      *key = (PlLong) Functor_And_Arity(UnTag_STC(word));
       break;
     }
 
@@ -481,7 +503,7 @@ Remove_From_2Chain(D2ChHdr *hdr, DynCInf *clause, Bool in_seq_chain)
   D2ChCell *cell = (in_seq_chain) ? &clause->seq_chain : &clause->ind_chain;
   DynCInf *prev = cell->prev;
   DynCInf *next = cell->next;
-  
+
   if (prev == NULL)		/* first cell ? */
     hdr->first = next;
   else
@@ -540,6 +562,35 @@ Pl_Delete_Dynamic_Clause(DynCInf *clause)
 
 
 
+/*-------------------------------------------------------------------------*
+ * ERASE_ALL_CLAUSES_OF_FILE                                               *
+ *                                                                         *
+ * This function is called to erase all clauses associated to a given file *
+ * (this is for consult/1 on a multifile pred).                            *
+ *-------------------------------------------------------------------------*/
+static void
+Erase_All_Clauses_Of_File(DynPInf *dyn, int pl_file)
+{
+  DynCInf *clause;
+
+  if (dyn == NULL)
+    return;
+
+  for (clause = dyn->seq_chain.first; clause;
+       clause = clause->seq_chain.next)
+    {
+      if (clause->erase_stamp == DYN_STAMP_NONE && clause->pl_file == pl_file)
+	Pl_Delete_Dynamic_Clause(clause);
+    }
+
+#if 0
+  Clean_Erased_Clauses();
+#endif
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * ERASE_ALL                                                               *
  *                                                                         *
@@ -564,7 +615,7 @@ Erase_All(DynPInf *dyn)
       first_dyn_with_erase = dyn;
     }
 
-  for (clause = dyn->seq_chain.first; clause; 
+  for (clause = dyn->seq_chain.first; clause;
        clause = clause->seq_chain.next)
     {
       if (clause->erase_stamp == DYN_STAMP_NONE)
@@ -576,6 +627,7 @@ Erase_All(DynPInf *dyn)
 
 
 
+
 /*-------------------------------------------------------------------------*
  * CLEAN_ERASED_CLAUSES                                                    *
  *                                                                         *
@@ -602,7 +654,7 @@ Clean_Erased_Clauses(void)
 
       if (dyn->first_erased_cl)	/* we must keep it - free impossible */
 	dyn->first_erased_cl = (DynCInf *)
-	  ((unsigned long) (dyn->first_erased_cl) | 1);	/* mark it */
+	  ((PlULong) (dyn->first_erased_cl) | 1);	/* mark it */
     }
 
 
@@ -610,10 +662,10 @@ Clean_Erased_Clauses(void)
   for (dyn = first_dyn_with_erase; dyn; dyn = dyn1)
     {
       dyn1 = dyn->next_dyn_with_erase;
-      if ((long) (dyn->first_erased_cl) & 1)	/* marked ? */
+      if ((PlLong) (dyn->first_erased_cl) & 1)	/* marked ? */
 	{			/* cannot free it */
 	  dyn->first_erased_cl = (DynCInf *)
-	    ((unsigned long) (dyn->first_erased_cl) & (~1));
+	    ((PlULong) (dyn->first_erased_cl) & (~1));
 	  prev = &(dyn->next_dyn_with_erase);
 	  continue;
 	}
@@ -685,7 +737,7 @@ static void
 Unlink_Clause(DynCInf *clause)
 {
   DynPInf *dyn = clause->dyn;
-  long *p_key;
+  PlLong *p_key;
   DSwtInf swt_info;
 
   Remove_From_2Chain(&dyn->seq_chain, clause, TRUE);
@@ -695,10 +747,10 @@ Unlink_Clause(DynCInf *clause)
   if (clause->p_ind_htbl && clause->ind_chain.prev == NULL &&
       clause->ind_chain.next == NULL)
     {
-      p_key = (long *) ((char *) clause->p_ind_hdr -
+      p_key = (PlLong *) ((char *) clause->p_ind_hdr -
 	((char *) &(swt_info.ind_chain) - (char *) &(swt_info.key)));
 #ifdef DEBUG1
-      DBGPRINTF("Removing last ind key in a hash table  (%ld)\n", *p_key);
+      DBGPRINTF("Removing last ind key in a hash table  (%" PL_FMT_d ")\n", *p_key);
 #endif
       Pl_Hash_Delete(*clause->p_ind_htbl, *p_key);
     }
@@ -714,8 +766,8 @@ Unlink_Clause(DynCInf *clause)
 static void
 Free_Clause(DynCInf *clause)
 {
-  if (clause->pl_byte_code)
-    Free(clause->pl_byte_code);
+  if (clause->byte_code)
+    Free(clause->byte_code);
 
   Free(clause);
 }
@@ -733,10 +785,14 @@ Free_Clause(DynCInf *clause)
  *             2 for '$remove_predicate'/2                                 *
  *             3 for abolish/1                                             *
  *                                                                         *
+ * pl_file_for_multi is for consulting a multifle pred defined in this file*
+ *    (else pl_file_for_multi = -1). In this case, and if a previous       *
+ *    predicate exists, only the clauses defined in this file are removed. *
+ *                                                                         *
  * returns a pointer to associated pred or NULL if it does not exist.      *
  *-------------------------------------------------------------------------*/
 PredInf *
-Pl_Update_Dynamic_Pred(int func, int arity, int what_to_do)
+Pl_Update_Dynamic_Pred(int func, int arity, int what_to_do, int pl_file_for_multi)
 {
   WamWord word;
   PredInf *pred;
@@ -754,8 +810,15 @@ Pl_Update_Dynamic_Pred(int func, int arity, int what_to_do)
 			pl_permission_type_static_procedure, word);
     }
 
-  Erase_All((DynPInf *) (pred->dyn));
-  pred->dyn = NULL;
+  if (pl_file_for_multi >= 0 && (pred->prop & MASK_PRED_MULTIFILE))
+    {
+      Erase_All_Clauses_Of_File((DynPInf *) (pred->dyn), pl_file_for_multi);
+    }
+  else
+    {
+      Erase_All((DynPInf *) (pred->dyn));
+      pred->dyn = NULL;
+    }
 
   if ((what_to_do & 2))
     {
@@ -798,12 +861,12 @@ Get_Scan_Choice_Point(WamWord *b)
  *-------------------------------------------------------------------------*/
 DynCInf *
 Pl_Scan_Dynamic_Pred(int owner_func, int owner_arity,
-		  DynPInf *dyn, WamWord first_arg_word,
-		  ScanFct alt_fct, int alt_fct_type,
-		  int alt_info_size, WamWord *alt_info)
+		     DynPInf *dyn, WamWord first_arg_word,
+		     ScanFct alt_fct, int alt_fct_type,
+		     int alt_info_size, WamWord *alt_info)
 {
   int index_no;
-  long key;
+  PlLong key;
   char **p_ind_htbl;
   DSwtInf *swt;
   DynScan scan;
@@ -852,13 +915,13 @@ Pl_Scan_Dynamic_Pred(int owner_func, int owner_arity,
     case STC_INDEX:
       p_ind_htbl = &(dyn->stc_htbl);
       break;
- 
+
     }
 
   if (p_ind_htbl)
     {
       scan.xxx_is_seq_chain = FALSE;
-      if (*p_ind_htbl && 
+      if (*p_ind_htbl &&
 	  (swt = (DSwtInf *) Pl_Hash_Find(*p_ind_htbl, key)) != NULL)
 	scan.xxx_ind_chain = swt->ind_chain.first;
       else
@@ -909,12 +972,12 @@ Scan_Dynamic_Pred_Next(DynScan *scan)
 {
   DynCInf *xxx_ind_chain, *var_ind_chain;
   DynCInf *xxx_clause, *var_clause;
-  long xxx_nb, var_nb;
+  PlLong xxx_nb, var_nb;
   DynCInf *clause;
 
 #ifdef DEBUG
-  DBGPRINTF("Looking for next clause stamp:%ld",
-	    (long) (scan->erase_stamp));
+  DBGPRINTF("Looking for next clause stamp:%" PL_FMT_d,
+	    (PlLong) (scan->erase_stamp));
   Check_Dynamic_Clauses(scan->dyn);
 #endif
 
@@ -974,7 +1037,7 @@ start:
  * PL_SCAN_DYNAMIC_PRED_ALT_0                                              *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-long
+PlLong
 Pl_Scan_Dynamic_Pred_Alt_0(void)
 {
   WamWord *alt_info;
@@ -1057,11 +1120,11 @@ static void
 Check_Dynamic_Clauses(DynPInf *dyn)
 {
   DBGPRINTF("\nFirst dyn with erase:%p\n",
-	    (long) first_dyn_with_erase);
+	    first_dyn_with_erase);
   DBGPRINTF("Dyn:%p  arity:%d  count_a:%d  count_z:%d  "
 	    "1st erased:%p  next dyn with erase:%p\n",
-	    (long) dyn, dyn->arity, dyn->count_a, dyn->count_z, 
-	    (long) dyn->first_erased_cl, (long) dyn->next_dyn_with_erase);
+	    dyn, dyn->arity, dyn->count_a, dyn->count_z,
+	    dyn->first_erased_cl, dyn->next_dyn_with_erase);
 
   Check_Chain(&dyn->seq_chain, NO_INDEX);
   Check_Chain(&dyn->var_ind_chain, VAR_INDEX);
@@ -1110,7 +1173,7 @@ Check_Hash(char *t, int index_no)
 	DBGPRINTF("val <%s>\n", pl_atom_tbl[swt->key].name);
 
       if (index_no == INT_INDEX)
-	DBGPRINTF("val <%ld>\n", swt->key);
+	DBGPRINTF("val <%" PL_FMT_d ">\n", swt->key);
 
       if (index_no == STC_INDEX)
 	DBGPRINTF("val <%s/%d>\n", pl_atom_tbl[Functor_Of(swt->key)].name,
@@ -1166,14 +1229,14 @@ Check_Chain(D2ChHdr *hdr, int index_no)
 	}
 
       DBGPRINTF(" %3d  %3d  %p  %p <-> %p  ",
-		clause->cl_no, clause->term_size, (long) clause,
-		(long) clause_b, (long) clause_f);
+		clause->cl_no, clause->term_size, clause,
+		clause_b, clause_f);
       Pl_Write_Simple(clause->head_word);
       DBGPRINTF(":-");
       Pl_Write_Simple(clause->body_word);
       if (clause->erase_stamp != DYN_STAMP_NONE)
-	DBGPRINTF("  erased at:%ld   next erased: %p",
-		  clause->erase_stamp, (long) (clause->next_erased_cl));
+	DBGPRINTF("  erased at:%" PL_FMT_d "   next erased: %p",
+		  clause->erase_stamp, (clause->next_erased_cl));
       DBGPRINTF("\n");
     }
 }
