@@ -1,3 +1,4 @@
+
 /*-------------------------------------------------------------------------*
  * GNU Prolog                                                              *
  *                                                                         *
@@ -6,23 +7,36 @@
  * Descr.: dynamic predicate support - header file                         *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: dynam_supp.h,v 1.13 2009/01/23 11:24:13 diaz Exp $ */
+/* $Id: dynam_supp.h,v 1.17 2011/03/28 16:20:05 diaz Exp $ */
 
 /*---------------------------------*
  * Constants                       *
@@ -38,9 +52,9 @@
  * Type Definitions                *
  *---------------------------------*/
 
-typedef long (*ScanFct) ();
+typedef PlLong (*ScanFct) ();
 
-typedef unsigned long DynStamp;
+typedef PlULong DynStamp;
 
 typedef struct dynpinf *DynPInfP;
 
@@ -68,9 +82,10 @@ typedef struct dyncinf		/* Dynamic clause information     */
   D2ChHdr *p_ind_hdr;		/* back ptr to ind_chain header   */
   char **p_ind_htbl;		/* back ptr to ind htbl (or NULL) */
   int cl_no;			/* clause number                  */
+  int pl_file;			/* file name of its definition    */
   DynStamp erase_stamp;		/* FFF...F if not erased or stamp */
   DynCInfP next_erased_cl;	/* pointer to next erased clause  */
-  unsigned *pl_byte_code;		/* bc pointer (NULL=interpreted)  */
+  unsigned *byte_code;		/* bc pointer (NULL=interpreted)  */
   int term_size;		/* size of the term of the clause */
   WamWord term_word;		/* clause [Head|Body]=<LST,adr+1> */
   WamWord head_word;		/* adr+1 = Car = clause term Head */
@@ -83,7 +98,7 @@ DynCInf;
 
 typedef struct			/* Dynamic switch item info       */
 {				/* ------------------------------ */
-  long key;			/* key: atm, int, f/n             */
+  PlLong key;			/* key: atm, int, f/n             */
   D2ChHdr ind_chain;		/* indexical chain                */
 }
 DSwtInf;
@@ -119,11 +134,11 @@ DynPInf;
  *---------------------------------*/
 
 DynCInf *Pl_Add_Dynamic_Clause(WamWord head_word, WamWord body_word,
-			    Bool asserta, Bool check_perm);
+			       Bool asserta, Bool check_perm, int pl_file);
 
 void Pl_Delete_Dynamic_Clause(DynCInf *clause);
 
-PredInf *Pl_Update_Dynamic_Pred(int func, int arity, int what_to_do);
+PredInf *Pl_Update_Dynamic_Pred(int func, int arity, int what_to_do, int pl_file_for_multi);
 
 DynCInf *Pl_Scan_Dynamic_Pred(int owner_func, int owner_arity,
 			   DynPInf *dyn, WamWord first_arg_word,
