@@ -6,23 +6,36 @@
  * Descr.: Prolog errors support                                           *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: error_supp.c,v 1.17 2009/01/23 11:24:13 diaz Exp $ */
+/* $Id: error_supp.c,v 1.27 2011/05/27 16:13:52 diaz Exp $ */
 
 #include <errno.h>
 #include <string.h>
@@ -74,29 +87,32 @@ static char *Context_Error_String(void);
 
 
 
-#define PL_ERR_INSTANTIATION       X24706C5F6572725F696E7374616E74696174696F6E
+#define PL_ERR_INSTANTIATION       X1_24706C5F6572725F696E7374616E74696174696F6E
 
-#define PL_ERR_TYPE                X24706C5F6572725F74797065
+#define PL_ERR_UNINSTANTIATION     X1_24706C5F6572725F756E696E7374616E74696174696F6E
 
-#define PL_ERR_DOMAIN              X24706C5F6572725F646F6D61696E
+#define PL_ERR_TYPE                X1_24706C5F6572725F74797065
 
-#define PL_ERR_EXISTENCE           X24706C5F6572725F6578697374656E6365
+#define PL_ERR_DOMAIN              X1_24706C5F6572725F646F6D61696E
 
-#define PL_ERR_PERMISSION          X24706C5F6572725F7065726D697373696F6E
+#define PL_ERR_EXISTENCE           X1_24706C5F6572725F6578697374656E6365
 
-#define PL_ERR_REPRESENTATION      X24706C5F6572725F726570726573656E746174696F6E
+#define PL_ERR_PERMISSION          X1_24706C5F6572725F7065726D697373696F6E
 
-#define PL_ERR_EVALUATION          X24706C5F6572725F6576616C756174696F6E
+#define PL_ERR_REPRESENTATION      X1_24706C5F6572725F726570726573656E746174696F6E
 
-#define PL_ERR_RESOURCE            X24706C5F6572725F7265736F75726365
+#define PL_ERR_EVALUATION          X1_24706C5F6572725F6576616C756174696F6E
 
-#define PL_ERR_SYNTAX              X24706C5F6572725F73796E746178
+#define PL_ERR_RESOURCE            X1_24706C5F6572725F7265736F75726365
 
-#define PL_ERR_SYSTEM              X24706C5F6572725F73797374656D
+#define PL_ERR_SYNTAX              X1_24706C5F6572725F73796E746178
+
+#define PL_ERR_SYSTEM              X1_24706C5F6572725F73797374656D
 
 
 
 Prolog_Prototype(PL_ERR_INSTANTIATION, 0);
+Prolog_Prototype(PL_ERR_UNINSTANTIATION, 1);
 Prolog_Prototype(PL_ERR_TYPE, 2);
 Prolog_Prototype(PL_ERR_DOMAIN, 2);
 Prolog_Prototype(PL_ERR_EXISTENCE, 2);
@@ -132,7 +148,8 @@ Error_Supp_Initializer(void)
   pl_type_list = Pl_Create_Atom("list");
   pl_type_number = Pl_Create_Atom("number");
   pl_type_predicate_indicator = Pl_Create_Atom("predicate_indicator");
-  pl_type_variable = Pl_Create_Atom("variable");
+  pl_type_variable = Pl_Create_Atom("variable"); /* deprecated: new code should emit an uninstantiation_error */
+  pl_type_pair = Pl_Create_Atom("pair");
   if (pl_fd_init_solver)		/* FD solver linked */
     {
       pl_type_fd_variable = Pl_Create_Atom("fd_variable"); /* for FD */
@@ -157,6 +174,7 @@ Error_Supp_Initializer(void)
   pl_domain_stream_position = Pl_Create_Atom("stream_position");
   pl_domain_stream_property = Pl_Create_Atom("stream_property");
   pl_domain_write_option = Pl_Create_Atom("write_option");
+  pl_domain_order = Pl_Create_Atom("order");
   pl_domain_term_stream_or_alias = Pl_Create_Atom("term_stream_or_alias");
 				/* for term_streams */
   pl_domain_g_array_index = Pl_Create_Atom("g_array_index");/* for g_vars */
@@ -231,7 +249,6 @@ Error_Supp_Initializer(void)
 
 
 
-
 /*-------------------------------------------------------------------------*
  * PL_SET_BIP_NAME_2                                                       *
  *                                                                         *
@@ -252,6 +269,19 @@ Pl_Set_Bip_Name_2(WamWord func_word, WamWord arity_word)
 
 
 
+/*-------------------------------------------------------------------------*
+ * PL_SET_BIP_NAME_UNTAGGED_2                                              *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Pl_Set_Bip_Name_Untagged_2(int func, int arity)
+{
+  cur_bip_func = func;
+  cur_bip_arity = arity;
+}
+
+
+
 /*-------------------------------------------------------------------------*
  * PL_CURRENT_BIP_NAME_2                                                   *
  *                                                                         *
@@ -380,7 +410,7 @@ Context_Error_String(void)
  *-------------------------------------------------------------------------*/
 void
 Pl_Set_Last_Syntax_Error(char *file_name, int err_line, int err_col,
-		      char *err_msg)
+			 char *err_msg)
 {
   last_err_file = file_name;
   last_err_line = err_line;
@@ -397,7 +427,7 @@ Pl_Set_Last_Syntax_Error(char *file_name, int err_line, int err_col,
  *-------------------------------------------------------------------------*/
 Bool
 Pl_Syntax_Error_Info_4(WamWord file_name_word,
-		    WamWord line_word, WamWord char_word, WamWord msg_word)
+		       WamWord line_word, WamWord char_word, WamWord msg_word)
 {
   Pl_Check_For_Un_Atom(file_name_word);
   Pl_Check_For_Un_Integer(line_word);
@@ -505,6 +535,21 @@ Pl_Err_Instantiation(void)
 
 
 
+/*-------------------------------------------------------------------------*
+ * PL_ERR_UNINSTANTIATION                                                  *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Pl_Err_Uninstantiation(WamWord term)
+{
+  Update_Cur_From_C_Bip();
+  A(0) = term;
+  Pl_Execute_A_Continuation(Prolog_Predicate(PL_ERR_UNINSTANTIATION, 1));
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * PL_ERR_TYPE                                                             *
  *                                                                         *
