@@ -6,23 +6,36 @@
  * Descr.: expand term management - C part                                 *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: expand_c.c,v 1.16 2009/01/23 11:24:13 diaz Exp $ */
+/* $Id: expand_c.c,v 1.23 2011/06/09 14:54:37 diaz Exp $ */
 
 #include <sys/types.h>
 
@@ -71,7 +84,7 @@ static WamWord Dcg_Head(WamWord dcg_head_word, WamWord *in_word,
 			WamWord *out_word, WamWord **end_lst_adr);
 
 static WamWord Dcg_Body(WamWord dcg_body_word, Bool for_alt,
-			WamWord in_word, WamWord out_word, 
+			WamWord in_word, WamWord out_word,
 			WamWord *end_lst_adr);
 
 static void Dcg_Body_On_Stack(WamWord dcg_body_word, WamWord in_word,
@@ -144,16 +157,15 @@ Pl_Dcg_Trans_Rule_2(WamWord rule_word, WamWord clause_word)
  *-------------------------------------------------------------------------*/
 Bool
 Pl_Dcg_Trans_Body_4(WamWord dcg_body_word, WamWord in_word, WamWord out_word,
-		 WamWord body_word)
+		    WamWord body_word)
 {
   top = Local_Top;		/* use local stack for the stack */
   opt_term_unif = TRUE;
 
-  in_word = Pl_Globalize_If_In_Local(in_word); 
+  in_word = Pl_Globalize_If_In_Local(in_word);
   out_word = Pl_Globalize_If_In_Local(out_word);
 
-  return Pl_Unify(body_word,
-	       Dcg_Body(dcg_body_word, FALSE, in_word, out_word, NULL));
+  return Pl_Unify(body_word, Dcg_Body(dcg_body_word, FALSE, in_word, out_word, NULL));
 }
 
 
@@ -192,7 +204,7 @@ start:
 	{
 	  if (tag_mask != TAG_LST_MASK)
 	    Pl_Err_Type(pl_type_list, word);
-	  
+
 	  *end_lst_adr = UnTag_LST(word);
 	}
       goto start;
@@ -339,12 +351,12 @@ Dcg_Body_On_Stack(WamWord dcg_body_word, WamWord in_word, WamWord out_word)
 	  return;
 	}
 
-      if (func == ATOM_CHAR(';'))
+      if (func == ATOM_CHAR(';') || func == ATOM_CHAR('|'))
 	{
 	  w1 = Dcg_Body(*adr++, TRUE, in_word, out_word, NULL);
 	  w2 = Dcg_Body(*adr, TRUE, in_word, out_word, NULL);
 
-	  *top++ = Dcg_Compound2(func, w1, w2);
+	  *top++ = Dcg_Compound2(ATOM_CHAR(';'), w1, w2);
 	  return;
 	}
     }
