@@ -6,23 +6,36 @@
  * Descr.: Prolog flag and system variable management - C Part             *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: flag_c.c,v 1.21 2009/02/06 12:20:14 diaz Exp $ */
+/* $Id: flag_c.c,v 1.30 2011/06/01 12:50:20 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -33,6 +46,7 @@
 #define FLAG_C_FILE
 
 #include "engine_pl.h"
+#include "gprolog_cst.h"
 #include "bips_pl.h"
 
 #ifndef _WIN32
@@ -43,7 +57,6 @@ extern char **environ;
 
 
 
-
 /*---------------------------------*
  * Constants                       *
  *---------------------------------*/
@@ -88,7 +101,10 @@ static int atom_chars_no_escape;
 static int atom_codes_no_escape;
 static int atom_atom_no_escape;
 
-static int atom_prolog[4];
+static int atom_the_dialect;
+static int atom_the_home;
+
+static int atom_prolog[11];
 
 
 
@@ -101,9 +117,9 @@ static Bool Unif_Flag(int i, WamWord value_word);
 
 
 
-#define CURRENT_PROLOG_FLAG_ALT    X2463757272656E745F70726F6C6F675F666C61675F616C74
+#define CURRENT_PROLOG_FLAG_ALT    X1_2463757272656E745F70726F6C6F675F666C61675F616C74
 
-#define ENVIRON_ALT                X24656E7669726F6E5F616C74
+#define ENVIRON_ALT                X1_24656E7669726F6E5F616C74
 
 Prolog_Prototype(CURRENT_PROLOG_FLAG_ALT, 0);
 Prolog_Prototype(ENVIRON_ALT, 0);
@@ -121,8 +137,7 @@ Flag_Initializer(void)
   atom_flag_tbl[FLAG_BOUNDED] = Pl_Create_Atom("bounded");
   atom_flag_tbl[FLAG_MAX_INTEGER] = Pl_Create_Atom("max_integer");
   atom_flag_tbl[FLAG_MIN_INTEGER] = Pl_Create_Atom("min_integer");
-  atom_flag_tbl[FLAG_ROUNDING_FCT] =
-    Pl_Create_Atom("integer_rounding_function");
+  atom_flag_tbl[FLAG_ROUNDING_FCT] = Pl_Create_Atom("integer_rounding_function");
 
   atom_flag_tbl[FLAG_CHAR_CONVERSION] = Pl_Create_Atom("char_conversion");
   atom_flag_tbl[FLAG_DEBUG] = Pl_Create_Atom("debug");
@@ -144,7 +159,19 @@ Flag_Initializer(void)
   atom_flag_tbl[FLAG_PROLOG_VERSION] = Pl_Create_Atom("prolog_version");
   atom_flag_tbl[FLAG_PROLOG_DATE] = Pl_Create_Atom("prolog_date");
   atom_flag_tbl[FLAG_PROLOG_COPYRIGHT] = Pl_Create_Atom("prolog_copyright");
+  atom_flag_tbl[FLAG_DIALECT] = Pl_Create_Atom("dialect");
+  atom_flag_tbl[FLAG_HOME] = Pl_Create_Atom("home");
+  atom_flag_tbl[FLAG_HOST_OS] = Pl_Create_Atom("host_os");
+  atom_flag_tbl[FLAG_HOST_VENDOR] = Pl_Create_Atom("host_vendor");
+  atom_flag_tbl[FLAG_HOST_CPU] = Pl_Create_Atom("host_cpu");
+  atom_flag_tbl[FLAG_HOST] = Pl_Create_Atom("host");
+  atom_flag_tbl[FLAG_ARCH] = Pl_Create_Atom("arch");
 
+  atom_flag_tbl[FLAG_VERSION] = Pl_Create_Atom("version");
+  atom_flag_tbl[FLAG_VERSION_DATA] = Pl_Create_Atom("version_data");
+  atom_flag_tbl[FLAG_UNIX] = Pl_Create_Atom("unix");
+
+  atom_flag_tbl[FLAG_ARGV] = Pl_Create_Atom("argv");
 
   atom_down = Pl_Create_Atom("down");
   atom_toward_zero = Pl_Create_Atom("toward_zero");
@@ -162,10 +189,21 @@ Flag_Initializer(void)
   atom_codes_no_escape = Pl_Create_Atom("codes_no_escape");
   atom_atom_no_escape = Pl_Create_Atom("atom_no_escape");
 
+  atom_the_dialect = Pl_Create_Atom(PROLOG_DIALECT);
+
+  atom_the_home = Pl_Create_Atom(pl_home ? pl_home : "");
+
   atom_prolog[0] = Pl_Create_Atom(PROLOG_NAME);
   atom_prolog[1] = Pl_Create_Atom(PROLOG_VERSION);
   atom_prolog[2] = Pl_Create_Atom(PROLOG_DATE);
   atom_prolog[3] = Pl_Create_Atom(PROLOG_COPYRIGHT);
+  atom_prolog[4] = atom_the_dialect;
+  atom_prolog[5] = atom_the_home;
+  atom_prolog[6] = Pl_Create_Atom(M_CPU);
+  atom_prolog[7] = Pl_Create_Atom(M_VENDOR);
+  atom_prolog[8] = Pl_Create_Atom(M_OS);
+  atom_prolog[9] = Pl_Create_Atom(M_CPU "-" M_VENDOR "-" M_OS);
+  atom_prolog[10] = Pl_Create_Atom(M_CPU "-" M_OS);
 
   Flag_Value(FLAG_SINGLETON_WARNING) = 1;
   Flag_Value(FLAG_SUSPICIOUS_WARNING) = 1;
@@ -173,7 +211,12 @@ Flag_Initializer(void)
   Flag_Value(FLAG_STRICT_ISO) = 1;
 
   Flag_Value(FLAG_DOUBLE_QUOTES) = FLAG_AS_CODES;
-  Flag_Value(FLAG_BACK_QUOTES) = FLAG_AS_ATOM | FLAG_NO_ESCAPE_MASK;
+
+  /* DON'T CHANGE this FLAG_BACK_QUOTES default: NO_ESCAPE is useful under
+   * Windows when assoc .pl to gprolog (see InnoSetup) and avoid \
+   * to be misinterpreted in pathnames (e.g. c:\foo\bar).
+   */
+  Flag_Value(FLAG_BACK_QUOTES) = FLAG_AS_ATOM | FLAG_NO_ESCAPE_MASK; 
 
 #ifndef NO_USE_LINEDIT
   SYS_VAR_LINEDIT = 1;
@@ -193,6 +236,7 @@ Bool
 Pl_Set_Prolog_Flag_2(WamWord flag_word, WamWord value_word)
 {
   WamWord word, tag_mask;
+  WamWord *adr;
   WamWord term;
   int atom;
   int i;
@@ -216,6 +260,7 @@ Pl_Set_Prolog_Flag_2(WamWord flag_word, WamWord value_word)
   switch (i)
     {
     case FLAG_BOUNDED:
+    case FLAG_UNIX:
       if (tag_mask != TAG_ATM_MASK ||
 	  (atom != pl_atom_true && atom != pl_atom_false))
 	goto err_value;
@@ -226,6 +271,7 @@ Pl_Set_Prolog_Flag_2(WamWord flag_word, WamWord value_word)
     case FLAG_MAX_ARITY:
     case FLAG_MAX_ATOM:
     case FLAG_MAX_UNGET:
+    case FLAG_VERSION:
       if (tag_mask != TAG_INT_MASK)
 	goto err_value;
       goto err_perm;
@@ -306,9 +352,25 @@ Pl_Set_Prolog_Flag_2(WamWord flag_word, WamWord value_word)
     case FLAG_PROLOG_VERSION:
     case FLAG_PROLOG_DATE:
     case FLAG_PROLOG_COPYRIGHT:
+    case FLAG_DIALECT:
+    case FLAG_HOME:
+    case FLAG_HOST_OS:
+    case FLAG_HOST_VENDOR:
+    case FLAG_HOST_CPU:
+    case FLAG_HOST:
+    case FLAG_ARCH:
       if (tag_mask != TAG_ATM_MASK)
 	goto err_value;
       goto err_perm;
+
+
+    case FLAG_VERSION_DATA:
+      if (tag_mask != TAG_STC_MASK)
+	goto err_value;
+      adr = UnTag_STC(word);
+      if (Functor(adr) != atom_the_dialect || Arity(adr) != 4)
+	goto err_value;
+      goto err_perm;
     }
 
   return TRUE;
@@ -419,7 +481,7 @@ static Bool
 Unif_Flag(int i, WamWord value_word)
 {
   int atom = -1;
-  long n;
+  PlLong n;
 
   switch (i)
     {
@@ -452,6 +514,14 @@ Unif_Flag(int i, WamWord value_word)
       n = MAX_ARITY;
       break;
 
+    case FLAG_MAX_ATOM:
+      n = MAX_ATOM;
+      break;
+
+    case FLAG_MAX_UNGET:
+      n = STREAM_PB_SIZE;
+      break;
+
     case FLAG_UNKNOWN:
     case FLAG_SYNTAX_ERROR:
     case FLAG_OS_ERROR:
@@ -505,22 +575,50 @@ Unif_Flag(int i, WamWord value_word)
     case FLAG_PROLOG_VERSION:
     case FLAG_PROLOG_DATE:
     case FLAG_PROLOG_COPYRIGHT:
+    case FLAG_DIALECT:
+    case FLAG_HOME:
+    case FLAG_HOST_OS:
+    case FLAG_HOST_CPU:
+    case FLAG_HOST_VENDOR:
+    case FLAG_HOST:
+    case FLAG_ARCH:
       atom = atom_prolog[i - FLAG_PROLOG_NAME];
       break;
 
-    case FLAG_MAX_ATOM:
-      n = MAX_ATOM;
+    case FLAG_VERSION:
+      n = __GPROLOG_VERSION__;
       break;
 
-    case FLAG_MAX_UNGET:
-      n = STREAM_PB_SIZE;
+    case FLAG_VERSION_DATA:
+      return Pl_Get_Structure(atom_the_dialect, 4, value_word) &&
+	Pl_Unify_Integer(__GPROLOG__) &&
+	Pl_Unify_Integer(__GPROLOG_MINOR__) &&
+	Pl_Unify_Integer(__GPROLOG_PATCHLEVEL__) &&
+	Pl_Unify_Nil();
+
+    case FLAG_UNIX:
+#if defined(__unix__) || defined(__CYGWIN__) || defined(unix)
+      atom = pl_atom_true;
+#else
+      atom = pl_atom_false;
+#endif
       break;
+
+    case FLAG_ARGV:
+      for (i = 0; i < pl_os_argc; i++)
+	{
+	  if (!Pl_Get_List(value_word) || !Pl_Unify_Atom(Pl_Create_Atom(pl_os_argv[i])))
+	    return FALSE;
+
+	  value_word = Pl_Unify_Variable();
+	}
+      return Pl_Get_Nil(value_word);
     }
 
   if (atom < 0)
     return Pl_Get_Integer(n, value_word);
 
- return Pl_Get_Atom(atom, value_word);
+  return Pl_Get_Atom(atom, value_word);
 }
 
 
@@ -713,7 +811,7 @@ void
 Pl_Set_Current_B_1(WamWord b_word)
 {
   WamWord word, tag_mask;
-  
+
   DEREF(b_word, word, tag_mask);
   Pl_Cut(word);
 }
@@ -727,8 +825,8 @@ Pl_Set_Current_B_1(WamWord b_word)
  *-------------------------------------------------------------------------*/
 
 /* these macros are to avoid gcc warning warn_unused_result */
-#define FWRITE(b, sz, n, f) if (fwrite(b, sz, n, f) != n) ; 
-#define FREAD(b, sz, n, f)  if (fread(b, sz, n, f) != n) ; 
+#define FWRITE(b, sz, n, f) if (fwrite(b, sz, n, f) != n) ;
+#define FREAD(b, sz, n, f)  if (fread(b, sz, n, f) != n) ;
 
 
 Bool
@@ -785,6 +883,9 @@ Pl_Write_Pl_State_File(WamWord file_word)
   i = Flag_Value(FLAG_MULTIFILE_WARNING);
   FWRITE(&i, sizeof(i), 1, f);
 
+  i = Flag_Value(FLAG_STRICT_ISO);
+  FWRITE(&i, sizeof(i), 1, f);
+
   i = SYS_VAR_SAY_GETC;
   FWRITE(&i, sizeof(i), 1, f);
 
@@ -859,6 +960,9 @@ Pl_Read_Pl_State_File(WamWord file_word)
   FREAD(&i, sizeof(i), 1, f);
   Flag_Value(FLAG_MULTIFILE_WARNING) = i;
 
+  FREAD(&i, sizeof(i), 1, f);
+  Flag_Value(FLAG_STRICT_ISO) = i;
+
   FREAD(&i, sizeof(i), 1, f);
   SYS_VAR_SAY_GETC = i;
 
