@@ -6,23 +6,36 @@
  * Descr.: Prolog flag and system variable support - header file           *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: flag_supp.h,v 1.17 2009/01/23 11:24:13 diaz Exp $ */
+/* $Id: flag_supp.h,v 1.22 2011/03/28 16:20:05 diaz Exp $ */
 
 /*---------------------------------*
  * Constants                       *
@@ -55,6 +68,19 @@ enum
   FLAG_PROLOG_VERSION,
   FLAG_PROLOG_DATE,
   FLAG_PROLOG_COPYRIGHT,
+  FLAG_DIALECT,
+  FLAG_HOME,
+  FLAG_HOST_CPU,
+  FLAG_HOST_VENDOR,
+  FLAG_HOST_OS,
+  FLAG_HOST,
+  FLAG_ARCH,
+
+  FLAG_VERSION_DATA,
+  FLAG_VERSION,
+  FLAG_UNIX,
+
+  FLAG_ARGV,
 
   NB_OF_FLAGS			/* this gives us the number of used flags */
 };
@@ -67,7 +93,7 @@ enum
 
 
      /* values for double_quotes and back_quotes */
-#define FLAG_AS_CODES              0	/* bit 2 is set if no_escape */	
+#define FLAG_AS_CODES              0	/* bit 2 is set if no_escape */
 #define FLAG_AS_CHARS              1
 #define FLAG_AS_ATOM               2
 #define FLAG_NO_ESCAPE_BIT         2
@@ -111,11 +137,11 @@ enum
 
 #ifdef FLAG_C_FILE
 
-long pl_sys_var[MAX_SYS_VARS];
+PlLong pl_sys_var[MAX_SYS_VARS];
 
 #else
 
-extern long pl_sys_var[];
+extern PlLong pl_sys_var[];
 
 #endif
 
@@ -128,7 +154,7 @@ extern long pl_sys_var[];
 
 Bool Pl_Read_Pl_State_File(WamWord file_word);
 Bool Pl_Write_Pl_State_File(WamWord file_word);
-  
+
 
 
 
@@ -150,6 +176,6 @@ Bool Pl_Write_Pl_State_File(WamWord file_word);
  *                                                                         *
  *  20: permanent: should stream fcts emit a char before calling fgetc ?   *
  *                                                                         *
- * 100..199: free for users (who know pl_sys_var[] exists !)                  *
+ * 100..199: free for users (who know pl_sys_var[] exists !)               *
  * 200..: some prolog flag values.                                         *
  *-------------------------------------------------------------------------*/
