@@ -6,23 +6,36 @@
  * Descr.: foreign interface support                                       *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: foreign_supp.c,v 1.15 2009/01/23 11:24:13 diaz Exp $ */
+/* $Id: foreign_supp.c,v 1.22 2011/06/10 13:58:09 diaz Exp $ */
 
 #include <string.h>
 
@@ -53,10 +66,10 @@
  * Global Variables                *
  *---------------------------------*/
 
-long pl_foreign_long[NB_OF_X_REGS];
+PlLong pl_foreign_long[NB_OF_X_REGS];
 double pl_foreign_double[NB_OF_X_REGS];
 
-long *pl_base_fl = pl_foreign_long;	/* overwrite var of engine.c */
+PlLong *pl_base_fl = pl_foreign_long;	/* overwrite var of engine.c */
 double *pl_base_fd = pl_foreign_double;	/* overwrite var of engine.c */
 
 static PlFIOArg fio_arg_array[NB_OF_X_REGS];
@@ -68,7 +81,7 @@ static WamWord **query_stack_top = query_stack;
 static WamWord *goal_H;
 
 WamWord *pl_query_top_b;		/* overwrite var of throw_c.c */
-WamWord pl_query_exception;	/* overwrite var of throw_c.c */
+WamWord pl_query_exception;		/* overwrite var of throw_c.c */
 
 
 
@@ -81,9 +94,9 @@ static CodePtr Prepare_Call(int func, int arity, WamWord *arg_adr);
 
 
 
-#define CALL_INTERNAL              X2463616C6C5F696E7465726E616C
+#define CALL_INTERNAL              X1_2463616C6C5F696E7465726E616C
 
-#define PL_QUERY_RECOVER_ALT       X24706C5F71756572795F7265636F7665725F616C74
+#define PL_QUERY_RECOVER_ALT       X1_24706C5F71756572795F7265636F7665725F616C74
 
 Prolog_Prototype(PL_QUERY_RECOVER_ALT, 0);
 Prolog_Prototype(CALL_INTERNAL, 2);
@@ -160,8 +173,8 @@ Pl_Foreign_Jump_Ret(CodePtr codep)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 PlFIOArg *
-Pl_Foreign_Rd_IO_Arg(int arg_long, WamWord start_word, long (*rd_fct) (),
-		  int fio_arg_index)
+Pl_Foreign_Rd_IO_Arg(int arg_long, WamWord start_word, PlLong (*rd_fct) (),
+		     int fio_arg_index)
 {
   WamWord word, tag_mask;
   PlFIOArg *fa = fio_arg_array + fio_arg_index;
@@ -171,7 +184,7 @@ Pl_Foreign_Rd_IO_Arg(int arg_long, WamWord start_word, long (*rd_fct) (),
   fa->is_var = fa->unify = (tag_mask == TAG_REF_MASK);
 
   if (rd_fct == NULL)
-    fa->value.l = (long) word;
+    fa->value.l = (PlLong) word;
   else if (!fa->is_var)
     {
       if (arg_long)
@@ -196,7 +209,7 @@ Pl_Foreign_Rd_IO_Arg(int arg_long, WamWord start_word, long (*rd_fct) (),
  *-------------------------------------------------------------------------*/
 Bool
 Pl_Foreign_Un_IO_Arg(int arg_long, Bool (*un_fct) (), PlFIOArg *fa,
-		  WamWord start_word)
+		     WamWord start_word)
 {
   if (!fa->unify)
     return TRUE;
@@ -237,7 +250,8 @@ Prepare_Call(int func, int arity, WamWord *arg_adr)
   int bip_func, bip_arity;
 
   pred = Pl_Lookup_Pred(func, arity);
-  if (pred == NULL || !(pred->prop & MASK_PRED_NATIVE_CODE))
+  if (pred == NULL || !(pred->prop & MASK_PRED_NATIVE_CODE) || 
+      (pred->prop & MASK_PRED_CONTROL_CONSTRUCT))
     {
       if (arity == 0)
 	A(0) = Tag_ATM(func);
@@ -415,7 +429,7 @@ Pl_Get_Exception(void)
  * PL_NO_MORE_CHOICE                                                       *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-void 
+void
 Pl_No_More_Choice(void)
 {
   Delete_Last_Choice_Point();
