@@ -6,23 +6,36 @@
  * Descr.: inline predicate prototypes - header file                       *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: inl_protos.h,v 1.18 2009/01/23 11:24:13 diaz Exp $ */
+/* $Id: inl_protos.h,v 1.23 2011/03/28 16:20:05 diaz Exp $ */
 
 /*---------------------------------*
  * Constants                       *
@@ -60,6 +73,8 @@ Bool FC Pl_Blt_Compound(WamWord x);
 
 Bool FC Pl_Blt_Callable(WamWord x);
 
+Bool FC Pl_Blt_Ground(WamWord x);
+
 Bool FC Pl_Blt_Fd_Var(WamWord x);
 
 Bool FC Pl_Blt_Non_Fd_Var(WamWord x);
@@ -166,6 +181,8 @@ WamWord FC Pl_Fct_Fast_Rem(WamWord x, WamWord y);
 
 WamWord FC Pl_Fct_Fast_Mod(WamWord x, WamWord y);
 
+WamWord FC Pl_Fct_Fast_Div2(WamWord x, WamWord y);
+
 WamWord FC Pl_Fct_Fast_And(WamWord x, WamWord y);
 
 WamWord FC Pl_Fct_Fast_Or(WamWord x, WamWord y);
@@ -178,10 +195,23 @@ WamWord FC Pl_Fct_Fast_Shl(WamWord x, WamWord y);
 
 WamWord FC Pl_Fct_Fast_Shr(WamWord x, WamWord y);
 
+WamWord FC Pl_Fct_Fast_LSB(WamWord x);
+
+WamWord FC Pl_Fct_Fast_MSB(WamWord x);
+
+WamWord FC Pl_Fct_Fast_Popcount(WamWord x);
+
 WamWord FC Pl_Fct_Fast_Abs(WamWord x);
 
 WamWord FC Pl_Fct_Fast_Sign(WamWord x);
 
+WamWord FC Pl_Fct_Fast_GCD(WamWord b, WamWord x);
+
+WamWord FC Pl_Fct_Fast_Log_Radix(WamWord b, WamWord x);
+
+
+
+
 WamWord FC Pl_Fct_Neg(WamWord x);
 
 WamWord FC Pl_Fct_Inc(WamWord x);
@@ -202,6 +232,8 @@ WamWord FC Pl_Fct_Rem(WamWord x, WamWord y);
 
 WamWord FC Pl_Fct_Mod(WamWord x, WamWord y);
 
+WamWord FC Pl_Fct_Div2(WamWord x, WamWord y);
+
 WamWord FC Pl_Fct_And(WamWord x, WamWord y);
 
 WamWord FC Pl_Fct_Or(WamWord x, WamWord y);
@@ -216,20 +248,34 @@ WamWord FC Pl_Fct_Shr(WamWord x, WamWord y);
 
 WamWord FC Pl_Fct_Abs(WamWord x);
 
+WamWord FC Pl_Fct_LSB(WamWord x);
+
+WamWord FC Pl_Fct_MSB(WamWord x);
+
+WamWord FC Pl_Fct_Popcount(WamWord x);
+
 WamWord FC Pl_Fct_Sign(WamWord x);
 
 
 
+WamWord FC Pl_Fct_GCD(WamWord x, WamWord y);
+
 WamWord FC Pl_Fct_Min(WamWord x, WamWord y);
 
 WamWord FC Pl_Fct_Max(WamWord x, WamWord y);
 
+WamWord FC Pl_Fct_Integer_Pow(WamWord x, WamWord y);
+
 WamWord FC Pl_Fct_Pow(WamWord x, WamWord y);
 
 WamWord FC Pl_Fct_Sqrt(WamWord x);
 
+WamWord FC Pl_Fct_Tan(WamWord x);
+
 WamWord FC Pl_Fct_Atan(WamWord x);
 
+WamWord FC Pl_Fct_Atan2(WamWord x, WamWord y);
+
 WamWord FC Pl_Fct_Cos(WamWord x);
 
 WamWord FC Pl_Fct_Acos(WamWord x);
@@ -238,10 +284,26 @@ WamWord FC Pl_Fct_Sin(WamWord x);
 
 WamWord FC Pl_Fct_Asin(WamWord x);
 
+WamWord FC Pl_Fct_Tanh(WamWord x);
+
+WamWord FC Pl_Fct_Atanh(WamWord x);
+
+WamWord FC Pl_Fct_Cosh(WamWord x);
+
+WamWord FC Pl_Fct_Acosh(WamWord x);
+
+WamWord FC Pl_Fct_Sinh(WamWord x);
+
+WamWord FC Pl_Fct_Asinh(WamWord x);
+
 WamWord FC Pl_Fct_Exp(WamWord x);
 
 WamWord FC Pl_Fct_Log(WamWord x);
 
+WamWord FC Pl_Fct_Log10(WamWord x);
+
+WamWord FC Pl_Fct_Log_Radix(WamWord b, WamWord x);
+
 WamWord FC Pl_Fct_Float(WamWord x);
 
 WamWord FC Pl_Fct_Ceiling(WamWord x);
@@ -258,6 +320,15 @@ WamWord FC Pl_Fct_Float_Integ_Part(WamWord x);
 
 WamWord FC Pl_Fct_Identity(WamWord x);
 
+WamWord FC Pl_Fct_PI(void);
+
+WamWord FC Pl_Fct_E(void);
+
+WamWord FC Pl_Fct_Epsilon(void);
+
+
+
+
 Bool FC Pl_Blt_Fast_Eq(WamWord x, WamWord y);
 
 Bool FC Pl_Blt_Fast_Neq(WamWord x, WamWord y);
