@@ -6,23 +6,36 @@
  * Descr.: operator management - C part                                    *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: oper_c.c,v 1.14 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: oper_c.c,v 1.21 2011/03/28 16:20:05 diaz Exp $ */
 
 #include <string.h>
 
@@ -72,7 +85,7 @@ static int Detect_Oper_Specif(OperInf *oper);
 
 
 
-#define CURRENT_OP_ALT             X2463757272656E745F6F705F616C74
+#define CURRENT_OP_ALT             X1_2463757272656E745F6F705F616C74
 
 Prolog_Prototype(CURRENT_OP_ALT, 0);
 
@@ -149,14 +162,18 @@ Pl_Op_3(WamWord prec_word, WamWord specif_word, WamWord oper_word)
       type = left = right = 0;	/* only for the compiler */
     }
 
-  if (type != PREFIX
-      && Check_Oper(atom_op, (type == POSTFIX) ? INFIX : POSTFIX))
-    Pl_Err_Permission(pl_permission_operation_create, pl_permission_type_operator,
+  if ((type != PREFIX && Check_Oper(atom_op, (type == POSTFIX) ? INFIX : POSTFIX)) || /* infix + postfix invalid */
+      (atom_op == ATOM_CHAR('|') && (type != INFIX || (prec > 0 && prec <= 1000))) || /* | no infix or prec <= 1000 */
+      (atom_op == ATOM_NIL || atom_op == pl_atom_curly_brackets)) /* [] or {} forbidden */
+    Pl_Err_Permission(pl_permission_operation_create,
+		      pl_permission_type_operator,
 		      oper_word);
 
+
   if (atom_op == ATOM_CHAR(','))
     Pl_Err_Permission(pl_permission_operation_modify,
-		      pl_permission_type_operator, oper_word);
+		      pl_permission_type_operator,
+		      oper_word);
 
   if (prec > 0)
     Pl_Create_Oper(atom_op, type, prec, left, right);
@@ -176,7 +193,7 @@ Pl_Current_Op_3(WamWord prec_word, WamWord specif_word, WamWord oper_word)
 {
   WamWord word, tag_mask;
   HashScan scan;
-  int prec;
+  PlLong prec;
   int atom_specif;
   OperInf *oper;
   int atom;
@@ -191,7 +208,7 @@ Pl_Current_Op_3(WamWord prec_word, WamWord specif_word, WamWord oper_word)
 
   DEREF(prec_word, word, tag_mask);
   prec = UnTag_INT(word);
-  if (tag_mask != TAG_REF_MASK && 
+  if (tag_mask != TAG_REF_MASK &&
       (tag_mask != TAG_INT_MASK || prec < 0 || prec > MAX_PREC))
     Pl_Err_Domain(pl_domain_operator_priority, word);
   prec_word = word;
