@@ -1,28 +1,41 @@
-/*-------------------------------------------------------------------------* 
- * GNU Prolog                                                              * 
- *                                                                         * 
- * Part  : Prolog buit-in predicates                                       * 
- * File  : os_interf.pl                                                    * 
- * Descr.: operating system interface management                           * 
- * Author: Daniel Diaz                                                     * 
- *                                                                         * 
- * Copyright (C) 1999-2009 Daniel Diaz                                     * 
- *                                                                         * 
- * GNU Prolog is free software; you can redistribute it and/or modify it   * 
- * under the terms of the GNU General Public License as published by the   * 
- * Free Software Foundation; either version 2, or any later version.       * 
- *                                                                         * 
- * GNU Prolog is distributed in the hope that it will be useful, but       * 
- * WITHOUT ANY WARRANTY; without even the implied warranty of              * 
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        * 
- * General Public License for more details.                                * 
- *                                                                         * 
- * You should have received a copy of the GNU General Public License along * 
- * with this program; if not, write to the Free Software Foundation, Inc.  * 
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               * 
+/*-------------------------------------------------------------------------*
+ * GNU Prolog                                                              *
+ *                                                                         *
+ * Part  : Prolog buit-in predicates                                       *
+ * File  : os_interf.pl                                                    *
+ * Descr.: operating system interface management                           *
+ * Author: Daniel Diaz                                                     *
+ *                                                                         *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ *                                                                         *
+ * This file is part of GNU Prolog                                         *
+ *                                                                         *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
+ * General Public License for more details.                                *
+ *                                                                         *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: os_interf.pl,v 1.15 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: os_interf.pl,v 1.19 2011/03/28 16:20:05 diaz Exp $ */
 
 :-	built_in.
 
@@ -66,7 +79,7 @@ directory_files(PathName, List) :-
 rename_file(PathName1, PathName2) :-
 	set_bip_name(rename_file, 2),
 	'$call_c_test'('Pl_Rename_File_2'(PathName1, PathName2)).
-	
+
 
 
 
@@ -212,7 +225,7 @@ architecture(Architecture) :-
 	set_bip_name(architecture, 1),
 	'$call_c_test'('Pl_Architecture_1'(Architecture)).
 
-	
+
 
 
 shell :-
@@ -267,14 +280,14 @@ popen(Cmd, Mode, Stream) :-
 
 
           % exec mask in sys_var[0]:
-          %     b0 
+          %     b0
           %     0/1
           %  Pid used
 
 exec(Cmd, StreamIn, StreamOut, StreamErr, Pid) :-
 	set_bip_name(exec, 5),
 	(   nonvar(Pid) ->
-	    '$pl_err_type'(variable, Pid)
+	    '$pl_err_uninstantiation'(Pid)
 	;   true
 	),
 	'$sys_var_write'(0, 0),
@@ -313,7 +326,7 @@ create_pipe(StreamIn, StreamOut) :-
 fork_prolog(Pid) :-
 	set_bip_name(fork_prolog, 1),
 	(   nonvar(Pid) ->
-	    '$pl_err_type'(variable, Pid)
+	    '$pl_err_uninstantiation'(Pid)
 	;   true
 	),
 	'$call_c_test'('Pl_Fork_Prolog_1'(Pid)).
