@@ -6,23 +6,36 @@
  * Descr.: parser support                                                  *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: parse_supp.c,v 1.16 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: parse_supp.c,v 1.24 2011/05/16 13:36:07 diaz Exp $ */
 
 #include <stdio.h>
 #include <string.h>
@@ -64,10 +77,12 @@
 static StmInf *pstm_i;
 
 static Bool tok_present;
+static TokInf unget_tok;
+
 
 static jmp_buf jumper;
 
-#if !defined(NO_USE_REGS) && NB_OF_USED_MACHINE_REGS>0
+#if !defined(NO_USE_REGS) && NB_OF_USED_MACHINE_REGS > 0
 static WamWord buff_save_machine_regs[NB_OF_USED_MACHINE_REGS];
 #endif
 
@@ -103,15 +118,22 @@ static int Lookup_In_Dico_Var(char *name);
 static void Parse_Error(char *err_msg);
 
 
+/* we simply save line/col (for error report) to avoid to 
+ * duplicate the entire token (with the big buffer for names).
+ * After Unget_Token it is possible to restore pl_token.line/col */
 
-#define Unget_Token           tok_present = TRUE
+#define Unget_Token				\
+do {						\
+  tok_present = TRUE;				\
+  unget_tok = pl_token;				\
+} while(0)
 
 
 
 #define Update_Last_Read_Position		\
 {						\
-  pl_last_read_line = pl_token.line;			\
-  pl_last_read_col = pl_token.col;			\
+  pl_last_read_line = pl_token.line;		\
+  pl_last_read_col = pl_token.col;		\
 }
 
 
@@ -146,7 +168,24 @@ Read_Next_Token(Bool comma_is_punct)
   char *err_msg;
 
   if (tok_present)
-    tok_present = FALSE;
+    {
+      tok_present = FALSE;
+      pl_token = unget_tok;
+      if (comma_is_punct && pl_token.type == TOKEN_NAME &&  !pl_token.quoted &&
+	  pl_token.name[0] == ',' && pl_token.name[1] == '\0')
+	{
+	  pl_token.type = TOKEN_PUNCTUATION;
+	  pl_token.punct = ',';
+	}
+      else if (!comma_is_punct && pl_token.type == TOKEN_PUNCTUATION && 
+	       pl_token.punct == ',')
+	{
+	  pl_token.type = TOKEN_NAME;
+	  pl_token.quoted = FALSE;
+	  pl_token.name[0] = ',';
+	  pl_token.name[1] = '\0';
+	}
+    }
   else if ((err_msg = Pl_Scan_Token(pstm_i, comma_is_punct)) != NULL)
     Parse_Error(err_msg);
 }
@@ -185,8 +224,8 @@ Pl_Read_Term(StmInf *pstm, int parse_end_of_term)
   if (jmp_val == 0)
     {
       term = Parse_Term(MAX_PREC, GENERAL_TERM, COMMA_ANY);
-      Read_Next_Token(COMMA_ANY);
 
+      Read_Next_Token(COMMA_ANY);
       if (term == NOT_A_WAM_WORD)
 	{
 	  if (pl_token.type == TOKEN_END_OF_FILE)
@@ -195,7 +234,10 @@ Pl_Read_Term(StmInf *pstm, int parse_end_of_term)
 	      goto finish;
 	    }
 	  else
-	    Parse_Error("expression expected");
+	    {
+	      /* Unget_Token; */ /* useless if followed by Parse_Error */
+	      Parse_Error("expression expected");
+	    }
 	}
 
       if (parse_end_of_term == PARSE_END_OF_TERM_DOT)
@@ -204,7 +246,7 @@ Pl_Read_Term(StmInf *pstm, int parse_end_of_term)
 	    goto finish;
 	  else
 	    {
-	      Unget_Token;
+	      /* Unget_Token; */ /* useless if followed by Parse_Error */
 	      Parse_Error(". or operator expected after expression");
 	    }
 	}
@@ -213,7 +255,7 @@ Pl_Read_Term(StmInf *pstm, int parse_end_of_term)
 	goto finish;
       else
 	{
-	  Unget_Token;
+          /* Unget_Token; */ /* useless if followed by Parse_Error */
 	  Parse_Error("eof or operator expected after expression");
 	}
     }
@@ -223,7 +265,7 @@ Pl_Read_Term(StmInf *pstm, int parse_end_of_term)
       term = NOT_A_WAM_WORD;
     }
 
-finish:
+ finish:
   pl_use_le_prompt = save_use_le_prompt;
   return term;
 }
@@ -296,9 +338,9 @@ Parse_Term(int cur_prec, int context, Bool comma_is_punct)
 
     case TOKEN_STRING:
     case TOKEN_BACK_QUOTED:	/* undefined in ISO */
-      flag_value = (pl_token.type == TOKEN_STRING) ? 
+      flag_value = (pl_token.type == TOKEN_STRING) ?
 	Flag_Value(FLAG_DOUBLE_QUOTES) :
-	Flag_Value(FLAG_BACK_QUOTES);
+      Flag_Value(FLAG_BACK_QUOTES);
 
       flag_value &= FLAG_AS_PART_MASK;
       if (flag_value == FLAG_AS_ATOM)
@@ -350,24 +392,47 @@ Parse_Term(int cur_prec, int context, Bool comma_is_punct)
 	  break;
 	}
 
-      /* maybe a prefix operator */
-      if ((oper = Pl_Lookup_Oper(atom, PREFIX)) && cur_prec >= oper->prec)
-	{			/* negative number */
-	  if (pl_token.name[0] == '-' && pl_token.name[1] == '\0' &&
-	      isdigit(Pl_Scan_Peek_Char(pstm_i, TRUE)))
+      /* test if it is a negative number */
+      if (pl_token.name[0] == '-' && pl_token.name[1] == '\0'
+#ifdef MINUS_SIGN_CANNOT_BE_FOLLOWED_BY_SPACES
+	  && isdigit(Pl_Scan_Peek_Char(pstm_i, TRUE))
+#endif
+	  )
+	{
+	  int save_line = pl_token.line;
+	  int save_col = pl_token.col;
+
+	  Read_Next_Token(COMMA_ANY);
+	  if (pl_token.type == TOKEN_INTEGER)
 	    {
-	      Read_Next_Token(COMMA_ANY);
-	      if (pl_token.type == TOKEN_INTEGER)
-		{
-		  if (pl_token.int_num > -INT_LOWEST_VALUE)
-		    Parse_Error("integer underflow (exceeds min_integer)");
-		  term = Pl_Put_Integer(-pl_token.int_num);
-		}
-	      else
-		term = Pl_Put_Float(-pl_token.float_num);
+	      if (pl_token.int_num > -INT_LOWEST_VALUE)
+		Parse_Error("integer underflow (exceeds min_integer)");
+	      term = Pl_Put_Integer(-pl_token.int_num);
+	      break;
+	    }
+	  
+	  if (pl_token.type == TOKEN_FLOAT)
+	    {
+	      term = Pl_Put_Float(-pl_token.float_num);
 	      break;
 	    }
 
+	  /* '-' not followed by a number, pushback this token */
+	  /* (cannot occur ifdef MINUS_SIGN_CANNOT_BE_FOLLOWED_BY_SPACES) */
+
+	  Unget_Token;
+
+	  /* restore token */
+	  pl_token.type = TOKEN_NAME;
+	  strcpy(pl_token.name, "-");
+	  pl_token.line = save_line;
+	  pl_token.col = save_col;
+	}
+
+
+      /* maybe a prefix operator */
+      if ((oper = Pl_Lookup_Oper(atom, PREFIX)) && cur_prec >= oper->prec)
+	{
 	  /* try a prefix operator */
 	  cur_left = oper->prec;
 	  term = Parse_Term(oper->right, TRYING_PREFIX, comma_is_punct);
@@ -404,16 +469,24 @@ Parse_Term(int cur_prec, int context, Bool comma_is_punct)
     {
       Read_Next_Token(comma_is_punct);
 
-      if (pl_token.type != TOKEN_NAME)
-	break;
-
-      atom = Pl_Create_Allocate_Atom(pl_token.name);
-      if ((oper = Pl_Lookup_Oper(atom, INFIX)))
+#if 1 /* to allow | to be unquoted if it is an infix operator with prec > 1000 */
+      if (pl_token.type == TOKEN_PUNCTUATION && pl_token.punct == '|' &&
+	  (oper = Pl_Lookup_Oper(atom = ATOM_CHAR('|'), INFIX)) && oper->prec > 1000 && cur_prec >= oper->prec)
 	infix_op = TRUE;
-      else if ((oper = Pl_Lookup_Oper(atom, POSTFIX)))
-	infix_op = FALSE;
       else
-	break;
+#endif
+	{
+	  if (pl_token.type != TOKEN_NAME)
+	    break;
+
+	  atom = Pl_Create_Allocate_Atom(pl_token.name);
+	  if ((oper = Pl_Lookup_Oper(atom, INFIX)))
+	    infix_op = TRUE;
+	  else if ((oper = Pl_Lookup_Oper(atom, POSTFIX)))
+	    infix_op = FALSE;
+	  else
+	    break;
+	}
 
       if (left_is_op)
 	Parse_Error("previous operator needs brackets");
@@ -564,13 +637,12 @@ Parse_List(Bool can_be_empty)
 
   switch (pl_token.punct)
     {
-    case ',':			/* [X,[...]] */
+    case ',':			/* [X,Y...] */
       cdr_word = Parse_List(FALSE);
       break;
 
     case '|':			/* [X|Y] */
-      cdr_word =
-	Parse_Term(MAX_ARG_OF_FUNCTOR_PREC, GENERAL_TERM, COMMA_ANY);
+      cdr_word = Parse_Term(MAX_ARG_OF_FUNCTOR_PREC, GENERAL_TERM, COMMA_ANY);
       if (cdr_word == NOT_A_WAM_WORD)
 	Parse_Error("expression expected in list");
 
@@ -674,9 +746,10 @@ static void
 Parse_Error(char *err_msg)
 {
   Pl_Set_Last_Syntax_Error(pl_atom_tbl[pstm_i->atom_file_name].name,
-			pl_token.line, pl_token.col, err_msg);
+			   pl_token.line, pl_token.col, err_msg);
 
-  Pl_Recover_After_Error(pstm_i);
+  if (pl_token.type != TOKEN_FULL_STOP)
+    Pl_Recover_After_Error(pstm_i);
 
   Save_Machine_Regs(buff_save_machine_regs);
   longjmp(jumper, 1);
@@ -780,7 +853,7 @@ Pl_Read_Token(StmInf *pstm)
   if ((err_msg = Pl_Scan_Token(pstm, FALSE)) != NULL)
     {
       Pl_Set_Last_Syntax_Error(pl_atom_tbl[pstm->atom_file_name].name,
-			    pl_token.line, pl_token.col, err_msg);
+			       pl_token.line, pl_token.col, err_msg);
       return NOT_A_WAM_WORD;
     }
 
