@@ -6,23 +6,36 @@
  * Descr.: predicate manipulation management - C part                      *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: pred_c.c,v 1.17 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: pred_c.c,v 1.26 2011/06/10 16:24:15 diaz Exp $ */
 
 #include <stdlib.h>
 #include <string.h>
@@ -49,7 +62,7 @@
  * Function Prototypes             *
  *---------------------------------*/
 
-#define CURRENT_PREDICATE_ALT   X2463757272656E745F7072656469636174655F616C74
+#define CURRENT_PREDICATE_ALT   X1_2463757272656E745F7072656469636174655F616C74
 
 Prolog_Prototype(CURRENT_PREDICATE_ALT, 0);
 
@@ -82,7 +95,7 @@ Pl_Current_Predicate_2(WamWord pred_indic_word, WamWord which_preds_word)
 
 #define Pred_Is_Ok(pred, func, which_preds) \
   (which_preds == 2 || (pl_atom_tbl[func].name[0] != '$' && \
-   (which_preds == 1 || !(pred->prop & MASK_PRED_ANY_BUILTIN))))
+   (which_preds == 1 || !(pred->prop & MASK_PRED_BUILTIN))))
 
   if (func >= 0 && arity >= 0)
     {
@@ -116,8 +129,8 @@ Pl_Current_Predicate_2(WamWord pred_indic_word, WamWord which_preds_word)
   A(3) = (WamWord) scan.endt;
   A(4) = (WamWord) scan.cur_t;
   A(5) = (WamWord) scan.cur_p;
-  Pl_Create_Choice_Point((CodePtr) Prolog_Predicate(CURRENT_PREDICATE_ALT, 0),
-		      6);
+  Pl_Create_Choice_Point((CodePtr) Prolog_Predicate(CURRENT_PREDICATE_ALT, 0), 6);
+
   return Pl_Get_Atom(Functor_Of(pred->f_n), name_word) &&
     Pl_Get_Integer(Arity_Of(pred->f_n), arity_word);
   /*
@@ -195,211 +208,240 @@ Pl_Current_Predicate_Alt_0(void)
 
 
 
+
 /*-------------------------------------------------------------------------*
- * PL_PRED_PROP_STATIC_1                                                   *
+ * PL_PRED_PROP_STATIC_2                                                   *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool
-Pl_Pred_Prop_Static_1(WamWord pred_indic_word)
+Pl_Pred_Prop_Static_2(WamWord func_word, WamWord arity_word)
 {
-  int func, arity;
-  PredInf *pred;
+  int func = Pl_Rd_Atom(func_word);
+  int arity = Pl_Rd_Integer(arity_word);
+  PredInf *pred = Pl_Lookup_Pred(func, arity);
 
-  func = Pl_Get_Pred_Indicator(pred_indic_word, TRUE, &arity);
-
-  if ((pred = Pl_Lookup_Pred(func, arity)) == NULL)
-    return FALSE;
-
-  return (pred->prop & MASK_PRED_DYNAMIC) == 0;
+  return pred != NULL && (pred->prop & MASK_PRED_DYNAMIC) == 0;
 }
 
 
 
 
 /*-------------------------------------------------------------------------*
- * PL_PRED_PROP_DYNAMIC_1                                                  *
+ * PL_PRED_PROP_DYNAMIC_2                                                  *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool
-Pl_Pred_Prop_Dynamic_1(WamWord pred_indic_word)
+Pl_Pred_Prop_Dynamic_2(WamWord func_word, WamWord arity_word)
 {
-  int func, arity;
-  PredInf *pred;
+  int func = Pl_Rd_Atom(func_word);
+  int arity = Pl_Rd_Integer(arity_word);
+  PredInf *pred = Pl_Lookup_Pred(func, arity);
 
-  func = Pl_Get_Pred_Indicator(pred_indic_word, TRUE, &arity);
-
-  if ((pred = Pl_Lookup_Pred(func, arity)) == NULL)
-    return FALSE;
-
-  return (pred->prop & MASK_PRED_DYNAMIC) != 0;
+  return pred != NULL && (pred->prop & MASK_PRED_DYNAMIC) != 0;
 }
 
 
 
 
 /*-------------------------------------------------------------------------*
- * PL_PRED_PROP_PRIVATE_1                                                  *
+ * PL_PRED_PROP_PRIVATE_2                                                  *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool
-Pl_Pred_Prop_Private_1(WamWord pred_indic_word)
+Pl_Pred_Prop_Private_2(WamWord func_word, WamWord arity_word)
 {
-  int func, arity;
-  PredInf *pred;
+  int func = Pl_Rd_Atom(func_word);
+  int arity = Pl_Rd_Integer(arity_word);
+  PredInf *pred = Pl_Lookup_Pred(func, arity);
 
-  func = Pl_Get_Pred_Indicator(pred_indic_word, TRUE, &arity);
-
-  if ((pred = Pl_Lookup_Pred(func, arity)) == NULL)
-    return FALSE;
-
-  return (pred->prop & MASK_PRED_PUBLIC) == 0;
+  return pred != NULL && (pred->prop & MASK_PRED_PUBLIC) == 0;
 }
 
 
 
 
 /*-------------------------------------------------------------------------*
- * PL_PRED_PROP_PUBLIC_1                                                   *
+ * PL_PRED_PROP_PUBLIC_2                                                   *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool
-Pl_Pred_Prop_Public_1(WamWord pred_indic_word)
+Pl_Pred_Prop_Public_2(WamWord func_word, WamWord arity_word)
 {
-  int func, arity;
-  PredInf *pred;
+  int func = Pl_Rd_Atom(func_word);
+  int arity = Pl_Rd_Integer(arity_word);
+  PredInf *pred = Pl_Lookup_Pred(func, arity);
 
-  func = Pl_Get_Pred_Indicator(pred_indic_word, TRUE, &arity);
-
-  if ((pred = Pl_Lookup_Pred(func, arity)) == NULL)
-    return FALSE;
-
-  return (pred->prop & MASK_PRED_PUBLIC) != 0;
+  return pred != NULL && (pred->prop & MASK_PRED_PUBLIC) != 0;
 }
 
 
 
 
 /*-------------------------------------------------------------------------*
- * PL_PRED_PROP_USER_1                                                     *
+ * PL_PRED_PROP_MONOFILE_2                                                 *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool
-Pl_Pred_Prop_User_1(WamWord pred_indic_word)
+Pl_Pred_Prop_Monofile_2(WamWord func_word, WamWord arity_word)
 {
-  int func, arity;
-  PredInf *pred;
+  int func = Pl_Rd_Atom(func_word);
+  int arity = Pl_Rd_Integer(arity_word);
+  PredInf *pred = Pl_Lookup_Pred(func, arity);
 
-  func = Pl_Get_Pred_Indicator(pred_indic_word, TRUE, &arity);
-
-  if ((pred = Pl_Lookup_Pred(func, arity)) == NULL)
-    return FALSE;
-
-  return (pred->prop & MASK_PRED_ANY_BUILTIN) == 0;
+  return pred != NULL && (pred->prop & MASK_PRED_MULTIFILE) == 0;
 }
 
 
 
 
 /*-------------------------------------------------------------------------*
- * PL_PRED_PROP_BUILT_IN_1                                                 *
+ * PL_PRED_PROP_MULTIFILE_2                                                *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool
-Pl_Pred_Prop_Built_In_1(WamWord pred_indic_word)
+Pl_Pred_Prop_Multifile_2(WamWord func_word, WamWord arity_word)
 {
-  int func, arity;
-  PredInf *pred;
+  int func = Pl_Rd_Atom(func_word);
+  int arity = Pl_Rd_Integer(arity_word);
+  PredInf *pred = Pl_Lookup_Pred(func, arity);
 
-  func = Pl_Get_Pred_Indicator(pred_indic_word, TRUE, &arity);
-
-  if ((pred = Pl_Lookup_Pred(func, arity)) == NULL)
-    return FALSE;
-
-  return (pred->prop & MASK_PRED_BUILTIN) != 0;
+  return pred != NULL && (pred->prop & MASK_PRED_MULTIFILE) != 0;
 }
 
 
 
 
 /*-------------------------------------------------------------------------*
- * PL_PRED_PROP_BUILT_IN_FD_1                                              *
+ * PL_PRED_PROP_USER_2                                                     *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool
-Pl_Pred_Prop_Built_In_Fd_1(WamWord pred_indic_word)
+Pl_Pred_Prop_User_2(WamWord func_word, WamWord arity_word)
 {
-  int func, arity;
-  PredInf *pred;
+  int func = Pl_Rd_Atom(func_word);
+  int arity = Pl_Rd_Integer(arity_word);
+  PredInf *pred = Pl_Lookup_Pred(func, arity);
 
-  func = Pl_Get_Pred_Indicator(pred_indic_word, TRUE, &arity);
-
-  if ((pred = Pl_Lookup_Pred(func, arity)) == NULL)
-    return FALSE;
-
-  return (pred->prop & MASK_PRED_BUILTIN_FD) != 0;
+  return pred != NULL && (pred->prop & MASK_PRED_BUILTIN) == 0;
 }
 
 
 
 
 /*-------------------------------------------------------------------------*
- * PL_PRED_PROP_NATIVE_CODE_1                                              *
+ * PL_PRED_PROP_BUILT_IN_2                                                 *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool
-Pl_Pred_Prop_Native_Code_1(WamWord pred_indic_word)
+Pl_Pred_Prop_Built_In_2(WamWord func_word, WamWord arity_word)
 {
-  int func, arity;
-  PredInf *pred;
+  int func = Pl_Rd_Atom(func_word);
+  int arity = Pl_Rd_Integer(arity_word);
+  PredInf *pred = Pl_Lookup_Pred(func, arity);
 
-  func = Pl_Get_Pred_Indicator(pred_indic_word, TRUE, &arity);
-
-  if ((pred = Pl_Lookup_Pred(func, arity)) == NULL)
-    return FALSE;
-
-  return (pred->prop & MASK_PRED_NATIVE_CODE) != 0;
+  return pred != NULL && (pred->prop & MASK_PRED_BUILTIN) != 0;
 }
 
 
 
 
 /*-------------------------------------------------------------------------*
- * PL_PRED_PROP_PROLOG_FILE_2                                              *
+ * PL_PRED_PROP_BUILT_IN_FD_2                                              *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool
-Pl_Pred_Prop_Prolog_File_2(WamWord pred_indic_word, WamWord pl_file_word)
+Pl_Pred_Prop_Built_In_Fd_2(WamWord func_word, WamWord arity_word)
 {
-  int func, arity;
-  PredInf *pred;
+  int func = Pl_Rd_Atom(func_word);
+  int arity = Pl_Rd_Integer(arity_word);
+  PredInf *pred = Pl_Lookup_Pred(func, arity);
 
-  func = Pl_Get_Pred_Indicator(pred_indic_word, TRUE, &arity);
-
-  if ((pred = Pl_Lookup_Pred(func, arity)) == NULL)
-    return FALSE;
-
-  return Pl_Un_Atom_Check(pred->pl_file, pl_file_word);
+  return pred != NULL && (pred->prop & MASK_PRED_BUILTIN_FD) != 0;
 }
 
 
 
 
 /*-------------------------------------------------------------------------*
- * PL_PRED_PROP_PROLOG_LINE_2                                              *
+ * PL_PRED_PROP_CONTROL_CONSTRUCT_2                                        *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool
-Pl_Pred_Prop_Prolog_Line_2(WamWord pred_indic_word, WamWord pl_line_word)
+Pl_Pred_Prop_Control_Construct_2(WamWord func_word, WamWord arity_word)
+{
+  int func = Pl_Rd_Atom(func_word);
+  int arity = Pl_Rd_Integer(arity_word);
+  PredInf *pred = Pl_Lookup_Pred(func, arity);
+
+  return pred != NULL && (pred->prop & MASK_PRED_CONTROL_CONSTRUCT) != 0;
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * PL_PRED_PROP_NATIVE_CODE_2                                              *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+Bool
+Pl_Pred_Prop_Native_Code_2(WamWord func_word, WamWord arity_word)
+{
+  int func = Pl_Rd_Atom(func_word);
+  int arity = Pl_Rd_Integer(arity_word);
+  PredInf *pred = Pl_Lookup_Pred(func, arity);
+
+  return pred != NULL && (pred->prop & MASK_PRED_NATIVE_CODE) != 0;
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * PL_PRED_PROP_PROLOG_FILE_3                                              *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+Bool
+Pl_Pred_Prop_Prolog_File_3(WamWord func_word, WamWord arity_word, WamWord pl_file_word)
+{
+  int func = Pl_Rd_Atom(func_word);
+  int arity = Pl_Rd_Integer(arity_word);
+  PredInf *pred = Pl_Lookup_Pred(func, arity);
+
+  return pred != NULL && Pl_Un_Atom_Check(pred->pl_file, pl_file_word);
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * PL_PRED_PROP_PROLOG_LINE_3                                              *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+Bool
+Pl_Pred_Prop_Prolog_Line_3(WamWord func_word, WamWord arity_word, WamWord pl_line_word)
+{
+  int func = Pl_Rd_Atom(func_word);
+  int arity = Pl_Rd_Integer(arity_word);
+  PredInf *pred = Pl_Lookup_Pred(func, arity);
+
+  return pred != NULL && Pl_Un_Integer_Check(pred->pl_line, pl_line_word);
+}
+
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * PL_GET_PRED_INDICATOR_3                                                 *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+Bool
+Pl_Get_Pred_Indicator_3(WamWord pred_indic_word, WamWord func_word, WamWord arity_word)
 {
   int func, arity;
-  PredInf *pred;
 
   func = Pl_Get_Pred_Indicator(pred_indic_word, TRUE, &arity);
-
-  if ((pred = Pl_Lookup_Pred(func, arity)) == NULL)
-    return FALSE;
-
-  return Pl_Un_Integer_Check(pred->pl_line, pl_line_word);
+  return Pl_Get_Atom(func, func_word) && Pl_Get_Integer(arity, arity_word);
 }
 
 
@@ -411,7 +453,7 @@ Pl_Pred_Prop_Prolog_Line_2(WamWord pred_indic_word, WamWord pl_line_word)
  *-------------------------------------------------------------------------*/
 Bool
 Pl_Get_Predicate_File_Info_3(WamWord pred_indic_word,
-			  WamWord pl_file_word, WamWord pl_line_word)
+			     WamWord pl_file_word, WamWord pl_line_word)
 {
   int func, arity;
   PredInf *pred;
@@ -437,7 +479,7 @@ Pl_Get_Predicate_File_Info_3(WamWord pred_indic_word,
  *-------------------------------------------------------------------------*/
 Bool
 Pl_Set_Predicate_File_Info_3(WamWord pred_indic_word,
-			  WamWord pl_file_word, WamWord pl_line_word)
+			     WamWord pl_file_word, WamWord pl_line_word)
 {
   int func, arity;
   int pl_file, pl_line;
@@ -485,7 +527,7 @@ Pl_Aux_Name_1(WamWord name_word)
  *-------------------------------------------------------------------------*/
 Bool
 Pl_Father_Of_Aux_Name_3(WamWord name_word, WamWord father_name_word,
-		     WamWord father_arity_word)
+			WamWord father_arity_word)
 {
   int func, father_func, father_arity;
 
@@ -508,7 +550,7 @@ Pl_Father_Of_Aux_Name_3(WamWord name_word, WamWord father_name_word,
  *-------------------------------------------------------------------------*/
 Bool
 Pl_Pred_Without_Aux_4(WamWord name_word, WamWord arity_word,
-		   WamWord name1_word, WamWord arity1_word)
+		      WamWord name1_word, WamWord arity1_word)
 {
   int func, arity;
   int func1, arity1;
@@ -533,7 +575,7 @@ Pl_Pred_Without_Aux_4(WamWord name_word, WamWord arity_word,
  *-------------------------------------------------------------------------*/
 Bool
 Pl_Make_Aux_Name_4(WamWord name_word, WamWord arity_word,
-		WamWord aux_nb_word, WamWord aux_name_word)
+		   WamWord aux_nb_word, WamWord aux_name_word)
 {
   int func, arity;
   int aux_nb;
