@@ -6,23 +6,36 @@
  * Descr.: predicate management support                                    *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: pred_supp.c,v 1.12 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: pred_supp.c,v 1.17 2011/04/08 16:16:16 diaz Exp $ */
 
 #include <stdlib.h>
 #include <string.h>
@@ -42,7 +55,6 @@
 
 
 
-
 /*---------------------------------*
  * Type Definitions                *
  *---------------------------------*/
@@ -158,3 +170,18 @@ Pl_Make_Aux_Name(int func, int arity, int aux_nb)
 	  aux_nb);
   return Pl_Create_Allocate_Atom(pl_glob_buff);
 }
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * PL_CREATE_PRED_MULTIFILE                                                *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Pl_Emit_BC_Execute_Wrapper(int func, int arity, PlLong *codep)
+{
+  Pl_BC_Start_Emit_0();
+  Pl_BC_Emit_Inst_Execute_Native(func, arity, codep);
+  Pl_BC_Stop_Emit_0();
+}
