@@ -6,23 +6,36 @@
  * Descr.: pretty print clause management - C part                         *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: pretty_c.c,v 1.16 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: pretty_c.c,v 1.22 2011/05/27 16:13:52 diaz Exp $ */
 
 #define OBJ_INIT Pretty_Initializer
 
@@ -69,7 +82,7 @@ static WamWord dollar_var_1;
 static WamWord dollar_varname_1;
 static WamWord equal_2;
 
-static long *singl_var_ptr;
+static PlLong *singl_var_ptr;
 static int nb_singl_var;
 
 static int nb_to_try;
@@ -92,7 +105,7 @@ static void Show_Body(StmInf *pstm, int level, int context, WamWord body_word);
 
 static void Start_Line(StmInf *pstm, int level, char c_before);
 
-static void Collect_Singleton(WamWord *adr);
+static Bool Collect_Singleton(WamWord *adr);
 
 static int Var_Name_To_Var_Number(int atom);
 
@@ -100,7 +113,7 @@ static void Exclude_A_Var_Number(int n);
 
 static void Collect_Excluded_Rec(WamWord start_word);
 
-static void Bind_Variable(WamWord *adr, WamWord word);
+static Bool Bind_Variable(WamWord *adr, WamWord word);
 
 
 
@@ -191,6 +204,7 @@ Portray_Clause(StmInf *pstm, WamWord term_word)
 	  Show_Body(pstm, 0, GENERAL_BODY, arg_word[1]);
 	}
       Pl_Write_A_Char(pstm, '.');
+      Pl_Write_A_Char(pstm, '\n');
       return;
     }
 
@@ -206,6 +220,7 @@ Portray_Clause(StmInf *pstm, WamWord term_word)
 	  Show_Body(pstm, 0, GENERAL_BODY, arg_word[1]);
 	}
       Pl_Write_A_Char(pstm, '.');
+      Pl_Write_A_Char(pstm, '\n');
       return;
     }
 
@@ -214,11 +229,13 @@ Portray_Clause(StmInf *pstm, WamWord term_word)
       Pl_Stream_Puts(":-\t", pstm);
       Show_Body(pstm, 0, GENERAL_BODY, arg_word[0]);
       Pl_Write_A_Char(pstm, '.');
+      Pl_Write_A_Char(pstm, '\n');
       return;
     }
 
   Pl_Write_Term(pstm, -1, MAX_PREC, WRITE_MASK, term_word);
   Pl_Write_A_Char(pstm, '.');
+  Pl_Write_A_Char(pstm, '\n');
 }
 
 
@@ -282,7 +299,7 @@ static void
 Show_Body(StmInf *pstm, int level, int context, WamWord body_word)
 {
   WamWord arg_word[2];
-  static 
+  static
     int prec[] = { 1200 - 1, 1000 - 1, 1000, 1100 - 1, 1100, 1050 - 1, 1050 };
 
 
@@ -418,27 +435,28 @@ Pl_Name_Singleton_Vars_1(WamWord start_word)
  * COLLECT_SINGLETON                                                       *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-static void
+static Bool
 Collect_Singleton(WamWord *adr)
 {
-  long *p;
+  PlLong *p;
 
   for (p = pl_glob_dico_var; p < singl_var_ptr; p++)
-    if ((*p & ~1) == (long) adr)	/* not a singleton */
+    if ((*p & ~1) == (PlLong) adr)	/* not a singleton */
       {
 	if ((*p & 1) == 0)	/* not yet marked - mark it */
 	  {
 	    *p |= 1;
 	    nb_singl_var--;
 	  }
-	return;
+	return TRUE;
       }
 
   if (singl_var_ptr - pl_glob_dico_var >= MAX_VAR_IN_TERM)
     Pl_Err_Representation(pl_representation_too_many_variables);
 
-  *singl_var_ptr++ = (long) adr;
+  *singl_var_ptr++ = (PlLong) adr;
   nb_singl_var++;
+  return TRUE;
 }
 
 
@@ -515,7 +533,7 @@ Pl_Name_Query_Vars_2(WamWord query_list_word, WamWord rest_list_word)
  *-------------------------------------------------------------------------*/
 Bool
 Pl_Bind_Variables_4(WamWord term_word, WamWord exclude_list_word,
-		 WamWord from_word, WamWord next_word)
+		    WamWord from_word, WamWord next_word)
 {
   WamWord word, tag_mask;
   WamWord save_list_word;
@@ -621,7 +639,7 @@ Collect_Excluded_Rec(WamWord start_word)
  terminal_rec:
 
   DEREF(start_word, word, tag_mask);
-  
+
   if (tag_mask == TAG_LST_MASK)
     {
       adr = UnTag_LST(word);
@@ -671,7 +689,7 @@ Collect_Excluded_Rec(WamWord start_word)
  * BIND_VARIABLE                                                           *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-static void
+static Bool
 Bind_Variable(WamWord *adr, WamWord word)
 {
   int i, j;
@@ -684,7 +702,7 @@ Bind_Variable(WamWord *adr, WamWord word)
     {
       Pl_Get_Structure(atom_dollar_var, 1, word);
       Pl_Unify_Integer(nb_to_try++);
-      return;
+      return TRUE;
     }
 
   i = nb_to_try % 26;
@@ -700,4 +718,6 @@ Bind_Variable(WamWord *adr, WamWord word)
 
   Pl_Get_Structure(atom_dollar_varname, 1, word);
   Pl_Unify_Atom(Pl_Create_Allocate_Atom(buff));
+
+  return TRUE;
 }
