@@ -6,23 +6,36 @@
  * Descr.: random number generator management - C part                     *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: random_c.c,v 1.12 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: random_c.c,v 1.15 2011/03/28 16:20:05 diaz Exp $ */
 
 #include "engine_pl.h"
 #include "bips_pl.h"
@@ -97,8 +110,8 @@ Bool
 Pl_Random_3(WamWord l_word, WamWord u_word, WamWord n_word)
 {
   double l, u;
-  long l1, u1;
-  long i;
+  PlLong l1, u1;
+  PlLong i;
   double d;
 
 
@@ -109,8 +122,8 @@ Pl_Random_3(WamWord l_word, WamWord u_word, WamWord n_word)
   if (l >= u)
     return FALSE;
 
-  l1 = (long) l;
-  u1 = (long) u;
+  l1 = (PlLong) l;
+  u1 = (PlLong) u;
 
   if (l1 == l && u1 == u)
     {
