@@ -6,23 +6,36 @@
  * Descr.: scanner support                                                 *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: scan_supp.c,v 1.14 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: scan_supp.c,v 1.29 2011/06/10 13:58:09 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -69,8 +82,7 @@ static void Scan_Number(StmInf *pstm, Bool integer_only);
 
 static void Scan_Quoted(StmInf *pstm);
 
-static int Scan_Quoted_Char(StmInf *pstm, Bool convert, int c0, 
-			    Bool no_escape);
+static int Scan_Quoted_Char(StmInf *pstm, Bool convert, int c0, Bool no_escape);
 
 
 
@@ -156,6 +168,7 @@ start_scan:
     }
 
 
+  pl_token.quoted = FALSE;
   pl_token.line = pstm->line_count + 1;
   pl_token.col = pstm->line_pos;
 
@@ -247,7 +260,7 @@ start_scan:
       do
 	Read_Next_Char(pstm, TRUE);
       while (c != '\n' && c != EOF);
-#if 0  // what says standard ? EOF allowad at end of %... comment ?
+#if 0  // what says standard ? EOF allowed at end of %... comment ?
       if (c == EOF)
 	{
 	  pl_token.type = TOKEN_END_OF_FILE;
@@ -266,9 +279,34 @@ start_scan:
 	  pl_token.type = TOKEN_IMMEDIAT_OPEN;
 	  break;
 	}
-
       pl_token.type = TOKEN_PUNCTUATION;
       pl_token.punct = c;
+
+#if 0  /* to return [] and {} as a token (else [ is a token and ] is another token) */
+      if (c == '[')
+	{
+	  Read_Next_Char(pstm, TRUE);
+	  if (c == ']')
+	    {
+	      pl_token.type = TOKEN_NAME;
+	      strcpy(pl_token.name, "[]");
+	      break;
+	    }
+	  Unget_Last_Char;
+	}
+
+      if (c == '{')
+	{
+	  Read_Next_Char(pstm, TRUE);
+	  if (c == '}')
+	    {
+	      pl_token.type = TOKEN_NAME;
+	      strcpy(pl_token.name, "{}");
+	      break;
+	    }
+	  Unget_Last_Char;
+	}
+#endif
       break;
 
     case SC:			/* solo character */
@@ -324,7 +362,7 @@ Scan_Number(StmInf *pstm, Bool integer_only)
       c == '.' && isdigit(Pl_Scan_Peek_Char(pstm, TRUE)))
     goto is_a_float;
 
-				/* integer number */
+  /* integer number */
   pl_token.type = TOKEN_INTEGER;
   *p = '\0';
 
@@ -334,9 +372,9 @@ Scan_Number(StmInf *pstm, Bool integer_only)
    * and LONG_MAX is > INT_GREATEST_VALUE. We will detect it at return from
    * this function.
    */
-  pl_token.int_num = strtol(pl_token.name, &p, 10);
+  pl_token.int_num = Str_To_PlLong(pl_token.name, &p, 10);
 
-  if (lg != 1 || pl_token.int_num != 0 || strchr("'bBoOxX", c) == NULL)
+  if (lg != 1 || pl_token.int_num != 0 || strchr("'box", c) == NULL)
     goto push_back;
 
   if (c == '\'')		/* 0'<character> */
@@ -344,15 +382,35 @@ Scan_Number(StmInf *pstm, Bool integer_only)
       c = Scan_Quoted_Char(pstm, TRUE, '\'', FALSE);
       if (c == -1)		/* <character> is ' */
 	{
-	  pl_token.line = pstm->line_count + 1;
-	  pl_token.col = pstm->line_pos + 1;
-	  err_msg = "quote character expected here";
+	  /* STRICT ISO does not allow 0'' one should write 0''' or 0'\' */
+	  if (Flag_Value(FLAG_STRICT_ISO))
+	    {
+
+	    /* do not emit an error since 0'' is valid if '' is a postif/infix op 
+	     * (this is the only case) - simply return the integer 0 
+	     */
+#if 1
+	      if (Check_Oper(pl_atom_void, INFIX) || Check_Oper(pl_atom_void, POSTFIX))
+		{
+		  Pl_Stream_Ungetc('\'', pstm);	/* push back last ' */
+		  Pl_Stream_Ungetc('\'', pstm); /* push back first ' */
+		  return;
+		}
+#endif
+	      pl_token.line = pstm->line_count + 1;
+	      pl_token.col = pstm->line_pos + 1;
+	      err_msg = "quote character expected here";
+	      return;
+	    }
+	  else
+	    c = '\'';
 	}
 
-      if (c == -2 || c == -3)
+      if (c < 0) 		/* \ newline   EOF   newline   tab    other error */
 	{
 	  Unget_Last_Char;
 
+	  pl_token.type = TOKEN_FULL_STOP; /* to stop immediately Pl_Recover_After_Error */
 	  pl_token.line = pstm->line_count + 1;
 	  pl_token.col = pstm->line_pos + 1;
 	  err_msg = "character expected here";
@@ -362,8 +420,8 @@ Scan_Number(StmInf *pstm, Bool integer_only)
       return;
     }
 
-  radix = (c == 'b' || c == 'B') ? (f = "01", 2) :
-    (c == 'o' || c == 'O') ? (f = "01234567", 8)
+  radix = (c == 'b') ? (f = "01", 2) :
+    (c == 'o') ? (f = "01234567", 8)
     : (f = "0123456789abcdefABCDEF", 16);
   p = pl_token.name;
   Read_Next_Char(pstm, TRUE);
@@ -374,11 +432,11 @@ Scan_Number(StmInf *pstm, Bool integer_only)
     }
   *p = '\0';
 
-  pl_token.int_num = strtol(pl_token.name, &p, radix);
+  pl_token.int_num = Str_To_PlLong(pl_token.name, &p, radix);
   goto push_back;
 
 
-is_a_float:			/* float number */
+ is_a_float:			/* float number */
 
   pl_token.type = TOKEN_FLOAT;
   *p++ = '.';
@@ -413,11 +471,11 @@ is_a_float:			/* float number */
     }
 
 
-end_float:
+ end_float:
   *p = '\0';
   sscanf(pl_token.name, "%lf", &pl_token.float_num);
 
-push_back:
+ push_back:
   Unget_Last_Char;
 }
 
@@ -435,11 +493,13 @@ Scan_Quoted(StmInf *pstm)
   char *s;
   Bool convert = (c_orig != '\'');
   Bool no_escape;
+  Bool error_found = FALSE;
   int i = 0;
 
   if (c_type == QT)
     {
       pl_token.type = TOKEN_NAME;
+      pl_token.quoted = TRUE;
       i = 0;
     }
   else if (c_type == DQ)
@@ -460,26 +520,44 @@ Scan_Quoted(StmInf *pstm)
   for (;;)
     {
       c = Scan_Quoted_Char(pstm, convert, c0, no_escape);
-      if (c == -1)
+      if (c == -1)		/* closing quote */
 	{
+	  if (error_found)
+	    break;
+
 	  *s = '\0';
 	  return;
 	}
 
-      if (c == -2)		/* EOF or \n */
-	break;
-
-      if (c == -3)		/* \ followed by \n */
+      if (c == -2)		/* \ followed by newline */
 	continue;
 
-      *s++ = c;
+      if (c == -3 || c == -4)	/* EOF   newline */
+	{
+	  pl_token.type = TOKEN_FULL_STOP; /* to stop immediately Pl_Recover_After_Error */
+	  *s = '\0';
+	  return;
+	}
+
+      if (c == -5 || c == -6)	/* tab or other error */
+	{
+	  error_found = TRUE;
+	  continue;	        /* continue to try to catch the closing quote */
+	}
+
+      if (!error_found)
+	*s++ = c;
     }
+
   /* error */
+
   *s = '\0';
 
-  if (err_msg)
+  if (err_msg != NULL)		/* this test should now always succeed */
     return;
 
+  /* thus this should never been used - to be checked */
+
   Unget_Last_Char;
 
   pl_token.line = pstm->line_count + 1;
@@ -499,7 +577,7 @@ Scan_Quoted(StmInf *pstm)
       err_msg = "double quote character expected here";
       break;
 
-    default:			/* to avoid compiler warning */
+    default:                    /* to avoid compiler warning */
       ;
     }
 }
@@ -522,21 +600,57 @@ Scan_Quoted_Char(StmInf *pstm, Bool convert, int c0, Bool no_escape)
   if (c == c0)
     {
       if (Pl_Scan_Peek_Char(pstm, convert) != c0)	/* '' or "" or `` */
-	return -1;
+	return -1;					/* closing quote */
 
       Read_Next_Char(pstm, convert);
       return c;
     }
 
-  if (c == EOF || c == '\n')
-    return -2;
+  if (c == EOF)
+    {
+      if (err_msg == NULL)
+	{
+	  Unget_Last_Char;
+	  pl_token.line = pstm->line_count + 1;
+	  pl_token.col = pstm->line_pos + 1;
+	  err_msg = "unexpected end of file";
+	}
+      return -3;		/* -3 means EOF */
+    }
+
+  if (c == '\n')
+    {
+      if (err_msg == NULL)
+	{
+	  Unget_Last_Char;
+	  pl_token.line = pstm->line_count + 1;
+	  pl_token.col = pstm->line_pos + 1;
+	  err_msg = "unexpected newline";
+	}
+      return -4;		/* -4 means newline */
+    }
+
+  if (c == '\t')
+    {
+      if (err_msg == NULL)
+	{
+	  Unget_Last_Char;
+	  pl_token.line = pstm->line_count + 1;
+	  pl_token.col = pstm->line_pos + 1;
+	  err_msg = "unexpected tab";
+	}
+      return -5;		/* -5 means tab */
+    }
 
   if (c != '\\' || no_escape)
     return c;
-  /* escape sequence */
+
+  				/* \...  escape sequences */
+
   Read_Next_Char(pstm, convert);
-  if (c == '\n')		/* \ followed by \n */
-    return -3;
+
+  if (c == '\n')		/* \ followed by newline */
+    return -2;			/* -2 means \ newline */
 
   if (strchr("\\'\"`", c))	/* \\ or \' or \" or \` */
     return c;
@@ -569,12 +683,17 @@ Scan_Quoted_Char(StmInf *pstm, Bool convert, int c0, Bool no_escape)
 	  Read_Next_Char(pstm, convert);
 	}
 
-      if (!Is_Valid_Code(x) && err_msg == NULL)
+      if (!Is_Valid_Code(x))
 	{
-	  pl_token.line = pstm->line_count + 1;
-	  pl_token.col = pstm->line_pos;
-	  err_msg = "invalid character code in \\constant\\ sequence";
+	  if (err_msg == NULL)
+	    {
+	      pl_token.line = pstm->line_count + 1;
+	      pl_token.col = pstm->line_pos;
+	      err_msg = "invalid character code in \\constant\\ sequence";
+	    }
+	  goto pump;
 	}
+
       if (c != '\\')
 	{
 	  if (err_msg == NULL)
@@ -584,7 +703,14 @@ Scan_Quoted_Char(StmInf *pstm, Bool convert, int c0, Bool no_escape)
 	      err_msg = "\\ expected in \\constant\\ sequence";
 	    }
 
-	  Unget_Last_Char;
+	  /* pump until \ or closing quote or newline is found */
+	pump:
+	  while(c != '\\' && c != c0 && c != EOF && c != '\n')
+	    Read_Next_Char(pstm, convert);
+	  if (c == c0)
+	    Unget_Last_Char;	/* to be able to continue in the parent's loop */
+
+	  return -6;		/* -6 means other error */
 	}
 
       return (int) (unsigned char) x;
@@ -597,7 +723,7 @@ Scan_Quoted_Char(StmInf *pstm, Bool convert, int c0, Bool no_escape)
       err_msg = "unknown escape sequence";
     }
 
-  return 0;
+  return -6;		/* -6 means other error */
 }
 
 
@@ -614,6 +740,7 @@ Pl_Recover_After_Error(StmInf *pstm)
 {
   int c0;
   Bool convert;
+  Bool dot_found;
 
   if (pstm->eof_reached)
     return;
@@ -638,19 +765,35 @@ Pl_Recover_After_Error(StmInf *pstm)
       c0 = c;
       convert = (c_orig != '\'');
 
+      dot_found = FALSE;
       for (;;)
 	{
 	  Next_Char;
 	  if (c == c0)		/* detect end of pl_token - also for  '' or "" or `` */
 	    break;
+	  
+	  if (c == '.')
+	    dot_found = TRUE;
+	  else if ((c_type & (LA | CM)) == 0)
+	    dot_found = FALSE;
+
+	  if (c == '\n')	/* detect newline inside a quoted token: stop */
+	    {
+	      if (dot_found)	/* consider 'xxxx. followed by newline as a full stop */
+		return;
+	      break;		/* else break the quoted token traversal */
+	    }
 
 	  if (c != '\\')
 	    continue;
 	  /* escape sequence */
 	  Next_Char;
-	  if (c == '\n')	/* \ followed by \n */
+	  if (c == '\n')	/* \ followed by newline */
 	    continue;
 
+	  if (c == '.')
+	    dot_found = TRUE;
+
 	  if (strchr("\\'\"`", c))	/* \\ or \' or \" or \`  " */
 	    continue;
 
@@ -691,6 +834,8 @@ Pl_Scan_Next_Atom(StmInf *pstm)
 {
   char *s;
 
+  err_msg = NULL;
+
   do
     Read_Next_Char(pstm, TRUE);
   while (c_type == LA);		/* layout character */
@@ -767,6 +912,8 @@ Pl_Scan_Next_Number(StmInf *pstm, Bool integer_only)
 {
   Bool minus_op = FALSE;
 
+  err_msg = NULL;
+
   for (;;)
     {
       Read_Next_Char(pstm, TRUE);
@@ -779,9 +926,19 @@ Pl_Scan_Next_Number(StmInf *pstm, Bool integer_only)
   pl_token.col = pstm->line_pos;
 
 
-  if (c == '-' && isdigit(Pl_Scan_Peek_Char(pstm, FALSE)))	/* negative number */
+  if (c == '-'
+#ifdef MINUS_SIGN_CANNOT_BE_FOLLOWED_BY_SPACES
+      && isdigit(Pl_Scan_Peek_Char(pstm, FALSE))	/* negative number */
+#endif
+      )
     {
-      Read_Next_Char(pstm, TRUE);
+
+      for (;;)
+	{
+	  Read_Next_Char(pstm, TRUE);
+	  if (c_type != LA)		/* layout character */
+	    break;
+	}
       minus_op = TRUE;
     }
 
