@@ -6,23 +6,36 @@
  * Descr.: stream support - header file                                    *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: stream_supp.h,v 1.18 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: stream_supp.h,v 1.22 2011/03/28 16:20:05 diaz Exp $ */
 
 #include <stdio.h>
 
@@ -125,7 +138,7 @@ typedef struct stm_lst		/* Chained stream list            */
 typedef struct stm_inf		/* Stream information             */
 {				/* ------------------------------ */
   int atom_file_name;		/* atom associated to filename    */
-  long file;			/* accessor (FILE *,TTYInf *) != 0*/
+  PlLong file;		/* accessor (FILE *,TTYInf *) != 0*/
   StmProp prop;			/* assoctiated properties         */
   StmLst *mirror;		/* mirror streams                 */
   StmLst *mirror_of;            /* streams this stream as mirror  */
@@ -153,7 +166,7 @@ StmInf;
 
 typedef struct			/* Alias information              */
 {				/* ------------------------------ */
-  long atom;			/* atom of the alias (the key)    */
+  PlLong atom;			/* atom of the alias (the key)    */
   int stm;			/* associated stream              */
 }
 AliasInf;
@@ -189,9 +202,11 @@ WamWord pl_last_output_sora;
 
 int pl_stm_stdin;
 int pl_stm_stdout;
+int pl_stm_stderr;
 
 int pl_stm_input;
 int pl_stm_output;
+int pl_stm_error;
 
 int pl_stm_top_level_input;
 int pl_stm_top_level_output;
@@ -205,6 +220,7 @@ int pl_atom_stream;
 
 int pl_atom_user_input;
 int pl_atom_user_output;
+int pl_atom_user_error;
 
 int pl_atom_top_level_input;
 int pl_atom_top_level_output;
@@ -253,9 +269,11 @@ extern WamWord pl_last_output_sora;
 
 extern int pl_stm_stdin;
 extern int pl_stm_stdout;
+extern int pl_stm_stderr;
 
 extern int pl_stm_input;
 extern int pl_stm_output;
+extern int pl_stm_error;
 
 extern int pl_stm_top_level_input;
 extern int pl_stm_top_level_output;
@@ -312,7 +330,7 @@ extern int pl_atom_eof;
  * Function Prototypes             *
  *---------------------------------*/
 
-int Pl_Add_Stream(int atom_file_name, long file, StmProp prop,
+int Pl_Add_Stream(int atom_file_name, PlLong file, StmProp prop,
 	       StmFct fct_getc, StmFct fct_putc,
 	       StmFct fct_flush, StmFct fct_close,
 	       StmFct fct_tell, StmFct fct_seek, StmFct fct_clearerr);
