@@ -6,23 +6,36 @@
  * Descr.: term (inline) management - C part                               *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: term_inl_c.c,v 1.15 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: term_inl_c.c,v 1.21 2011/05/27 16:13:52 diaz Exp $ */
 
 #include "engine_pl.h"
 #include "bips_pl.h"
@@ -42,11 +55,20 @@
  * Global Variables                *
  *---------------------------------*/
 
+static PlLong *var_ptr;
+static PlLong *base_var_ptr;
 
 /*---------------------------------*
  * Function Prototypes             *
  *---------------------------------*/
 
+static Bool Collect_Variable(WamWord *adr);
+
+static Bool Check_Variable(WamWord *adr, WamWord var_word);
+
+
+
+
 	  /* Term comparison inlines */
 
 
@@ -147,6 +169,16 @@ Pl_Blt_Compare(WamWord cmp_word, WamWord x, WamWord y)
   c = (cmp < 0) ? '<' : (cmp == 0) ? '=' : '>';
 
   res = Pl_Un_Atom_Check(ATOM_CHAR(c), cmp_word);
+  if (!res)			/* check if it is one of < = > */
+    {
+      WamWord word, tag_mask;
+      char *s;
+
+      DEREF(cmp_word, word, tag_mask); /* we know it is an atom */
+      s = pl_atom_tbl[UnTag_ATM(word)].name;
+      if ((s[0] != '<' && s[0] != '=' && s[0] != '>') || s[1] != '\0')
+	Pl_Err_Domain(pl_domain_order, cmp_word);
+    }
 
   Pl_Unset_C_Bip_Name();
 
@@ -222,7 +254,7 @@ Pl_Blt_Functor(WamWord term_word, WamWord functor_word, WamWord arity_word)
   if (tag_mask == TAG_REF_MASK)
     Pl_Err_Instantiation();
 
-  if (tag_mask != TAG_ATM_MASK && tag_mask != TAG_INT_MASK && 
+  if (tag_mask != TAG_ATM_MASK && tag_mask != TAG_INT_MASK &&
       tag_mask != TAG_FLT_MASK)
     Pl_Err_Type(pl_type_atomic, functor_word);
 
@@ -309,7 +341,7 @@ Pl_Blt_Univ(WamWord term_word, WamWord list_word)
       adr = UnTag_FDV(word);
       car_word = Tag_REF(adr);	/* since Dont_Separate_Tag */
       lst_length = 1 + 0;
-    } 
+    }
 #endif
   else				/* TAG_ATM/INT/FLT_MASK */
     {
@@ -439,7 +471,7 @@ Pl_Copy_Term_2(WamWord u_word, WamWord v_word)
   static WamWord fix_bug;
 
   size = Pl_Term_Size(u_word);
-  fix_bug = u_word;	
+  fix_bug = u_word;
   Pl_Copy_Term(H, &fix_bug);
   word = *H;
   H += size;
@@ -527,3 +559,153 @@ Pl_Term_Ref_2(WamWord term_word, WamWord ref_word)
 
   return Pl_Un_Positive_Check(ref, ref_word);
 }
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * PL_TERM_VARIABLES_3                                                     *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+Bool
+Pl_Term_Variables_3(WamWord start_word, WamWord list_word, WamWord tail_word)
+{
+  PlLong *p;
+
+  /* only check if no Tail since if there is no vars in Term
+   * then List = Tail and Tail can be any term */
+
+  if (tail_word == NOT_A_WAM_WORD)
+    Pl_Check_For_Un_List(list_word);
+
+  var_ptr = pl_glob_dico_var;	/* pl_glob_dico_var: stores variables */
+
+  Pl_Treat_Vars_Of_Term(start_word, TRUE, Collect_Variable);
+
+  for(p = pl_glob_dico_var; p < var_ptr; p++)
+    {
+      if (!Pl_Get_List(list_word) || !Pl_Unify_Value(*p))
+	return FALSE;
+      list_word = Pl_Unify_Variable();
+    }
+
+  if (tail_word == NOT_A_WAM_WORD)
+    return Pl_Get_Nil(list_word);
+
+  return Pl_Unify(list_word, tail_word);
+}
+
+
+
+/*-------------------------------------------------------------------------*
+ * PL_TERM_VARIABLES_2                                                     *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+Bool
+Pl_Term_Variables_2(WamWord start_word, WamWord list_word)
+{
+  return Pl_Term_Variables_3(start_word, list_word, NOT_A_WAM_WORD);
+}
+
+
+
+/*-------------------------------------------------------------------------*
+ * COLLECT_VARIABLE                                                        *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+static Bool
+Collect_Variable(WamWord *adr)
+{
+  PlLong *p;
+
+  for (p = pl_glob_dico_var; p < var_ptr; p++)
+    if (*p == (PlLong) adr)	/* already present */
+      return TRUE;
+
+  if (var_ptr - pl_glob_dico_var >= MAX_VAR_IN_TERM)
+    Pl_Err_Representation(pl_representation_too_many_variables);
+
+  *var_ptr++ = (PlLong) adr;
+
+  return TRUE;
+}
+
+
+
+/*-------------------------------------------------------------------------*
+ * PL_SUBSUMES_TERM_2                                                      *
+ *                                                                         *
+ * mostly implements:                                                      *
+ *                                                                         *
+ * subsumes_term(Generic, Specific) :-                                     *
+ * 	\+ \+ subsumes(Generic, Specific).                                 *
+ *                                                                         *
+ * subsumes(General, Specific) :-                                          *
+ * 	term_variables(Specific, SVars),                                   *
+ * 	unify_with_occurs_check(General, Specific),                        *
+ * 	term_variables(SVars, SVars2),                                     *
+ * 	SVars == SVars2.                                                   *
+ *                                                                         *
+ * TODO: what to do with FD vars ? (a var subsumes a FD var, what else ?)  *
+ *-------------------------------------------------------------------------*/
+Bool
+Pl_Subsumes_Term_2(WamWord general_word, WamWord specific_word)
+{
+  Bool ret = FALSE;;
+
+  Pl_Defeasible_Open();
+
+  base_var_ptr = var_ptr = pl_glob_dico_var;	/* pl_glob_dico_var: stores variables */
+
+
+  Pl_Treat_Vars_Of_Term(specific_word, TRUE, Collect_Variable);
+
+  /* TODO: improve FD vars (possible ?) */
+
+  ret = Pl_Unify_Occurs_Check(general_word, specific_word) &&
+    Pl_Treat_Vars_Of_Term(specific_word, TRUE, Check_Variable) &&
+    base_var_ptr == var_ptr;
+
+  Pl_Defeasible_Close(FALSE);	/* undo bindings */
+  return ret;
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * CHECK_VARIABLE                                                          *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+static Bool
+Check_Variable(WamWord *adr, WamWord var_word)
+{
+  WamWord word, tag_mask;
+  WamWord *adr1;
+  PlLong *p;
+
+  if (Tag_Of(var_word) == FDV)	/* improve FDV */
+    return FALSE;
+
+  for (p = pl_glob_dico_var; p < base_var_ptr; p++) /* check if already found until now */
+    if (*p == (PlLong) adr)	/* test if already present (thus well dereferenced) */
+      return TRUE;
+
+  if (base_var_ptr >= var_ptr)
+    return FALSE; 			/* not found */
+
+  /* check if Specific has been modified (also deref which is important) */
+
+  DEREF(*base_var_ptr, word, tag_mask);
+  if (Tag_Of(word) != REF)	/* TODO: treat FD vars */
+    return FALSE;		/* specific has been instantiated - no longer a var */
+
+  adr1 = UnTag_REF(word); /* save dereferenced adr */
+
+  if (adr1 != adr) /*  check if it is the current variable */
+    return FALSE;  /* not ok */
+
+  *base_var_ptr = (PlLong) adr1;	/* replace adr by dereferenced adr */
+  base_var_ptr++;
+  return TRUE;
+}
