@@ -6,23 +6,36 @@
  * Descr.: term support                                                    *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: term_supp.c,v 1.17 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: term_supp.c,v 1.24 2011/06/10 16:44:02 diaz Exp $ */
 
 #include <string.h>
 
@@ -70,8 +83,7 @@ static void Copy_Term_Rec(WamWord *dst_adr, WamWord *src_adr, WamWord **p);
  * PL_TERM_COMPARE                                                         *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-long
-Pl_Term_Compare(WamWord start_u_word, WamWord start_v_word)
+PlLong Pl_Term_Compare(WamWord start_u_word, WamWord start_v_word)
 {
   WamWord u_word, u_tag_mask;
   WamWord v_word, v_tag_mask;
@@ -169,8 +181,8 @@ Pl_Term_Compare(WamWord start_u_word, WamWord start_v_word)
  *                                                                         *
  * Call fct for each variable found in a term.                             *
  *-------------------------------------------------------------------------*/
-void
-Pl_Treat_Vars_Of_Term(WamWord start_word, Bool generic_var, void (*fct) ())
+Bool
+Pl_Treat_Vars_Of_Term(WamWord start_word, Bool generic_var, Bool (*fct) ())
 {
   WamWord word, tag_mask;
   WamWord *adr;
@@ -183,20 +195,23 @@ terminal_rec:
   switch (Tag_Of(word))
     {
     case REF:
-      (*fct) (UnTag_REF(word), word);
+      if (!(*fct) (UnTag_REF(word), word))
+	return FALSE;
       break;
 
 #ifndef NO_USE_FD_SOLVER
     case FDV:
       if (generic_var)
-	(*fct) (UnTag_FDV(word), word);
+	if (!(*fct) (UnTag_FDV(word), word))
+	  return FALSE;
       break;
 #endif
 
     case LST:
       adr = UnTag_LST(word);
       adr = &Car(adr);
-      Pl_Treat_Vars_Of_Term(*adr++, generic_var, fct);
+      if (!Pl_Treat_Vars_Of_Term(*adr++, generic_var, fct))
+	return FALSE;
 
       start_word = *adr;
       goto terminal_rec;
@@ -206,11 +221,14 @@ terminal_rec:
       i = Arity(adr);
       adr = &Arg(adr, 0);
       while (--i)
-	Pl_Treat_Vars_Of_Term(*adr++, generic_var, fct);
+	if (!Pl_Treat_Vars_Of_Term(*adr++, generic_var, fct))
+	  return FALSE;
 
       start_word = *adr;
       goto terminal_rec;
     }
+
+  return TRUE;
 }
 
 
@@ -265,7 +283,7 @@ Pl_Term_Size(WamWord start_word)
 terminal_rec:
 
   DEREF(start_word, word, tag_mask);
-  
+
   switch (Tag_From_Tag_Mask(tag_mask))
     {
 #ifndef NO_USE_FD_SOLVER
@@ -553,10 +571,11 @@ Pl_Get_Pred_Indicator(WamWord pred_indic_word, Bool must_be_ground, int *arity)
 
   if (!Pl_Get_Structure(ATOM_CHAR('/'), 2, pred_indic_word))
     {
+#if 0 /* no longer accept a callable when a predicate indicator is expected */
       if (!Flag_Value(FLAG_STRICT_ISO) &&
 	  Pl_Rd_Callable(word, &func, arity) != NULL)
 	return func;
-
+#endif
       Pl_Err_Type(pl_type_predicate_indicator, pred_indic_word);
     }
 
@@ -607,7 +626,7 @@ Pl_Get_Pred_Indicator(WamWord pred_indic_word, Bool must_be_ground, int *arity)
  *-------------------------------------------------------------------------*/
 Bool
 Pl_Get_Pred_Indic_3(WamWord pred_indic_word, WamWord func_word,
-		 WamWord arity_word)
+		    WamWord arity_word)
 {
   int func, arity;
 
@@ -615,3 +634,64 @@ Pl_Get_Pred_Indic_3(WamWord pred_indic_word, WamWord func_word,
 
   return Pl_Get_Atom(func, func_word) && Pl_Get_Integer(arity, arity_word);
 }
+
+
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * PL_ACYCLIC_TERM_1                                                       *
+ *                                                                         *
+ * This implementation is not very satisfactory because:                   *
+ * - it does not handle terminal recursion (useful for lists).             *
+ * - it does not take into account sharing.                                *
+ * However, it is simple and enough until a full support for cyclic term   *
+ * is implemented (at least in the unification).                           *
+ *-------------------------------------------------------------------------*/
+Bool
+Pl_Acyclic_Term_1(WamWord start_word)
+
+#define MARK Tag_LST(0)
+
+{
+  WamWord word, tag_mask;
+  WamWord *adr;
+  int arity;
+  Bool ret;
+
+  DEREF(start_word, word, tag_mask);
+
+  if (tag_mask == TAG_LST_MASK)
+    {
+      adr = UnTag_LST(word);
+      arity = 2;
+      adr = &Car(adr);
+    }
+  else if (tag_mask == TAG_STC_MASK)
+    {
+      adr = UnTag_STC(word);
+      arity = Arity(adr);
+      adr = &Arg(adr, 0);
+    }
+  else
+    return TRUE;
+
+
+  while (--arity >= 0)
+    {
+      word = *adr;
+      if (word == MARK)		/* marked = cyclic */
+	return FALSE;
+
+      *adr = MARK;		/* mark it */
+
+      ret = Pl_Acyclic_Term_1(word);
+      
+      *adr++ = word;		/* unmark it */
+      if (!ret)
+	return FALSE;
+    }
+
+  return TRUE;
+}
