@@ -6,23 +6,36 @@
  * Descr.: term support - header file                                      *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: term_supp.h,v 1.14 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: term_supp.h,v 1.18 2011/03/28 16:20:05 diaz Exp $ */
 
 /*---------------------------------*
  * Constants                       *
@@ -41,14 +54,14 @@
 WamWord pl_pi_name_word;
 WamWord pl_pi_arity_word;
 
-long pl_glob_dico_var[MAX_VAR_IN_TERM];	/* a general purpose dico */
+PlLong pl_glob_dico_var[MAX_VAR_IN_TERM];	/* a general purpose dico */
 
 #else
 
 extern WamWord pl_pi_name_word;
 extern WamWord pl_pi_arity_word;
 
-extern long pl_glob_dico_var[];
+extern PlLong pl_glob_dico_var[];
 
 #endif
 
@@ -59,7 +72,7 @@ extern long pl_glob_dico_var[];
  * Function Prototypes             *
  *---------------------------------*/
 
-long Pl_Term_Compare(WamWord start_u_word, WamWord start_v_word);
+PlLong Pl_Term_Compare(WamWord start_u_word, WamWord start_v_word);
 
 Bool Is_List(WamWord start_word);
 
@@ -67,8 +80,8 @@ Bool Is_Partial_List(WamWord start_word);
 
 Bool Is_List_Or_Partial(WamWord start_word);
 
-void Pl_Treat_Vars_Of_Term(WamWord start_word, Bool generic_var,
-			void (*fct) ());
+Bool Pl_Treat_Vars_Of_Term(WamWord start_word, Bool generic_var,
+			   Bool (*fct) ());
 
 int Pl_List_Length(WamWord start_word);
 
@@ -79,4 +92,4 @@ void Pl_Copy_Term(WamWord *dst_adr, WamWord *src_adr);
 void Pl_Copy_Contiguous_Term(WamWord *dst_adr, WamWord *src_adr);
 
 int Pl_Get_Pred_Indicator(WamWord pred_indic_word, Bool must_be_ground,
-		       int *arity);
+			  int *arity);
