@@ -1,28 +1,41 @@
-/*-------------------------------------------------------------------------* 
- * GNU Prolog                                                              * 
- *                                                                         * 
- * Part  : Prolog buit-in predicates                                       * 
- * File  : top_level.pl                                                    * 
- * Descr.: top Level                                                       * 
- * Author: Daniel Diaz                                                     * 
- *                                                                         * 
- * Copyright (C) 1999-2009 Daniel Diaz                                     * 
- *                                                                         * 
- * GNU Prolog is free software; you can redistribute it and/or modify it   * 
- * under the terms of the GNU General Public License as published by the   * 
- * Free Software Foundation; either version 2, or any later version.       * 
- *                                                                         * 
- * GNU Prolog is distributed in the hope that it will be useful, but       * 
- * WITHOUT ANY WARRANTY; without even the implied warranty of              * 
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        * 
- * General Public License for more details.                                * 
- *                                                                         * 
- * You should have received a copy of the GNU General Public License along * 
- * with this program; if not, write to the Free Software Foundation, Inc.  * 
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               * 
+/*-------------------------------------------------------------------------*
+ * GNU Prolog                                                              *
+ *                                                                         *
+ * Part  : Prolog buit-in predicates                                       *
+ * File  : top_level.pl                                                    *
+ * Descr.: top Level                                                       *
+ * Author: Daniel Diaz                                                     *
+ *                                                                         *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ *                                                                         *
+ * This file is part of GNU Prolog                                         *
+ *                                                                         *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
+ * General Public License for more details.                                *
+ *                                                                         *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: top_level.pl,v 1.18 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: top_level.pl,v 1.26 2011/05/27 16:13:52 diaz Exp $ */
 
 :-	built_in.
 
@@ -52,6 +65,8 @@ break :-
 	;   true
 	),
 	'$sys_var_inc'(10),
+	g_read('$cmd_line_consult_file', LFile),
+	'$exec_cmd_line_consult_files'(LFile),
 	g_read('$cmd_line_entry_goal', LGoal),
 	'$exec_cmd_line_entry_goals'(LGoal),
 	g_assign('$cmd_line_entry_goal', []),
@@ -77,10 +92,10 @@ break :-
 '$top_level_abort' :-
 	'$reinit_after_exception',
 	'$sys_var_read'(11, B),
-	format(top_level_output, 'execution aborted~n', []),
+	write(top_level_output, 'execution aborted\n'),
 	'$catch_sync_for_fail_at'(B).
 
-	
+
 
 
 '$top_level_stop' :-
@@ -94,7 +109,7 @@ break :-
 '$top_level_exception'(X) :-
 	'$reinit_after_exception',
 	format(top_level_output, '~Nuncaught exception: ', []),
-	write_term(top_level_output, X, [quoted(true), numbervars(false)]),
+	write_term(top_level_output, X, [quoted(true), numbervars(false), namevars(false)]),
 	nl(top_level_output),
 	fail.
 
@@ -114,8 +129,10 @@ break :-
 
 '$top_level2' :-
 	repeat,
-	'$get_current_B'(B),
+	'$get_current_B'(B),   % the current choice-point
+%	'$sys_var_read'(7, B), % the last Handler created by catch/3 (what is better ???)
 	'$sys_var_write'(11, B),
+%write('top-level catcher'(B)), nl,
 	'$write_indicator',
 % current_prolog_flag(char_conversion, CharConv),
 % g_assign('$char_conv', CharConv),
@@ -155,6 +172,7 @@ break :-
 
 
 
+
 '$write_indicator' :-
 	g_read('$debug_mode', DebugMode),
 	'$dbg_indicator'(DebugMode, A),
@@ -191,7 +209,7 @@ break :-
 	g_assign('$cmd_line_query_goal', LGoal),
 	!,
 	Prompt = '| ?- ',
-	(   '$sys_var_read'(12, 1) -> 
+	(   '$sys_var_read'(12, 1) ->
 	    write(top_level_output, Prompt)
 	;   true),
 	format(top_level_output, '~a.~n', [Goal]),
@@ -208,7 +226,9 @@ break :-
 	g_assign('$debug_next', DebugMode),
 	g_assign('$all_solutions', f),
 	'$get_current_B'(B),
+	'$call_c'('Pl_Save_Regs_For_Signal'),  % save some registers in case of CTRL+C
 	'$call'(X, top_level, 0, true),
+	'$call_c'('Pl_Save_Regs_For_Signal'),  % save some registers in case of CTRL+C
 	'$get_current_B'(B1),
 	format(top_level_output, '~N', []),
 	'$set_query_vars_names'(QueryVars, ToDispVars),
@@ -220,7 +240,7 @@ break :-
 	    (   B1 > B ->
 	        g_read('$all_solutions', f),          % fail for previous 'a'
 	        write(top_level_output, ' ? '),
-	        '$read_return'                        % fail for ';' and  'a'
+	        '$read_return'                        % fail for ';' ' ' and  'a'
 	    ;   true
 	    )
 	).
@@ -245,7 +265,7 @@ break :-
 	    sub_atom(Name1, 0, 1, _, '_')),
 	!,
 	'$remove_underscore_vars'(ToDispVars, ToDispVars1).
-	
+
 '$remove_underscore_vars'([X|ToDispVars], [X|ToDispVars1]) :-
 	'$remove_underscore_vars'(ToDispVars, ToDispVars1).
 
@@ -260,15 +280,17 @@ break :-
 	).
 
 '$write_solution'(ToDispVars, _, _) :-
-	'$write_solution1'(ToDispVars).
+	( current_op(Prior, xfx, =) ; Prior = 700), !,
+	Prior1 is Prior - 1,
+	'$write_solution1'(ToDispVars, Prior1).
 
 
-'$write_solution1'([]).
+'$write_solution1'([], _).
 
-'$write_solution1'([Name = Value|ToDispVars]) :-
+'$write_solution1'([Name = Value|ToDispVars], Prior) :-
 	format(top_level_output, '~n~a = ', [Name]),
-	write_term(top_level_output, Value, [quoted(true), numbervars(false)]),
-	'$write_solution1'(ToDispVars).
+	write_term(top_level_output, Value, [quoted(true), numbervars(false), namevars(true), priority(Prior)]),
+	'$write_solution1'(ToDispVars, Prior).
 
 
 
@@ -281,18 +303,22 @@ break :-
 
 
 
-'$read_return'(10).
+'$read_return'(10).		% newline
 
-'$read_return'(13).
+'$read_return'(13).		% carriage-return
 
-'$read_return'(97) :-
+'$read_return'(97) :-		% 'a'
 	g_assign('$all_solutions', t), !,
 	fail.
 
-'$read_return'(59) :-
+'$read_return'(59) :-		% ';'
 	format(top_level_output, '~N', []), !,
 	fail.
 
+'$read_return'(32) :-		% ' ' (simulate a ';')
+	format(top_level_output, '\b;~N', []), !,
+	fail.
+
 '$read_return'(_) :-
 	nl(top_level_output),
 	write(top_level_output, 'Action (; for next solution, a for all solutions, RET to stop) ? '),
@@ -300,6 +326,21 @@ break :-
 
 
 
+/* interface with command-line option consulting files */
+
+'$exec_cmd_line_consult_files'([File|_LFile]) :-
+	'$catch_internal'('$consult2'(File), error(Err, _), true, false),
+	nonvar(Err),
+	format('~Nwarning: command-line consulting file ~q failed due to ~q~n', [File, Err]),
+	fail.
+	
+'$exec_cmd_line_consult_files'([_|LFile]) :-
+	!,
+	'$exec_cmd_line_consult_files'(LFile).
+
+'$exec_cmd_line_consult_files'(_).		% can be another term than []
+
+
 /* interface with command-line options executing goals */
 
 '$exec_cmd_line_entry_goals'([Goal|LGoal]):-
@@ -310,7 +351,7 @@ break :-
 '$exec_cmd_line_entry_goals'(_).		% can be another term than []
 
 
-						
+
 
 '$exec_cmd_line_goal'(Goal) :-		% called by top_level.c
 	(   '$catch'('$exec_cmd1'(Goal), Err, '$exec_cmd_err'(Goal, Err), 'command-line', -1, false) ->
