@@ -6,23 +6,36 @@
  * Descr.: top Level - C part                                              *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: top_level_c.c,v 1.17 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: top_level_c.c,v 1.22 2011/03/28 16:20:05 diaz Exp $ */
 
 
 #include "gp_config.h"
@@ -62,18 +75,23 @@
  * Global Variables                *
  *---------------------------------*/
 
+
+#if !defined(NO_USE_REGS) && NB_OF_USED_MACHINE_REGS > 0
+static WamWord buff_save_machine_regs[NB_OF_USED_MACHINE_REGS];
+#endif
+
 /*---------------------------------*
  * Function Prototypes             *
  *---------------------------------*/
 
-#define ABORT                      X61626F7274
+#define ABORT                      X0_abort
 
-#define BREAK                      X627265616B
+#define BREAK                      X0_break
 
 Prolog_Prototype(ABORT, 0);
 Prolog_Prototype(BREAK, 0);
 
-static long Ctrl_C_Manager(int from_callback);
+static PlLong Ctrl_C_Manager(int from_callback);
 
 
 
@@ -90,12 +108,24 @@ Pl_Set_Ctrl_C_Handler_0(void)
 
 
 
+/*-------------------------------------------------------------------------*
+ * CTRL_C_MANAGER                                                          *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Pl_Save_Regs_For_Signal(void)
+{
+  Save_Machine_Regs(buff_save_machine_regs);
+}
+
+
+
 
 /*-------------------------------------------------------------------------*
  * CTRL_C_MANAGER                                                          *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-static long
+static PlLong
 Ctrl_C_Manager(int from_callback)
 {
   StmInf *pstm = pl_stm_tbl[pl_stm_top_level_output];
@@ -103,7 +133,8 @@ Ctrl_C_Manager(int from_callback)
   int c;
   CodePtr to_execute;
 
-  Pl_Reset_Prolog_In_Signal();
+  //  Pl_Reset_Prolog_In_Signal();
+  Restore_Machine_Regs(buff_save_machine_regs);
 
 start:
   Pl_Stream_Printf(pstm, "\nProlog interruption (h for help) ? ");
@@ -117,7 +148,7 @@ start:
     case 'a':			/* abort */
       to_execute = Prolog_Predicate(ABORT, 0);
       if (from_callback)
-	return (long) to_execute;
+	return (PlLong) to_execute;
       Pl_Execute_A_Continuation(to_execute);
       break;
 
@@ -136,8 +167,7 @@ start:
     case 'd':			/* debug */
       if (SYS_VAR_DEBUGGER)
 	{
-	  pred =
-	    Pl_Lookup_Pred(Pl_Create_Atom((c == 't') ? "trace" : "debug"), 0);
+	  pred = Pl_Lookup_Pred(Pl_Create_Atom((c == 't') ? "trace" : "debug"), 0);
 	  if (pred == NULL)
 	    Pl_Fatal_Error(ERR_DEBUGGER_NOT_FOUND);	/* should not occur */
 
