@@ -6,23 +6,36 @@
  * Descr.: type testing (inline) management - C part                       *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: type_inl_c.c,v 1.15 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: type_inl_c.c,v 1.19 2011/03/28 16:20:05 diaz Exp $ */
 
 #include "engine_pl.h"
 #include "bips_pl.h"
@@ -168,6 +181,56 @@ Pl_Blt_Non_Generic_Var(WamWord x)
 
 
 
+/*-------------------------------------------------------------------------*
+ * PL_BLT_GROUND                                                           *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+Bool FC
+Pl_Blt_Ground(WamWord start_word)
+{
+  WamWord word, tag_mask;
+  WamWord *adr;
+  int arity;
+
+ terminal_rec:
+  DEREF(start_word, word, tag_mask);
+
+  if (Tag_Is_Generic_Var(tag_mask))
+    return FALSE;
+
+  if (tag_mask == TAG_LST_MASK)
+    {
+      adr = UnTag_LST(word);
+      if (!Pl_Blt_Ground(Car(adr)))
+	return FALSE;
+
+      start_word = Cdr(adr);
+      goto terminal_rec;
+    }
+
+  if (tag_mask == TAG_STC_MASK)
+    {
+      adr = UnTag_LST(word);
+      arity = Arity(adr);
+
+      adr = &Arg(adr, 0);
+      while(--arity > 0)
+	{
+	  if (!Pl_Blt_Ground(*adr++))
+	    return FALSE;
+
+	}
+
+      start_word = *adr;
+      goto terminal_rec;
+    }
+
+  return TRUE;
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * PL_BLT_LIST                                                             *
  *                                                                         *
