@@ -6,23 +6,36 @@
  * Descr.: write term support                                              *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: write_supp.c,v 1.18 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: write_supp.c,v 1.28 2011/06/09 14:47:38 diaz Exp $ */
 
 #include <string.h>
 #include <ctype.h>
@@ -52,9 +65,10 @@
 
 #define W_NOTHING                  0	/* for pl_last_writing */
 #define W_NUMBER                   1
-#define W_IDENTIFIER               2
-#define W_QUOTED                   3
-#define W_GRAPHIC                  4
+#define W_NUMBER_0                 2    /* to avoid 0'f ' if 'f ' is an op (avoid 0'char) */
+#define W_IDENTIFIER               3
+#define W_QUOTED                   4
+#define W_GRAPHIC                  5
 
 
 
@@ -67,8 +81,8 @@
 
 
 #define GENERAL_TERM               0
-#define INSIDE_LEFT_ASSOC_OP       2
 #define INSIDE_ANY_OP              1
+#define INSIDE_LEFT_ASSOC_OP       2
 
 
 
@@ -96,6 +110,7 @@ static Bool space_args;
 static Bool portrayed;
 
 static int last_prefix_op = W_NO_PREFIX_OP;
+static Bool *p_bracket_minus;
 
 
 
@@ -118,7 +133,7 @@ static void Show_Global_Var(WamWord *adr);
 
 static void Show_Atom(int context, int atom);
 
-static void Show_Integer(long x);
+static void Show_Integer(PlLong x);
 
 #ifndef NO_USE_FD_SOLVER
 static void Show_Fd_Variable(WamWord *fdv_adr);
@@ -171,7 +186,7 @@ Pl_Write_Term(StmInf *pstm, int depth, int prec, int mask, WamWord term_word)
 
   pl_last_writing = W_NOTHING;
 
-  Show_Term(depth, prec, GENERAL_TERM, term_word);
+  Show_Term(depth, prec, (prec >= 1200) ? GENERAL_TERM : INSIDE_ANY_OP, term_word);
 }
 
 
@@ -248,6 +263,12 @@ Need_Space(int c)
 
   switch (pl_last_writing)
     {
+    case W_NUMBER_0:
+      if (c_type == QT)
+	{
+	  space = TRUE;
+	  break;
+	} /* then in W_NUMBER */
     case W_NUMBER:
       space = (c_type & (UL | CL | SL | DI)) || c == '.';
       break;
@@ -268,9 +289,21 @@ Need_Space(int c)
       space = FALSE;
     }
 
-  if (space || (c == '(' && last_prefix_op) ||
-      (c_type == DI && last_prefix_op == W_PREFIX_OP_MINUS))
+  if (space || (c == '(' && last_prefix_op != W_NO_PREFIX_OP))
     Pl_Stream_Putc(' ', pstm_o);
+  else if (c_type == DI && last_prefix_op == W_PREFIX_OP_MINUS)
+    {
+#ifdef MINUS_SIGN_CANNOT_BE_FOLLOWED_BY_SPACES
+      Pl_Stream_Putc(' ', pstm_o);
+#else
+      (*p_bracket_minus)++;
+      /* to show it is an operator notation we also display a space (not strictly necessary) */
+#if 1
+      Pl_Stream_Putc(' ', pstm_o); 
+#endif
+      Pl_Stream_Putc('(', pstm_o);
+#endif
+    }
 
   last_prefix_op = W_NO_PREFIX_OP;
   pl_last_writing = W_NOTHING;
@@ -542,14 +575,17 @@ Show_Atom(int context, int atom)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 static void
-Show_Integer(long x)
+Show_Integer(PlLong x)
 {
   char str[32];
 
-  sprintf(str, "%ld", x);
+  sprintf(str, "%" PL_FMT_d, x);
   Out_String(str);
 
-  pl_last_writing = W_NUMBER;
+  if (*str == '0' && str[1] == '\0')
+    pl_last_writing = W_NUMBER_0;
+  else
+    pl_last_writing = W_NUMBER;
 }
 
 
@@ -675,7 +711,7 @@ Show_List_Arg(int depth, WamWord *lst_adr)
       if (Try_Portray(word))
 	return;
 
-      Show_Structure(depth, MAX_ARG_OF_FUNCTOR_PREC, GENERAL_TERM, 
+      Show_Structure(depth, MAX_ARG_OF_FUNCTOR_PREC, GENERAL_TERM,
 		     UnTag_STC(word));
       break;
     }
@@ -684,6 +720,33 @@ Show_List_Arg(int depth, WamWord *lst_adr)
 
 
 
+/*-------------------------------------------------------------------------*
+ * IS_VALID_VAR_NAME                                                       *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+static Bool
+Is_Valid_Var_Name(char *str)
+
+{
+  int c_type;
+
+  c_type = pl_char_type[(unsigned) *str];
+  if ((c_type & (UL | CL)) == 0) /* neither underline nor capital letter */
+    return FALSE;
+
+  while(*++str != '\0')
+    {
+      c_type = pl_char_type[(unsigned) *str];
+      if ((c_type & (UL | CL | SL | DI)) == 0)
+	return FALSE;
+    }
+
+  return TRUE;
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * SHOW_STRUCTURE                                                          *
  *                                                                         *
@@ -702,6 +765,7 @@ Show_Structure(int depth, int prec, int context, WamWord *stc_adr)
   char str[32];
   Bool bracket;
   Bool surround_space;
+  char *p;
 
 
   depth--;
@@ -709,9 +773,10 @@ Show_Structure(int depth, int prec, int context, WamWord *stc_adr)
   if (name_vars && f_n == dollar_varname_1)
     {
       DEREF(Arg(stc_adr, 0), word, tag_mask);
-      if (tag_mask == TAG_ATM_MASK)
+      p = pl_atom_tbl[UnTag_ATM(word)].name;
+      if (tag_mask == TAG_ATM_MASK && Is_Valid_Var_Name(p))
 	{
-	  Out_String(pl_atom_tbl[UnTag_ATM(word)].name);
+	  Out_String(p);
 	  pl_last_writing = W_IDENTIFIER;
 	  return;
 	}
@@ -793,12 +858,25 @@ Show_Structure(int depth, int prec, int context, WamWord *stc_adr)
 	Out_Space();
       else
 	if (strcmp(pl_atom_tbl[functor].name, "-") == 0)
-	  last_prefix_op = W_PREFIX_OP_MINUS;
+	  {
+	    last_prefix_op = W_PREFIX_OP_MINUS;
+	    p_bracket_minus = &bracket;
+	  }
 
       Show_Term(depth, oper->right, INSIDE_ANY_OP, Arg(stc_adr, 0));
       last_prefix_op = W_NO_PREFIX_OP;
 
-      if (bracket)
+      /* Here we need a while(bracket--) instead of if(bracket) because
+       * in some cases with the minus op and additional bracket is needed.
+       * Example: with op(100, xfx, &) (recall the prec of - is 200). 
+       * The term ((-(1)) & b must be displayed as: (- (1)) & b
+       * Concerning the sub-term - (1), the first ( is emitted  10 lines above
+       * because the precedence of - (200) is > precedence of & (100).
+       * The second ( is emitted by Need_Space() because the argument of - begins 
+       * by a digit. At the return we have to close 2 ).
+       */
+
+      while (bracket--)	
 	Out_Char(')');
 
       return;
@@ -850,33 +928,44 @@ Show_Structure(int depth, int prec, int context, WamWord *stc_adr)
 
       Show_Term(depth, oper->left, context, Arg(stc_adr, 0));
 
-      if (functor == ATOM_CHAR(','))
+#if 1 /* to show | unquoted if it is an infix operator with prec > 1000 */
+      if (functor == ATOM_CHAR('|') && oper->prec > 1000)
 	{
-	  Out_Char(',');
+	  if (space_args)
+	    Out_Space();
+	  Out_Char('|');
 	  if (space_args)
 	    Out_Space();
 	}
       else
-	{
-	  surround_space = FALSE;
-
-	  if (pl_atom_tbl[functor].prop.type == IDENTIFIER_ATOM ||
-	      pl_atom_tbl[functor].prop.type == OTHER_ATOM ||
-	      (space_args 
-#ifdef SPACE_ARGS_RESTRICTED	/* space_args -> space around xfx operators */
-	       && oper->left != oper->prec && oper->right != oper->prec
 #endif
-	       ))
-	    {
-	      surround_space = TRUE;
+	if (functor == ATOM_CHAR(','))
+	  {
+	    Out_Char(',');
+	    if (space_args)
 	      Out_Space();
-	    }
+	  }
+	else
+	  {
+	    surround_space = FALSE;
 
-	  Show_Atom(GENERAL_TERM, functor);
+	    if (pl_atom_tbl[functor].prop.type == IDENTIFIER_ATOM ||
+		pl_atom_tbl[functor].prop.type == OTHER_ATOM ||
+		(space_args
+#ifdef SPACE_ARGS_RESTRICTED	/* space_args -> space around xfx operators */
+		 && oper->left != oper->prec && oper->right != oper->prec
+#endif
+		 ))
+	      {
+		surround_space = TRUE;
+		Out_Space();
+	      }
 
-	  if (surround_space)
-	    Out_Space();
-	}
+	    Show_Atom(GENERAL_TERM, functor);
+
+	    if (surround_space)
+	      Out_Space();
+	  }
 
       Show_Term(depth, oper->right, INSIDE_ANY_OP, Arg(stc_adr, 1));
 
@@ -888,7 +977,7 @@ Show_Structure(int depth, int prec, int context, WamWord *stc_adr)
 
 
 
-functional:			/* functional notation */
+ functional:			/* functional notation */
 
   Show_Atom(GENERAL_TERM, functor);
   Out_Char('(');
