@@ -6,23 +6,36 @@
  * Descr.: FD Range Implementation                                         *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: fd_range.c,v 1.18 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: fd_range.c,v 1.22 2011/03/28 16:20:05 diaz Exp $ */
 
 #include <stdio.h>
 #include <string.h>
@@ -44,11 +57,11 @@
  * INTERVAL_MAX_INTEGER: an integer constant corresponding to the greatest *
  *                       value for intervals (i.e. 0..INTERVAL_MAX_INTEGER)*
  *                                                                         *
- * pl_vec_max_integer     : an integer variable corresponding to the greatest *
- *                       value for vectors   (i.e. 0..pl_vec_max_integer).    *
- * pl_vec_size            : an integer variable corresponding to the size of a*
- *                       vector in words (i.e. pl_vec_max_integer/WORD_SIZE)  *
- *                       (see Pl_Define_Vector_Size() function).              *
+ * pl_vec_max_integer  : an integer variable corresponding to the greatest *
+ *                       value for vectors   (i.e. 0..pl_vec_max_integer). *
+ * pl_vec_size         : an integer variable corresponding to the size of a*
+ *                       vector in words(i.e. pl_vec_max_integer/WORD_SIZE)*
+ *                       (see Pl_Define_Vector_Size() function).           *
  *                                                                         *
  * RANGE_TOP_STACK     : a long * variable corresponding to the top of the *
  *                       stack where are allocated the bit-vectors.        *
@@ -60,7 +73,7 @@
  *                                                                         *
  * The following macros can be redefined:                                  *
  *                                                                         *
- * WORD_SIZE           : a constant defining sizeof(long) in bits (32/64). *
+ * WORD_SIZE           : a constant defining sizeof(void*) in bits (32/64).*
  *-------------------------------------------------------------------------*/
 
 
@@ -106,68 +119,6 @@
 
 
 
-/*-------------------------------------------------------------------------*
- * PL_LEAST_SIGNIFICANT_BIT                                                *
- *                                                                         *
- *-------------------------------------------------------------------------*/
-int
-Pl_Least_Significant_Bit(VecWord x)
-{
-  int bit = 0;
-
-#if WORD_SIZE == 64
-  if (x << 32 == 0)
-    bit += 32, x >>= 32;
-#endif
-
-  if (x << (WORD_SIZE - 32 + 16) == 0)
-    bit += 16, x >>= 16;
-  if (x << (WORD_SIZE - 32 + 16 + 8) == 0)
-    bit += 8, x >>= 8;
-  if (x << (WORD_SIZE - 32 + 16 + 8 + 4) == 0)
-    bit += 4, x >>= 4;
-  if (x << (WORD_SIZE - 32 + 16 + 8 + 4 + 2) == 0)
-    bit += 2, x >>= 2;
-  if (x << (WORD_SIZE - 32 + 16 + 8 + 4 + 2 + 1) == 0)
-    bit += 1;
-
-  return bit;
-}
-
-
-
-
-/*-------------------------------------------------------------------------*
- * PL_MOST_SIGNIFICANT_BIT                                                 *
- *                                                                         *
- *-------------------------------------------------------------------------*/
-int
-Pl_Most_Significant_Bit(VecWord x)
-{
-  int bit = WORD_SIZE - 1;
-
-#if WORD_SIZE == 64
-  if (x >> 32 == 0)
-    bit -= 32, x <<= 32;
-#endif
-
-  if (x >> (WORD_SIZE - 32 + 16) == 0)
-    bit -= 16, x <<= 16;
-  if (x >> (WORD_SIZE - 32 + 16 + 8) == 0)
-    bit -= 8, x <<= 8;
-  if (x >> (WORD_SIZE - 32 + 16 + 8 + 4) == 0)
-    bit -= 4, x <<= 4;
-  if (x >> (WORD_SIZE - 32 + 16 + 8 + 4 + 2) == 0)
-    bit -= 2, x <<= 2;
-  if (x >> (WORD_SIZE - 32 + 16 + 8 + 4 + 2 + 1) == 0)
-    bit -= 1;
-
-  return bit;
-}
-
-
-
-
 /*-------------------------------------------------------------------------*
  * PL_DEFINE_VECTOR_SIZE                                                   *
  *                                                                         *
@@ -225,41 +176,12 @@ Pl_Vector_From_Interval(Vector vec, int min, int max)
 int
 Pl_Vector_Nb_Elem(Vector vec)
 {
-  static int nb_bits_in_byte[256] =
-    { 0, 1, 1, 2, 1, 2, 2, 3, 1, 2, 2, 3, 2, 3, 3, 4, 1, 2, 2, 3, 2, 3, 3,
-    4, 2, 3, 3, 4, 3, 4, 4, 5,
-    1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5,
-    3, 4, 4, 5, 4, 5, 5, 6,
-    1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5,
-    3, 4, 4, 5, 4, 5, 5, 6,
-    2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6,
-    4, 5, 5, 6, 5, 6, 6, 7,
-    1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5,
-    3, 4, 4, 5, 4, 5, 5, 6,
-    2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6,
-    4, 5, 5, 6, 5, 6, 6, 7,
-    2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6,
-    4, 5, 5, 6, 5, 6, 6, 7,
-    3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7,
-    5, 6, 6, 7, 6, 7, 7, 8
-  };
   register Vector end = vec + pl_vec_size;
-  register VecWord vec_word;
   register int nb_elem = 0;
 
   do
     {
-      vec_word = *vec;
-      nb_elem += nb_bits_in_byte[vec_word & 0xFF];
-      nb_elem += nb_bits_in_byte[(vec_word >> 8) & 0xFF];
-      nb_elem += nb_bits_in_byte[(vec_word >> 16) & 0xFF];
-      nb_elem += nb_bits_in_byte[(vec_word >> 24) & 0xFF];
-#if WORD_SIZE == 64
-      nb_elem += nb_bits_in_byte[(vec_word >> 32) & 0xFF];
-      nb_elem += nb_bits_in_byte[(vec_word >> 40) & 0xFF];
-      nb_elem += nb_bits_in_byte[(vec_word >> 48) & 0xFF];
-      nb_elem += nb_bits_in_byte[(vec_word >> 56) & 0xFF];
-#endif
+      nb_elem += Pl_Count_Set_Bits(*vec++);
       vec++;
     }
   while (vec < end);
@@ -319,7 +241,7 @@ Pl_Vector_Next_After(Vector vec, int n)
 
       start = vec + word_no;
 
-      word = (bit_no == WORD_SIZE) ? 0 : *start & ~((1L << bit_no) - 1);
+      word = (bit_no == WORD_SIZE) ? 0 : *start & ~(((PlLong)1 << bit_no) - 1);
     }
   else				/* n < 0 find first */
     {
@@ -371,7 +293,7 @@ Pl_Vector_Next_Before(Vector vec, int n)
 
       end = vec + word_no;
 
-      word = *end & ((1L << bit_no) - 1);
+      word = *end & (((PlLong)1 << bit_no) - 1);
     }
   else				/* n > pl_vec_max_integer find last */
     {
