@@ -6,25 +6,39 @@
  * Descr.: FD Range Implementation - header file                           *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: fd_range.h,v 1.13 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: fd_range.h,v 1.17 2011/03/28 16:20:05 diaz Exp $ */
 
 #include "bool.h"
+#include "pl_long.h"
 
 /*---------------------------------*
  * Constants                       *
@@ -34,7 +48,7 @@
  * Type Definitions                *
  *---------------------------------*/
 
-typedef unsigned long VecWord;
+typedef PlULong VecWord;
 
 typedef VecWord *Vector;
 
@@ -81,10 +95,6 @@ Range;
  * Function Prototypes             *
  *---------------------------------*/
 
-int Pl_Least_Significant_Bit(VecWord x);
-
-int Pl_Most_Significant_Bit(VecWord x);
-
 void Pl_Define_Vector_Size(int max_val);
 
 void Pl_Vector_From_Interval(Vector vec, int min, int max);
