@@ -6,23 +6,36 @@
  * Descr.: FD to C macros - header file                                    *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: fd_to_c.h,v 1.14 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: fd_to_c.h,v 1.17 2011/03/28 16:20:05 diaz Exp $ */
 
 
 #ifndef _FD_TO_C_H
@@ -166,14 +179,14 @@
 
 #define fd_call_internal_and_test_switch_simple(fct_name)	\
 {								\
-  long (*fct) () = (long (*)()) fct_name(AF);			\
+  PlLong (*fct) () = (PlLong (*)()) fct_name(AF);			\
 								\
-  if (fct == (long (*)()) FALSE)				\
+  if (fct == (PlLong (*)()) FALSE)				\
     {								\
       ret_val = FALSE;						\
       goto lab_exit;						\
     }								\
-  if (fct != (long (*)()) TRUE)	/* FD switch case triggered */	\
+  if (fct != (PlLong (*)()) TRUE)	/* FD switch case triggered */	\
     {								\
       if ((*fct) (AF) == FALSE)					\
 	{							\
@@ -188,14 +201,14 @@
 
 #define fd_call_internal_and_test_switch(fct_name)		\
 {								\
-  long (*fct) () = (long (*)()) fct_name(AF);			\
+  PlLong (*fct) () = (PlLong (*)()) fct_name(AF);			\
 								\
-  if (fct == (long (*)()) FALSE)				\
+  if (fct == (PlLong (*)()) FALSE)				\
     {								\
       ret_val = FALSE;						\
       goto lab_exit;						\
     }								\
-  if (fct != (long (*)()) TRUE)	/* FD switch case triggered */	\
+  if (fct != (PlLong (*)()) TRUE)	/* FD switch case triggered */	\
     {								\
       if ((*fct) (AF) == FALSE)					\
 	{							\
@@ -386,7 +399,7 @@
 #define fd_test_switch_condition(t, fct_name)	\
   if (t)					\
     {						\
-      ret_val = (long) fct_name;		\
+      ret_val = (PlLong) fct_name;		\
       goto lab_exit;				\
     }
 
@@ -648,7 +661,7 @@ Bool						\
 name_args					\
 {						\
   WamWord *AF;					\
-  long ret_val = TRUE;
+  PlLong ret_val = TRUE;
 
 
 
@@ -660,10 +673,10 @@ name_args					\
 
 
 #define fd_begin_internal(fct_name)		\
-static long					\
+static PlLong					\
 fct_name(WamWord *AF)				\
 {						\
-  long ret_val = TRUE;
+  PlLong ret_val = TRUE;
 
 
 
