@@ -6,23 +6,36 @@
  * Descr.: general engine                                                  *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: engine.c,v 1.20 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: engine.c,v 1.27 2011/06/01 09:20:40 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -34,6 +47,10 @@
 
 #if defined(_WIN32) || defined(__CYGWIN__)
 #include <windows.h>
+#define READ_REGISTRY_ONLY
+#include "../TopComp/prolog_path.c"
+#else
+#include <sys/param.h>
 #endif
 
 #define ENGINE_FILE
@@ -44,6 +61,9 @@
 #include "../Linedit/linedit.h"
 #endif
 
+#include "../TopComp/prolog_path.c"
+
+
 
 /*---------------------------------*
  * Constants                       *
@@ -62,7 +82,7 @@
  * Global Variables                *
  *---------------------------------*/
 
-void (*pl_init_stream_supp)();	/* overwritten by foreign if present */
+void (*pl_init_stream_supp)();  /* overwritten by foreign if present */
 
 #if !defined(NO_USE_REGS) && NB_OF_USED_MACHINE_REGS > 0
 static WamWord init_buff_regs[NB_OF_USED_MACHINE_REGS];
@@ -76,8 +96,7 @@ static int nb_user_directives = 0;
 static jmp_buf *p_jumper;
 static WamWord *p_buff_save;
 
-static CodePtr cont_jmp;	/* we use a global var to support DEC alpha */
-
+static CodePtr cont_jmp;        /* we use a global var to support DEC alpha */
 
 
 
@@ -91,7 +110,7 @@ static void Call_Prolog_Success(void);
 
 static Bool Call_Next(CodePtr codep);
 
-void Pl_Call_Compiled(CodePtr codep);	/* defined in engine1.c */
+void Pl_Call_Compiled(CodePtr codep);   /* defined in engine1.c */
 
 
 
@@ -105,39 +124,50 @@ Pl_Start_Prolog(int argc, char *argv[])
 {
   int i, x;
   char *p;
+  void (*copy_of_pl_init_stream_supp)() = Pl_Dummy_Ptr(pl_init_stream_supp);
 
   setlocale(LC_ALL, "");
-  setlocale(LC_NUMERIC, "C");	/* make sure floats come out right... */
-
-  Pl_Init_Machine();
+  setlocale(LC_NUMERIC, "C");   /* make sure floats come out right... */
 
   pl_os_argc = argc;
   pl_os_argv = argv;
 
+  pl_home = Get_Prolog_Path(argv[0], &pl_devel_mode);
+
+  Pl_Init_Machine();
+
   Set_Line_Buf(stdout);
   Set_Line_Buf(stderr);
 
   for (i = 0; i < NB_OF_STACKS; i++)
     {
       if (pl_fd_init_solver == NULL && strcmp(pl_stk_tbl[i].name, "cstr") == 0)
-	{			/* FD solver not linked */
-	  pl_stk_tbl[i].size = 0;
-	  continue;
-	}
+        {                       /* FD solver not linked */
+          pl_stk_tbl[i].size = 0;
+          continue;
+        }
 
-      if ((pl_stk_tbl[i].size = KBytes_To_Wam_Words(*(pl_stk_tbl[i].p_def_size)))
-	  == 0)
-	pl_stk_tbl[i].size = pl_stk_tbl[i].default_size;
+      if ((pl_stk_tbl[i].size = KBytes_To_Wam_Words(*(pl_stk_tbl[i].p_def_size))) == 0)
+        pl_stk_tbl[i].size = pl_stk_tbl[i].default_size;
 
       if (!pl_fixed_sizes && *pl_stk_tbl[i].env_var_name)
-	{
-	  p = (char *) getenv(pl_stk_tbl[i].env_var_name);
-	  if (p && *p)
-	    {
-	      sscanf(p, "%d", &x);
-	      pl_stk_tbl[i].size = KBytes_To_Wam_Words(x);
-	    }
-	}
+        {
+          p = (char *) getenv(pl_stk_tbl[i].env_var_name);
+          if (p && *p)
+            {
+              sscanf(p, "%d", &x);
+              pl_stk_tbl[i].size = KBytes_To_Wam_Words(x);
+            }
+        }
+
+#if defined(_WIN32) || defined(__CYGWIN__)
+      if (!pl_fixed_sizes && *pl_stk_tbl[i].env_var_name)
+        {
+          DWORD x;
+          if (Read_Windows_Registry(pl_stk_tbl[i].env_var_name, REG_DWORD, &x, sizeof(x)))
+            pl_stk_tbl[i].size = KBytes_To_Wam_Words(x);
+        }
+#endif
     }
 
   Pl_M_Allocate_Stacks();
@@ -163,8 +193,8 @@ Pl_Start_Prolog(int argc, char *argv[])
 #endif
     pl_use_gui = 0;
 
-  if (pl_init_stream_supp)
-    (*pl_init_stream_supp)();
+  if (copy_of_pl_init_stream_supp)
+    (*copy_of_pl_init_stream_supp)();
 
   Pl_Reset_Prolog();
   Pl_Fd_Init_Solver();
@@ -186,7 +216,7 @@ Pl_Stop_Prolog(void)
 {
 #ifdef DEREF_STATS
   double d = (double) chain_len / (double) nb_deref;
-  fprintf(stderr, "Deref: nb: %ld  avg len = %g\n", nb_deref, d);
+  fprintf(stderr, "Deref: nb: %" PL_FMT_d "  avg len = %g\n", nb_deref, d);
 #endif
   Restore_Machine_Regs(init_buff_regs);
 }
@@ -203,14 +233,14 @@ void
 Pl_Reset_Prolog(void)
 {
   E = B = LSSA = Local_Stack;
-  H = heap_actual_start;	/* restart after needed global terms */
+  H = heap_actual_start;        /* restart after needed global terms */
   TR = Trail_Stack;
   CP = NULL;
   STAMP = 0;
   CS = Cstr_Stack;
-  BCI = 0;			/* BCI only needed for byte-code (cf. bips prolog) */
+  BCI = 0;                      /* BCI only needed for byte-code (cf. bips prolog) */
 
-  Pl_Create_Choice_Point(Call_Prolog_Fail, 0);	/* 1st choice point */
+  Pl_Create_Choice_Point(Call_Prolog_Fail, 0);  /* 1st choice point */
 
   Pl_Fd_Reset_Solver();
 }
@@ -260,8 +290,8 @@ Pl_Execute_Directive(int pl_file, int pl_line, Bool is_system, CodePtr proc)
 
   if (!Pl_Call_Prolog(proc))
     fprintf(stderr,
-	    ERR_DIRECTIVE_FAILED, pl_atom_tbl[pl_file].name, pl_line,
-	    (is_system) ? "system" : "user");
+            ERR_DIRECTIVE_FAILED, pl_atom_tbl[pl_file].name, pl_line,
+            (is_system) ? "system" : "user");
 
   Pl_Reset_Prolog();
 }
@@ -316,20 +346,20 @@ Pl_Call_Prolog(CodePtr codep)
   WamCont save_ALTB = ALTB(query_b);
   Bool ok;
 
-  ALTB(query_b) = (CodePtr) Call_Prolog_Fail;	/* modify choice point */
+  ALTB(query_b) = (CodePtr) Call_Prolog_Fail;   /* modify choice point */
 
   CP = Adjust_CP(Call_Prolog_Success);
 
-#if defined(_WIN32) || defined(__CYGWIN__)
+#if defined(USE_SEH)
   SEH_PUSH(Win32_SEH_Handler);
 #endif
   ok = Call_Next(codep);
-#if defined(_WIN32) || defined(__CYGWIN__)
+#if defined(USE_SEH)
   SEH_POP;
 #endif
 
-  CP = save_CP;			/* restore continuation */
-  ALTB(query_b) = save_ALTB;	/* restore choice point */
+  CP = save_CP;                 /* restore continuation */
+  ALTB(query_b) = save_ALTB;    /* restore choice point */
 
   return ok;
 }
@@ -349,15 +379,15 @@ Pl_Call_Prolog_Next_Sol(WamWord *query_b)
   WamCont save_ALTB = ALTB(query_b);
   Bool ok;
 
-  ALTB(query_b) = (CodePtr) Call_Prolog_Fail;	/* modify choice point */
+  ALTB(query_b) = (CodePtr) Call_Prolog_Fail;   /* modify choice point */
 
-  CP = Adjust_CP(Call_Prolog_Success);	/* should be useless since */
+  CP = Adjust_CP(Call_Prolog_Success);  /* should be useless since */
   /* alternative will restore CP */
 
   ok = Call_Next(ALTB(B));
 
-  CP = save_CP;			/* restore continuation */
-  ALTB(query_b) = save_ALTB;	/* restore choice point */
+  CP = save_CP;                 /* restore continuation */
+  ALTB(query_b) = save_ALTB;    /* restore choice point */
 
   return ok;
 }
@@ -405,7 +435,7 @@ Call_Next(CodePtr codep)
   jmp_buf *old_jumper = p_jumper;
   jmp_buf new_jumper;
   WamWord *old_buff_save = p_buff_save;
-  WamWord buff_save_machine_regs[NB_OF_USED_MACHINE_REGS + 1];	/* +1 if = 0 */
+  WamWord buff_save_machine_regs[NB_OF_USED_MACHINE_REGS + 1];  /* +1 if = 0 */
 #if 0
   WamWord buff_save_all_regs[NB_OF_REGS];
 #endif
@@ -422,17 +452,17 @@ Call_Next(CodePtr codep)
 
   Restore_Machine_Regs(buff_save_machine_regs);
 
-  if (jmp_val == 0)		/* normal call to codep */
+  if (jmp_val == 0)             /* normal call to codep */
     Pl_Call_Compiled(codep);
 
-  if (jmp_val == 3)		/* return with a continuation in jmp_val */
+  if (jmp_val == 3)             /* return with a continuation in jmp_val */
     Pl_Call_Compiled(cont_jmp);
 
-				/* normal return */
+                                /* normal return */
   p_jumper = old_jumper;
   p_buff_save = old_buff_save;
 
-  if (jmp_val < 0)		/* false: restore WAM registers */
+  if (jmp_val < 0)              /* false: restore WAM registers */
     {
 #if 0
       Restore_All_Regs(buff_save_all_regs);
@@ -440,23 +470,24 @@ Call_Next(CodePtr codep)
       return FALSE;
     }
 
-  return jmp_val;		/* 1 (TRUE) or 3 (exception) */
+  return jmp_val;               /* 1 (TRUE) or 3 (exception) */
 }
 
 
 
 
-	  /*------------------------------------------------------------*
+          /*------------------------------------------------------------*
            * Call_Prolog_Fail: Prolog continuation after failure.       *
            * Return in Call_Next with a longjmp (value -1)              *
            *------------------------------------------------------------*/
+#if 1//!(defined(M_x86_64) && defined(_MSC_VER))/* see file engine_asm.s */
 
 static void
 Call_Prolog_Fail(void)
 {
-#ifdef M_ix86_darwin		/* see comment in Ma2Asm/ix86_any.c */
+#ifdef M_ix86_darwin            /* see comment in Ma2Asm/ix86_any.c */
   asm("subl $4,%esp");
-#elif defined(M_x86_64) 	/* see comment in Ma2Asm/x86_64_any.c */
+#elif defined(M_x86_64) && !defined(_MSC_VER)   /* see comment in Ma2Asm/x86_64_any.c */
   asm("subq $8,%rsp");
 #endif
   Save_Machine_Regs(p_buff_save);
@@ -466,7 +497,7 @@ Call_Prolog_Fail(void)
 
 
 
-	  /*------------------------------------------------------------*
+          /*------------------------------------------------------------*
            * Call_Prolog_Success: Prolog continuation after success.    *
            * Return in Call_Next with a longjmp (value 1)               *
            *------------------------------------------------------------*/
@@ -474,19 +505,18 @@ Call_Prolog_Fail(void)
 static void
 Call_Prolog_Success(void)
 {
-#ifdef M_ix86_darwin		/* see comment in Ma2Asm/ix86_any.c */
+#ifdef M_ix86_darwin            /* see comment in Ma2Asm/ix86_any.c */
   asm("subl $4,%esp");
-#elif defined(M_x86_64) 	/* see comment in Ma2Asm/x86_64_any.c */
+#elif defined(M_x86_64) && !defined(_MSC_VER)   /* see comment in Ma2Asm/x86_64_any.c */
   asm("subq $8,%rsp");
 #endif
   Save_Machine_Regs(p_buff_save);
   longjmp(*p_jumper, 1);
 }
 
+#endif
 
-
-
-	  /*------------------------------------------------------------*
+          /*------------------------------------------------------------*
            * Exit_With_Exception:                                       *
            * Similar to a success but Call_Prolog returns 2 instead of 1*
            * (i.e. TRUE)                                                *
@@ -503,7 +533,7 @@ Pl_Exit_With_Exception(void)
 
 
 
-	  /*------------------------------------------------------------*
+          /*------------------------------------------------------------*
            * Execute_A_Continuation:                                    *
            * Similar to a nested Call_Prolog but faster, and if a fail  *
            * occurs it is normally handled by the prolog engine, i.e.   *
