@@ -6,23 +6,36 @@
  * Descr.: general engine - header file                                    *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: engine.h,v 1.14 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: engine.h,v 1.22 2011/06/28 13:40:31 diaz Exp $ */
 
 #ifdef NO_STACK_TEST
 #   undef  M_Check_Stacks()
@@ -37,7 +50,7 @@
  *---------------------------------*/
 
 
-#define cpp_recurs(p, n)           p##_##n
+#define cpp_recurs(p, n)           p##__a##n
 #define Prolog_Predicate(p, n)     cpp_recurs(p, n)
 
 #define Prolog_Prototype(p, n)     void Prolog_Predicate(p, n)()
@@ -60,9 +73,12 @@
 int pl_os_argc;
 char **pl_os_argv;
 
+char *pl_home;
+int pl_devel_mode;
+
 char pl_glob_buff[10240];
 
-long *pl_base_fl;			/* overwritten by foreign if present */
+PlLong *pl_base_fl;			/* overwritten by foreign if present */
 double *pl_base_fd;		/* overwritten by foreign if present */
 
 int pl_use_gui;
@@ -72,9 +88,12 @@ int pl_use_gui;
 extern int pl_os_argc;
 extern char **pl_os_argv;
 
+extern char *pl_home;
+extern int pl_devel_mode;
+
 extern char pl_glob_buff[];
 
-extern long *pl_base_fl;
+extern PlLong *pl_base_fl;
 extern double *pl_base_fd;
 
 extern int pl_use_gui;
