@@ -6,29 +6,44 @@
  * Descr.: general engine                                                  *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: engine1.c,v 1.22 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: engine1.c,v 1.28 2011/06/28 13:40:31 diaz Exp $ */
 
 #include "engine_pl.h"
 
 
 WamWord *pl_ensure_reserved;
 
+#if !(defined(M_x86_64) && defined(_MSC_VER))/* see file eng1-x86_64_win.s */
+
 
 /*-------------------------------------------------------------------------*
  * Call_Compiled invokes a Prolog code.                                    *
@@ -45,33 +60,34 @@ WamWord *pl_ensure_reserved;
  *                                                                         *
  * This functions is in a separate file for historical reasons. In versions*
  * <= 1.2.4, this file was compiled without any C compiler optimization to *
- * ensure reserved_stack_space was not removed but the C compiler. In order*
+ * ensure reserved_stack_space was not removed by the C compiler. In order *
  * to use ebp under ix86 it must be compiled with -fomit-frame-pointer. The*
  * simpliest way was to use the same C compiler invocation but adding a    *
  * global variable to ensure the stack is not removed.                     *
  *-------------------------------------------------------------------------*/
 
+static long x;
 void
 Pl_Call_Compiled(CodePtr codep)
 {
   WamWord reserved_stack_space[1024];
 
 #if defined(M_sparc) && !defined(M_sparc_bsd)
-  register long * __attribute__ ((unused)) rfl asm("%l2") = pl_base_fl;
+  register PlLong * __attribute__ ((unused)) rfl asm("%l2") = pl_base_fl;
   register double * __attribute__ ((unused)) rfd asm("%l3") = pl_base_fd;
-  pl_ensure_reserved = (WamWord *) rfl + (long) rfd; /* to avoid gcc remove 2 previous inits ! */
+  pl_ensure_reserved = (WamWord *) rfl + (PlLong) rfd; /* to avoid gcc remove 2 previous inits ! */
 #endif
 
 #if !defined(NO_MACHINE_REG_FOR_REG_BANK) && !defined(MAP_REG_BANK)
 
 #if defined(M_ix86_linux) || \
     defined(M_ix86_cygwin) || defined(M_ix86_mingw) || \
-    defined(M_ix86_sco) || defined(M_ix86_bsd) 
+    defined(M_ix86_sco) || defined(M_ix86_bsd)
 
   register WamWord *rb asm("%ebx") = pl_reg_bank;
   pl_ensure_reserved = (WamWord *) rb; /* to avoid gcc warning */
 
-#elif defined(_MSC_VER)
+#elif defined(_MSC_VER) && defined(M_ix86)
 
   _asm mov ebx, pl_reg_bank
 
@@ -100,11 +116,6 @@ Pl_Call_Compiled(CodePtr codep)
   register WamWord *rb asm("%l0") = pl_reg_bank;
   pl_ensure_reserved = (WamWord *) rb; /* to avoid gcc warning */
 
-#elif defined(M_x86_64)
-
-  register WamWord *rb asm("%r12") = pl_reg_bank;
-  pl_ensure_reserved = (WamWord *) rb; /* to avoid gcc warning */
-
 #endif
 
 #endif
@@ -113,11 +124,12 @@ Pl_Call_Compiled(CodePtr codep)
 #if defined(M_ix86_darwin)	/* see comment in Ma2Asm/ix86_any.c */
   asm("andl $0xfffffff0,%esp");
   asm("addl $4,%esp");
-#elif defined(M_x86_64)		/* see comment in Ma2Asm/x86_64_any.c */
+#elif defined(M_x86_64) && !defined(_MSC_VER)		/* see comment in Ma2Asm/x86_64_any.c */
   asm("andq $0xfffffffffffffff0,%rsp");
   asm("addq $8,%rsp");
 #endif
 
   (*codep) ();
-
 }
+#endif
+
