@@ -6,23 +6,36 @@
  * Descr.: machine dependent features - Header file                        *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: machine.h,v 1.22 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: machine.h,v 1.26 2011/04/08 16:16:16 diaz Exp $ */
 
 #ifndef _MACHINE_H
 #define _MACHINE_H
@@ -30,7 +43,6 @@
 #include "bool.h"
 
 
-
 /*---------------------------------*
  * Constants                       *
  *---------------------------------*/
@@ -53,11 +65,11 @@ void Pl_M_Allocate_Stacks(void);
 
 char *Pl_M_Sys_Err_String(int err_no);
 
-long Pl_M_User_Time(void);
+PlLong Pl_M_User_Time(void);
 
-long Pl_M_System_Time(void);
+PlLong Pl_M_System_Time(void);
 
-long Pl_M_Real_Time(void);
+PlLong Pl_M_Real_Time(void);
 
 void Pl_M_Randomize(void);
 
@@ -122,7 +134,7 @@ void M_Check_Magic_Words(void); /* not compiled if not needed */
 
 #    define M_USED_REGS            {"15", "20", 0}
 
-#elif defined(M_x86_64)
+#elif defined(M_x86_64) && !defined(_MSC_VER) 
 
 #    define M_USED_REGS            {"r12", "r13", "r14", "r15", 0}
 
@@ -138,6 +150,11 @@ void M_Check_Magic_Words(void); /* not compiled if not needed */
 #endif
 
 
+//#if defined(_MSC_VER) && defined(M_x86_64)
+//#define NO_MACHINE_REG_FOR_REG_BANK
+//#endif
+
+
 
 
 /*---------------------------------*
