@@ -6,24 +6,38 @@
  * Descr.: object chaining management - header file                        *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: obj_chain.h,v 1.15 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: obj_chain.h,v 1.20 2011/04/08 16:16:16 diaz Exp $ */
 
+#include "pl_long.h"
 
 /*---------------------------------*
  * Constants                       *
@@ -66,7 +80,7 @@ OBJ_CTOR(void)
 
 #else /* _MSC_VER */
 
-static void 
+static void
 OBJ_CTOR(void)
 {
   Pl_New_Object(OBJ_INIT, NULL, NULL);
@@ -74,7 +88,7 @@ OBJ_CTOR(void)
 
 #pragma data_seg(".GPLC$m")
 
-static long obj_chain_start = (long) OBJ_CTOR;
+static PlLong obj_chain_start = (PlLong) OBJ_CTOR;
 
 #pragma data_seg()
 
@@ -84,6 +98,6 @@ static long obj_chain_start = (long) OBJ_CTOR;
 
 
 
-#if defined(_MSC_VER) || defined(M_ix86_darwin)
+#if defined(_MSC_VER) || defined(M_darwin)
 #define OBJ_CHAIN_REVERSE_ORDER
 #endif
