@@ -6,23 +6,36 @@
  * Descr.: operator table management                                       *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: oper.c,v 1.14 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: oper.c,v 1.20 2011/03/28 16:20:05 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -84,6 +97,8 @@ Pl_Init_Oper(void)
 
   ADD_OPER(1200, FX, ":-");
 
+  ADD_OPER(1105, XFY, "|");
+
   ADD_OPER(1100, XFY, ";");
 
   ADD_OPER(1050, XFY, "->");
@@ -124,11 +139,11 @@ Pl_Init_Oper(void)
   ADD_OPER(400, YFX, "//");
   ADD_OPER(400, YFX, "rem");
   ADD_OPER(400, YFX, "mod");
+  ADD_OPER(400, YFX, "div");
   ADD_OPER(400, YFX, "<<");
   ADD_OPER(400, YFX, ">>");
 
-  ADD_OPER(200, XFY, "**");
-
+  ADD_OPER(200, XFX, "**");
   ADD_OPER(200, XFY, "^");
 
   ADD_OPER(200, FY, "+");
@@ -243,7 +258,7 @@ Pl_Lookup_Oper_Any_Type(int atom_op)
 OperInf *
 Pl_Delete_Oper(int atom_op, int type)
 {
-  long key = Make_Oper_Key(atom_op, type);
+  PlLong key = Make_Oper_Key(atom_op, type);
 
   pl_atom_tbl[atom_op].prop.op_mask &= ~Make_Op_Mask(type);
 
