@@ -6,23 +6,36 @@
  * Descr.: operator table management - header file                         *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: oper.h,v 1.13 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: oper.h,v 1.16 2011/03/28 16:20:05 diaz Exp $ */
 
 /*---------------------------------*
  * Constants                       *
@@ -32,9 +45,9 @@
 #define MAX_ARG_OF_FUNCTOR_PREC    999
 
 
-#define Make_Oper_Key(a, t)        (((unsigned long) (a) << 2) | (t))
-#define Atom_Of_Oper(k)            ((unsigned long) (k) >> 2)
-#define Type_Of_Oper(k)            ((unsigned long) (k) & 3)
+#define Make_Oper_Key(a, t)        (((PlULong) (a) << 2) | (t))
+#define Atom_Of_Oper(k)            ((PlULong) (k) >> 2)
+#define Type_Of_Oper(k)            ((PlULong) (k) & 3)
 
 
 
@@ -59,7 +72,7 @@
 
 typedef struct			/* Operator information           */
 {				/* ------------------------------ */
-  long a_t;			/* key is <atom,operator type>    */
+  PlLong a_t;			/* key is <atom,operator type>    */
   int prec;			/* precedence of the operator     */
   int left;			/* precedence of the operator lhs */
   int right;			/* precedence of the operator rhs */
