@@ -6,23 +6,36 @@
  * Descr.: C Compiler options and WAM Configuration                        *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: pl_config.c,v 1.20 2009/02/10 12:48:54 diaz Exp $ */
+/* $Id: pl_config.c,v 1.27 2011/05/11 15:11:11 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -42,11 +55,13 @@
 
 #define STR_LENGTH                 512
 
-#define FILE_WAM_REGS_H            "wam_regs.h"
-
 #define FILE_WAM_ARCHI_DEF         "wam_archi.def"
 #define FILE_WAM_ARCHI_H           "wam_archi.h"
 
+#define FILE_WAM_REGS_H            "wam_regs.h"
+
+#define FILE_WAM_STACKS_H          "wam_stacks.h"
+
 #define FILE_GPROLOG_CST_H         "gprolog_cst.h"
 
 
@@ -96,6 +111,7 @@ TagInf;
 typedef struct
 {
   char name[32];
+  char desc[64];
   int def_size;
   char top_macro[128];
 }
@@ -113,6 +129,7 @@ int nb_of_used_mach_regs;
 
 char save_str[STR_LENGTH];
 FILE *fw_r;
+FILE *fw_s;
 FILE *fg_c;
 
 
@@ -128,6 +145,8 @@ void Write_GProlog_Cst(void);
 
 char *Read_Identifier(char *s, int fail_if_error, char **end);
 
+char *Read_String(char *s, char **end);
+
 int Pl_Read_Integer(char *s, char **end);
 
 void Generate_Regs(FILE *f, FILE *g);
@@ -159,10 +178,10 @@ main(void)
     }
 
 
-  if (WORD_SIZE != (sizeof(long) * 8))
+  if (WORD_SIZE != (sizeof(PlLong) * 8))
     {
       printf("Bad Value of WORD_SIZE - should be %d\n",
-	     (int) (sizeof(long) * 8));
+             (int) (sizeof(PlLong) * 8));
 
       return 1;
     }
@@ -170,11 +189,15 @@ main(void)
   if ((fw_r = fopen(FILE_WAM_REGS_H, "wb")) == NULL)
     Pl_Fatal_Error("cannot open %s", FILE_WAM_REGS_H);
 
-  fprintf(fw_r,
-	  "/* this file is automatically generated by pl_config.c */\n");
+  fprintf(fw_r, "/* this file is automatically generated by pl_config.c */\n");
   fprintf(fw_r, "\n#include \"gp_config.h\"\n\n");
 
 
+  if ((fw_s = fopen(FILE_WAM_STACKS_H, "wb")) == NULL)
+    Pl_Fatal_Error("cannot open %s", FILE_WAM_STACKS_H);
+
+  fprintf(fw_s, "/* this file is automatically generated by pl_config.c */\n");
+
   if ((fg_c = fopen(FILE_GPROLOG_CST_H, "wb")) == NULL)
     Pl_Fatal_Error("cannot open %s", FILE_GPROLOG_CST_H);
 #if 0
@@ -188,63 +211,88 @@ main(void)
   printf("GNU Prolog version: %s (%s)\n", PROLOG_VERSION, PROLOG_DATE);
   printf("Operating system  : %s\n", M_OS);
   printf("Processor         : %s\n", M_CPU);
-  printf("Size of a word    : %d bits\n", (int) WORD_SIZE);
+  printf("Size of a WAM word: %d bits\n", (int) WORD_SIZE);
   printf("C compiler        : %s\n", CC);
   printf("C flags           : %s\n", CFLAGS);
   printf("C flags machine   : %s\n", CFLAGS_MACHINE);
+  printf("Assembler         : %s\n", AS);
+  printf("Assembler flags   : %s\n", ASFLAGS);
   printf("Loader flags      : %s\n", LDFLAGS);
   printf("Loader libraries  : %s\n", LDLIBS);
   printf("Use line editor   : %s\n",
 #ifndef NO_USE_LINEDIT
-	 "Yes"
+         "Yes"
 #else
-	 "No"
+         "No"
 #endif
     );
+
   printf("Use piped consult : %s\n",
 #ifndef NO_USE_PIPED_STDIN_FOR_CONSULT
-	 "Yes"
+         "Yes"
 #else
-	 "No"
+         "No"
 #endif
     );
+
 #ifdef _WIN32
   printf("Use GUI console   : %s\n",
 #ifdef W32_GUI_CONSOLE
-	 "Yes"
+         "Yes"
 #else
-	 "No"
+         "No"
 #endif
     );
 #endif
+
+#ifdef W32_GUI_CONSOLE
+  printf("Use HtmlHelp      : %s\n",
+#if defined(WITH_HTMLHELP) && WITH_HTMLHELP == 1
+         "Yes (statically linked)"
+#elif defined(WITH_HTMLHELP) && WITH_HTMLHELP == 2
+         "Yes (dynamically loaded)"
+#else
+         "No"
+#endif
+    );
+#endif
+
   printf("Use sockets       : %s\n",
 #ifndef NO_USE_SOCKETS
-	 "Yes"
+         "Yes"
 #else
-	 "No"
+         "No"
 #endif
     );
+
   printf("Use FD solver     : %s\n",
 #ifndef NO_USE_FD_SOLVER
-	 "Yes"
+         "Yes"
 #else
-	 "No"
+         "No"
 #endif
     );
+
+#if defined(__unix__) && !defined(__CYGWIN__)
+  printf("Working sigaction : %s\n", (Detect_Sigaction()) ? "Yes" : "No");
+#endif
+
+
 #ifdef COULD_COMPILE_FOR_FC
   printf("Use fast call     : %s\n",
 #ifndef NO_USE_FAST_CALL
-	 "Yes"
+         "Yes"
 #else
-	 "No"
+         "No"
 #endif
     );
 #endif
+
   printf("Use machine regs. : %s\n",
 #ifndef NO_USE_REGS
-	 "Yes"
+         "Yes"
 #else
-	 "No"
+         "No"
 #endif
     );
 
@@ -255,15 +303,12 @@ main(void)
   Write_C_Compiler_Info();
 
 
-#if defined(__unix__) && !defined(__CYGWIN__)
-  printf("Working sigaction : %s\n", (Detect_Sigaction()) ? "Yes" : "No");
-#endif
-
 #if 0
   fprintf(fg_c, "/* end of automatically generated part */\n");
 #endif
 
   fclose(fw_r);
+  fclose(fw_s);
   fclose(fg_c);
 
   printf("Used register(s)  : ");
@@ -295,14 +340,16 @@ Write_GProlog_Cst(void)
 
   fputc('\n', fg_c);
 
+  fprintf(fg_c, "#define __GNU_PROLOG__        \t%d\n", major);
   fprintf(fg_c, "#define __GPROLOG__           \t%d\n", major);
   fprintf(fg_c, "#define __GPROLOG_MINOR__     \t%d\n", minor);
   fprintf(fg_c, "#define __GPROLOG_PATCHLEVEL__\t%d\n", patch_level);
-  fprintf(fg_c, "#define __GPROLOG_VERSION__   \t%d\n", 
-	  major * 10000 + minor * 100 + patch_level);
+  fprintf(fg_c, "#define __GPROLOG_VERSION__   \t%d\n",
+          major * 10000 + minor * 100 + patch_level);
 
   fputc('\n', fg_c);
 
+  fprintf(fg_c, "#define PROLOG_DIALECT  \t\"" PROLOG_DIALECT "\"\n");
   fprintf(fg_c, "#define PROLOG_NAME     \t\"" PROLOG_NAME "\"\n");
   fprintf(fg_c, "#define PROLOG_VERSION  \t\"" PROLOG_VERSION "\"\n");
   fprintf(fg_c, "#define PROLOG_DATE     \t\"" PROLOG_DATE "\"\n");
@@ -356,37 +403,37 @@ Generate_Archi(void)
   while (!feof(f) && fgets(str, sizeof(str), f))
     {
       if (*str != '@')
-	{
-	  fputs(str, g);
-	  continue;
-	}
+        {
+          fputs(str, g);
+          continue;
+        }
 
       strcpy(save_str, str);
       p1 = Read_Identifier(str + 1, 1, &p2);
 
       if (strcmp(p1, "begin") != 0)
-	Pl_Fatal_Error("Syntax error: incorrect @ declaration in: %s",
-		    save_str);
+        Pl_Fatal_Error("Syntax error: incorrect @ declaration in: %s",
+                    save_str);
 
       p1 = Read_Identifier(p2 + 1, 1, &p2);
 
       if (strcmp(p1, "regs") == 0)
-	{
-	  Generate_Regs(f, g);
-	  continue;
-	}
+        {
+          Generate_Regs(f, g);
+          continue;
+        }
 
       if (strcmp(p1, "tags") == 0)
-	{
-	  Generate_Tags(f, g);
-	  continue;
-	}
+        {
+          Generate_Tags(f, g);
+          continue;
+        }
 
       if (strcmp(p1, "stacks") == 0)
-	{
-	  Generate_Stacks(f, g);
-	  continue;
-	}
+        {
+          Generate_Stacks(f, g);
+          continue;
+        }
 
 
       Pl_Fatal_Error("Syntax error: unknown section in: %s", save_str);
@@ -413,9 +460,9 @@ Read_Identifier(char *s, int fail_if_error, char **end)
   if (!isalpha(**end))
     {
       if (fail_if_error)
-	Pl_Fatal_Error("Syntax error: identifier expected in: %s", save_str);
+        Pl_Fatal_Error("Syntax error: identifier expected in: %s", save_str);
       else
-	return NULL;
+        return NULL;
     }
 
   do
@@ -424,7 +471,7 @@ Read_Identifier(char *s, int fail_if_error, char **end)
 
   if (!isspace(**end))
     Pl_Fatal_Error("Syntax error: space expected after identifier in: %s",
-		save_str);
+                save_str);
 
   **end = '\0';
 
@@ -434,6 +481,32 @@ Read_Identifier(char *s, int fail_if_error, char **end)
 
 
 
+/*-------------------------------------------------------------------------*
+ * READ_STRING                                                             *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+char *
+Read_String(char *s, char **end)
+{
+  while (isspace(*s))
+    s++;
+
+  if (*s != '"')
+    Pl_Fatal_Error("Syntax error: string expected in: %s", save_str);
+
+  for(*end = s + 1; **end != '"'; (*end)++)
+    if (**end == '\0')
+      Pl_Fatal_Error("Syntax error: string expected in: %s", save_str);
+
+  (*end)++;
+  **end = '\0';
+
+  return s;                     /* NB: returned string contains double-quotes */
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * PL_READ_INTEGER                                                         *
  *                                                                         *
@@ -459,7 +532,7 @@ Pl_Read_Integer(char *s, char **end)
 
   if (!isspace(**end))
     Pl_Fatal_Error("Syntax error: space expected after identifier in: %s",
-		save_str);
+                save_str);
   **end = '\0';
   return x;
 }
@@ -502,43 +575,43 @@ Generate_Regs(FILE *f, FILE *g)
   for (;;)
     {
       if (feof(f) || !fgets(str, sizeof(str), f))
-	Pl_Fatal_Error("Syntax error: end expected for @begin reg");
+        Pl_Fatal_Error("Syntax error: end expected for @begin reg");
 
       if (*str != '@')
-	{
-	  if (*str != '\n')
-	    fputs(str, g);
-	  continue;
-	}
+        {
+          if (*str != '\n')
+            fputs(str, g);
+          continue;
+        }
 
       strcpy(save_str, str);
       p1 = Read_Identifier(str + 1, 1, &p2);
 
       if (strcmp(p1, "end") == 0)
-	break;
+        break;
 
       if (strcmp(p1, "filler") == 0)
-	{
-	  p1 = Read_Identifier(p2 + 1, 0, &p2);
-	  if (!p1)
-	    {
-	      i = Pl_Read_Integer(p2, &p2);
-	      p1 = str;
-	      sprintf(p1, "%d", i);
-	    }
+        {
+          p1 = Read_Identifier(p2 + 1, 0, &p2);
+          if (!p1)
+            {
+              i = Pl_Read_Integer(p2, &p2);
+              p1 = str;
+              sprintf(p1, "%d", i);
+            }
 
-	  sprintf(str_base + strlen(str_base), "%s+", p1);
-	  continue;
-	}
+          sprintf(str_base + strlen(str_base), "%s+", p1);
+          continue;
+        }
 
       if (strcmp(p1, "reg") == 0)
-	{
-	  i = Pl_Read_Integer(p2 + 1, &p2);
-	  strcpy(reg[i][nb_reg[i]].type, Read_Identifier(p2 + 1, 1, &p2));
-	  strcpy(reg[i][nb_reg[i]].name, Read_Identifier(p2 + 1, 1, &p2));
-	  nb_reg[i]++;
-	  continue;
-	}
+        {
+          i = Pl_Read_Integer(p2 + 1, &p2);
+          strcpy(reg[i][nb_reg[i]].type, Read_Identifier(p2 + 1, 1, &p2));
+          strcpy(reg[i][nb_reg[i]].name, Read_Identifier(p2 + 1, 1, &p2));
+          nb_reg[i]++;
+          continue;
+        }
 
       Pl_Fatal_Error("Syntax error: incorrect @ declaration in: %s", save_str);
     }
@@ -572,27 +645,27 @@ Generate_Regs(FILE *f, FILE *g)
   for (i = 0; i < 10; i++)
     for (j = 0, total_nb_reg += nb_reg[i]; j < nb_reg[i]; j++)
       {
-	dp = &reg[i][j];
-	if (*p)
-	  {
-	    used_mach_reg[nb_of_used_mach_regs].mach_reg_name = *p;
-	    used_mach_reg[nb_of_used_mach_regs].pl_reg_name = dp->name;
-	    used_mach_reg[nb_of_used_mach_regs].type = dp->type;
-	    nb_of_used_mach_regs++;
-	    fprintf(g, "register %s\t\t%-3s asm (\"%s\");\n",
-		    dp->type, dp->name, *p);
-	    fprintf(fw_r, "#define MAP_REG_%-10s\t\"%s\"\n", dp->name,
-		    *p++);
-	    if (!*p)
-	      fprintf(g, "\n\n");
-	  }
-	else
-	  {
-	    fprintf(g, "#define %s\t\t\t(((%-8s *) pl_reg_bank)[%s%d])\n",
-		    dp->name, dp->type, str_base, nb_not_alloc);
-	    fprintf(fw_r, "#define MAP_OFFSET_%-6s\t((%s%d)*%d)\n",
-		    dp->name, str_base, nb_not_alloc++, (int) sizeof(long));
-	  }
+        dp = &reg[i][j];
+        if (*p)
+          {
+            used_mach_reg[nb_of_used_mach_regs].mach_reg_name = *p;
+            used_mach_reg[nb_of_used_mach_regs].pl_reg_name = dp->name;
+            used_mach_reg[nb_of_used_mach_regs].type = dp->type;
+            nb_of_used_mach_regs++;
+            fprintf(g, "register %s\t\t%-3s asm (\"%s\");\n",
+                    dp->type, dp->name, *p);
+            fprintf(fw_r, "#define MAP_REG_%-10s\t\"%s\"\n", dp->name,
+                    *p++);
+            if (!*p)
+              fprintf(g, "\n\n");
+          }
+        else
+          {
+            fprintf(g, "#define %s\t\t\t(((%-8s *) pl_reg_bank)[%s%d])\n",
+                    dp->name, dp->type, str_base, nb_not_alloc);
+            fprintf(fw_r, "#define MAP_OFFSET_%-6s\t((%s%d)*%d)\n",
+                    dp->name, str_base, nb_not_alloc++, (int) sizeof(PlLong));
+          }
       }
 
 
@@ -601,9 +674,9 @@ Generate_Regs(FILE *f, FILE *g)
   fprintf(g, "#define NB_OF_ALLOC_REGS    \t%d\n", total_nb_reg - nb_not_alloc);
   fprintf(g, "#define NB_OF_NOT_ALLOC_REGS\t%d\n", nb_not_alloc);
   fprintf(g, "#define REG_BANK_SIZE       \t(%sNB_OF_NOT_ALLOC_REGS)\n",
-	  str_base);
+          str_base);
   fprintf(g, "\n\n\n\n#define NB_OF_USED_MACHINE_REGS %d\n",
-	  nb_of_used_mach_regs);	/* same as NB_OF_ALLOC_REGS :-) ? */
+          nb_of_used_mach_regs);        /* same as NB_OF_ALLOC_REGS :-) ? */
 
 
 #ifndef NO_MACHINE_REG_FOR_REG_BANK /* pl_reg_bank restored anyway */
@@ -611,7 +684,7 @@ Generate_Regs(FILE *f, FILE *g)
 #else
   regs_to_save_for_signal = (nb_of_used_mach_regs >= 1);
 #endif
-    
+
   fprintf(g, "\n");
   fprintf(g, "#ifdef ENGINE_FILE\n\n");
 #ifdef NO_MACHINE_REG_FOR_REG_BANK
@@ -628,10 +701,10 @@ Generate_Regs(FILE *f, FILE *g)
   for (i = 0; i < 10; i++)
     for (j = 0; j < nb_reg[i]; j++)
       {
-	dp = &reg[i][j];
-	fprintf(g, "\"%s\"%s", dp->name,
-		k < total_nb_reg - 1 ? ", " : "};\n");
-	k++;
+        dp = &reg[i][j];
+        fprintf(g, "\"%s\"%s", dp->name,
+                k < total_nb_reg - 1 ? ", " : "};\n");
+        k++;
       }
 
   fprintf(g, "\n#else\n\n");
@@ -643,7 +716,7 @@ Generate_Regs(FILE *f, FILE *g)
 
   if (regs_to_save_for_signal)
     fprintf(g, "extern WamWord pl_buff_signal_reg[];\n\n");
-  
+
   fprintf(g, "extern char *pl_reg_tbl[];\n");
 
   fprintf(g, "\n#endif\n\n");
@@ -661,12 +734,12 @@ Generate_Regs(FILE *f, FILE *g)
   for (i = 0; i < 10; i++)
     for (j = 0; j < nb_reg[i]; j++)
       {
-	dp = &reg[i][j];
-	if (k < total_nb_reg - 1)
-	  fprintf(g, "((i)==%d) ? (WamWord) %-3s\t: \\\n\t\t\t\t ",
-		  k++, dp->name);
-	else
-	  fprintf(g, "           (WamWord) %s)\n", dp->name);
+        dp = &reg[i][j];
+        if (k < total_nb_reg - 1)
+          fprintf(g, "((i)==%d) ? (WamWord) %-3s\t: \\\n\t\t\t\t ",
+                  k++, dp->name);
+        else
+          fprintf(g, "           (WamWord) %s)\n", dp->name);
       }
 
 
@@ -677,10 +750,10 @@ Generate_Regs(FILE *f, FILE *g)
   for (i = 0; i < 10; i++)
     for (j = 0; j < nb_reg[i]; j++)
       {
-	dp = &reg[i][j];
-	fprintf(g, "    buff_save[%d] = (WamWord) %s; \\\n", k,
-		dp->name);
-	k++;
+        dp = &reg[i][j];
+        fprintf(g, "    buff_save[%d] = (WamWord) %s; \\\n", k,
+                dp->name);
+        k++;
       }
 
   fprintf(g, "  } while(0)\n");
@@ -693,10 +766,10 @@ Generate_Regs(FILE *f, FILE *g)
   for (i = 0; i < 10; i++)
     for (j = 0; j < nb_reg[i]; j++)
       {
-	dp = &reg[i][j];
-	fprintf(g, "    %-6s = (%-8s) buff_save[%d]; \\\n",
-		dp->name, dp->type, k);
-	k++;
+        dp = &reg[i][j];
+        fprintf(g, "    %-6s = (%-8s) buff_save[%d]; \\\n",
+                dp->name, dp->type, k);
+        k++;
       }
 
   fprintf(g, "  } while(0)\n");
@@ -778,49 +851,49 @@ Generate_Tags(FILE *f, FILE *g)
   int nb_tag = 0;
   int tag_size, tag_size_low, tag_size_high, value_size;
   int max_value = 0;
-  unsigned long tag_mask;
-  long min_integer, max_integer;
+  PlULong tag_mask;
+  PlLong min_integer, max_integer;
   int i;
 
 
   for (;;)
     {
       if (feof(f) || !fgets(str, sizeof(str), f))
-	Pl_Fatal_Error("Syntax error: end expected for @begin tag");
+        Pl_Fatal_Error("Syntax error: end expected for @begin tag");
 
       if (*str != '@')
-	{
-	  if (*str != '\n')
-	    fputs(str, g);
-	  continue;
-	}
+        {
+          if (*str != '\n')
+            fputs(str, g);
+          continue;
+        }
 
       strcpy(save_str, str);
       p1 = Read_Identifier(str + 1, 1, &p2);
 
       if (strcmp(p1, "end") == 0)
-	break;
+        break;
 
       if (strcmp(p1, "tag") == 0)
-	{
-	  strcpy(tag[nb_tag].name, Read_Identifier(p2 + 1, 1, &p2));
-	  p1 = Read_Identifier(p2 + 1, 1, &p2);
+        {
+          strcpy(tag[nb_tag].name, Read_Identifier(p2 + 1, 1, &p2));
+          p1 = Read_Identifier(p2 + 1, 1, &p2);
 
-	  if (strcmp(p1, "long_int") == 0)
-	    tag[nb_tag].type = LONG_INT;
-	  else if (strcmp(p1, "short_uns") == 0)
-	    tag[nb_tag].type = SHORT_UNS;
-	  else if (strcmp(p1, "address") == 0)
-	    tag[nb_tag].type = ADDRESS;
-	  else 
-	    Pl_Fatal_Error("Syntax error: wrong tag type in: %s", save_str);
+          if (strcmp(p1, "long_int") == 0)
+            tag[nb_tag].type = LONG_INT;
+          else if (strcmp(p1, "short_uns") == 0)
+            tag[nb_tag].type = SHORT_UNS;
+          else if (strcmp(p1, "address") == 0)
+            tag[nb_tag].type = ADDRESS;
+          else
+            Pl_Fatal_Error("Syntax error: wrong tag type in: %s", save_str);
 
-	  tag[nb_tag].value = Pl_Read_Integer(p2 + 1, &p2);
-	  if (tag[nb_tag].value > max_value)
-	    max_value = tag[nb_tag].value;
-	  nb_tag++;
-	  continue;
-	}
+          tag[nb_tag].value = Pl_Read_Integer(p2 + 1, &p2);
+          if (tag[nb_tag].value > max_value)
+            max_value = tag[nb_tag].value;
+          nb_tag++;
+          continue;
+        }
 
       Pl_Fatal_Error("Syntax error: incorrect @ declaration in: %s", save_str);
     }
@@ -828,7 +901,7 @@ Generate_Tags(FILE *f, FILE *g)
 
   fprintf(g, "\n\n   /*--- Begin Tag Generation ---*/\n\n");
 
-#define Mk_Tag_Mask(x) ((((unsigned long) (x) >> tag_size_low) << (value_size + tag_size_low)) | ((x) & ((1 << tag_size_low) - 1)))
+#define Mk_Tag_Mask(x) ((((PlULong) (x) >> tag_size_low) << (value_size + tag_size_low)) | ((x) & ((1 << tag_size_low) - 1)))
 
 #if 0
   tag_size = 4;
@@ -852,11 +925,11 @@ Generate_Tags(FILE *f, FILE *g)
 
   tag_mask = Mk_Tag_Mask((1 << tag_size) - 1);
 
-  max_integer = (1L << (WORD_SIZE - tag_size - 1)) - 1;
+  max_integer = (1 << (WORD_SIZE - tag_size - 1)) - 1;
   min_integer = -max_integer - 1;
 
-  fprintf(fg_c, "#define PL_MIN_INTEGER\t\t%ldL\n", min_integer);
-  fprintf(fg_c, "#define PL_MAX_INTEGER\t\t%ldL\n", max_integer);
+  fprintf(fg_c, "#define PL_MIN_INTEGER\t\t%" PL_FMT_d "\n",  min_integer);
+  fprintf(fg_c, "#define PL_MAX_INTEGER\t\t%" PL_FMT_d "\n",  max_integer);
   fputc('\n', fg_c);
 
 
@@ -864,23 +937,23 @@ Generate_Tags(FILE *f, FILE *g)
   fprintf(g, "#define TAG_SIZE_LOW \t\t%d\n", tag_size_low);
   fprintf(g, "#define TAG_SIZE_HIGH\t\t%d\n", tag_size_high);
   fprintf(g, "#define VALUE_SIZE   \t\t%d\n", value_size);
-  fprintf(g, "#define TAG_MASK     \t\t%#lxUL\n", tag_mask);
-  fprintf(g, "#define VALUE_MASK   \t\t%#lxUL\n", ~tag_mask);
+  fprintf(g, "#define TAG_MASK     \t\t(PlULong)%#" PL_FMT_x "\n", tag_mask);
+  fprintf(g, "#define VALUE_MASK   \t\t(PlULong)%#" PL_FMT_x "\n", ~tag_mask);
 
-  fprintf(g, "#define Tag_Mask_Of(w)\t\t((unsigned long) (w) & (TAG_MASK))\n");
+  fprintf(g, "#define Tag_Mask_Of(w)\t\t((PlLong) (w) & (TAG_MASK))\n");
 
-  fprintf(g, "#define Tag_From_Tag_Mask(w) \t(((unsigned long) (w) >> %d) | ((w) & %d))\n", value_size, (1 << tag_size_low) -1);
+  fprintf(g, "#define Tag_From_Tag_Mask(w) \t(((PlULong) (w) >> %d) | ((w) & %d))\n", value_size, (1 << tag_size_low) -1);
 
   if (tag_size_high > 0)
-    fprintf(g, "#define Tag_Of(w)     \t\t((((unsigned long) (w) >> %d) << %d) | ((w) & %d))\n",
-	    WORD_SIZE-tag_size_high, tag_size_low, (1 << tag_size_low) -1);
+    fprintf(g, "#define Tag_Of(w)     \t\t((((PlULong) (w) >> %d) << %d) | ((w) & %d))\n",
+            WORD_SIZE-tag_size_high, tag_size_low, (1 << tag_size_low) -1);
   else
     fprintf(g, "#define Tag_Of(w)     \t\tTag_Mask_Of(w)\n");
 
-  for (i = 0; i < nb_tag; i++)
-    fprintf(g, "#define TAG_%s_MASK\t\t%#lxUL\n", tag[i].name, 
-	    Mk_Tag_Mask(tag[i].value));
-
+  for (i = 0; i < nb_tag; i++) {
+    fprintf(g, "#define TAG_%s_MASK\t\t(PlULong)%#" PL_FMT_x "\n",
+            tag[i].name, Mk_Tag_Mask(tag[i].value));
+  }
   fprintf(g, "\n");
 
   fprintf(g, "#define NB_OF_TAGS       \t%d\n", nb_tag);
@@ -893,21 +966,21 @@ Generate_Tags(FILE *f, FILE *g)
 
   fprintf(g, "\n");
   fprintf(g, "\t/* General Tag/UnTag macros */\n\n");
-  fprintf(g, "#define Tag_Long_Int(tm, v)  \t((((unsigned long) ((v) << %d)) >> %d) | (tm))\n",
-	  tag_size, tag_size_high);
+  fprintf(g, "#define Tag_Long_Int(tm, v)  \t((((PlLong) ((v) << %d)) >> %d) | (tm))\n",
+          tag_size, tag_size_high);
 
 
-  fprintf(g, "#define Tag_Short_Uns(tm, v)\t(((unsigned long) (v) << %d) + (tm))\n", tag_size_low);
+  fprintf(g, "#define Tag_Short_Uns(tm, v)\t(((PlLong) (v) << %d) + (tm))\n", tag_size_low);
 
 /* For Tag_Address the + (tm) is better than | (tm) since the C compiler can
  * optimizes things like Tag_Address(2, H + 1) with only 1 instruction (+ 6)
- * instead of 2 (1 for + 4, 1 for | TAG_STC_MASK) 
+ * instead of 2 (1 for + 4, 1 for | TAG_STC_MASK)
  */
-  fprintf(g, "#define Tag_Address(tm, v)  \t((unsigned long) (v) + (tm))\n");
+  fprintf(g, "#define Tag_Address(tm, v)  \t((PlLong) (v) + (tm))\n");
 
   fprintf(g, "\n");
-  fprintf(g, "#define UnTag_Long_Int(w)    \t((long) ((w) << %d) >> %d)\n",
-	  tag_size_high, tag_size);
+  fprintf(g, "#define UnTag_Long_Int(w)    \t((PlLong) ((w) << %d) >> %d)\n",
+          tag_size_high, tag_size);
 
   fprintf(g, "#define UnTag_Short_Uns(w)\tUnTag_Long_Int(w)\n");
 
@@ -923,27 +996,27 @@ Generate_Tags(FILE *f, FILE *g)
     {
       fprintf(g, "#define Tag_%s(v)  \t\t", tag[i].name);
       switch(tag[i].type)
-	{
-	case LONG_INT:
-	  if (tag[i].value == 0)
-	    fprintf(g, "(((unsigned long) (v) << %d) & VALUE_MASK)\n", 
-		    tag_size_low);
-	  /* testing if high bits are 1 should suffice below - TO DO */
-	  else if (tag[i].value == (1 << tag_size) - 1)
-	    fprintf(g, "(((unsigned long) (v) << %d) | TAG_MASK)\n", 
-		    tag_size_low);
-	  else
-	    fprintf(g, "Tag_Long_Int(TAG_%s_MASK, v)\n", tag[i].name);
-	  break;
+        {
+        case LONG_INT:
+          if (tag[i].value == 0)
+            fprintf(g, "(((PlULong) (v) << %d) & VALUE_MASK)\n",
+                    tag_size_low);
+          /* testing if high bits are 1 should suffice below - TO DO */
+          else if (tag[i].value == (1 << tag_size) - 1)
+            fprintf(g, "(((PlULong) (v) << %d) | TAG_MASK)\n",
+                    tag_size_low);
+          else
+            fprintf(g, "Tag_Long_Int(TAG_%s_MASK, v)\n", tag[i].name);
+          break;
 
-	case SHORT_UNS:
-	  fprintf(g, "Tag_Short_Uns(TAG_%s_MASK, v)\n", tag[i].name);
-	  break;
+        case SHORT_UNS:
+          fprintf(g, "Tag_Short_Uns(TAG_%s_MASK, v)\n", tag[i].name);
+          break;
 
-	case ADDRESS:
-	  fprintf(g, "Tag_Address(TAG_%s_MASK, v)\n", tag[i].name);
-	  break;
-	}
+        case ADDRESS:
+          fprintf(g, "Tag_Address(TAG_%s_MASK, v)\n", tag[i].name);
+          break;
+        }
     }
 
   fprintf(g, "\n");
@@ -952,25 +1025,25 @@ Generate_Tags(FILE *f, FILE *g)
     {
       fprintf(g, "#define UnTag_%s(w)  \t\t", tag[i].name);
       switch(tag[i].type)
-	{
-	case LONG_INT:
-	  fprintf(g, "UnTag_Long_Int(w)\n");
-	  break;
+        {
+        case LONG_INT:
+          fprintf(g, "UnTag_Long_Int(w)\n");
+          break;
 
-	case SHORT_UNS:
-	  if (tag[i].value <= 3)
-	    fprintf(g, "((unsigned long) (w) >> %d)\n", tag_size_low);
-	  else
-	    fprintf(g, "UnTag_Short_Uns(w)\n");
-	  break;
+        case SHORT_UNS:
+          if (tag[i].value <= 3)
+            fprintf(g, "((PlULong) (w) >> %d)\n", tag_size_low);
+          else
+            fprintf(g, "UnTag_Short_Uns(w)\n");
+          break;
 
-	case ADDRESS:
-	  if (tag[i].value == 0)
-	    fprintf(g, "((WamWord *) (w))\n");
-	  else
-	    fprintf(g, "UnTag_Address(w)\n");
-	  break;
-	}
+        case ADDRESS:
+          if (tag[i].value == 0)
+            fprintf(g, "((WamWord *) (w))\n");
+          else
+            fprintf(g, "UnTag_Address(w)\n");
+          break;
+        }
     }
 
   fprintf(g, "\n");
@@ -993,7 +1066,7 @@ Generate_Tags(FILE *f, FILE *g)
   fprintf(g, "  char *name;\n");
   fprintf(g, "  TypTag type;\n");
   fprintf(g, "  int value;\n");
-  fprintf(g, "  long tag_mask;\n");
+  fprintf(g, "  PlLong tag_mask;\n");
   fprintf(g, "}InfTag;\n\n\n");
 
 
@@ -1002,12 +1075,11 @@ Generate_Tags(FILE *f, FILE *g)
 
   for (i = 0; i < nb_tag; i++)
     {
-      fprintf(g, "  { \"%s\", %s, %d, %#lxUL }%s", tag[i].name,
-	      (tag[i].type == LONG_INT) ? "LONG_INT" :
-	      (tag[i].type == SHORT_UNS) ? "SHORT_UNS" : "ADDRESS",
-	      tag[i].value,
-	      Mk_Tag_Mask(tag[i].value),
-	      (i < nb_tag - 1) ? ",\n" : "\n};\n");
+      fprintf(g, "  { \"%s\", %s, %d, %" PL_FMT_x "}%s", tag[i].name,
+              (tag[i].type == LONG_INT) ? "LONG_INT" :
+              (tag[i].type == SHORT_UNS) ? "SHORT_UNS" : "ADDRESS",
+              tag[i].value, Mk_Tag_Mask(tag[i].value),
+              (i < nb_tag - 1) ? ",\n" : "\n};\n");
     }
 
   fprintf(g, "\n#else\n\n");
@@ -1025,7 +1097,7 @@ Generate_Tags(FILE *f, FILE *g)
  * GENERATE_STACKS                                                         *
  *                                                                         *
  * stack description:                                                      *
- *    @stack name default_size stack_top_macro                             *
+ *    @stack name "description" default_size stack_top_macro               *
  *-------------------------------------------------------------------------*/
 void
 Generate_Stacks(FILE *f, FILE *g)
@@ -1039,32 +1111,33 @@ Generate_Stacks(FILE *f, FILE *g)
   for (;;)
     {
       if (feof(f) || !fgets(str, sizeof(str), f))
-	Pl_Fatal_Error("Syntax error: end expected for @begin stack");
+        Pl_Fatal_Error("Syntax error: end expected for @begin stack");
 
       if (*str != '@')
-	{
-	  if (*str != '\n')
-	    fputs(str, g);
-	  continue;
-	}
+        {
+          if (*str != '\n')
+            fputs(str, g);
+          continue;
+        }
 
       strcpy(save_str, str);
       p1 = Read_Identifier(str + 1, 1, &p2);
 
       if (strcmp(p1, "end") == 0)
-	break;
+        break;
 
       if (strcmp(p1, "stack") == 0)
-	{
-	  strcpy(stack[nb_stack].name, Read_Identifier(p2 + 1, 1, &p2));
-	  i = Pl_Read_Integer(p2 + 1, &p2);
-	  stack[nb_stack].def_size = i * 1024 / sizeof(long);
+        {
+          strcpy(stack[nb_stack].name, Read_Identifier(p2 + 1, 1, &p2));
+          strcpy(stack[nb_stack].desc, Read_String(p2 + 1, &p2));
+          i = Pl_Read_Integer(p2 + 1, &p2);
+          stack[nb_stack].def_size = i * 1024 / sizeof(PlLong);
 
-	  strcpy(stack[nb_stack].top_macro,
-		 Read_Identifier(p2 + 1, 1, &p2));
-	  nb_stack++;
-	  continue;
-	}
+          strcpy(stack[nb_stack].top_macro,
+                 Read_Identifier(p2 + 1, 1, &p2));
+          nb_stack++;
+          continue;
+        }
 
       Pl_Fatal_Error("Syntax error: incorrect @ declaration in: %s", save_str);
     }
@@ -1072,7 +1145,9 @@ Generate_Stacks(FILE *f, FILE *g)
 
   fprintf(g, "\n\n   /*--- Begin Stack Generation ---*/\n\n");
 
-  fprintf(g, "#define NB_OF_STACKS \t\t%d\n\n", nb_stack);
+  fprintf(g, "#include \"wam_stacks.h\"\n");
+
+  fprintf(fw_s, "#define NB_OF_STACKS \t\t%d\n\n", nb_stack);
 
 
   for (i = 0; i < nb_stack; i++)
@@ -1080,61 +1155,61 @@ Generate_Stacks(FILE *f, FILE *g)
       strcpy(str, stack[i].name);
       *str = toupper(*str);
 
-      fprintf(g, "#define %s_Stack       \t(pl_stk_tbl[%d].stack)\n", str, i);
-      fprintf(g, "#define %s_Size        \t(pl_stk_tbl[%d].size)\n", str, i);
-      fprintf(g, "#define %s_Offset(adr) \t((WamWord *)(adr) - %s_Stack)\n",
-	      str, str);
-      fprintf(g, "#define %s_Used_Size   \t%s_Offset(%s)\n\n", str, str,
-	      stack[i].top_macro);
+      fprintf(fw_s, "#define %s_Stack       \t(pl_stk_tbl[%d].stack)\n", str, i);
+      fprintf(fw_s, "#define %s_Size        \t(pl_stk_tbl[%d].size)\n", str, i);
+      fprintf(fw_s, "#define %s_Offset(adr) \t((WamWord *)(adr) - %s_Stack)\n",
+              str, str);
+      fprintf(fw_s, "#define %s_Used_Size   \t%s_Offset(%s)\n\n", str, str,
+              stack[i].top_macro);
     }
 
-  fprintf(g, "\n#define Stack_Top(s)       \t(");
+  fprintf(fw_s, "\n#define Stack_Top(s)       \t(");
   for (i = 0; i < nb_stack - 1; i++)
-    fprintf(g, "((s) == %d) ? %s : ", i, stack[i].top_macro);
+    fprintf(fw_s, "((s) == %d) ? %s : ", i, stack[i].top_macro);
 
-  fprintf(g, "%s)\n", stack[nb_stack - 1].top_macro);
+  fprintf(fw_s, "%s)\n", stack[nb_stack - 1].top_macro);
 
 
-  fprintf(g, "\ntypedef struct\n");
-  fprintf(g, "{\n");
-  fprintf(g, "  char *name;\n");
-  fprintf(g, "  char *env_var_name;\n");
-  fprintf(g, "  long *p_def_size;\n");
-  fprintf(g, "  int default_size; \t/* in WamWords */\n");
-  fprintf(g, "  int size;         \t/* in WamWords */\n");
-  fprintf(g, "  WamWord *stack;\n");
-  fprintf(g, "}InfStack;\n\n\n");
+  fprintf(fw_s, "\ntypedef struct\n");
+  fprintf(fw_s, "{\n");
+  fprintf(fw_s, "  char *name;\n");
+  fprintf(fw_s, "  char *desc;\n");
+  fprintf(fw_s, "  char *env_var_name;\n");
+  fprintf(fw_s, "  PlLong *p_def_size;\t/* used for fixed_sizes */\n");
+  fprintf(fw_s, "  int default_size; \t/* in WamWords */\n");
+  fprintf(fw_s, "  int size;         \t/* in WamWords */\n");
+  fprintf(fw_s, "  WamWord *stack;\n");
+  fprintf(fw_s, "}InfStack;\n\n\n");
 
 
-  fprintf(g, "#ifdef ENGINE_FILE\n\n");
+  fprintf(fw_s, "#ifdef ENGINE_FILE\n\n");
 
-  fprintf(g,
-	  "    /* these variables can be overwritten by top_comp.c (see stack size file) */\n");
+  fprintf(fw_s, "    /* these variables can be overwritten by top_comp.c (see stack size file) */\n");
 
   for (i = 0; i < nb_stack; i++)
-    fprintf(g, "long pl_def_%s_size;\n", stack[i].name);
-  fprintf(g, "long pl_fixed_sizes;\n\n");
+    fprintf(fw_s, "PlLong pl_def_%s_size;\n", stack[i].name);
+  fprintf(fw_s, "PlLong pl_fixed_sizes;\n\n");
 
-  fprintf(g, "InfStack pl_stk_tbl[] =\n{\n");
+  fprintf(fw_s, "InfStack pl_stk_tbl[] =\n{\n");
 
   for (i = 0; i < nb_stack; i++)
     {
       strcpy(str, stack[i].name);
       for (p1 = str; *p1; p1++)
-	*p1 = toupper(*p1);
+        *p1 = toupper(*p1);
 
-      fprintf(g, " { \"%s\", \"%sSZ\", &pl_def_%s_size, %d, 0, NULL }%s",
-	      stack[i].name, str, stack[i].name, stack[i].def_size, 
+      fprintf(fw_s, " { \"%s\", %s, \"%sSZ\", &pl_def_%s_size, %d, 0, NULL }%s",
+              stack[i].name, stack[i].desc, str, stack[i].name, stack[i].def_size,
               (i < nb_stack - 1) ? ",\n" : "\n};\n");
     }
 
 
-  fprintf(g, "\n#else\n\n");
+  fprintf(fw_s, "\n#else\n\n");
   for (i = 0; i < nb_stack; i++)
-    fprintf(g, "extern long pl_def_%s_size;\n", stack[i].name);
-  fprintf(g, "extern long pl_fixed_sizes;\n\n\n");
-  fprintf(g, "extern InfStack pl_stk_tbl[];\n");
-  fprintf(g, "\n#endif\n");
+    fprintf(fw_s, "extern PlLong pl_def_%s_size;\n", stack[i].name);
+  fprintf(fw_s, "extern PlLong pl_fixed_sizes;\n\n\n");
+  fprintf(fw_s, "extern InfStack pl_stk_tbl[];\n");
+  fprintf(fw_s, "\n#endif\n");
 
 
   fprintf(g, "\n\n   /*--- End Stack Generation ---*/\n\n");
@@ -1152,7 +1227,7 @@ Detect_Sigaction(void)
 {
   if (system("make try_sigaction 2>/dev/null 1>&2"))
     return 0;
-  
+
   if (system("./try_sigaction") != 0)
     return 0;
 
