@@ -6,40 +6,68 @@
  * Descr.: predicate table management - header file                        *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: pred.h,v 1.15 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: pred.h,v 1.20 2011/06/10 14:48:48 diaz Exp $ */
+
 
 /*---------------------------------*
  * Constants                       *
  *---------------------------------*/
 
+					  /* if modified -> modif wam2ma.c */
+#define MASK_PRED_NATIVE_CODE       1	  /* codep is set, except for control constructs */
+#define MASK_PRED_DYNAMIC           2	  /* dynamic or static */
+#define MASK_PRED_PUBLIC            4	  /* public or private */
+#define MASK_PRED_BUILTIN           8	  /* built-in (procedure provided by the system) */
+#define MASK_PRED_BUILTIN_FD        16	  /* FD built-in pred  (==> MASK_PRED_BUILTIN) */
+#define MASK_PRED_CONTROL_CONSTRUCT 32	  /* control_construct (==> MASK_PRED_BUILTIN) */
+#define MASK_PRED_MULTIFILE         64	  /* multifile or monofile */
+#define MASK_PRED_EXPORTED          128	  /* exported by module not yet used - for future */
+
+
+
+
+
 /*---------------------------------*
  * Type Definitions                *
  *---------------------------------*/
 
 typedef struct			/* Predicate information          */
 {				/* ------------------------------ */
-  long f_n;			/* key is <functor_atom,arity>    */
+  PlLong f_n;			/* key is <functor_atom,arity>    */
   int pl_file;			/* atom pl file of its definiton  */
   int pl_line;			/* pl file line of its definition */
   int prop;			/* predicate props (cf BipsPl)    */
-  long *codep;			/* compiled code                  */
-  long *dyn;			/* dynamic info (cf BipsPl)       */
+  PlLong *codep;		/* compiled code                  */
+  PlLong *dyn;			/* dynamic info (cf BipsPl)       */
 }
 PredInf;
 
@@ -70,7 +98,7 @@ extern char *pl_pred_tbl;
 void Pl_Init_Pred(void);
 
 PredInf * FC Pl_Create_Pred(int func, int arity, int pl_file, int pl_line,
-		     int prop, long *codep);
+			    int prop, PlLong *codep);
 
 PredInf * FC Pl_Lookup_Pred(int func, int arity);
 
