@@ -6,23 +6,36 @@
  * Descr.: line editor                                                     *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: linedit.c,v 1.17 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: linedit.c,v 1.21 2011/04/08 16:16:16 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -54,19 +67,19 @@
 
 
 #if 1
-#define TREAT_BUFFERED_CHARS	/* treat buffered chars at start (X paste) */
+#define TREAT_BUFFERED_CHARS    /* treat buffered chars at start (X paste) */
 #endif
 
 #if 1
-#define NO_DUP_IN_HISTORY	/* do not put in history line == the last */
+#define NO_DUP_IN_HISTORY       /* do not put in history line == the last */
 #endif
 
 #if 1
-#define IGNORE_QUOTED_PART	/* ingore quoted item in bracket matching */
+#define IGNORE_QUOTED_PART      /* ingore quoted item in bracket matching */
 #endif
 
 #if 0
-#define NO_USE_SELECT		/* no use select(2) for temporisation */
+#define NO_USE_SELECT           /* no use select(2) for temporisation */
 #endif
 
 
@@ -91,9 +104,9 @@
 #define CLOSE_BRACKET              ")]}"
 
 #ifdef NO_USE_SELECT
-#define BRACKET_TIMMING            300000	/* in microseconds */
+#define BRACKET_TIMMING            300000       /* in microseconds */
 #else
-#define BRACKET_TIMMING            900000	/* in microseconds */
+#define BRACKET_TIMMING            900000       /* in microseconds */
 #endif
 
 
@@ -138,11 +151,10 @@ static char *global_end;
 
 static int prompt_length;
 
-static long ctrl_c_ret_val;
+static PlLong ctrl_c_ret_val;
 
 static char clipboard[4096] = "";
 
-
 static HistCell hist_tbl[MAX_HISTORY_LINES];
 static int hist_start = 0;
 static int hist_end = 0;
@@ -168,6 +180,8 @@ static char *Skip(char *from, char *limit, int res_sep_cmp, int direction);
 
 static int Is_A_Separator(char c);
 
+static int Search_Bracket(char *brackets, char c);
+
 static int Tab_To_Spaces(int p);
 
 
@@ -181,7 +195,7 @@ static int History_Get_Line(char *str, int hist_no);
 
 
 static char *Completion_Do_Match(char *prefix, int prefix_length,
-				 int *rest_length);
+                                 int *rest_length);
 
 static void Completion_Print_All(void);
 
@@ -232,9 +246,9 @@ Pl_LE_Gets(char *str)
 
   if ((str = Pl_LE_FGets(str, big_size, NULL, 0)) != NULL)
     {
-      l = strlen(str) - 1;	/* for gets remove last \n */
+      l = strlen(str) - 1;      /* for gets remove last \n */
       if (l >= 0 && str[l] == '\n')
-	str[l] = '\0';
+        str[l] = '\0';
     }
 
   return str;
@@ -255,7 +269,7 @@ Pl_LE_FGets(char *str, int size, char *prompt, int display_prompt)
   char *mark = NULL;
   char *p, *q, *start, *stop;
   char w;
-  int c, n;
+  int c, n, n1;
   int last_was_eof = 0;
   int h_no = Hist_End_Entry();
   int rest_length;
@@ -264,27 +278,27 @@ Pl_LE_FGets(char *str, int size, char *prompt, int display_prompt)
 
   Pl_LE_Initialize();
 
-  size--;			/* -1 for '\0' */
+  size--;                       /* -1 for '\0' */
 
   prompt_length = (prompt && display_prompt) ? strlen(prompt) : 0;
 
   Pl_LE_Open_Terminal();
   global_str = str;
 
-#ifdef TREAT_BUFFERED_CHARS	/* treat buffered lines (for paste) */
+#ifdef TREAT_BUFFERED_CHARS     /* treat buffered lines (for paste) */
   while (KBD_IS_NOT_EMPTY)
     {
       if (end - str >= size || ((c = Pl_LE_Get_Char()) == '\n') ||
-	  c == '\r')
-	{
-	  RE_DISPLAY_LINE;
-	  goto return_is_read;
-	}
-      if (c == '\t')		/* '\t' on output would cause trouble */
-	for(n = Tab_To_Spaces(end - str); n; n--)
-	  *end++ = ' ';
+          c == '\r')
+        {
+          RE_DISPLAY_LINE;
+          goto return_is_read;
+        }
+      if (c == '\t')            /* '\t' on output would cause trouble */
+        for(n = Tab_To_Spaces(end - str); n; n--)
+          *end++ = ' ';
       else
-	*end++ = c;
+        *end++ = c;
     }
 
   if (end != str)
@@ -304,504 +318,497 @@ Pl_LE_FGets(char *str, int size, char *prompt, int display_prompt)
 
       c = Pl_LE_Get_Char();
     one_char:
-      *end = ' ';		/* to allow for separator test */
+      *end = ' ';               /* to allow for separator test */
 
       if (Pl_LE_Is_Interrupt_Key(c))
-	{			/* save global vars for reentrancy */
-	  int save_prompt_length = prompt_length;
+        {                       /* save global vars for reentrancy */
+          int save_prompt_length = prompt_length;
 
-	  FORWD(end - pos, pos); /* go to EOL to avoid multi-line */
-				/* truncation on the output */
-	  Pl_LE_Close_Terminal();
-	  c = *end;
-	  *end = '\0';		/* to allow the handler to use/test str */
-	  if ((ctrl_c_ret_val = Pl_Emit_Ctrl_C()) != 0)
-	    return (char *) -2;
+          FORWD(end - pos, pos); /* go to EOL to avoid multi-line */
+          /* truncation on the output */
+          Pl_LE_Close_Terminal();
+          c = *end;
+          *end = '\0';          /* to allow the handler to use/test str */
+          if ((ctrl_c_ret_val = Pl_Emit_Ctrl_C()) != 0)
+            return (char *) -2;
 
-	  Pl_LE_Open_Terminal();
-	  *end = c;
+          Pl_LE_Open_Terminal();
+          *end = c;
 
-	  global_str = str;	/* restore global vars for reentrancy */
-	  prompt_length = save_prompt_length;
+          global_str = str;     /* restore global vars for reentrancy */
+          prompt_length = save_prompt_length;
 
-	re_display_line:	/* display prompt + full line */
-	  RE_DISPLAY_LINE;
-	  continue;
-	}
+        re_display_line:        /* display prompt + full line */
+          RE_DISPLAY_LINE;
+          continue;
+        }
 
-      if (KEY_IS_EOF(c))	/* to avoid EOF when typing too much ^D */
-	{
-	  if (end == str)
-	    {
-	      if (c == KEY_CTRL('D') && last_was_eof)
-		goto error;
-	      else
-		{
-		  str = NULL;
-		  goto finish;
-		}
-	    }
-	  last_was_eof = (c == KEY_CTRL('D'));
-	}
+      if (KEY_IS_EOF(c))        /* to avoid EOF when typing too much ^D */
+        {
+          if (end == str)
+            {
+              if (c == KEY_CTRL('D') && last_was_eof)
+                goto error;
+              else
+                {
+                  str = NULL;
+                  goto finish;
+                }
+            }
+          last_was_eof = (c == KEY_CTRL('D'));
+        }
       else
-	last_was_eof = 0;
+        last_was_eof = 0;
 
       if (c != '\t')
-	tab_count = 0;
+        tab_count = 0;
 
       switch (c)
-	{
-	case KEY_CTRL('A'):	/* go to begin of line */
-	case KEY_EXT_HOME:
-	  BACKD(pos - str);
-	  pos = str;
-	  continue;
-
-
-	case KEY_CTRL('E'):	/* go to end of line */
-	case KEY_EXT_END:
-	  FORWD(end - pos, pos);
-	  pos = end;
-	  continue;
-
-
-	case KEY_CTRL('B'):	/* go to 1 char backward */
-	case KEY_EXT_LEFT:
-	  if (pos == str)
-	    goto error;
-	  BACKD(1);
-	  pos--;
-	  continue;
-
-
-	case KEY_CTRL('F'):	/* go to 1 char forward */
-	case KEY_EXT_RIGHT:
-	  if (pos == end)
-	    goto error;
-	  FORWD(1, pos);
-	  pos++;
-	  continue;
-
-
-	case KEY_BACKSPACE:	/* erase previous char */
-	case KEY_DELETE:
-	  if (pos == str)
-	    goto error;
-	del_last:
-	  for (p = pos; p < end; p++)
-	    p[-1] = *p;
-	  BACKD(1);
-	  pos--;
-	  end--;
-	  DISPL(end - pos, pos);
-	  ERASE(1);
-	  BACKD(end - pos);
-	  continue;
-
-
-	case KEY_CTRL('D'):	/* erase current char */
-	case KEY_EXT_DELETE:
-	  if (pos == end)
-	    goto error;
-	  /* simply equivalent to ^F + BACKSPACE */
-	  FORWD(1, pos);
-	  pos++;
-	  goto del_last;
-
-
-	case KEY_CTRL('U'):	/* erase begin of line */
-	case KEY_CTRL_EXT_HOME:
-	  q = clipboard;
-	  p = str;
-	  while (p < pos)	/* add deleted part to clipboard */
-	    *q++ = *p++;
-	  *q = '\0';
-	  n = pos - str;
-	  for (p = pos; p < end; p++)
-	    p[-n] = *p;
-
-	  pos = str;
-	  end -= n;
-	  BACKD(n);
-	  DISPL(end - pos, pos);
-	  ERASE(n);
-	  BACKD(end - pos);
-	  continue;
-
-
-	case KEY_CTRL('K'):	/* erase end of line */
-	case KEY_CTRL_EXT_END:
-	  q = clipboard;
-	  p = pos;
-	  while (p < end)	/* add deleted part to clipboard */
-	    *q++ = *p++;
-	  *q = '\0';
-	  ERASE(end - pos);
-	  end = pos;
-	  continue;
-
-
-	case KEY_CTRL('Y'):	/* paste from clipboard */
-	  for (p = clipboard; *p; p++)
-	    if (!New_Char(*p, str, size, &pos, &end))
-	      goto error;
-	  continue;
-
-
-	case KEY_CTRL(' '):	/* mark begin selection */
-	  mark = pos;
-	  continue;
-
-
-	case KEY_ESC('W'):	/* copy (from mark) to clipboard */
-	case KEY_CTRL('W'):	/* cut (from mark) to clipboard */
-	  if (mark == NULL)
-	    goto error;
-
-	  if (mark < pos)
-	    {
-	      start = mark;
-	      stop = pos;
-	    }
-	  else
-	    {
-	      start = pos;
-	      stop = mark;
-	    }
-	  q = clipboard;
-	  p = start;
-	  while (p < stop)
-	    *q++ = *p++;
-	  *q = '\0';
-	  if (c == KEY_ESC('W'))
-	    continue;
-
-	  n = stop - start;
-	  for (p = stop; p < end; p++)
-	    p[-n] = *p;
-
-	  if (mark < pos)
-	    BACKD(n);
-	  pos = start;
-	  end -= n;
-	  DISPL(end - pos, pos);
-	  ERASE(n);
-	  BACKD(end - pos);
-	  continue;
-
-
-	case KEY_ESC('B'):	/* go to previous word */
-	case KEY_CTRL_EXT_LEFT:
-	  p = (pos == str) ? pos : pos - 1; /* to avoid start of a word */
-	  p = Skip(p, str, 1, -1);	/* skip separators */
-	  p = Skip(p, str, 0, -1);	/* skip non separators */
-	  p = Skip(p, end, 1, +1);	/* skip separators */
-	  BACKD(pos - p);
-	  pos = p;
-	  continue;
-
-
-	case KEY_ESC('F'):	/* go to next word */
-	case KEY_CTRL_EXT_RIGHT:
-	  p = pos;
-	  p = Skip(p, end, 0, +1);	/* skip non separators */
-	  p = Skip(p, end, 1, +1);	/* skip separators */
-	  FORWD(p - pos, pos);
-	  pos = p;
-	  continue;
-
-
-	case KEY_ESC('C'):	/* capitalize word */
-	  p = pos;
-	  p = Skip(p, end, 1, +1);	/* skip separators */
-	  if (islower(*p))
-	    *p = *p - 'a' + 'A';
-	  p = Skip(p, end, 0, +1);	/* skip non separators */
-	  DISPL(p - pos, pos);
-	  pos = p;
-	  continue;
-
-
-	case KEY_ESC('L'):	/* convert to lower case */
-	  p = pos;
-	  p = Skip(p, end, 1, +1);	/* skip separators */
-	  for (; p < end && !Is_A_Separator(*p); p++)
-	    *p = tolower(*p);
-	  DISPL(p - pos, pos);
-	  pos = p;
-	  continue;
-
-
-	case KEY_ESC('U'):	/* convert to upper case */
-	  p = pos;
-	  p = Skip(p, end, 1, +1);	/* skip separators */
-	  for (; p < end && !Is_A_Separator(*p); p++)
-	    *p = toupper(*p);
-	  DISPL(p - pos, pos);
-	  pos = p;
-	  continue;
-
-
-	case '\t':		/* TAB: completion */
-	  if (tab_count != 0)	/* already a TAB */
-	    {
-	      if (++tab_count > NB_TAB_BEFORE_LIST)
-		{
-		  NewLn();
-		  Completion_Print_All();
-		  goto re_display_line;
-		}
-	      goto error;
-	    }
-	  p = (pos == str) ? pos : pos - 1; /* to avoid start of a word */
-	  p = Skip(p, str, 0, -1);	/* skip non separators */
-	  p = Skip(p, end, 1, +1);	/* skip separators */
-	  w = *pos;		/* prefix from p to pos */
-	  *pos = '\0';
-	  p = Completion_Do_Match(p, pos - p, &rest_length);
-	  *pos = w;
-	  if (p == NULL)
-	    goto error;
-
-	  while (rest_length--)
-	    if (!New_Char(*p++, str, size, &pos, &end))
-	      goto error;
-
-	  if (comp_first_match != comp_last_match)
-	    {
-	      tab_count = 1;
-	      goto error;	/* for the beep */
-	    }
-	  tab_count = 0;
-	  continue;
-
-
-	case KEY_ESC('\t'):	/* transform a tab to spaces */
-	  for (n = Tab_To_Spaces(pos - str); n; n--)
-	    if (!New_Char(' ', str, size, &pos, &end))
-	      goto error;
-	  continue;
-
-
-	case KEY_CTRL('V'):	/* switch insert mode (on/off) */
-	case KEY_EXT_INSERT:
-	  ins_mode = 1 - ins_mode;
-	  INS_MODE(ins_mode);
-	  continue;
-
-
-	case KEY_CTRL('T'):	/* swap last and current char */
-	  if (pos == str || pos == end)
-	    goto error;
-	  w = pos[0];
-	  pos[0] = pos[-1];
-	  pos[-1] = w;
-	  BACKD(1);
-	  DISPL(2, pos - 1);
-	  pos++;
-	  continue;
-
-	case '\n':
-	case '\r':
-	return_is_read:
-  	  FORWD(end - pos, pos); /* go to EOL to avoid multi-line */
-				/* truncation on the output */
-	  *end = '\0';
-	  History_Add_Line(str, end - str);
-	  if (end - str < size)	/* '\n' can be added */
-	    *end++ = '\n';
-	  *end = '\0';
-	  goto finish;
-
-
-	case KEY_CTRL('P'):	/* history: recall previous line */
-	case KEY_EXT_UP:
-	  if (Hist_Is_Empty() || h_no == Hist_Start_Entry())
-	    goto error;
-	  *end = '\0';
-	  History_Update_Line(str, end - str, h_no);
-	  Hist_Dec(h_no);
-	write_hist_line:
-	  p = end;
-	  end = str + History_Get_Line(str, h_no);
-	  BACKD(pos - str);
-	  DISPL(end - str, str);
-	  if (end < p)
-	    ERASE(p - end);
-	  pos = end;
-	  continue;
-
-
-	case KEY_CTRL('N'):	/* history: recall next line */
-	case KEY_EXT_DOWN:
-	  if (Hist_Is_Empty() || h_no == Hist_End_Entry())
-	    goto error;
-	  *end = '\0';
-	  History_Update_Line(str, end - str, h_no);
-	  Hist_Inc(h_no);
-	  goto write_hist_line;
-
-
-	case KEY_ESC('P'):	/* history: recall previous matching line */
-	  if (Hist_Is_Empty() || pos == str)
-	    goto error;
-	  *end = '\0';
-	  History_Update_Line(str, end - str, h_no);
-	try_previous:
-	  if (h_no == Hist_Start_Entry())
-	    goto error;
-	  Hist_Dec(h_no);
-	  if (hist_tbl[h_no].line == NULL ||
-	      strncmp(str, hist_tbl[h_no].line, pos - str) != 0)
-	    goto try_previous;
-	write_hist_match_line:
-	  p = end;
-	  end = str + History_Get_Line(str, h_no);
-	  DISPL(end - pos, pos);
-	  if (end < p)
-	    ERASE(p - end);
-	  BACKD(end - pos);
-	  continue;
-
-
-	case KEY_ESC('N'):	/* history: recall next matching line */
-	  if (Hist_Is_Empty() || pos == str)
-	    goto error;
-	  *end = '\0';
-	  History_Update_Line(str, end - str, h_no);
-	try_next:
-	  if (Hist_Is_Empty() || h_no == Hist_End_Entry())
-	    goto error;
-	  Hist_Inc(h_no);
-	  if (hist_tbl[h_no].line == NULL ||
-	      strncmp(str, hist_tbl[h_no].line, pos - str) != 0)
-	    goto try_next;
-	  goto write_hist_match_line;
-
-
-	case KEY_ESC('<'):	/* history: recall first line */
-	case KEY_EXT_PAGE_UP:
-	  if (Hist_Is_Empty() || h_no == Hist_Start_Entry())
-	    goto error;
-	  *end = '\0';
-	  History_Update_Line(str, end - str, h_no);
-	  Hist_First(h_no);
-	  goto write_hist_line;
-
-
-	case KEY_ESC('>'):	/* history: recall last line */
-	case KEY_EXT_PAGE_DOWN:
-	  if (Hist_Is_Empty() || h_no == Hist_End_Entry())
-	    goto error;
-	  *end = '\0';
-	  History_Update_Line(str, end - str, h_no);
-	  Hist_Last(h_no);
-	  goto write_hist_line;
-
-
-	case KEY_ESC('?'):	/* display help */
-	display_help:
-	  NewLn();
-	  Display_Help();
-	  goto re_display_line;
-
-
-	default:
-	  if ((unsigned) c > 255 || !isprint(c))
-	    {
-	      n = c;
-	      EMIT_BEEP;
-	      c = Pl_LE_Get_Char();
-	      if (c != n)
-		goto one_char;
-	      goto display_help;
-	    }
-
-	  if (!New_Char(c, str, size, &pos, &end))
-	    goto error;
-	  /* brackets ([{ }]): matching */
-	  if (KBD_IS_NOT_EMPTY ||
-	      (q = strchr(CLOSE_BRACKET, c)) == NULL)
-	    continue;
-
-	  n = q - CLOSE_BRACKET;
-
-	  count_bracket[0] = count_bracket[1] = count_bracket[2] = 0;
-	  count_bracket[n]--;
-
-	  p = pos - 1;
-	  for (; count_bracket[n] != 0;)
-	    {
-	      if (--p < str)
-		goto bracket_exit;
-
-	      c = *p;
-	      if ((q = strchr(CLOSE_BRACKET, c)) != NULL)
-		{
-		  count_bracket[q - CLOSE_BRACKET]--;
-		  continue;
-		}
-
-	      if ((q = strchr(OPEN_BRACKET, c)) != NULL)
-		if (++count_bracket[q - OPEN_BRACKET] > 0)
-		  goto bracket_exit;
+        {
+        case KEY_CTRL('A'):     /* go to begin of line */
+        case KEY_EXT_HOME:
+          BACKD(pos - str);
+          pos = str;
+          continue;
+
+
+        case KEY_CTRL('E'):     /* go to end of line */
+        case KEY_EXT_END:
+          FORWD(end - pos, pos);
+          pos = end;
+          continue;
+
+
+        case KEY_CTRL('B'):     /* go to 1 char backward */
+        case KEY_EXT_LEFT:
+          if (pos == str)
+            goto error;
+          BACKD(1);
+          pos--;
+          continue;
+
+
+        case KEY_CTRL('F'):     /* go to 1 char forward */
+        case KEY_EXT_RIGHT:
+          if (pos == end)
+            goto error;
+          FORWD(1, pos);
+          pos++;
+          continue;
+
+
+        case KEY_BACKSPACE:     /* erase previous char */
+        case KEY_DELETE:
+          if (pos == str)
+            goto error;
+        del_last:
+          for (p = pos; p < end; p++)
+            p[-1] = *p;
+          BACKD(1);
+          pos--;
+          end--;
+          DISPL(end - pos, pos);
+          ERASE(1);
+          BACKD(end - pos);
+          continue;
+
+
+        case KEY_CTRL('D'):     /* erase current char */
+        case KEY_EXT_DELETE:
+          if (pos == end)
+            goto error;
+          /* simply equivalent to ^F + BACKSPACE */
+          FORWD(1, pos);
+          pos++;
+          goto del_last;
+
+
+        case KEY_CTRL('U'):     /* erase begin of line */
+        case KEY_CTRL_EXT_HOME:
+          q = clipboard;
+          p = str;
+          while (p < pos)       /* add deleted part to clipboard */
+            *q++ = *p++;
+          *q = '\0';
+          n = pos - str;
+          for (p = pos; p < end; p++)
+            p[-n] = *p;
+
+          pos = str;
+          end -= n;
+          BACKD(n);
+          DISPL(end - pos, pos);
+          ERASE(n);
+          BACKD(end - pos);
+          continue;
+
+
+        case KEY_CTRL('K'):     /* erase end of line */
+        case KEY_CTRL_EXT_END:
+          q = clipboard;
+          p = pos;
+          while (p < end)       /* add deleted part to clipboard */
+            *q++ = *p++;
+          *q = '\0';
+          ERASE(end - pos);
+          end = pos;
+          continue;
+
+
+        case KEY_CTRL('Y'):     /* paste from clipboard */
+          for (p = clipboard; *p; p++)
+            if (!New_Char(*p, str, size, &pos, &end))
+              goto error;
+          continue;
+
+
+        case KEY_CTRL(' '):     /* mark begin selection */
+          mark = pos;
+          continue;
+
+
+        case KEY_ESC('W'):      /* copy (from mark) to clipboard */
+        case KEY_CTRL('W'):     /* cut (from mark) to clipboard */
+          if (mark == NULL)
+            goto error;
+
+          if (mark < pos)
+            {
+              start = mark;
+              stop = pos;
+            }
+          else
+            {
+              start = pos;
+              stop = mark;
+            }
+          q = clipboard;
+          p = start;
+          while (p < stop)
+            *q++ = *p++;
+          *q = '\0';
+          if (c == KEY_ESC('W'))
+            continue;
+
+          n = stop - start;
+          for (p = stop; p < end; p++)
+            p[-n] = *p;
+
+          if (mark < pos)
+            BACKD(n);
+          pos = start;
+          end -= n;
+          DISPL(end - pos, pos);
+          ERASE(n);
+          BACKD(end - pos);
+          continue;
+
+
+        case KEY_ESC('B'):      /* go to previous word */
+        case KEY_CTRL_EXT_LEFT:
+          p = (pos == str) ? pos : pos - 1; /* to avoid start of a word */
+          p = Skip(p, str, 1, -1);      /* skip separators */
+          p = Skip(p, str, 0, -1);      /* skip non separators */
+          p = Skip(p, end, 1, +1);      /* skip separators */
+          BACKD(pos - p);
+          pos = p;
+          continue;
+
+
+        case KEY_ESC('F'):      /* go to next word */
+        case KEY_CTRL_EXT_RIGHT:
+          p = pos;
+          p = Skip(p, end, 0, +1);      /* skip non separators */
+          p = Skip(p, end, 1, +1);      /* skip separators */
+          FORWD(p - pos, pos);
+          pos = p;
+          continue;
+
+
+        case KEY_ESC('C'):      /* capitalize word */
+          p = pos;
+          p = Skip(p, end, 1, +1);      /* skip separators */
+          if (islower(*p))
+            *p = *p - 'a' + 'A';
+          p = Skip(p, end, 0, +1);      /* skip non separators */
+          DISPL(p - pos, pos);
+          pos = p;
+          continue;
+
+
+        case KEY_ESC('L'):      /* convert to lower case */
+          p = pos;
+          p = Skip(p, end, 1, +1);      /* skip separators */
+          for (; p < end && !Is_A_Separator(*p); p++)
+            *p = tolower(*p);
+          DISPL(p - pos, pos);
+          pos = p;
+          continue;
+
+
+        case KEY_ESC('U'):      /* convert to upper case */
+          p = pos;
+          p = Skip(p, end, 1, +1);      /* skip separators */
+          for (; p < end && !Is_A_Separator(*p); p++)
+            *p = toupper(*p);
+          DISPL(p - pos, pos);
+          pos = p;
+          continue;
+
+
+        case '\t':              /* TAB: completion */
+          if (tab_count != 0)   /* already a TAB */
+            {
+              if (++tab_count > NB_TAB_BEFORE_LIST)
+                {
+                  NewLn();
+                  Completion_Print_All();
+                  goto re_display_line;
+                }
+              goto error;
+            }
+          p = (pos == str) ? pos : pos - 1; /* to avoid start of a word */
+          p = Skip(p, str, 0, -1);      /* skip non separators */
+          p = Skip(p, end, 1, +1);      /* skip separators */
+          w = *pos;             /* prefix from p to pos */
+          *pos = '\0';
+          p = Completion_Do_Match(p, pos - p, &rest_length);
+          *pos = w;
+          if (p == NULL)
+            goto error;
+
+          while (rest_length--)
+            if (!New_Char(*p++, str, size, &pos, &end))
+              goto error;
+
+          if (comp_first_match != comp_last_match)
+            {
+              tab_count = 1;
+              goto error;       /* for the beep */
+            }
+          tab_count = 0;
+          continue;
+
+
+        case KEY_ESC('\t'):     /* transform a tab to spaces */
+          for (n = Tab_To_Spaces(pos - str); n; n--)
+            if (!New_Char(' ', str, size, &pos, &end))
+              goto error;
+          continue;
+
+
+        case KEY_CTRL('V'):     /* switch insert mode (on/off) */
+        case KEY_EXT_INSERT:
+          ins_mode = 1 - ins_mode;
+          INS_MODE(ins_mode);
+          continue;
+
+
+        case KEY_CTRL('T'):     /* swap last and current char */
+          if (pos == str || pos == end)
+            goto error;
+          w = pos[0];
+          pos[0] = pos[-1];
+          pos[-1] = w;
+          BACKD(1);
+          DISPL(2, pos - 1);
+          pos++;
+          continue;
+
+        case '\n':
+        case '\r':
+        return_is_read:
+          FORWD(end - pos, pos); /* go to EOL to avoid multi-line */
+        /* truncation on the output */
+        *end = '\0';
+        History_Add_Line(str, end - str);
+        if (end - str < size)   /* '\n' can be added */
+          *end++ = '\n';
+        *end = '\0';
+        goto finish;
+
+
+        case KEY_CTRL('P'):     /* history: recall previous line */
+        case KEY_EXT_UP:
+          if (Hist_Is_Empty() || h_no == Hist_Start_Entry())
+            goto error;
+          *end = '\0';
+          History_Update_Line(str, end - str, h_no);
+          Hist_Dec(h_no);
+        write_hist_line:
+          p = end;
+          end = str + History_Get_Line(str, h_no);
+          BACKD(pos - str);
+          DISPL(end - str, str);
+          if (end < p)
+            ERASE(p - end);
+          pos = end;
+          continue;
+
+
+        case KEY_CTRL('N'):     /* history: recall next line */
+        case KEY_EXT_DOWN:
+          if (Hist_Is_Empty() || h_no == Hist_End_Entry())
+            goto error;
+          *end = '\0';
+          History_Update_Line(str, end - str, h_no);
+          Hist_Inc(h_no);
+          goto write_hist_line;
+
+
+        case KEY_ESC('P'):      /* history: recall previous matching line */
+          if (Hist_Is_Empty() || pos == str)
+            goto error;
+          *end = '\0';
+          History_Update_Line(str, end - str, h_no);
+        try_previous:
+          if (h_no == Hist_Start_Entry())
+            goto error;
+          Hist_Dec(h_no);
+          if (hist_tbl[h_no].line == NULL ||
+              strncmp(str, hist_tbl[h_no].line, pos - str) != 0)
+            goto try_previous;
+        write_hist_match_line:
+          p = end;
+          end = str + History_Get_Line(str, h_no);
+          DISPL(end - pos, pos);
+          if (end < p)
+            ERASE(p - end);
+          BACKD(end - pos);
+          continue;
+
+
+        case KEY_ESC('N'):      /* history: recall next matching line */
+          if (Hist_Is_Empty() || pos == str)
+            goto error;
+          *end = '\0';
+          History_Update_Line(str, end - str, h_no);
+        try_next:
+          if (Hist_Is_Empty() || h_no == Hist_End_Entry())
+            goto error;
+          Hist_Inc(h_no);
+          if (hist_tbl[h_no].line == NULL ||
+              strncmp(str, hist_tbl[h_no].line, pos - str) != 0)
+            goto try_next;
+          goto write_hist_match_line;
+
+
+        case KEY_ESC('<'):      /* history: recall first line */
+        case KEY_EXT_PAGE_UP:
+          if (Hist_Is_Empty() || h_no == Hist_Start_Entry())
+            goto error;
+          *end = '\0';
+          History_Update_Line(str, end - str, h_no);
+          Hist_First(h_no);
+          goto write_hist_line;
+
+
+        case KEY_ESC('>'):      /* history: recall last line */
+        case KEY_EXT_PAGE_DOWN:
+          if (Hist_Is_Empty() || h_no == Hist_End_Entry())
+            goto error;
+          *end = '\0';
+          History_Update_Line(str, end - str, h_no);
+          Hist_Last(h_no);
+          goto write_hist_line;
+
+
+        case KEY_ESC('?'):      /* display help */
+        display_help:
+          NewLn();
+          Display_Help();
+          goto re_display_line;
+
+
+        default:
+          if ((unsigned) c > 255 || !isprint(c))
+            {
+              n = c;
+              EMIT_BEEP;
+              c = Pl_LE_Get_Char();
+              if (c != n)
+                goto one_char;
+              goto display_help;
+            }
+
+          if (!New_Char(c, str, size, &pos, &end))
+            goto error;
+          /* brackets ([{ }]): matching */
+          if (KBD_IS_NOT_EMPTY || (n = Search_Bracket(CLOSE_BRACKET, c)) < 0)
+            continue;
+
+          count_bracket[0] = count_bracket[1] = count_bracket[2] = 0;
+          count_bracket[n]--;
+
+          p = pos - 1;
+          for (; count_bracket[n] != 0;)
+            {
+              if (--p < str)
+                goto bracket_exit;
+
+              c = *p;
+              if ((n1 = Search_Bracket(CLOSE_BRACKET, c)) >= 0)
+                {
+                  count_bracket[n1]--;
+                  continue;
+                }
+
+              if ((n1 = Search_Bracket(OPEN_BRACKET, c)) >= 0)
+                if (++count_bracket[n1] > 0)
+                  goto bracket_exit;
 
 #ifdef IGNORE_QUOTED_PART
-	      if (p > str && (c == '\'' || c == '"') && p[-1] != '\\')
-		{		/* ignore quoted part */
-		  while (--p > str && (*p != c || p[-1] == '\\'))
-		    ;
-		}
+              if (p > str && (c == '\'' || c == '"') && p[-1] != '\\')
+                {               /* ignore quoted part */
+                  while (--p > str && (*p != c || p[-1] == '\\'))
+                    ;
+                }
 #endif
-	    }
+            }
 
-	  if (KBD_IS_NOT_EMPTY)
-	    continue;
+          if (KBD_IS_NOT_EMPTY)
+            continue;
 
-	  n = pos - p;
-	  q = pos;
-	  BACKD(n);
+          n = pos - p;
+          q = pos;
+          BACKD(n);
 #if defined(_WIN32) && !defined(__CYGWIN__)
 
-	  {
-	    long t0 = clock(), t1;
+          {
+            PlLong t0 = clock(), t1;
 
-	    do
-	      t1 = clock();
-	    while (!KBD_IS_NOT_EMPTY &&
-		   (t1 - t0) * 1000000 / CLOCKS_PER_SEC < BRACKET_TIMMING);
-	  }
+            do
+              t1 = clock();
+            while (!KBD_IS_NOT_EMPTY &&
+                   (t1 - t0) * 1000000 / CLOCKS_PER_SEC < BRACKET_TIMMING);
+          }
 
 #elif !defined(NO_USE_SELECT)
-	  {
-	    fd_set set;
-	    struct timeval t;
+          {
+            fd_set set;
+            struct timeval t;
 
-	    t.tv_sec = 0;
-	    t.tv_usec = BRACKET_TIMMING;
+            t.tv_sec = 0;
+            t.tv_usec = BRACKET_TIMMING;
 
-	    FD_ZERO(&set);
-	    FD_SET(0, &set);
-	    select(1, &set, NULL, NULL, &t);
-	  }
+            FD_ZERO(&set);
+            FD_SET(0, &set);
+            select(1, &set, NULL, NULL, &t);
+          }
 #else
-	  usleep(BRACKET_TIMMING);
+          usleep(BRACKET_TIMMING);
 #endif
-	  pos = p;
-	  FORWD(n, pos);
-	  pos = q;
-	bracket_exit:
-	  continue;
-	}
+          pos = p;
+          FORWD(n, pos);
+          pos = q;
+        bracket_exit:
+          continue;
+        }
     error:
-#ifndef _WIN32
       EMIT_BEEP;
-#else
-      ;
-#endif
     }
 
-finish:
+ finish:
   NewLn();
 
   Pl_LE_Close_Terminal();
@@ -830,13 +837,13 @@ New_Char(int c, char *str, int size, char **p_pos, char **p_end)
     {
       *pos = (char) c;
       if (++pos > end)
-	end = pos;
+        end = pos;
       PUT_CHAR(c);
     }
   else
     {
       for (p = end; p > pos; p--)
-	*p = p[-1];
+        *p = p[-1];
 
       *pos = (char) c;
       end++;
@@ -853,7 +860,6 @@ New_Char(int c, char *str, int size, char **p_pos, char **p_end)
 
 
 
-
 /*-------------------------------------------------------------------------*
  * PL_LE_GET_PROMPT_LENGTH                                                 *
  *                                                                         *
@@ -892,11 +898,11 @@ Pl_LE_Get_Current_Word(char *word)
   char *end = global_end;
   char *p, *q;
 
-  p = Skip(pos, str, 0, -1);	/* skip non separators */
+  p = Skip(pos, str, 0, -1);    /* skip non separators */
   if (Is_A_Separator(*p))
     p++;
 
-  q = Skip(pos, end, 0, +1);	/* skip non separators */
+  q = Skip(pos, end, 0, +1);    /* skip non separators */
 
   while (p < q)
     *word++ = *p++;
@@ -937,7 +943,7 @@ Pl_LE_Set_Separators(char *sep_str)
  * PL_LE_GET_CTRL_C_RETURN_VALUE                                           *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-long
+PlLong
 Pl_LE_Get_Ctrl_C_Return_Value(void)
 {
   return ctrl_c_ret_val;
@@ -956,7 +962,7 @@ Skip(char *from, char *limit, int res_sep_cmp, int direction)
   while (from != limit)
     {
       if (Is_A_Separator(*from) != res_sep_cmp)
-	break;			/* exit since *from does not satisfy res_sep_cmp */
+        break;                  /* exit since *from does not satisfy res_sep_cmp */
 
       from = from + direction;
     }
@@ -987,6 +993,25 @@ Is_A_Separator(char c)
 
 
 
+/*-------------------------------------------------------------------------*
+ * SEARCH_BRACKET                                                          *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+static int
+Search_Bracket(char *brackets, char c)
+{
+  int n;
+
+  for(n = 0; brackets[n] != '\0'; n++)
+    if (brackets[n] == c)
+      return n;
+
+  return -1;
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * TAB_TO_SPACES                                                           *
  *                                                                         *
@@ -1013,7 +1038,7 @@ History_Add_Line(char *line, int length)
 
   while (*p == ' ')
     p++;
-  if (*p == '\0')		/* do not add an empty line */
+  if (*p == '\0')               /* do not add an empty line */
     return;
 
 #ifdef NO_DUP_IN_HISTORY
@@ -1045,13 +1070,13 @@ History_Update_Line(char *line, int length, int hist_no)
   if (h->line != NULL && h->buff_length < length)
     {
       free(h->line);
-      h->line = NULL;		/* to ensure future malloc */
+      h->line = NULL;           /* to ensure future malloc */
     }
 
-  if (h->line == NULL)		/* not yet allocated */
+  if (h->line == NULL)          /* not yet allocated */
     {
       if ((h->line = (char *) malloc(length + 1)) == NULL)
-	exit(1);
+        exit(1);
       h->buff_length = length;
     }
 
@@ -1094,10 +1119,10 @@ Pl_LE_Compl_Add_Word(char *word, int word_length)
     {
       cmp = strcmp((*p)->word, word);
       if (cmp == 0)
-	return word;
+        return word;
 
       if (cmp > 0)
-	break;
+        break;
     }
 
   if ((q = (CompNode *) malloc(sizeof(CompNode))) == NULL)
@@ -1129,10 +1154,10 @@ Pl_LE_Compl_Del_Word(char *word)
     {
       cmp = strcmp((*p)->word, word);
       if (cmp == 0)
-	break;
+        break;
 
       if (cmp > 0)
-	return NULL;
+        return NULL;
     }
 
   q = *p;
@@ -1226,17 +1251,17 @@ Completion_Do_Match(char *prefix, int prefix_length, int *rest_length)
     {
       cmp = strncmp(p->word, prefix, prefix_length);
       if (cmp == 0)
-	{
-	  if (comp_first_match == NULL)
-	    comp_first_match = p;
+        {
+          if (comp_first_match == NULL)
+            comp_first_match = p;
 
-	  comp_last_match = p;
-	  comp_nb_match++;
-	  if (p->word_length > comp_match_max_lg)
-	    comp_match_max_lg = p->word_length;
-	}
+          comp_last_match = p;
+          comp_nb_match++;
+          if (p->word_length > comp_match_max_lg)
+            comp_match_max_lg = p->word_length;
+        }
       else if (cmp > 0)
-	break;
+        break;
     }
 
   if (comp_first_match == NULL)
@@ -1245,24 +1270,24 @@ Completion_Do_Match(char *prefix, int prefix_length, int *rest_length)
   if (comp_first_match == comp_last_match)
     *rest_length = comp_first_match->word_length - prefix_length;
   else
-    {				/* determine longest common suffix */
+    {                           /* determine longest common suffix */
       l = prefix_length;
       for (;;)
-	{
-	  w = comp_first_match->word[l];
-	  p = comp_first_match->next;
-	  for (;;)
-	    {
-	      if (p->word[l] != w)	/* also deals with '\0' */
-		goto diff_found;
+        {
+          w = comp_first_match->word[l];
+          p = comp_first_match->next;
+          for (;;)
+            {
+              if (p->word[l] != w)      /* also deals with '\0' */
+                goto diff_found;
 
-	      if (p == comp_last_match)
-		break;
-	      p = p->next;
-	    }
+              if (p == comp_last_match)
+                break;
+              p = p->next;
+            }
 
-	  l++;
-	}
+          l++;
+        }
     diff_found:
       *rest_length = l - prefix_length;
     }
@@ -1292,7 +1317,7 @@ Completion_Print_All(void)
 
   SCREEN_SIZE(&row, &col);
 
-  nb_in_a_line = col / (comp_match_max_lg + 2);	/* at least 2 chars to separate */
+  nb_in_a_line = col / (comp_match_max_lg + 2); /* at least 2 chars to separate */
 
   if (nb_in_a_line <= 1)
     nb_in_a_line = 1;
@@ -1305,14 +1330,14 @@ Completion_Print_All(void)
     ? 0 : (col - nb_in_a_line * comp_match_max_lg) / nb_in_a_line;
 
 
-  if (nb_lines > NB_MATCH_LINES_BEFORE_ASK)	/* too many matchings ? */
+  if (nb_lines > NB_MATCH_LINES_BEFORE_ASK)     /* too many matchings ? */
     {
       sprintf(buff, "Show all %d possibilities (y/n) ? ", comp_nb_match);
       DISPL_STR(buff);
       c = Pl_LE_Get_Char();
       NewLn();
       if (c != 'y')
-	return;
+        return;
     }
 
   p = comp_first_match;
@@ -1322,26 +1347,26 @@ Completion_Print_All(void)
       p1 = p;
       c = 0;
       for (;;)
-	{
-	  DISPL_STR(p1->word);
+        {
+          DISPL_STR(p1->word);
 
-	  if (++c == ((l < nb_lines - 1) ? nb_in_a_line : nb_in_last_line))
-	    break;
+          if (++c == ((l < nb_lines - 1) ? nb_in_a_line : nb_in_last_line))
+            break;
 
-	  sprintf(buff, "%*s", comp_match_max_lg - p1->word_length + spaces,
-		  "");
-	  DISPL_STR(buff);
+          sprintf(buff, "%*s", comp_match_max_lg - p1->word_length + spaces,
+                  "");
+          DISPL_STR(buff);
 
-	  skip = nb_lines;
-	  if (c > nb_in_a_line - nb_miss_in_last_line)
-	    skip--;
-	  for (k = 0; k < skip; k++)
-	    p1 = p1->next;
-	}
+          skip = nb_lines;
+          if (c > nb_in_a_line - nb_miss_in_last_line)
+            skip--;
+          for (k = 0; k < skip; k++)
+            p1 = p1->next;
+        }
 
       NewLn();
       if (++l == nb_lines)
-	break;
+        break;
 
       p = p->next;
     }
@@ -1362,8 +1387,8 @@ Display_Help(void)
 
   L("");
   sprintf(buff,
-	  "   linedit %-25s Copyright (C) 1999-2009 Daniel Diaz",
-	  LINEDIT_VERSION);
+          "   linedit %-25s Copyright (C) 1999-2011 Daniel Diaz",
+          LINEDIT_VERSION);
   L(buff);
   L("");
   L("                              Moving");
@@ -1419,7 +1444,7 @@ start:
     {
       Pl_LE_Close_Terminal();
       if ((ctrl_c_ret_val = Pl_Emit_Ctrl_C()) != 0)
-	return -2;
+        return -2;
       goto start;
     }
 
@@ -1445,7 +1470,7 @@ int
 Pl_LE_Printf(char *format, ...)
 {
   va_list arg_ptr;
-  char buff[1024];
+  static char buff[65535];
   int ret;
 
   Pl_LE_Initialize();
