@@ -6,23 +6,36 @@
  * Descr.: line editor - header file                                       *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: linedit.h,v 1.15 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: linedit.h,v 1.18 2011/03/28 16:20:05 diaz Exp $ */
 
 
 /* Windows uses 2 codepages (which give the meaning of 0x80..0xFF chars):
@@ -107,9 +120,9 @@ char *Pl_LE_Gets(char *str);
 char *Pl_LE_FGets(char *str, int size, char *prompt, int display_prompt);
 
 
-long Pl_LE_Get_Ctrl_C_Return_Value(void);
+PlLong Pl_LE_Get_Ctrl_C_Return_Value(void);
 
-#define LE_Interrupted_By_Ctrl_C(r)  ((long) r == -2L)
+#define LE_Interrupted_By_Ctrl_C(r)  ((PlLong) r == (PlLong) -2)
 
 
 int Pl_LE_Get_Prompt_Length(void);
