@@ -6,23 +6,36 @@
  * Descr.: test file                                                       *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: test_linedit.c,v 1.18 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: test_linedit.c,v 1.21 2011/03/28 16:20:05 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -32,6 +45,15 @@
 
 #include "../EnginePl/gp_config.h"
 
+
+#include "../W32GUICons/w32gc_interf.h" /* only to test GUI Console stack size dialog box */
+#ifdef GUI_CONSOLE_WITH_STACK_SIZES
+#define ENGINE_FILE                      /* to define stacks data */
+typedef PlLong WamWord;
+#include "../EnginePl/wam_stacks.h"
+#endif
+
+
 #if defined(__unix__) || defined(__CYGWIN__)
 #include <unistd.h>
 #else
@@ -69,7 +91,7 @@
  * CTRL_C_MANAGER                                                          *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-long
+PlLong
 Ctrl_C_Manager(int from_callback)
 {
   int c;
@@ -78,7 +100,7 @@ Ctrl_C_Manager(int from_callback)
   int nb, max_lg, is_last;
 
   printf("\nCATCHING CTRL+C prompt length: %d   current pos: %d\n",
-	 Pl_LE_Get_Prompt_Length(), Pl_LE_Get_Current_Position());
+         Pl_LE_Get_Prompt_Length(), Pl_LE_Get_Current_Position());
 
   printf("e: exit, c: continue, C: completions, w: current word: ");
   fflush(stdout);
@@ -101,13 +123,13 @@ Ctrl_C_Manager(int from_callback)
       printf("Enter a prefix:");
       Pl_LE_Gets(prefix);
       if ((str = Pl_LE_Compl_Init_Match(prefix, &nb, &max_lg)) == NULL)
-	printf("no matching\n");
+        printf("no matching\n");
       else
-	{
-	  printf("common=<%s> nb=%d max_lg=%d\n", str, nb, max_lg);
-	  while ((str = Pl_LE_Compl_Find_Match(&is_last)) != NULL)
-	    printf("matching: <%s>\n", str);
-	}
+        {
+          printf("common=<%s> nb=%d max_lg=%d\n", str, nb, max_lg);
+          while ((str = Pl_LE_Compl_Find_Match(&is_last)) != NULL)
+            printf("matching: <%s>\n", str);
+        }
       break;
     }
 
@@ -126,7 +148,7 @@ Set_Test_Locale(void)
 {
   time_t ltime;
   struct tm *thetime;
-  unsigned char str[100];
+  char str[100];
 
   setlocale(LC_ALL, "");
   time(&ltime);
@@ -171,16 +193,16 @@ main(int argc, char *argv[])
   printf("enter lines (EOF to finish)\n");
 
 #if 0
-  {				/* test space overflow in WIN32 GUI console */
+  {                             /* test space overflow in WIN32 GUI console */
     int i;
     char buf[256];
 
     for (i = 0; i < 280; i++)
       {
-	sprintf(buf,
-		"line %3d tttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttt\n",
-		i);
-	printf(buf);
+        sprintf(buf,
+                "line %3d tttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttt\n",
+                i);
+        printf(buf);
       }
 
   }
@@ -190,24 +212,30 @@ main(int argc, char *argv[])
     {
 #ifdef __unix__
       if (tempo)
-	{
-	  printf("tempo %d secs to allow you to buffer some chars:\n");
-	  sleep(tempo);
-	}
+        {
+          printf("tempo %d secs to allow you to buffer some chars:\n");
+          sleep(tempo);
+        }
 #endif
 #if 0
       printf("enter a line:");
       if (Pl_LE_Gets(line) == NULL)
 #else
-      if (Pl_LE_FGets(line, MAX_SIZE, "enter a line:", 1) == NULL)
+        if (Pl_LE_FGets(line, MAX_SIZE, "enter a line:", 1) == NULL)
 #endif
-	break;
+          break;
       printf("Line:(%s) len:%d\n", line, strlen(line));
       for (p = line; (p = strtok(p, sep)) != NULL; p = NULL)
-	{
-	  printf("adding word (%s) for completion\n", p);
-	  Pl_LE_Compl_Add_Word(strdup(p), strlen(p));
-	}
+        {
+          printf("adding word (%s) for completion\n", p);
+          Pl_LE_Compl_Add_Word(strdup(p), strlen(p));
+        }
+#if 0 // test an exception (under Win32 relaunch automatically the main !!! why ?
+      if (*line == 'k' && line[1] == '\0')
+        {
+          *(int *) 12 = 45;
+        }
+#endif
     }
 
   printf("End of testing\n");
