@@ -6,23 +6,36 @@
  * Descr.: test file                                                       *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: test_noecho.c,v 1.13 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: test_noecho.c,v 1.16 2011/03/28 16:20:05 diaz Exp $ */
 
 #include <stdio.h>
 #include <locale.h>
@@ -34,6 +47,12 @@
 #define printf Pl_LE_Printf
 
 
+#include "../W32GUICons/w32gc_interf.h"
+#ifdef GUI_CONSOLE_WITH_STACK_SIZES       /* only to test GUI Console stack size dialog box */
+#define ENGINE_FILE                      /* to define stacks data */
+typedef PlLong WamWord;
+#include "../EnginePl/wam_stacks.h"
+#endif
 
 
 /*---------------------------------*
@@ -66,7 +85,7 @@ main(int argc, char *argv[])
 
   setlocale(LC_ALL, "");
 
-  while ((c = LE_FGetc_No_Echo(0, 1)) != EOF)
+  while ((c = Pl_LE_Get_Key(0, 1)) != EOF)
     {
       printf("Read Char (%d)", c);
       if (isprint(c))
