@@ -6,23 +6,36 @@
  * Descr.: translation file for Linux on AMD x86-64                        *
  * Author: Gwenole Beauchesne                                              *
  *                                                                         *
- * Copyright (C) 2002 Gwenole Beauchesne                                   *
+ * Copyright (C) 1999-2011 Daniel Diaz and Gwenole Beauchesne              *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: x86_64_any.c,v 1.7 2008/11/04 14:35:45 diaz Exp $ */
+/* $Id: x86_64_any.c,v 1.11 2011/05/11 15:11:11 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -30,15 +43,15 @@
 #include <limits.h>
 
 
-/* For M_x86_64_linux/solaris: an important point is that C stack must be 
+/* For M_x86_64_linux/solaris: an important point is that C stack must be
  * aligned on 16 bytes else some problems occurs with double.
  * If this is not done and if the called function performs a movaps %xmm0,xx
  * an error will occur.
  * Just before calling a function %rsp is 16bytes aligned, %rsp = 0x...0
  * (4 low bits = 0). The callq instruction pushes the return address, so at
- * the entry of a function, %rsp is 0x...8. Gcc then adjusts (via subq) 
+ * the entry of a function, %rsp is 0x...8. Gcc then adjusts (via subq)
  * %rsp to be 0x...0 before calling a function. We mimic the same modifying
- * Call_Compiled to force %rsp to be 0x...0 when arriving in a Prolog code. 
+ * Call_Compiled to force %rsp to be 0x...0 when arriving in a Prolog code.
  * So a Prolog code can call C functions safely.
  * When a Prolog code finishes it returns into C inside Call_Prolog_Success
  * or Call_Prolog_Fail. In both functions we re-adjust the stack (gcc thinks
@@ -86,6 +99,17 @@ char asm_reg_cp[20];
 
 int w_label = 0;
 
+#ifdef _WIN32
+#define MAX_PR_ARGS 4
+static int pr_arg_no;
+static const char *gpr_arg[MAX_PR_ARGS] = {
+  "%rcx", "%rdx", "%r8", "%r9"
+};
+
+static const char *fpr_arg[MAX_PR_ARGS] = {
+  "%xmm0", "%xmm1", "%xmm2", "%xmm3"
+};
+#else
 #define MAX_GPR_ARGS 6
 static int gpr_arg_no;
 static const char *gpr_arg[MAX_GPR_ARGS] = {
@@ -99,8 +123,8 @@ static const char *fpr_arg[MAX_FPR_ARGS] = {
   "%xmm0", "%xmm1", "%xmm2", "%xmm3",
   "%xmm4", "%xmm5", "%xmm6", "%xmm7"
 };
-
-	  /* variables for ma_parser.c / ma2asm.c */
+#endif
+          /* variables for ma_parser.c / ma2asm.c */
 
 char *comment_prefix = "#";
 char *local_symb_prefix = ".L";
@@ -211,9 +235,11 @@ Code_Start(char *label, int prolog, int global)
     {
       union
       {
-	double d;
-	unsigned int w[2];
-      } dbl = { .d = dbl_tbl[i] };
+        double d;
+        unsigned int w[2];
+      } dbl;
+
+      dbl.d = dbl_tbl[i];
 
       Label_Printf("%s%d:", DOUBLE_PREFIX, x++);
       Inst_Printf(".long", "%d", dbl.w[0]);
@@ -223,7 +249,9 @@ Code_Start(char *label, int prolog, int global)
 
   Label_Printf("");
   Inst_Printf(".align", "16");
+#if defined(M_x86_64_linux) || defined(M_x86_64_bsd) || defined(M_x86_64_sco)
   Inst_Printf(".type", "%s,@function", label);
+#endif
 
   if (global)
     Inst_Printf(".globl", "%s", label);
@@ -443,56 +471,100 @@ Move_To_Reg_Y(int index)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-Call_C_Start(char *fct_name, int fc, int nb_args, int nb_args_in_words, 
-	     char **p_inline)
+Call_C_Start(char *fct_name, int fc, int nb_args, int nb_args_in_words,
+             char **p_inline)
 {
+#ifdef _WIN32
+  pr_arg_no = 0;
+#else
   gpr_arg_no = 0;
   fpr_arg_no = 0;
+#endif
 }
 
-#define BEFORE_ARG						\
-{								\
-  char r[10], *r_aux;						\
-  int r_eq_r_aux = 0;						\
-								\
-  if (gpr_arg_no < MAX_GPR_ARGS)				\
-    {								\
-      strcpy(r, gpr_arg[gpr_arg_no++]);				\
-      r_aux = r;						\
-      r_eq_r_aux = 1;						\
-    }								\
-  else								\
-    {								\
-      int nwords = offset - gpr_arg_no - fpr_arg_no;		\
-								\
-      sprintf(r, "%d(%%rsp)", nwords * 8);			\
-      r_aux = "%rax";						\
+#ifdef _WIN32
+#define BEFORE_ARG                                      \
+{                                                       \
+  char r[10], *r_aux;                                   \
+  int r_eq_r_aux = 0;                                   \
+                                                        \
+  if (pr_arg_no < MAX_PR_ARGS)                          \
+    {                                                   \
+      strcpy(r, gpr_arg[pr_arg_no++]);                  \
+      r_aux = r;                                        \
+      r_eq_r_aux = 1;                                   \
+    }                                                   \
+  else                                                  \
+    {                                                   \
+      int nwords = offset;                              \
+                                                        \
+      sprintf(r, "%d(%%rsp)", nwords * 8);              \
+      r_aux = "%rax";                                   \
     }
 
+#define BEFORE_FPR_ARG                                  \
+{                                                       \
+  char r[10], *r_aux;                                   \
+  int r_eq_r_aux = 0;                                   \
+                                                        \
+  if (pr_arg_no < MAX_PR_ARGS)                          \
+    {                                                   \
+      strcpy(r, fpr_arg[pr_arg_no++]);                  \
+      r_aux = r;                                        \
+      r_eq_r_aux = 1;                                   \
+    }                                                   \
+  else                                                  \
+    {                                                   \
+      int nwords = offset;                              \
+                                                        \
+      sprintf(r, "%d(%%rsp)", nwords * 8);              \
+      r_aux = "%xmm8";                                  \
+    }
+#else
 
-#define AFTER_ARG						\
+#define BEFORE_ARG                                      \
+{                                                       \
+  char r[10], *r_aux;                                   \
+  int r_eq_r_aux = 0;                                   \
+                                                        \
+  if (gpr_arg_no < MAX_GPR_ARGS)                        \
+    {                                                   \
+      strcpy(r, gpr_arg[gpr_arg_no++]);                 \
+      r_aux = r;                                        \
+      r_eq_r_aux = 1;                                   \
+    }                                                   \
+  else                                                  \
+    {                                                   \
+      int nwords = offset - gpr_arg_no - fpr_arg_no;    \
+                                                        \
+      sprintf(r, "%d(%%rsp)", nwords * 8);              \
+      r_aux = "%rax";                                   \
+    }
+
+#define BEFORE_FPR_ARG                                  \
+{                                                       \
+  char r[10], *r_aux;                                   \
+  int r_eq_r_aux = 0;                                   \
+                                                        \
+  if (fpr_arg_no < MAX_FPR_ARGS)                        \
+    {                                                   \
+      strcpy(r, fpr_arg[fpr_arg_no++]);                 \
+      r_aux = r;                                        \
+      r_eq_r_aux = 1;                                   \
+    }                                                   \
+  else                                                  \
+    {                                                   \
+      int nwords = offset - gpr_arg_no - fpr_arg_no;    \
+                                                        \
+      sprintf(r, "%d(%%rsp)", nwords * 8);              \
+      r_aux = "%xmm8";                                  \
+    }
+#endif
+
+#define AFTER_ARG                                       \
 }
 
-#define BEFORE_FPR_ARG						\
-{								\
-  char r[10], *r_aux;						\
-  int r_eq_r_aux = 0;						\
-								\
-  if (fpr_arg_no < MAX_FPR_ARGS)				\
-    {								\
-      strcpy(r, fpr_arg[fpr_arg_no++]);				\
-      r_aux = r;						\
-      r_eq_r_aux = 1;						\
-    }								\
-  else								\
-    {								\
-      int nwords = offset - gpr_arg_no - fpr_arg_no;		\
-								\
-      sprintf(r, "%d(%%rsp)", nwords * 8);			\
-      r_aux = "%xmm8";						\
-    }
-
-#define AFTER_FPR_ARG						\
+#define AFTER_FPR_ARG                                   \
 }
 
 
@@ -501,17 +573,17 @@ Call_C_Start(char *fct_name, int fc, int nb_args, int nb_args_in_words,
  *                                                                         *
  *-------------------------------------------------------------------------*/
 int
-Call_C_Arg_Int(int offset, long int_val)
+Call_C_Arg_Int(int offset, PlLong int_val)
 {
   BEFORE_ARG;
 
   if (LITTLE_INT(int_val))
-    Inst_Printf("movq", "$%ld,%s", int_val, r);
+    Inst_Printf("movq", "$%" PL_FMT_d ",%s", int_val, r);
   else
     {
-      Inst_Printf("movabsq", "$%ld,%s", int_val, r_aux);
+      Inst_Printf("movabsq", "$%" PL_FMT_d ",%s", int_val, r_aux);
       if (!r_eq_r_aux)
-	Inst_Printf("movq", "%s,%s", r_aux, r);
+        Inst_Printf("movq", "%s,%s", r_aux, r);
     }
 
   AFTER_ARG;
@@ -531,7 +603,9 @@ Call_C_Arg_Double(int offset, double dbl_val)
 
   dbl_tbl[nb_dbl++] = dbl_val;
 
-  Inst_Printf("movsd", "%s%d(%%rip),%s", DOUBLE_PREFIX, dbl_lc_no++, r);
+  Inst_Printf("movsd", "%s%d(%%rip),%s", DOUBLE_PREFIX, dbl_lc_no++, r_aux);
+  if (!r_eq_r_aux)
+    Inst_Printf("movq", "%s,%s", r_aux, r);
 
   AFTER_FPR_ARG;
 
@@ -566,12 +640,12 @@ Call_C_Arg_Mem_L(int offset, int adr_of, char *name, int index)
   BEFORE_ARG;
 
   if (adr_of)
-    Inst_Printf("movq", "$%s+%d,%s", name, index * 8, r);
+    Inst_Printf("movq", "$" "%s+%d,%s", name, index * 8, r);
   else
     {
       Inst_Printf("movq", "%s+%d(%%rip),%s", name, index * 8, r_aux);
       if (!r_eq_r_aux)
-	Inst_Printf("movq", "%s,%s", r_aux, r);
+        Inst_Printf("movq", "%s,%s", r_aux, r);
     }
 
   AFTER_ARG;
@@ -592,14 +666,14 @@ Call_C_Arg_Reg_X(int offset, int adr_of, int index)
   if (adr_of)
     {
       if (!r_eq_r_aux && index == 0)
-	{
+        {
 #ifdef NO_MACHINE_REG_FOR_REG_BANK
-	  Inst_Printf("movq", "$%s,%s", ASM_REG_BANK, r);
+          Inst_Printf("movq", "$%s,%s", ASM_REG_BANK, r);
 #else
-	  Inst_Printf("movq", "%s,%s", ASM_REG_BANK, r);
+          Inst_Printf("movq", "%s,%s", ASM_REG_BANK, r);
 #endif
-	  goto finish;
-	}
+          goto finish;
+        }
       Inst_Printf("leaq", "%s,%s", Off_Reg_Bank(index * 8), r_aux);
     }
   else
@@ -609,7 +683,7 @@ Call_C_Arg_Reg_X(int offset, int adr_of, int index)
     Inst_Printf("movq", "%s,%s", r_aux, r);
 
 finish:
-  ;				/* gcc3 does not like use of label at end of compound statement */
+  ;                             /* gcc3 does not like use of label at end of compound statement */
   AFTER_ARG;
 
   return 1;
@@ -654,7 +728,7 @@ Call_C_Arg_Foreign_L(int offset, int adr_of, int index)
     {
       Inst_Printf("movq", "pl_foreign_long+%d(%%rip),%s", index * 8, r_aux);
       if (!r_eq_r_aux)
-	Inst_Printf("movq", "%s, %s", r_aux, r);
+        Inst_Printf("movq", "%s, %s", r_aux, r);
     }
 
   AFTER_ARG;
@@ -751,7 +825,7 @@ Fail_Ret(void)
 void
 Move_Ret_To_Mem_L(char *name, int index)
 {
-  Inst_Printf("movq", "%%rax,%s+%d(%%rip)", name, index * 8);
+  Inst_Printf("movq", "%%rax," "%s+%d(%%rip)", name, index * 8);
 }
 
 
@@ -763,7 +837,7 @@ Move_Ret_To_Mem_L(char *name, int index)
  *-------------------------------------------------------------------------*/
 void
 Move_Ret_To_Reg_X(int index)
-{				/* similar to Move_To_Reg_X */
+{                               /* similar to Move_To_Reg_X */
   Inst_Printf("movq", "%%rax,%s", Off_Reg_Bank(index * 8));
 }
 
@@ -776,7 +850,7 @@ Move_Ret_To_Reg_X(int index)
  *-------------------------------------------------------------------------*/
 void
 Move_Ret_To_Reg_Y(int index)
-{				/* similar to Move_To_Reg_Y */
+{                               /* similar to Move_To_Reg_Y */
   Inst_Printf("movq", "%%rax,%d(%s)", Y_OFFSET(index), asm_reg_e);
 }
 
@@ -790,7 +864,7 @@ Move_Ret_To_Reg_Y(int index)
 void
 Move_Ret_To_Foreign_L(int index)
 {
-  Inst_Printf("movq", "%%rax, pl_foreign_long+%d(%%rip)", index * 8);
+  Inst_Printf("movq", "%%rax," "pl_foreign_long+%d(%%rip)", index * 8);
 }
 
 
@@ -803,7 +877,7 @@ Move_Ret_To_Foreign_L(int index)
 void
 Move_Ret_To_Foreign_D(int index)
 {
-  Inst_Printf("movsd", "%%xmm0, pl_foreign_double+%d(%%rip)", index * 8);
+  Inst_Printf("movsd", "%%xmm0," "pl_foreign_double+%d(%%rip)", index * 8);
 }
 
 
@@ -814,18 +888,18 @@ Move_Ret_To_Foreign_D(int index)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-Cmp_Ret_And_Int(long int_val)
+Cmp_Ret_And_Int(PlLong int_val)
 {
   if (int_val == 0)
     Inst_Printf("testq", "%%rax,%%rax");
   else if (LITTLE_INT(int_val))
-    Inst_Printf("cmpq", "$%ld,%%rax", int_val);
+    Inst_Printf("cmpq", "$%" PL_FMT_d ",%%rax", int_val);
   else
     {
       /* %rdx is second integral return value. At this stage, it is
          bound to be dead since we only deal with primitive object
          types.  */
-      Inst_Printf("movabsq", "$%ld,%%rdx", int_val);
+      Inst_Printf("movabsq", "$%" PL_FMT_d ",%%rdx", int_val);
       Inst_Printf("cmpq", "%%rdx,%%rax", int_val);
     }
 }
@@ -926,23 +1000,29 @@ Dico_Long_Start(int nb_longs)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-Dico_Long(char *name, int global, VType vtype, long value)
+Dico_Long(char *name, int global, VType vtype, PlLong value)
 {
   switch (vtype)
     {
     case NONE:
-      value = 1;		/* then in case ARRAY_SIZE */
+      value = 1;                /* then in case ARRAY_SIZE */
     case ARRAY_SIZE:
+#if defined(M_x86_64_linux) || defined(M_x86_64_sco) || \
+    defined(M_x86_64_solaris) || defined(M_x86_64_bsd)
       if (!global)
-	Inst_Printf(".local", "%s", name);
-      Inst_Printf(".comm", "%s,%ld,8", name, value * 8);
+        Inst_Printf(".local", "%s", name);
+      Inst_Printf(".comm", "%s,%" PL_FMT_d ",8", name, value * 8);
+#else
+      if (!global)
+        Inst_Printf(".lcomm", "%s,%" PL_FMT_d, name, value * 8);
+#endif
       break;
 
     case INITIAL_VALUE:
       if (global)
-	Inst_Printf(".globl", "%s", name);
+        Inst_Printf(".globl", "%s", name);
       Label_Printf("%s:", name);
-      Inst_Printf(".quad", "%ld", value);
+      Inst_Printf(".quad", "%" PL_FMT_d, value);
       break;
     }
 }
@@ -970,7 +1050,13 @@ Data_Start(char *initializer_fct)
   if (initializer_fct == NULL)
     return;
 
+#ifdef _MSC_VER
+  Inst_Printf(".section", ".GPLC$m");
+#elif defined( __CYGWIN__) || defined (_WIN32)
+  Inst_Printf(".section", ".ctors,\"aw\"");
+#else
   Inst_Printf(".section", ".ctors,\"aw\",@progbits");
+#endif
   Inst_Printf(".align", "8");
   Inst_Printf(".quad", "%s", initializer_fct);
 }
