@@ -6,23 +6,36 @@
  * Descr.: instruction codification (needed for register allocation)       *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: inst_codif.pl,v 1.14 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: inst_codif.pl,v 1.18 2011/03/28 16:20:05 diaz Exp $ */
 
           % alias stopping instructions
 
@@ -102,10 +115,12 @@ codif(unify_value(x(Tmp)), [r(Tmp)]).
 
 codif(unify_local_value(x(Tmp)), [r(Tmp)]).
 
-codif(call(_ / N), LCode) :-
+codif(call(T), LCode) :-
+	( T = _ / N ; T = _:_/N ), !,
 	lst_r_for_call_execute(0, N, LCode).
 
-codif(execute(_ / N), LCode) :-
+codif(execute(T), LCode) :-
+	( T = _ / N ; T = _:_/N ), !,
 	lst_r_for_call_execute(0, N, LCode).
 
 codif(load_cut_level(Tmp), [w(Tmp)]).
