@@ -6,23 +6,41 @@
  * Descr.: main file                                                       *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: pl2wam.pl,v 1.15 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: pl2wam.pl,v 1.21 2011/06/10 14:48:48 diaz Exp $ */
+
+pl2wam(Arg) :-
+	atom(Arg),
+	Arg \== [], !,			% to call easily inder top-level
+	pl2wam([Arg]).
 
 pl2wam(LArg) :-
 	catch(pl2wam1(LArg), Err, exception(Err)).
@@ -33,18 +51,18 @@ pl2wam(LArg) :-
 pl2wam1(LArg) :-
 	cmd_line_args(LArg, PlFile, WamFile),
 	prolog_file_name(PlFile, PlFile1),
-	read_file_init(PlFile),
-	emit_code_init(WamFile, PlFile),
 	g_read(native_code, NativeCode),
 	compile_msg_start(PlFile1, NativeCode),
+	read_file_init(PlFile),
+	emit_code_init(WamFile, PlFile),
 	init_counters,
 	repeat,
 	read_predicate(Pred, N, LSrcCl),
 	add_counter(user_read_file, real_read_file),
-	(   LSrcCl = []                                   % [] at end of file
-	                ->
+	(   LSrcCl = [] ->	% [] at end of file
 	    !
-	;   read_file_error_nb(0),
+	;
+	    read_file_error_nb(0),
 	    compile_and_emit_pred(NativeCode, Pred, N, LSrcCl),
 	    fail
 	),
@@ -131,7 +149,8 @@ compile_msg_start(PlFile, NativeCode) :-
 	    Type = 'native code'
 	;   Type = 'byte code'
 	),
-	format('compiling ~a for ~a...~n', [PlFile, Type]).
+	format('compiling ~a for ~a...~n', [PlFile, Type]),
+	flush_output.
 
 
 
@@ -254,7 +273,6 @@ cmd_line_args(LArg, PlFile, WamFile) :-
 	g_assign(native_code, t),
 	g_assign(susp_warn, t),
 	g_assign(singl_warn, t),
-	g_assign(mult_warn, t),
 	g_assign(redef_error, t),
 	g_assign(foreign_only, f),
 	g_assign(call_c, t),
@@ -330,9 +348,6 @@ cmd_line_arg1('--no-susp-warn', LArg, LArg) :-
 cmd_line_arg1('--no-singl-warn', LArg, LArg) :-
 	g_assign(singl_warn, f).
 
-cmd_line_arg1('--no-mult-warn', LArg, LArg) :-
-	g_assign(mult_warn, f).
-
 cmd_line_arg1('--no-redef-error', LArg, LArg) :-
 	g_assign(redef_error, f).
 
@@ -416,7 +431,8 @@ display_copying :-
 	nl,
 	format('~a comes with ABSOLUTELY NO WARRANTY.~n', [Name]),
 	format('You may redistribute copies of ~a~n', [Name]),
-	format('under the terms of the GNU General Public License.~n', []),
+	format('under the terms of the GNU Lesser General Public License~n', []),
+	format('or of the terms of the GNU General Public License (or both in parallel)~n', []),
 	format('For more information about these matters, see the files named COPYING.~n', []).
 
 
@@ -448,7 +464,6 @@ h('  -w, --wam-for-byte-code     produce a WAM file for byte-code (force --no-ca
 h('  --pl-state FILE             read FILE to set the initial Prolog state').
 h('  --no-susp-warn              do not show warnings for suspicious predicates').
 h('  --no-singl-warn             do not show warnings for named singleton variables').
-h('  --no-mult-warn              do not show warnings for multifile directives').
 h('  --no-redef-error            do not show errors for built-in redefinitions').
 h('  --foreign-only              only compile foreign/1-2 directives').
 h('  --no-call-c                 do not allow the use of fd_tell, ''$call_c'',...').
