@@ -6,23 +6,36 @@
  * Descr.: source file reading                                             *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: read_file.pl,v 1.16 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: read_file.pl,v 1.25 2011/06/10 16:24:15 diaz Exp $ */
 
 /*-------------------------------------------------------------------------*
  * Data structures:                                                        *
@@ -79,6 +92,13 @@
  * ensure_linked(Pred,N):                                                  *
  *    asserted for each Pred/N occuring in a :- ensure_linked directive.   *
  *                                                                         *
+ * module_export(Pred,N,Module):                                           *
+ *    asserted for each imported Pred/N from Module.                       *
+ *    asserted for each exported Pred/N from Module.                       *
+ *                                                                         *
+ * meta_pred(Pred,N,MetaDecl):                                             *
+ *    asserted for each meta_predicate declaration.                        *
+ *                                                                         *
  * Buffers for executable directive management (with assert/retract):      *
  *                                                                         *
  * buff_exe_system(SrcDirec)                                               *
@@ -115,10 +135,15 @@ read_file_init(PlFile) :-
 	retractall(empty_dyn_pred(_, _, _)),
 	retractall(ensure_linked(_, _)),
 	retractall(pred_info(_, _, _)),
+	retractall(module_export(_, _, _)),
+	retractall(meta_pred(_, _, _)),
+	g_assign(module, user),
+	g_assign(module_already_seen, f),
 	g_assign(default_kind, user),
 	g_assign(reading_dyn_pred, f),
 	g_assign(eof_reached, f),
 	g_assign(open_file_stack, []),
+	g_assign(if_stack, []),
 	g_assign(where, 0),
 	g_assign(syn_error_nb, 0),
 	g_assign(in_lines, 0),
@@ -148,7 +173,14 @@ open_new_prolog_file(PlFile0) :-
 	;   open(PlFile, read, Stream)
 	),
 	g_read(open_file_stack, OpenFileStack),
-	g_assign(open_file_stack, [PlFile * Stream|OpenFileStack]).
+	g_assign(open_file_stack, [PlFile * Stream|OpenFileStack]),
+	(   peek_char(Stream, '#'), % ignore #! starting line (for shebang support)
+	    repeat,
+	    get_char(Stream, X),
+	    (X = '\n' ; X = end_of_file)
+	;
+	    true
+	).
 
 
 
@@ -185,7 +217,7 @@ read_predicate(Pred, N, LSrcCl) :-
 
 
 read_predicate_next(Pred, N, LSrcCl) :-
-	test_pred_info(dyn, Pred, N), !,
+	(test_pred_info(dyn, Pred, N) ; test_pred_info(multi, Pred, N)), !,
 	LSrcCl = [Where + _|_],
 	add_dyn_interf_clause(Pred, N, Where),
 	create_exe_clauses_for_dyn_pred(LSrcCl, Pred, N),
@@ -284,8 +316,7 @@ group_clauses_by_pred(Pred, N, SrcCl, [SrcCl|LSrcCl1]) :-
 
 
 add_dyn_interf_clause(Pred, N, _) :-
-	clause(buff_dyn_interf_clause(Pred, N, _), true),  % already asserted
-	                                                  !.
+	clause(buff_dyn_interf_clause(Pred, N, _), true), !.  % already asserted
 
 add_dyn_interf_clause(Pred, N, Where) :-
 	create_dyn_interf_clause(Pred, N, Where, SrcCl),
@@ -315,8 +346,9 @@ create_exe_clauses_for_dyn_pred([], _, _).
 
 create_exe_clauses_for_dyn_pred([SrcCl|LSrcCl], Pred, N) :-
 	SrcCl = Where + Cl,
+	get_file_name(Where, PlFile),
 	add_wrapper_to_dyn_clause(Pred, N, Where + Cl, AuxName),
-	handle_initialization(system, '$add_clause_term_and_bc'(Cl, [execute(AuxName / N)]), Where),
+	handle_initialization(system, ('$call_c'('Pl_Emit_BC_Execute_Wrapper'(Pred, N, '&', AuxName, N), [by_value]), '$add_clause_term'(Cl, PlFile)), Where),
 	create_exe_clauses_for_dyn_pred(LSrcCl, Pred, N).
 
 
@@ -325,10 +357,14 @@ create_exe_clauses_for_dyn_pred([SrcCl|LSrcCl], Pred, N) :-
 create_exe_clauses_for_pub_pred([]).
 
 create_exe_clauses_for_pub_pred([Where + Cl|LSrcCl]) :-
-	handle_initialization(system, '$add_clause_term'(Cl), Where),
+	get_file_name(Where, PlFile),
+	handle_initialization(system, '$add_clause_term'(Cl, PlFile), Where),
 	create_exe_clauses_for_pub_pred(LSrcCl).
 
 
+get_file_name([PlFile * _|_] + _, PlFile).
+
+
 
 
 get_next_clause(Pred, N, SrcCl) :-
@@ -354,6 +390,8 @@ get_next_clause(Pred, N, SrcCl) :-
 	;   get_next_clause(Pred, N, SrcCl)
 	), !.
 
+
+
 get_next_clause1(end_of_file, _, _, Pred, N, SrcCl) :-
 	close_last_prolog_file,
 	g_read(open_file_stack, OpenFileStack),
@@ -361,10 +399,69 @@ get_next_clause1(end_of_file, _, _, Pred, N, SrcCl) :-
 	    Pred = end_of_file,
 	    N = 0,
 	    SrcCl = _ + end_of_file,
-	    g_assign(eof_reached, t)
+	    g_assign(eof_reached, t),
+	    (   g_read(if_stack, []) ->
+		true
+	    ;
+		error('endif directive expected', [])
+	    )
+
 	;   get_next_clause(Pred, N, SrcCl)
 	).
 
+				% +++++ begin preprocessor management +++++
+get_next_clause1((:- if(Goal)), _, _, Pred, N, SrcCl) :-
+	!,
+	g_read(if_stack, IfStack),
+	(   '$catch'(Goal, Err, (warn('if directive caused exception: ~w', [Err]), fail), any, 0, false) ->
+	    g_assign(if_stack, [if(then, 1)|IfStack])
+	;
+	    g_assign(if_stack, [if(then, 0)|IfStack])
+	),
+	get_next_clause(Pred, N, SrcCl).
+
+get_next_clause1((:- elif(Goal)), _, _, Pred, N, SrcCl) :-
+	!,
+	(   g_read(if_stack, [if(then, Keep)|IfStack]) ->
+	    (   Keep = 0 ->
+		(   '$catch'(Goal, Err, (warn('elif directive caused exception: ~w', [Err]), fail), any, 0, false) ->
+		    g_assign(if_stack, [if(then, 1)|IfStack])
+		;
+		    g_assign(if_stack, [if(then, 0)|IfStack])
+		)
+	    ;
+		g_assign(if_stack, [if(then, 2)|IfStack])  % 2 means ignore both the then and else part
+	    )
+	;
+	    error('unexpected elif directive', [])
+	),
+	get_next_clause(Pred, N, SrcCl).
+
+get_next_clause1((:- else), _, _, Pred, N, SrcCl) :-
+	!,
+	(   g_read(if_stack, [if(then, Keep)|IfStack]) ->
+	    Keep1 is 1 - Keep,
+	    g_assign(if_stack, [if(else, Keep1)|IfStack])
+	;
+	    error('unexpected else directive', [])
+	),
+	get_next_clause(Pred, N, SrcCl).
+
+
+get_next_clause1((:- endif), _, _, Pred, N, SrcCl) :-
+	!,
+	(   g_read(if_stack, [if(_, _)|IfStack]) ->
+	    g_assign(if_stack, IfStack)
+	;
+	    error('unexpected endif directive', [])
+	),
+	get_next_clause(Pred, N, SrcCl).
+
+get_next_clause1(_, _, _, Pred, N, SrcCl) :- % preprocessor ignores a clause
+	g_read(if_stack, [if(_, Keep)|_]), Keep \== 1, !, % ignore Keep = 0 and Keep = 2 or 1-2 = -1
+	get_next_clause(Pred, N, SrcCl).
+				% +++++ end preprocessor management +++++
+
 get_next_clause1((:- D), Where, SingNames, Pred, N, SrcCl) :-
 	display_singletons(SingNames, directive),
 	(   g_read(foreign_only, f)
@@ -389,6 +486,8 @@ get_next_clause1(Cl, Where, SingNames, Pred, N, Where + Cl) :-
 	;   error('head is not a callable (~q)', [Head])
 	),
 	functor(Head, Pred, N),
+	check_head_is_module_free(Head),
+	check_module_clash(Pred, N),
 	check_predicate(Pred, N),
 	display_singletons(SingNames, Pred / N).
 
@@ -464,6 +563,11 @@ handle_directive(dynamic, DLst, Where) :-
 	set_flag_for_preds(DLst, pub),
 	add_empty_dyn(DLst, Where).
 
+handle_directive(multifile, DLst, _) :-
+	!,
+	DLst \== [],
+	set_flag_for_preds(DLst, multi).
+
 handle_directive(discontiguous, DLst, _) :-
 	!,
 	DLst \== [],
@@ -493,13 +597,9 @@ handle_directive(ensure_linked, DLst, _) :-
 	    add_ensure_linked(DLst)
 	).
 
-handle_directive(multifile, _, _) :-
+handle_directive(encoding, _, _) :-
 	!,
-	(   g_read(mult_warn, t) ->
-	    warn('multifile directive not supported - directive ignored', [])
-	;
-	    true
-	).
+	warn('encoding directive not supported - directive ignored', []).
 
 handle_directive(ensure_loaded, _, _) :-
 	!,
@@ -533,6 +633,32 @@ handle_directive(initialization, [Body], Where) :-
 	handle_initialization(user, Body, Where).
 
 
+handle_directive(module, [Module, DLst], _) :-
+	!,
+	(   g_read(module_already_seen, f) ->
+	    check_module_name(Module, false),
+	    g_assign(module_already_seen, t),
+	    g_assign(module, Module),
+	    add_module_export_info(DLst, Module)
+	;
+	    error('directive module/2 already declared', [])
+	).
+
+handle_directive(use_module, [Module, DLst], _) :-
+	!,
+	check_module_name(Module, false),
+	add_module_export_info(DLst, Module).
+
+handle_directive(meta_predicate, [MetaDecl], _) :-
+	!,
+	(   callable(MetaDecl) ->
+	    functor(MetaDecl, Pred, N),
+	    set_flag_for_preds(Pred/N, meta),
+	    assertz(meta_pred(Pred, N, MetaDecl))
+	;
+	    error('invalide directive meta_predicate/1 ~w', [MetaDecl])
+	).
+
 handle_directive(foreign, [Template], Where) :-
 	!,
 	handle_directive(foreign, [Template, []], Where).
@@ -729,6 +855,100 @@ add_ensure_linked(Pred / N) :-
 
 
 
+add_module_export_info([], _) :-
+	!.
+
+add_module_export_info([P1|P2], Module) :-
+	!,
+	add_module_export_info(P1, Module),
+	add_module_export_info(P2, Module).
+
+add_module_export_info((P1, P2), Module) :-
+	!,
+	add_module_export_info(P1, Module),
+	add_module_export_info(P2, Module).
+
+add_module_export_info(Pred / N, _) :-
+	clause(module_export(Pred, N, Module1), true), !,
+	error('predicate ~w already exported from module ~w', [Pred/N, Module1]).
+
+add_module_export_info(Pred / N, Module) :-
+	assertz(module_export(Pred, N, Module)),
+	(   test_pred_info(def, Pred, N) ->
+	    check_module_clash(Pred, N)
+	;
+	    true
+	).
+
+
+check_module_name(Module, true) :-
+	var(Module), !.
+
+check_module_name(Module, _) :-
+	atom(Module), !.
+
+check_module_name(Module, _) :-
+	error('invalid module name (~q) should be an atom', [Module]).
+
+/*
+check_module_name(Module, _) :-
+	atom(Module),
+	\+ atom_property(Module, needs_quotes), !.
+
+check_module_name(Module, _) :-
+	error('invalid module name (~q) should only containts lower chars', [Module]).
+*/
+
+
+
+check_head_is_module_free(Module:Head) :-
+	!,
+	error('module qualification is not allowed for the head of a clause (~w)', [Module:Head]).
+
+check_head_is_module_free(_).
+
+
+
+
+check_module_clash(Pred, N) :-  % Pred/N is defined in current module check for clash with an import
+	clause(module_export(Pred, N, Module), true),
+	g_read(module, Module1),
+	Module \== Module1, !,
+	error('clash on ~q - defined in module ~q (here) and imported from ~w', [Pred / N, Module1, Module]).
+
+check_module_clash(_, _).
+
+
+
+
+
+get_owner_module(Pred, N, Module) :-
+	clause(module_export(Pred, N, Module), true),
+	Module \== system, !.
+
+get_owner_module(_, _, _).
+
+
+
+is_exported(Pred, N) :-
+	clause(module_export(Pred, N, _), true), !.
+
+
+
+
+get_module_of_cur_pred(Module) :-
+	cur_pred(Pred, N),
+	(   test_pred_info(bpl, Pred, N) ->
+	    Module = system
+	;   test_pred_info(bfd, Pred, N) ->
+	    Module = system
+	;
+	    g_read(module, Module)
+	).
+
+
+
+
 set_flag_for_preds([], _) :-
 	!.
 
@@ -786,6 +1006,8 @@ flag_bit(bpl, 3).
 flag_bit(bfd, 4).
 flag_bit(discontig, 5).
 flag_bit(cut, 6).
+flag_bit(meta, 7).
+flag_bit(multi, 8).
 
 
 
@@ -822,13 +1044,13 @@ test_pred_info(Flag, F, N) :-
 
 check_predicate(Pred, N) :-
 	g_read(redef_error, t),
-	bip(Pred, N), !,
-	error('redefining built-in predicate ~q', [Pred / N]).
+	control_construct(Pred, N), !,
+	error('redefining control construct ~q', [Pred / N]).
 
 check_predicate(Pred, N) :-
 	g_read(redef_error, t),
-	control_construct(Pred, N), !,
-	error('redefining control construct ~q', [Pred / N]).
+	bip(Pred, N), !,
+	error('redefining built-in predicate ~q', [Pred / N]).
 
 check_predicate(Pred, N) :-
 	g_read(susp_warn, t),
@@ -843,6 +1065,13 @@ check_predicate(_, _).
 
 
 
+bip(F, N) :-
+	'$predicate_property1'(F, N, built_in), !.
+/* no longer needed built_in_fd ==> built_in
+bip(F, N) :-
+	'$predicate_property1'(F, N, built_in_fd).
+*/
+
 
 control_construct(',', 2).
 control_construct(;, 2).
@@ -854,15 +1083,15 @@ control_construct(call, 1).
 control_construct(catch, 3).
 control_construct(throw, 1).
 
-suspicious_predicate(',', 2).
-suspicious_predicate(;, 2).
-suspicious_predicate(->, 2).
-suspicious_predicate(!, 0).
+%suspicious_predicate(',', 2).
+%suspicious_predicate(;, 2).
+%suspicious_predicate(->, 2).
+%suspicious_predicate(!, 0).
+suspicious_predicate(:, 2).
 suspicious_predicate(:-, 1).
 suspicious_predicate(:-, 2).
 suspicious_predicate(-->, 2).
-suspicious_predicate({}, X) :-
-	X < 2 .
+suspicious_predicate({}, X) :- X < 2.
 suspicious_predicate(+, 2).
 suspicious_predicate(-, 2).
 suspicious_predicate(*, 2).
