@@ -6,23 +6,36 @@
  * Descr.: pass 1: syntactic sugar removing                                *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: syn_sugar.pl,v 1.16 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: syn_sugar.pl,v 1.21 2011/03/28 16:20:05 diaz Exp $ */
 
 syntactic_sugar_init_pred(Pred, _) :-
 	'$aux_name'(Pred), !.
@@ -52,9 +65,9 @@ normalize_cuts(Body, Body2) :-
 	).
 
 
-normalize_cuts1(X, VarCut, P) :-
+normalize_cuts1(X, CutVar, P) :-
 	var(X),
-	normalize_cuts1(call(X), VarCut, P).
+	normalize_cuts1(call(X), CutVar, P).
 
 normalize_cuts1((If ; R), CutVar, Body) :-
 	nonvar(If),
@@ -78,7 +91,18 @@ normalize_cuts1((P ; Q), CutVar, (P1 ; Q1)) :-
 	normalize_cuts1(P, CutVar, P1),
 	normalize_cuts1(Q, CutVar, Q1).
 
+normalize_cuts1(M:G, CutVar, Body) :-
+	check_module_name(M, true),
+	normalize_cuts1(G, CutVar, G1),
+	distrib_module_qualif(G1, M, G2),
+	(   G2 = M3:_, var(M3) ->
+	    normalize_cuts1(call(G2), CutVar, Body)
+	;
+	    Body = G2
+	).
+
 normalize_cuts1(call(G), _, '$call'(G, Func, Arity, true)) :-
+%	get_module_of_cur_pred(Module), % then use a '$call'(G, Module, Func, Arity, true)
 	cur_pred_without_aux(Func, Arity).
 
 normalize_cuts1(catch(G, C, R), _, '$catch'(G, C, R, Func, Arity, true)) :-
@@ -87,15 +111,50 @@ normalize_cuts1(catch(G, C, R), _, '$catch'(G, C, R, Func, Arity, true)) :-
 normalize_cuts1(throw(B), _, '$throw'(B, Func, Arity, true)) :-
 	cur_pred_without_aux(Func, Arity).
 
-normalize_cuts1(P, _, P) :-
+normalize_cuts1(P, _, P1) :-
 	(   callable(P) ->
-	    true
+	    meta_pred_rewriting(P, P1)
 	;   error('body goal is not callable (~q)', [P])
 	).
 
 
 
 
+distrib_module_qualif((P ; Q), M, (P1 ; Q1)) :-
+	!,
+	distrib_module_qualif(P, M, P1),
+	distrib_module_qualif(Q, M, Q1).
+
+distrib_module_qualif((P -> Q), M, (P1 -> Q1)) :-
+	!,
+	distrib_module_qualif(P, M, P1),
+	distrib_module_qualif(Q, M, Q1).
+
+distrib_module_qualif((P , Q), M, (P1 , Q1)) :-
+	!,
+	distrib_module_qualif(P, M, P1),
+	distrib_module_qualif(Q, M, Q1).
+
+distrib_module_qualif(M:G, _, G1) :-
+	!,
+	check_module_name(M, true),
+	distrib_module_qualif(G, M, G1).
+
+distrib_module_qualif(!, _, !) :-
+	!.
+
+distrib_module_qualif(P, M, M:P).
+
+distrib_module_qualif_goal(G, _, G) :-
+	nonvar(G),
+	G = M:_, !,		% already qualifed with a module
+	check_module_name(M, true).
+
+distrib_module_qualif_goal(G, M, M:G).
+
+
+
+
 normalize_alts(Body, Head, Body1) :-
 	functor(Head, Pred, N),
 	g_assign(head_functor, Pred),
@@ -192,15 +251,12 @@ pred_rewriting(fd_tell(X), T) :-                          % FD transformation
 
 pred_rewriting(set_bip_name(Name, Arity), Pred1) :-
 	g_read(inline, t),      % also if byte code since implies --no-inline
-	nonvar(Name),
-	nonvar(Arity),
-	(   atom(Name)
-	;   error('set_bip_name/2 inline: atom expected instead of ~q', [Name])
+	(   atom(Name), integer(Arity) ->
+	    CallC = '$call_c'('Pl_Set_Bip_Name_Untagged_2'(Name, Arity), [by_value])
+	;
+	    CallC = '$call_c'('Pl_Set_Bip_Name_2'(Name, Arity))
 	),
-	(   integer(Arity)
-	;   error('set_bip_name/2 inline: integer expected instead of ~q', [Arity])
-	),
-	pred_rewriting('$call_c'('Pl_Set_Bip_Name_2'(Name, Arity)), Pred1).
+	pred_rewriting(CallC, Pred1).
 
 pred_rewriting(Pred, Pred1) :-                     % math define current bip
 	g_read(inline, t),      % also if byte code since implies --no-inline
@@ -209,7 +265,7 @@ pred_rewriting(Pred, Pred1) :-                     % math define current bip
 	(   F = (is)
 	;   math_cmp_functor_name(F, _)
 	),					            % see code_gen.pl
-	pred_rewriting('$call_c'('Pl_Set_Bip_Name_2'(F, 2)), T),
+	pred_rewriting(set_bip_name(F, 2), T),
 	Pred1 = (T, Pred).
 
 
@@ -280,3 +336,46 @@ add_wrapper_to_dyn_clause(Pred, N, Where + Cl, AuxName) :-
 head_wrapper(Head, AuxName, Head1) :-
 	Head =.. [_|LArgs],
 	Head1 =.. [AuxName|LArgs].
+
+
+
+	% meta_predicate rewriting
+
+meta_pred_rewriting(P1, P2) :-
+	nonvar(P1),
+	functor(P1, Pred, N),
+	clause(meta_pred(Pred, N, MetaDecl), _), !,
+	functor(P2, Pred, N),
+	meta_pred_rewrite_args(1, N, P1, MetaDecl, P2).
+
+meta_pred_rewriting(P, P).
+
+
+
+
+meta_pred_rewrite_args(I, N, P1, MetaDecl, P2) :-
+	I =< N, !,
+	arg(I, P1, A1),
+	arg(I, P2, A2),
+	arg(I, MetaDecl, Spec),
+	meta_pred_rewrite_arg(Spec, A1, A2),
+	I1 is I + 1,
+	meta_pred_rewrite_args(I1, N, P1, MetaDecl, P2).
+
+meta_pred_rewrite_args(_, _, _, _, _).
+
+
+
+
+meta_pred_rewrite_arg(Spec, A, A) :-
+	var(Spec), !.
+
+meta_pred_rewrite_arg(:, A1, A2) :-
+	!,
+	meta_pred_rewrite_arg(0, A1, A2).
+
+meta_pred_rewrite_arg(Spec, A1, '$mt'(Module, A1)) :-
+	integer(Spec), !,
+	get_module_of_cur_pred(Module).
+
+meta_pred_rewrite_arg(_, A, A).
