@@ -6,23 +6,36 @@
  * Descr.: code emission                                                   *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: wam_emit.pl,v 1.16 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: wam_emit.pl,v 1.20 2011/03/28 16:20:05 diaz Exp $ */
 
 /*-------------------------------------------------------------------------*
  * WAM Instructions                                                        *
@@ -37,6 +50,7 @@
  * get_nil(A)                               put_nil(A)                     *
  * get_list(A)                              put_list(A)                    *
  * get_structure(F/N, A)                    put_structure(F/N, A)          *
+ *                                          hide_meta_term(A)              *
  *                                                                         *
  *                                          math_load_value(V, A)          *
  *                                          math_fast_load_value(V, A)     *
@@ -97,12 +111,8 @@ emit_code_init(WamFile0, PlFile0) :-
 	g_assign(cur_pl_file, ''),
 	prolog_name(Name),
 	prolog_version(Version),
-	prolog_date(Date),
-	date_time(dt(Yr, Mh, Dy, Hr, Me, Sd)),
-	format(Stream, '%% compiler: ~a ~a (~a)~n', [Name, Version, Date]),
-	format(Stream, '%% file    : ~a~n', [PlFile]),
-	format(Stream, '%% date    : ~d ~d ~d~n', [Mh, Dy, Yr]),
-	format(Stream, '%% time    : ~d:~d:~d~n', [Hr, Me, Sd]).
+	format(Stream, '%% compiler: ~a ~a~n', [Name, Version]),
+	format(Stream, '%% file    : ~a~n', [PlFile]).
 
 
 
@@ -166,13 +176,40 @@ emit_pred_start(Pred, N, PlFile, PlLine, Stream, _) :-
 	    PubPriv = public
 	;   PubPriv = private
 	),
-	(   test_pred_info(bpl, Pred, N) ->
-	    UsBplBfd = built_in
-	;   test_pred_info(bfd, Pred, N) ->
-	    UsBplBfd = built_in_fd
-	;   UsBplBfd = user
+	(   test_pred_info(multi, Pred, N) ->
+	    MonoMulti = multifile
+	;   MonoMulti = monofile
 	),
-	format(Stream, '~n~npredicate(~q,~d,~a,~a,~a,', [Pred / N, PlLine, StaDyn, PubPriv, UsBplBfd]).
+	g_read(module, Module0),
+	export_type(Pred, N, Module0, _Module, ExportBplBfd),
+  		% MODULES: then add Module:Pred/N instead of Pred/N in the next line
+	format(Stream, '~n~npredicate(~q,~d,~a,~a,~a,~a,',
+	       [Pred/N, PlLine, StaDyn, PubPriv, MonoMulti, ExportBplBfd]).
+
+
+
+export_type(Pred, _, Module, Module, local) :-
+	'$aux_name'(Pred), !.
+
+export_type(Pred, N, Module, Module, local) :-
+	test_pred_info(multi, Pred, N), !.
+
+export_type(Pred, N, _, system, built_in) :-
+	test_pred_info(bpl, Pred, N), !.
+
+export_type(Pred, N, _, system, built_in_fd) :-
+	test_pred_info(bfd, Pred, N), !.
+
+export_type(Pred, N, system, system, built_in) :-  % an exported pred in system is a built_in - remove if wanted
+	is_exported(Pred, N), !.
+
+export_type(_, _, Module, Module, global) :-
+	g_read(module_already_seen, f), !.
+
+export_type(Pred, N, Module, Module, global) :-
+	is_exported(Pred, N), !.
+
+export_type(_, _, Module, Module, local).
 
 
 
