@@ -6,31 +6,55 @@
  * Descr.: line editor <--> GUICons interface                              *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2011 Daniel Diaz                                     *
  *                                                                         *
- * GNU Prolog is free software; you can redistribute it and/or modify it   *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; either version 2, or any later version.       *
+ * This file is part of GNU Prolog                                         *
  *                                                                         *
- * GNU Prolog is distributed in the hope that it will be useful, but       *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        *
+ * GNU Prolog is free software: you can redistribute it and/or             *
+ * modify it under the terms of either:                                    *
+ *                                                                         *
+ *   - the GNU Lesser General Public License as published by the Free      *
+ *     Software Foundation; either version 3 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or                                                                      *
+ *                                                                         *
+ *   - the GNU General Public License as published by the Free             *
+ *     Software Foundation; either version 2 of the License, or (at your   *
+ *     option) any later version.                                          *
+ *                                                                         *
+ * or both in parallel, as here.                                           *
+ *                                                                         *
+ * GNU Prolog is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details.                                *
  *                                                                         *
- * You should have received a copy of the GNU General Public License along *
- * with this program; if not, write to the Free Software Foundation, Inc.  *
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.               *
+ * You should have received copies of the GNU General Public License and   *
+ * the GNU Lesser General Public License along with this program.  If      *
+ * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: w32gc_interf.c,v 1.15 2009/01/23 11:24:14 diaz Exp $ */
+/* $Id: w32gc_interf.c,v 1.18 2011/03/28 16:20:05 diaz Exp $ */
 
 #include <stdio.h>
 #include <windows.h>
 
-
 #include "../EnginePl/gp_config.h"
 
 
+/* we include the minimum from ../EnginePl to allow for testing (test_linedit, test_noecho)
+ * with Win GUI console separately (see project MS Visual C++ in ../Linedit)
+ * this implies some redundant definitions (WamWord, Wam_Words_To_KBytes)
+ */
+
+typedef PlLong WamWord;
+#include "../EnginePl/wam_stacks.h"
+#define Wam_Words_To_KBytes(ww)    (ww * sizeof(WamWord) / 1024)
+
+#include "w32gc_interf.h"
+
+
 /* from linedit.h */
 
 extern char *Pl_LE_Get_Separators(void);
@@ -51,6 +75,10 @@ extern int Pl_LE_Get_Prompt_Length(void);
 
 static void Start_GUI(void);
 
+#ifdef GUI_CONSOLE_WITH_STACK_SIZES
+static PlLong Query_Stack(QueryStackCmd cmd, int stack_no);
+#endif
+
 /* overwrite to customize linedit */
 
 void (*pl_le_hook_start) () = Start_GUI;
@@ -87,9 +115,11 @@ void (*pl_le_hook_exit_process) ();
 
 static FARPROC Find_Fct(HANDLE h, char *name);
 
-
+#ifdef __GNUC__
+typedef int (*Fct) ();
+#else
 typedef __declspec(dllimport) int (*Fct) ();
-
+#endif
 
 /*-------------------------------------------------------------------------*
  * START_GUI                                                               *
@@ -109,16 +139,14 @@ Start_GUI(void)
 
   if (h == NULL)
     {
-      fprintf(stderr,
-	      "warning: cannot load DLL " DLL_W32GUICONS
+      fprintf(stderr, "warning: cannot load DLL " DLL_W32GUICONS
 	      " - text console used\n");
       pl_le_hook_start = NULL;
       return;
     }
   pl_le_hook_put_char = (void (*)()) Find_Fct(h, "_W32GC_Put_Char");
   pl_le_hook_get_char0 = (int (*)()) Find_Fct(h, "_W32GC_Get_Char0");
-  pl_le_hook_kbd_is_not_empty =
-    (int (*)()) Find_Fct(h, "_W32GC_Kbd_Is_Not_Empty");
+  pl_le_hook_kbd_is_not_empty = (int (*)()) Find_Fct(h, "_W32GC_Kbd_Is_Not_Empty");
   pl_le_hook_screen_size = (void (*)()) Find_Fct(h, "_W32GC_Screen_Size");
 
   /* the following are not mandatory for linedit and could be commented out */
@@ -133,10 +161,8 @@ Start_GUI(void)
 
   /* the following are not used by linedit but by stream_supp.c */
 
-  pl_le_hook_set_line_buffering =
-    (void (*)()) Find_Fct(h, "_W32GC_Set_Line_Buffering");
-  pl_le_hook_get_line_buffering =
-    (int (*)()) Find_Fct(h, "_W32GC_Get_Line_Buffering");
+  pl_le_hook_set_line_buffering = (void (*)()) Find_Fct(h, "_W32GC_Set_Line_Buffering");
+  pl_le_hook_get_line_buffering = (int (*)()) Find_Fct(h, "_W32GC_Get_Line_Buffering");
   pl_le_hook_flush = (void (*)()) Find_Fct(h, "_W32GC_Flush");
 
   pl_le_hook_confirm_box = (int (*)()) Find_Fct(h, "_W32GC_Confirm_Box");
@@ -144,7 +170,13 @@ Start_GUI(void)
   pl_le_hook_exit_process = (void (*)()) Find_Fct(h, "_W32GC_Exit_Process");
 
   W32GC_Start_Window = (Fct) Find_Fct(h, "_W32GC_Start_Window");
-  (*W32GC_Start_Window) (Pl_LE_Get_Separators, Pl_LE_Get_Prompt_Length);
+  (*W32GC_Start_Window) (Pl_LE_Get_Separators, Pl_LE_Get_Prompt_Length,
+#ifdef GUI_CONSOLE_WITH_STACK_SIZES
+                         Query_Stack
+#else
+                         NULL
+#endif
+                         );
 }
 
 
@@ -177,3 +209,42 @@ Find_Fct(HANDLE h, char *name)
 
   return p;
 }
+
+
+#ifdef GUI_CONSOLE_WITH_STACK_SIZES
+
+/*-------------------------------------------------------------------------*
+ * QUERY_STACK                                                             *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+static PlLong
+Query_Stack(QueryStackCmd cmd, int stack_no)
+{
+  switch(cmd)
+    {
+    case QUERY_STACK_GET_NB_OF_STACKS:
+      return (PlLong) NB_OF_STACKS;
+
+    case QUERY_STACK_HAS_FIXED_SIZES:
+      return (PlLong) pl_fixed_sizes;
+
+    case QUERY_STACK_GET_NAME:
+      return (PlLong) pl_stk_tbl[stack_no].name;
+
+    case QUERY_STACK_GET_DESC:
+      return (PlLong) pl_stk_tbl[stack_no].desc;
+
+    case QUERY_STACK_GET_ENV_VAR_NAME:
+      return (PlLong) pl_stk_tbl[stack_no].env_var_name;
+
+    case QUERY_STACK_GET_DEFAULT_SIZE:
+      return (PlLong) Wam_Words_To_KBytes(pl_stk_tbl[stack_no].default_size);
+
+    case QUERY_STACK_GET_SIZE:
+      return (PlLong) Wam_Words_To_KBytes(pl_stk_tbl[stack_no].size);
+    }
+
+  return 0;
+}
+
+#endif  /* GUI_CONSOLE_WITH_STACK_SIZES */
